@@ -1,13 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
  * Copyright (C) 2016 Freescale Semiconductor, Inc.
  * Copyright 2017-2018 NXP
- *
- * The code contained herein is licensed under the GNU General Public
- * License. You may obtain a copy of the GNU General Public License
- * Version 2 or later at the following locations:
- *
- * http://www.opensource.org/licenses/gpl-license.html
- * http://www.gnu.org/copyleft/gpl.html
+ *   Author: Dong Aisheng <aisheng.dong@nxp.com>
  */
 
 #include <linux/delay.h>
@@ -52,18 +47,16 @@
 #define CSRE	0x34
 #define MR	0x40
 
-#define PMC_HSRUN		0x4
-#define PMC_RUN			0x8
-#define PMC_VLPR		0xc
-#define PMC_STOP		0x10
-#define PMC_VLPS		0x14
-#define PMC_LLS			0x18
-#define PMC_VLLS		0x1c
-#define PMC_STATUS		0x20
-#define PMC_CTRL		0x24
-#define PMC_SRAMCTRL_0		0x28
-#define PMC_SRAMCTRL_1		0x2c
-#define PMC_SRAMCTRL_2		0x30
+#define PMC1_HSRUN		0x4
+#define PMC1_RUN		0x8
+#define PMC1_VLPR		0xc
+#define PMC1_STOP		0x10
+#define PMC1_VLPS		0x14
+#define PMC1_LLS		0x18
+#define PMC1_VLLS		0x1c
+#define PMC1_STATUS		0x20
+#define PMC1_CTRL		0x24
+#define PMC0_CTRL		0x28
 
 #define BM_PMPROT_AHSRUN	(1 << 7)
 #define BM_PMPROT_AVLP		(1 << 5)
@@ -86,7 +79,7 @@
 #define BP_PMCTRL_STOPM		0
 #define BP_PMCTRL_PSTOPO	16
 
-#define MX7ULP_MAX_MMDC_IO_NUM		36
+#define MX7ULP_MAX_MMDC_IO_NUM		64
 #define MX7ULP_MAX_MMDC_NUM		50
 #define MX7ULP_MAX_IOMUX_NUM		116
 #define MX7ULP_MAX_SELECT_INPUT_NUM	78
@@ -124,6 +117,13 @@
 
 #define ADDR_1M_MASK	0xFFF00000
 
+#define WDOG_CS			0x0
+#define WDOG_CS_CMD32EN		BIT(13)
+#define WDOG_CNT	0x4
+#define REFRESH_SEQ0	0xA602
+#define REFRESH_SEQ1	0xB480
+#define REFRESH		((REFRESH_SEQ1 << 16) | REFRESH_SEQ0)
+
 static void __iomem *smc1_base;
 static void __iomem *pmc0_base;
 static void __iomem *pmc1_base;
@@ -134,10 +134,14 @@ static void __iomem *pcc2_base;
 static void __iomem *pcc3_base;
 static void __iomem *mu_base;
 static void __iomem *scg1_base;
+static void __iomem *wdog1_base;
 static void __iomem *gpio_base[4];
+static void __iomem *port_pcr_base[4];
 static void __iomem *suspend_ocram_base;
 static void (*imx7ulp_suspend_in_ocram_fn)(void __iomem *sram_base);
 
+static u32 port_pcr[4][20];
+static u32 port_num[4] = {20, 12, 16, 20};
 static u32 tpm5_regs[4];
 static u32 lpuart4_regs[4];
 static u32 pcc2_regs[24][2] = {
@@ -190,12 +194,22 @@ struct imx7ulp_pm_socdata {
 };
 
 static const u32 imx7ulp_mmdc_io_lpddr3_offset[] __initconst = {
-	0x128, 0xf8, 0xd8, 0x108,
-	0x104, 0x124, 0x80, 0x84,
-	0x88, 0x8c, 0x120, 0x10c,
-	0x110, 0x114, 0x118, 0x90,
-	0x94, 0x98, 0x9c, 0xe0,
-	0xe4,
+	0x0,  0x4,  0x8,  0xc,
+	0x10, 0x14, 0x18, 0x1c,
+	0x20, 0x24, 0x28, 0x2c,
+	0x30, 0x34, 0x38, 0x3c,
+	0x40, 0x44, 0x48, 0x4c,
+	0x50, 0x54, 0x58, 0x5c,
+	0x60, 0x64, 0x68, 0x6c,
+	0x70, 0x74, 0x78, 0x7c,
+	0x80, 0x84, 0x88, 0x8c,
+	0x90, 0x94, 0x98, 0x9c,
+	0xa0, 0xa4, 0xa8, 0xac,
+	0xb0, 0xb4, 0xb8, 0xbc,
+	0xc0, 0xc4, 0xc8, 0xcc,
+	0xd0, 0xd4, 0xd8, 0xdc,
+	0xe8, 0xf8, 0xfc, 0x120,
+	0x124,
 };
 
 static const u32 imx7ulp_mmdc_lpddr3_offset[] __initconst = {
@@ -273,11 +287,6 @@ static void __iomem *aips3_base;
 static void __iomem *aips4_base;
 static void __iomem *aips5_base;
 
-static const char * const low_power_ocram_match[] __initconst = {
-	"fsl,lpm-sram",
-	NULL
-};
-
 static void imx7ulp_gpio_save(void)
 {
 	int i;
@@ -288,6 +297,24 @@ static void imx7ulp_gpio_save(void)
 	}
 }
 
+static void imx7ulp_port_pcr_save(void)
+{
+	int i;
+	int j;
+	for (i = 0; i < 4; i++)
+		for (j=0; j < port_num[i]; j++)
+			port_pcr[i][j] = readl_relaxed(port_pcr_base[i] + j * 4);
+}
+
+static void imx7ulp_port_pcr_restore(void)
+{
+	int i;
+	int j;
+	for (i = 0; i < 4; i++)
+		for (j=0; j < port_num[i]; j++)
+			writel_relaxed(port_pcr[i][j], port_pcr_base[i] + j * 4);
+}
+
 static void imx7ulp_scg1_save(void)
 {
 	int i;
@@ -392,33 +419,33 @@ static void imx7ulp_set_dgo(u32 val)
 	writel_relaxed(val, pm_info->sim_base + DGO_GPR4);
 }
 
-int imx7ulp_set_lpm(enum imx7ulp_cpu_pwr_mode mode)
+int imx7ulp_set_lpm(enum ulp_cpu_pwr_mode mode)
 {
 	u32 val1 = BM_PMPROT_AHSRUN | BM_PMPROT_AVLP | BM_PMPROT_AVLLS;
 	u32 val2 = readl_relaxed(smc1_base + PMCTRL);
-	u32 val3 = readl_relaxed(pmc0_base + PMC_CTRL);
+	u32 val3 = readl_relaxed(pmc0_base + PMC0_CTRL);
 
 	val2 &= ~(BM_PMCTRL_RUNM |
 		BM_PMCTRL_STOPM | BM_PMCTRL_PSTOPO);
 	val3 |= BM_CTRL_LDOOKDIS;
 
 	switch (mode) {
-	case RUN:
+	case ULP_PM_RUN:
 		/* system/bus clock enabled */
 		val2 |= 0x3 << BP_PMCTRL_PSTOPO;
 		break;
-	case WAIT:
+	case ULP_PM_WAIT:
 		/* system clock disabled, bus clock enabled */
 		val2 |= 0x2 << BP_PMCTRL_PSTOPO;
 		break;
-	case STOP:
+	case ULP_PM_STOP:
 		/* system/bus clock disabled */
 		val2 |= 0x1 << BP_PMCTRL_PSTOPO;
 		break;
-	case VLPS:
+	case ULP_PM_VLPS:
 		val2 |= 0x2 << BP_PMCTRL_STOPM;
 		break;
-	case VLLS:
+	case ULP_PM_VLLS:
 		val2 |= 0x4 << BP_PMCTRL_STOPM;
 		break;
 	default:
@@ -427,7 +454,7 @@ int imx7ulp_set_lpm(enum imx7ulp_cpu_pwr_mode mode)
 
 	writel_relaxed(val1, smc1_base + PMPROT);
 	writel_relaxed(val2, smc1_base + PMCTRL);
-	writel_relaxed(val3, pmc0_base + PMC_CTRL);
+	writel_relaxed(val3, pmc0_base + PMC0_CTRL);
 
 	return 0;
 }
@@ -451,35 +478,48 @@ static int imx7ulp_suspend_finish(unsigned long val)
 	else
 		state = MX7ULP_SUSPEND_STANDBY_PARAM;
 
-	if (psci_ops.cpu_suspend) {
+	if (psci_ops.cpu_suspend)
 		return psci_ops.cpu_suspend(state, __pa(cpu_resume));
-	}
 
 	imx7ulp_suspend_in_ocram_fn(suspend_ocram_base);
 
 	return 0;
 }
 
+static void imx7ulp_wdog_refresh(void)
+{
+	/*
+	 * On revision 2.2, wdog2 is by default disabled when out of
+	 * reset, so here, we ONLY refresh wdog1.
+	 */
+	if (readl_relaxed(wdog1_base + WDOG_CS) & WDOG_CS_CMD32EN) {
+		writel(REFRESH, wdog1_base + WDOG_CNT);
+	} else {
+		writel_relaxed(REFRESH_SEQ0, wdog1_base + WDOG_CNT);
+		writel_relaxed(REFRESH_SEQ1, wdog1_base + WDOG_CNT);
+	}
+}
+
 static int imx7ulp_pm_enter(suspend_state_t state)
 {
 	switch (state) {
 	case PM_SUSPEND_STANDBY:
-		if (psci_ops.cpu_suspend) {
+		if (psci_ops.cpu_suspend)
 			/* Zzz ... */
 			cpu_suspend(1, imx7ulp_suspend_finish);
-		} else {
-			imx7ulp_set_lpm(VLPS);
+		else {
+			imx7ulp_set_lpm(ULP_PM_VLPS);
 			writel_relaxed(
-				readl_relaxed(pmc1_base + PMC_VLPS) | BM_VLPS_RBBEN,
-				pmc1_base + PMC_VLPS);
+				readl_relaxed(pmc1_base + PMC1_VLPS) | BM_VLPS_RBBEN,
+				pmc1_base + PMC1_VLPS);
 
 			/* Zzz ... */
 			cpu_suspend(0, imx7ulp_suspend_finish);
 
 			writel_relaxed(
-				readl_relaxed(pmc1_base + PMC_VLPS) & ~BM_VLPS_RBBEN,
-				pmc1_base + PMC_VLPS);
-			imx7ulp_set_lpm(RUN);
+				readl_relaxed(pmc1_base + PMC1_VLPS) & ~BM_VLPS_RBBEN,
+				pmc1_base + PMC1_VLPS);
+			imx7ulp_set_lpm(ULP_PM_RUN);
 		}
 		break;
 	case PM_SUSPEND_MEM:
@@ -488,6 +528,7 @@ static int imx7ulp_pm_enter(suspend_state_t state)
 			cpu_suspend(0, imx7ulp_suspend_finish);
 		} else {
 			imx7ulp_gpio_save();
+			imx7ulp_port_pcr_save();
 			imx7ulp_scg1_save();
 			imx7ulp_pcc2_save();
 			imx7ulp_pcc3_save();
@@ -495,31 +536,34 @@ static int imx7ulp_pm_enter(suspend_state_t state)
 			if (!console_suspend_enabled)
 				imx7ulp_lpuart_save();
 			imx7ulp_iomuxc_save();
-			imx7ulp_set_lpm(VLLS);
+			imx7ulp_set_lpm(ULP_PM_VLLS);
 
 			/* Zzz ... */
 			cpu_suspend(0, imx7ulp_suspend_finish);
 
 			imx7ulp_pcc2_restore();
 			imx7ulp_pcc3_restore();
+			imx7ulp_port_pcr_restore();
 			if (!console_suspend_enabled)
 				imx7ulp_lpuart_restore();
 			imx7ulp_set_dgo(0);
 			imx7ulp_tpm_restore();
-			imx7ulp_set_lpm(RUN);
+			imx7ulp_set_lpm(ULP_PM_RUN);
 	}
 		break;
 	default:
 		return -EINVAL;
 	}
 
+	imx7ulp_wdog_refresh();
+
 	return 0;
 }
 
 /* Put CA7 into VLLS mode before M4 power off CA7 */
 void imx7ulp_poweroff(void)
 {
-	imx7ulp_set_lpm(VLLS);
+	imx7ulp_set_lpm(ULP_PM_VLLS);
 	cpu_suspend(0, imx7ulp_suspend_finish);
 }
 
@@ -554,7 +598,7 @@ static int __init imx7ulp_dt_find_lpsram(unsigned long node, const char *uname,
 	unsigned long lpram_addr;
 	const __be32 *prop = of_get_flat_dt_prop(node, "reg", NULL);
 
-	if (of_flat_dt_match(node, low_power_ocram_match)) {
+	if (of_flat_dt_is_compatible(node, "fsl,lpm-sram")) {
 		if (!prop)
 			return -EINVAL;
 
@@ -668,6 +712,7 @@ void __init imx7ulp_pm_common_init(const struct imx7ulp_pm_socdata
 
 	np = of_find_compatible_node(NULL, NULL, "fsl,imx7ulp-smc1");
 	smc1_base = of_iomap(np, 0);
+	of_node_put(np);
 	WARN_ON(!smc1_base);
 
 	np = of_find_compatible_node(NULL, NULL, "fsl,imx7ulp-pmc0");
@@ -694,7 +739,7 @@ void __init imx7ulp_pm_common_init(const struct imx7ulp_pm_socdata
 	pcc3_base = of_iomap(np, 0);
 	WARN_ON(!pcc3_base);
 
-	np = of_find_compatible_node(NULL, NULL, "fsl,imx7ulp-iomuxc-1");
+	np = of_find_compatible_node(NULL, NULL, "fsl,imx7ulp-iomuxc1");
 	iomuxc1_base = of_iomap(np, 0);
 	WARN_ON(!iomuxc1_base);
 
@@ -702,10 +747,16 @@ void __init imx7ulp_pm_common_init(const struct imx7ulp_pm_socdata
 	scg1_base = of_iomap(np, 0);
 	WARN_ON(!scg1_base);
 
+	np = of_find_compatible_node(NULL, NULL, "fsl,imx7ulp-wdt");
+	wdog1_base = of_iomap(np, 0);
+	WARN_ON(!wdog1_base);
+
 	np = NULL;
 	for (i = 0; i < 4; i++) {
 		np = of_find_compatible_node(np, NULL, "fsl,vf610-gpio");
+		port_pcr_base[i] = of_iomap(np, 0);
 		gpio_base[i] = of_iomap(np, 1);
+		WARN_ON(!port_pcr_base[i]);
 		WARN_ON(!gpio_base[i]);
 	}
 
@@ -782,10 +833,20 @@ void __init imx7ulp_pm_common_init(const struct imx7ulp_pm_socdata
 	}
 }
 
+u32 imx7ulp_get_mode(void)
+{
+	u32 mode;
+
+	mode = readl_relaxed(smc1_base + PMCTRL) & BM_PMCTRL_RUNM;
+	mode >>= 8;
+
+	return mode;
+}
+
 void __init imx7ulp_pm_init(void)
 {
 	imx7ulp_pm_common_init(&imx7ulp_lpddr3_pm_data);
-	imx7ulp_set_lpm(RUN);
+	imx7ulp_set_lpm(ULP_PM_RUN);
 }
 
 static irqreturn_t imx7ulp_nmi_isr(int irq, void *param)
