@@ -17,6 +17,7 @@
 #include <linux/init.h>
 #include <linux/module.h>
 #include <linux/interrupt.h>
+#include <linux/iopoll.h>
 #include <linux/clk.h>
 #include <linux/dma-mapping.h>
 #include <linux/dmapool.h>
@@ -27,9 +28,8 @@
 #include <linux/of_address.h>
 #include <linux/of_irq.h>
 #include <linux/of_dma.h>
-
-#include <soc/imx/revision.h>
-#include <soc/imx8/soc.h>
+#include <linux/pm_runtime.h>
+#include <linux/pm_domain.h>
 
 #include "virt-dma.h"
 
@@ -38,6 +38,7 @@
 #define EDMA_CH_INT			0x08
 #define EDMA_CH_SBR			0x0C
 #define EDMA_CH_PRI			0x10
+#define EDMA_CH_MUX			0x14
 #define EDMA_TCD_SADDR			0x20
 #define EDMA_TCD_SOFF			0x24
 #define EDMA_TCD_ATTR			0x26
@@ -97,8 +98,8 @@
 #define EDMA_TCD_CSR_D_REQ		BIT(3)
 #define EDMA_TCD_CSR_E_SG		BIT(4)
 #define EDMA_TCD_CSR_E_LINK		BIT(5)
-#define EDMA_TCD_CSR_ACTIVE		BIT(6)
-#define EDMA_TCD_CSR_DONE		BIT(7)
+#define EDMA_TCD_CSR_EEOP		BIT(6)
+#define EDMA_TCD_CSR_ESDA		BIT(7)
 
 #define FSL_EDMA_BUSWIDTHS	(BIT(DMA_SLAVE_BUSWIDTH_1_BYTE) | \
 				BIT(DMA_SLAVE_BUSWIDTH_2_BYTES) | \
@@ -113,11 +114,9 @@
 /* channel name template define in dts */
 #define CHAN_PREFIX			"edma0-chan"
 #define CHAN_POSFIX			"-tx"
+#define CLK_POSFIX			"-clk"
 
-enum fsl_edma3_pm_state {
-	RUNNING = 0,
-	SUSPENDED,
-};
+#define EDMA_MINOR_LOOP_TIMEOUT		500 /* us */
 
 struct fsl_edma3_hw_tcd {
 	__le32	saddr;
@@ -131,6 +130,11 @@ struct fsl_edma3_hw_tcd {
 	__le32	dlast_sga;
 	__le16	csr;
 	__le16	biter;
+	/*
+	 * Store Dest Address if EDMA_TCD_CSR_ESDA enabled which's used as
+	 * 'eeop + cyclic'.
+	 */
+	__le32  sda;
 };
 
 struct fsl_edma3_sw_tcd {
@@ -150,9 +154,7 @@ struct fsl_edma3_slave_config {
 struct fsl_edma3_chan {
 	struct virt_dma_chan		vchan;
 	enum dma_status			status;
-	enum fsl_edma3_pm_state		pm_state;
 	bool				idle;
-	bool				used;
 	struct fsl_edma3_engine		*edma3;
 	struct fsl_edma3_desc		*edesc;
 	struct fsl_edma3_slave_config	fsc;
@@ -163,9 +165,20 @@ struct fsl_edma3_chan {
 	int				is_rxchan;
 	int				is_remote;
 	int				is_dfifo;
+	bool                            is_sw;
 	struct dma_pool			*tcd_pool;
 	u32				chn_real_count;
-	char				txirq_name[32];
+	char                            txirq_name[32];
+	struct platform_device		*pdev;
+	struct device			*dev;
+	struct work_struct		issue_worker;
+	u32				srcid;
+	struct clk			*clk;
+};
+
+struct fsl_edma3_drvdata {
+	bool has_pd;
+	u32 dmamuxs;
 };
 
 struct fsl_edma3_desc {
@@ -173,6 +186,7 @@ struct fsl_edma3_desc {
 	struct fsl_edma3_chan		*echan;
 	bool				iscyclic;
 	unsigned int			n_tcds;
+	u32				curidx;
 	struct fsl_edma3_sw_tcd		tcd[];
 };
 
@@ -183,15 +197,29 @@ struct fsl_edma3_reg_save {
 
 struct fsl_edma3_engine {
 	struct dma_device	dma_dev;
+	unsigned long		irqflag;
 	struct mutex		fsl_edma3_mutex;
 	u32			n_chans;
 	int			errirq;
 	#define MAX_CHAN_NUM	32
 	struct fsl_edma3_reg_save edma_regs[MAX_CHAN_NUM];
 	bool			swap;	/* remote/local swapped on Audio edma */
+	const struct fsl_edma3_drvdata *drvdata;
+	struct clk		*clk_mp;
 	struct fsl_edma3_chan	chans[];
 };
 
+
+static struct fsl_edma3_drvdata fsl_edma_imx8q = {
+	.has_pd = true,
+	.dmamuxs = 0,
+};
+
+static struct fsl_edma3_drvdata fsl_edma_imx8ulp = {
+	.has_pd = false,
+	.dmamuxs = 1,
+};
+
 static struct fsl_edma3_chan *to_fsl_edma3_chan(struct dma_chan *chan)
 {
 	return container_of(chan, struct fsl_edma3_chan, vchan.chan);
@@ -226,12 +254,13 @@ static void fsl_edma3_enable_request(struct fsl_edma3_chan *fsl_chan)
 
 	writel(val, addr + EDMA_CH_SBR);
 
+	if (fsl_chan->srcid && !readl(addr + EDMA_CH_MUX))
+		writel(fsl_chan->srcid, addr + EDMA_CH_MUX);
+
 	val = readl(addr + EDMA_CH_CSR);
 
 	val |= EDMA_CH_CSR_ERQ;
 	writel(val, addr + EDMA_CH_CSR);
-
-	fsl_chan->used = true;
 }
 
 static void fsl_edma3_disable_request(struct fsl_edma3_chan *fsl_chan)
@@ -239,6 +268,9 @@ static void fsl_edma3_disable_request(struct fsl_edma3_chan *fsl_chan)
 	void __iomem *addr = fsl_chan->membase;
 	u32 val = readl(addr + EDMA_CH_CSR);
 
+	if (fsl_chan->srcid)
+		writel(0, addr + EDMA_CH_MUX);
+
 	val &= ~EDMA_CH_CSR_ERQ;
 	writel(val, addr + EDMA_CH_CSR);
 }
@@ -282,16 +314,31 @@ static int fsl_edma3_terminate_all(struct dma_chan *chan)
 	struct fsl_edma3_chan *fsl_chan = to_fsl_edma3_chan(chan);
 	unsigned long flags;
 	LIST_HEAD(head);
+	u32 val;
+
+	fsl_edma3_disable_request(fsl_chan);
+
+	/*
+	 * Checking ACTIVE to ensure minor loop stop indeed to prevent the
+	 * potential illegal memory write if channel not stopped with buffer
+	 * freed. Ignore tx channel since no such risk.
+	 */
+	if (fsl_chan->is_rxchan)
+		readl_poll_timeout_atomic(fsl_chan->membase + EDMA_CH_CSR, val,
+			!(val & EDMA_CH_CSR_ACTIVE), 2,
+			EDMA_MINOR_LOOP_TIMEOUT);
 
 	spin_lock_irqsave(&fsl_chan->vchan.lock, flags);
-	fsl_edma3_disable_request(fsl_chan);
 	fsl_chan->edesc = NULL;
 	fsl_chan->idle = true;
-	fsl_chan->used = false;
 	fsl_chan->vchan.cyclic = NULL;
 	vchan_get_all_descriptors(&fsl_chan->vchan, &head);
 	spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
 	vchan_dma_desc_free_list(&fsl_chan->vchan, &head);
+
+	if (fsl_chan->edma3->drvdata->has_pd)
+		pm_runtime_allow(fsl_chan->dev);
+
 	return 0;
 }
 
@@ -307,6 +354,7 @@ static int fsl_edma3_pause(struct dma_chan *chan)
 		fsl_chan->idle = true;
 	}
 	spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
+
 	return 0;
 }
 
@@ -322,6 +370,7 @@ static int fsl_edma3_resume(struct dma_chan *chan)
 		fsl_chan->idle = false;
 	}
 	spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
+
 	return 0;
 }
 
@@ -379,8 +428,16 @@ static size_t fsl_edma3_desc_residue(struct fsl_edma3_chan *fsl_chan,
 	else
 		cur_addr = readl(addr + EDMA_TCD_DADDR);
 
+	/* skip the tcds before curidx in cyclic case */
+	if (edesc->iscyclic)
+		len -= edesc->curidx * le32_to_cpu(edesc->tcd[0].vtcd->nbytes)
+			* le16_to_cpu(edesc->tcd[0].vtcd->biter);
+
 	/* figure out the finished and calculate the residue */
-	for (i = 0; i < fsl_chan->edesc->n_tcds; i++) {
+	for (i = edesc->curidx; i < fsl_chan->edesc->n_tcds; i++) {
+		if (dir == DMA_DEV_TO_MEM)
+			cur_addr = le32_to_cpu(edesc->tcd[i].vtcd->sda);
+
 		size = le32_to_cpu(edesc->tcd[i].vtcd->nbytes)
 			* le16_to_cpu(edesc->tcd[i].vtcd->biter);
 		if (dir == DMA_MEM_TO_DEV)
@@ -389,8 +446,15 @@ static size_t fsl_edma3_desc_residue(struct fsl_edma3_chan *fsl_chan,
 			dma_addr = le32_to_cpu(edesc->tcd[i].vtcd->daddr);
 
 		len -= size;
-		if (cur_addr >= dma_addr && cur_addr < dma_addr + size) {
+		if (cur_addr > dma_addr && cur_addr <= dma_addr + size) {
 			len += dma_addr + size - cur_addr;
+			/*
+			 * mark curidx as the start point in the next time
+			 * fsl_edma3_desc_residue(called by fsl_edma3_tx_status
+			 * commonly per cyclic interrupt/callback)
+			 */
+			if (edesc->iscyclic)
+				edesc->curidx = (i + 1) % edesc->n_tcds;
 			break;
 		}
 	}
@@ -466,11 +530,13 @@ static void fsl_edma3_set_tcd_regs(struct fsl_edma3_chan *fsl_chan,
 
 static inline
 void fsl_edma3_fill_tcd(struct fsl_edma3_chan *fsl_chan,
-			struct fsl_edma3_hw_tcd *tcd, u32 src, u32 dst,
+			struct fsl_edma3_sw_tcd *sw_tcd, u32 src, u32 dst,
 			u16 attr, u16 soff, u32 nbytes, u32 slast, u16 citer,
 			u16 biter, u16 doff, u32 dlast_sga, bool major_int,
 			bool disable_req, bool enable_sg)
 {
+	struct fsl_edma3_hw_tcd *tcd = sw_tcd->vtcd;
+	u32 slast_sda = sw_tcd->ptcd + offsetof(struct fsl_edma3_hw_tcd, sda);
 	u16 csr = 0;
 
 	/*
@@ -500,7 +566,11 @@ void fsl_edma3_fill_tcd(struct fsl_edma3_chan *fsl_chan,
 	}
 
 	tcd->nbytes = cpu_to_le32(EDMA_TCD_NBYTES_NBYTES(nbytes));
-	tcd->slast = cpu_to_le32(EDMA_TCD_SLAST_SLAST(slast));
+
+	if (fsl_chan->is_rxchan)
+		tcd->slast = cpu_to_le32(EDMA_TCD_SLAST_SLAST(slast_sda));
+	else
+		tcd->slast = cpu_to_le32(EDMA_TCD_SLAST_SLAST(slast));
 
 	tcd->citer = cpu_to_le16(EDMA_TCD_CITER_CITER(citer));
 	tcd->doff = cpu_to_le16(EDMA_TCD_DOFF_DOFF(doff));
@@ -518,7 +588,10 @@ void fsl_edma3_fill_tcd(struct fsl_edma3_chan *fsl_chan,
 		csr |= EDMA_TCD_CSR_E_SG;
 
 	if (fsl_chan->is_rxchan)
-		csr |= EDMA_TCD_CSR_ACTIVE;
+		csr |= EDMA_TCD_CSR_EEOP | EDMA_TCD_CSR_ESDA;
+
+	if (fsl_chan->is_sw)
+		csr |= EDMA_TCD_CSR_START;
 
 	tcd->csr = cpu_to_le16(csr);
 }
@@ -563,6 +636,7 @@ static struct dma_async_tx_descriptor *fsl_edma3_prep_dma_cyclic(
 	int sg_len, i;
 	u32 src_addr, dst_addr, last_sg, nbytes;
 	u16 soff, doff, iter;
+	bool major_int = true;
 
 	sg_len = buf_len / period_len;
 	fsl_desc = fsl_edma3_alloc_desc(fsl_chan, sg_len);
@@ -572,6 +646,26 @@ static struct dma_async_tx_descriptor *fsl_edma3_prep_dma_cyclic(
 
 	dma_buf_next = dma_addr;
 	nbytes = fsl_chan->fsc.addr_width * fsl_chan->fsc.burst;
+
+	/*
+	 * Choose the suitable burst length if period_len is not multiple of
+	 * burst length so that the whole transfer length is multiple of minor
+	 * loop(burst length).
+	 */
+	if (period_len % nbytes) {
+		u32 width = fsl_chan->fsc.addr_width;
+
+		for (i = fsl_chan->fsc.burst; i > 1; i--) {
+			if (!(period_len % (i * width))) {
+				nbytes = i * width;
+				break;
+			}
+		}
+		/* if no chance to get suitable burst size, use it as 1 */
+		if (i == 1)
+			nbytes = width;
+	}
+
 	iter = period_len / nbytes;
 
 	for (i = 0; i < sg_len; i++) {
@@ -603,14 +697,17 @@ static struct dma_async_tx_descriptor *fsl_edma3_prep_dma_cyclic(
 			dst_addr = fsl_chan->fsc.dev_addr;
 			soff = 0;
 			doff = 0;
+			major_int = false;
 		}
 
-		fsl_edma3_fill_tcd(fsl_chan, fsl_desc->tcd[i].vtcd, src_addr,
+		fsl_edma3_fill_tcd(fsl_chan, &fsl_desc->tcd[i], src_addr,
 				dst_addr, fsl_chan->fsc.attr, soff, nbytes, 0,
-				iter, iter, doff, last_sg, true, false, true);
+				iter, iter, doff, last_sg, major_int, false, true);
 		dma_buf_next += period_len;
 	}
 
+	fsl_desc->curidx = 0;
+
 	return vchan_tx_prep(&fsl_chan->vchan, &fsl_desc->vdesc, flags);
 }
 
@@ -657,16 +754,36 @@ static struct dma_async_tx_descriptor *fsl_edma3_prep_slave_sg(
 			doff = 0;
 		}
 
+		/*
+		 * Choose the suitable burst length if sg_dma_len is not
+		 * multiple of burst length so that the whole transfer length is
+		 * multiple of minor loop(burst length).
+		 */
+		if (sg_dma_len(sg) % nbytes) {
+			u32 width = fsl_chan->fsc.addr_width;
+			int j;
+
+			for (j = fsl_chan->fsc.burst; j > 1; j--) {
+				if (!(sg_dma_len(sg) % (j * width))) {
+					nbytes = j * width;
+					break;
+				}
+			}
+			/* Set burst size as 1 if there's no suitable one */
+			if (j == 1)
+				nbytes = width;
+		}
+
 		iter = sg_dma_len(sg) / nbytes;
 		if (i < sg_len - 1) {
 			last_sg = fsl_desc->tcd[(i + 1)].ptcd;
-			fsl_edma3_fill_tcd(fsl_chan, fsl_desc->tcd[i].vtcd,
+			fsl_edma3_fill_tcd(fsl_chan, &fsl_desc->tcd[i],
 					src_addr, dst_addr, fsl_chan->fsc.attr,
 					soff, nbytes, 0, iter, iter, doff,
 					last_sg, false, false, true);
 		} else {
 			last_sg = 0;
-			fsl_edma3_fill_tcd(fsl_chan, fsl_desc->tcd[i].vtcd,
+			fsl_edma3_fill_tcd(fsl_chan, &fsl_desc->tcd[i],
 					src_addr, dst_addr, fsl_chan->fsc.attr,
 					soff, nbytes, 0, iter, iter, doff,
 					last_sg, true, true, false);
@@ -684,12 +801,35 @@ static void fsl_edma3_xfer_desc(struct fsl_edma3_chan *fsl_chan)
 	if (!vdesc)
 		return;
 	fsl_chan->edesc = to_fsl_edma3_desc(vdesc);
+
 	fsl_edma3_set_tcd_regs(fsl_chan, fsl_chan->edesc->tcd[0].vtcd);
 	fsl_edma3_enable_request(fsl_chan);
 	fsl_chan->status = DMA_IN_PROGRESS;
 	fsl_chan->idle = false;
 }
 
+static struct dma_async_tx_descriptor *fsl_edma3_prep_memcpy(
+		struct dma_chan *chan, dma_addr_t dma_dst,
+		dma_addr_t dma_src, size_t len, unsigned long flags)
+{
+	struct fsl_edma3_chan *fsl_chan = to_fsl_edma3_chan(chan);
+	struct fsl_edma3_desc *fsl_desc;
+
+	fsl_desc = fsl_edma3_alloc_desc(fsl_chan, 1);
+	if (!fsl_desc)
+		return NULL;
+	fsl_desc->iscyclic = false;
+
+	fsl_chan->is_sw = true;
+
+	/* To match with copy_align and max_seg_size so 1 tcd is enough */
+	fsl_edma3_fill_tcd(fsl_chan, &fsl_desc->tcd[0], dma_src, dma_dst,
+			EDMA_TCD_ATTR_SSIZE_64BYTE | EDMA_TCD_ATTR_DSIZE_64BYTE,
+			64, len, 0, 1, 1, 64, 0, true, true, false);
+
+	return vchan_tx_prep(&fsl_chan->vchan, &fsl_desc->vdesc, flags);
+}
+
 static size_t fsl_edma3_desc_residue(struct fsl_edma3_chan *fsl_chan,
 		struct virt_dma_desc *vdesc, bool in_progress);
 
@@ -704,16 +844,20 @@ static irqreturn_t fsl_edma3_tx_handler(int irq, void *dev_id)
 	unsigned int intr;
 	void __iomem *base_addr;
 
+	spin_lock(&fsl_chan->vchan.lock);
+
+	/* Ignore this interrupt since channel has been freeed with power off */
+	if (!fsl_chan->edesc && !fsl_chan->tcd_pool)
+		goto irq_handled;
+
 	base_addr = fsl_chan->membase;
 
 	intr = readl(base_addr + EDMA_CH_INT);
 	if (!intr)
-		return IRQ_NONE;
+		goto irq_handled;
 
 	writel(1, base_addr + EDMA_CH_INT);
 
-	spin_lock(&fsl_chan->vchan.lock);
-
 	/* Ignore this interrupt since channel has been disabled already */
 	if (!fsl_chan->edesc)
 		goto irq_handled;
@@ -740,20 +884,8 @@ irq_handled:
 static void fsl_edma3_issue_pending(struct dma_chan *chan)
 {
 	struct fsl_edma3_chan *fsl_chan = to_fsl_edma3_chan(chan);
-	unsigned long flags;
 
-	spin_lock_irqsave(&fsl_chan->vchan.lock, flags);
-
-	if (unlikely(fsl_chan->pm_state != RUNNING)) {
-		spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
-		/* cannot submit due to suspend */
-		return;
-	}
-
-	if (vchan_issue_pending(&fsl_chan->vchan) && !fsl_chan->edesc)
-		fsl_edma3_xfer_desc(fsl_chan);
-
-	spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
+	schedule_work(&fsl_chan->issue_worker);
 }
 
 static struct dma_chan *fsl_edma3_xlate(struct of_phandle_args *dma_spec,
@@ -773,7 +905,8 @@ static struct dma_chan *fsl_edma3_xlate(struct of_phandle_args *dma_spec,
 			continue;
 
 		fsl_chan = to_fsl_edma3_chan(chan);
-		if (fsl_chan->hw_chanid == dma_spec->args[0]) {
+		if (fsl_edma3->drvdata->dmamuxs == 0 &&
+		    fsl_chan->hw_chanid == dma_spec->args[0]) {
 			chan = dma_get_slave_channel(chan);
 			chan->device->privatecnt++;
 			fsl_chan->priority = dma_spec->args[1];
@@ -782,6 +915,16 @@ static struct dma_chan *fsl_edma3_xlate(struct of_phandle_args *dma_spec,
 			fsl_chan->is_dfifo = dma_spec->args[2] & ARGS_DFIFO;
 			mutex_unlock(&fsl_edma3->fsl_edma3_mutex);
 			return chan;
+		} else if (fsl_edma3->drvdata->dmamuxs && !fsl_chan->srcid) {
+			chan = dma_get_slave_channel(chan);
+			chan->device->privatecnt++;
+			fsl_chan->priority = dma_spec->args[1];
+			fsl_chan->srcid = dma_spec->args[0];
+			fsl_chan->is_rxchan = dma_spec->args[2] & ARGS_RX;
+			fsl_chan->is_remote = dma_spec->args[2] & ARGS_REMOTE;
+			fsl_chan->is_dfifo = dma_spec->args[2] & ARGS_DFIFO;
+			mutex_unlock(&fsl_edma3->fsl_edma3_mutex);
+			return chan;
 		}
 	}
 	mutex_unlock(&fsl_edma3->fsl_edma3_mutex);
@@ -791,10 +934,39 @@ static struct dma_chan *fsl_edma3_xlate(struct of_phandle_args *dma_spec,
 static int fsl_edma3_alloc_chan_resources(struct dma_chan *chan)
 {
 	struct fsl_edma3_chan *fsl_chan = to_fsl_edma3_chan(chan);
+	struct platform_device *pdev = fsl_chan->pdev;
+	int ret;
+
+	clk_prepare_enable(fsl_chan->clk);
 
 	fsl_chan->tcd_pool = dma_pool_create("tcd_pool", chan->device->dev,
 				sizeof(struct fsl_edma3_hw_tcd),
 				32, 0);
+
+	if (fsl_chan->edma3->drvdata->has_pd) {
+		pm_runtime_get_sync(fsl_chan->dev);
+		/* clear meaningless pending irq anyway */
+		if (readl(fsl_chan->membase + EDMA_CH_INT))
+			writel(1, fsl_chan->membase + EDMA_CH_INT);
+	}
+
+	ret = devm_request_irq(&pdev->dev, fsl_chan->txirq,
+			fsl_edma3_tx_handler, fsl_chan->edma3->irqflag,
+			fsl_chan->txirq_name, fsl_chan);
+	if (ret) {
+		dev_err(&pdev->dev, "Can't register %s IRQ.\n",
+			fsl_chan->txirq_name);
+		if (fsl_chan->edma3->drvdata->has_pd)
+			pm_runtime_put_sync_suspend(fsl_chan->dev);
+
+		return ret;
+	}
+
+	if (fsl_chan->edma3->drvdata->has_pd) {
+		pm_runtime_mark_last_busy(fsl_chan->dev);
+		pm_runtime_put_autosuspend(fsl_chan->dev);
+	}
+
 	return 0;
 }
 
@@ -804,6 +976,11 @@ static void fsl_edma3_free_chan_resources(struct dma_chan *chan)
 	unsigned long flags;
 	LIST_HEAD(head);
 
+	if (fsl_chan->edma3->drvdata->has_pd)
+		pm_runtime_get_sync(fsl_chan->dev);
+
+	devm_free_irq(&fsl_chan->pdev->dev, fsl_chan->txirq, fsl_chan);
+
 	spin_lock_irqsave(&fsl_chan->vchan.lock, flags);
 	fsl_edma3_disable_request(fsl_chan);
 	fsl_chan->edesc = NULL;
@@ -812,8 +989,20 @@ static void fsl_edma3_free_chan_resources(struct dma_chan *chan)
 
 	vchan_dma_desc_free_list(&fsl_chan->vchan, &head);
 	dma_pool_destroy(fsl_chan->tcd_pool);
+
+	spin_lock_irqsave(&fsl_chan->vchan.lock, flags);
 	fsl_chan->tcd_pool = NULL;
-	fsl_chan->used = false;
+	fsl_chan->srcid = 0;
+	/* Clear interrupt before power off */
+	if (readl(fsl_chan->membase + EDMA_CH_INT))
+		writel(1, fsl_chan->membase + EDMA_CH_INT);
+	spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
+
+	if (fsl_chan->edma3->drvdata->has_pd)
+		pm_runtime_put_sync_suspend(fsl_chan->dev);
+
+	clk_disable_unprepare(fsl_chan->clk);
+	fsl_chan->is_sw = false;
 }
 
 static void fsl_edma3_synchronize(struct dma_chan *chan)
@@ -823,15 +1012,77 @@ static void fsl_edma3_synchronize(struct dma_chan *chan)
 	vchan_synchronize(&fsl_chan->vchan);
 }
 
+static struct device *fsl_edma3_attach_pd(struct device *dev,
+					  struct device_node *np, int index)
+{
+	const char *domn = "edma0-chan01";
+	struct device *pd_chan;
+	struct device_link *link;
+	int ret;
+
+	ret = of_property_read_string_index(np, "power-domain-names", index,
+						&domn);
+	if (ret) {
+		dev_err(dev, "parse power-domain-names error.(%d)\n", ret);
+		return NULL;
+	}
+
+	pd_chan = dev_pm_domain_attach_by_name(dev, domn);
+	if (IS_ERR_OR_NULL(pd_chan))
+		return NULL;
+
+	link = device_link_add(dev, pd_chan, DL_FLAG_STATELESS |
+					     DL_FLAG_PM_RUNTIME |
+					     DL_FLAG_RPM_ACTIVE);
+	if (IS_ERR(link)) {
+		dev_err(dev, "Failed to add device_link to %s: %ld\n", domn,
+			PTR_ERR(link));
+		return NULL;
+	}
+
+	return pd_chan;
+}
+
+static void fsl_edma3_issue_work(struct work_struct *work)
+{
+	struct fsl_edma3_chan *fsl_chan = container_of(work,
+						       struct fsl_edma3_chan,
+						       issue_worker);
+	unsigned long flags;
+
+	if (fsl_chan->edma3->drvdata->has_pd)
+		pm_runtime_forbid(fsl_chan->dev);
+
+	spin_lock_irqsave(&fsl_chan->vchan.lock, flags);
+
+	if (vchan_issue_pending(&fsl_chan->vchan) && !fsl_chan->edesc)
+		fsl_edma3_xfer_desc(fsl_chan);
+
+	spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
+}
+
+static const struct of_device_id fsl_edma3_dt_ids[] = {
+	{ .compatible = "fsl,imx8qm-edma", .data = &fsl_edma_imx8q},
+	{ .compatible = "fsl,imx8qm-adma", .data = &fsl_edma_imx8q},
+	{ .compatible = "fsl,imx8ulp-edma", .data = &fsl_edma_imx8ulp},
+	{ /* sentinel */ }
+};
+MODULE_DEVICE_TABLE(of, fsl_edma3_dt_ids);
+
 static int fsl_edma3_probe(struct platform_device *pdev)
 {
 	struct device_node *np = pdev->dev.of_node;
+	const struct of_device_id *of_id =
+			of_match_device(fsl_edma3_dt_ids, &pdev->dev);
 	struct fsl_edma3_engine *fsl_edma3;
 	struct fsl_edma3_chan *fsl_chan;
+	struct resource *res_mp;
 	struct resource *res;
 	int len, chans;
 	int ret, i;
-	unsigned long irqflag = 0;
+
+	if (!of_id)
+		return -EINVAL;
 
 	ret = of_property_read_u32(np, "dma-channels", &chans);
 	if (ret) {
@@ -846,44 +1097,47 @@ static int fsl_edma3_probe(struct platform_device *pdev)
 
 	/* Audio edma rx/tx channel shared interrupt */
 	if (of_property_read_bool(np, "shared-interrupt"))
-		irqflag = IRQF_SHARED;
+		fsl_edma3->irqflag = IRQF_SHARED;
 
-	fsl_edma3->swap = false;
+	fsl_edma3->swap = of_device_is_compatible(np, "fsl,imx8qm-adma");
 	fsl_edma3->n_chans = chans;
-
-	/*
-	 * FIXUP: if this is the i.MX8QM TO1.0, need set the swap.
-	 * FIXME: This will be revisted to set the swap property
-	 * from the device-tree node later instead of revison check,
-	 * but, this will need add extra DT file, not perfect too.
-	 */
-
-	if ((of_device_is_compatible(np, "fsl,imx8qm-adma")) &&
-		cpu_is_imx8qm() &&
-		imx8_get_soc_revision() == IMX_CHIP_REVISION_1_0)
-		fsl_edma3->swap = true;
+	fsl_edma3->drvdata = (const struct fsl_edma3_drvdata *)of_id->data;
 
 	INIT_LIST_HEAD(&fsl_edma3->dma_dev.channels);
+
+	res_mp = platform_get_resource(pdev, IORESOURCE_MEM, 0);
+
+	fsl_edma3->clk_mp = devm_clk_get_optional(&pdev->dev, "edma-mp-clk");
+	if (IS_ERR(fsl_edma3->clk_mp)) {
+		dev_err(&pdev->dev, "Can't get mp clk.\n");
+		return PTR_ERR(fsl_edma3->clk_mp);
+	}
+	clk_prepare_enable(fsl_edma3->clk_mp);
+
 	for (i = 0; i < fsl_edma3->n_chans; i++) {
 		struct fsl_edma3_chan *fsl_chan = &fsl_edma3->chans[i];
-		const char *txirq_name = fsl_chan->txirq_name;
+		const char *txirq_name;
 		char chanid[3], id_len = 0;
+		char clk_name[18];
 		char *p = chanid;
 		unsigned long val;
 
 		fsl_chan->edma3 = fsl_edma3;
-		fsl_chan->pm_state = RUNNING;
+		fsl_chan->pdev = pdev;
 		fsl_chan->idle = true;
+		fsl_chan->srcid = 0;
 		/* Get per channel membase */
-		res = platform_get_resource(pdev, IORESOURCE_MEM, i);
+		res = platform_get_resource(pdev, IORESOURCE_MEM, i + 1);
 		fsl_chan->membase = devm_ioremap_resource(&pdev->dev, res);
 		if (IS_ERR(fsl_chan->membase))
 			return PTR_ERR(fsl_chan->membase);
 
 		/* Get the hardware chanel id by the channel membase
 		 * channel0:0x10000, channel1:0x20000... total 32 channels
+		 * Note: skip first res_mp which we don't care.
 		 */
-		fsl_chan->hw_chanid = (res->start >> 16) & 0x1f;
+		fsl_chan->hw_chanid = ((res->start - res_mp->start) >> 16) & 0x3f;
+		fsl_chan->hw_chanid--;
 
 		ret = of_property_read_string_index(np, "interrupt-names", i,
 							&txirq_name);
@@ -917,18 +1171,20 @@ static int fsl_edma3_probe(struct platform_device *pdev)
 			return fsl_chan->txirq;
 		}
 
-		ret = devm_request_irq(&pdev->dev, fsl_chan->txirq,
-				fsl_edma3_tx_handler, irqflag, txirq_name,
-				fsl_chan);
-		if (ret) {
-			dev_err(&pdev->dev, "Can't register %s IRQ.\n",
-				txirq_name);
-			return ret;
-		}
+		memcpy(fsl_chan->txirq_name, txirq_name, strlen(txirq_name));
+
+		strncpy(clk_name, txirq_name, strlen(CHAN_PREFIX) + id_len);
+		strcpy(clk_name + strlen(CHAN_PREFIX) + id_len, CLK_POSFIX);
+		fsl_chan->clk = devm_clk_get_optional(&pdev->dev,
+						     (const char *)clk_name);
+		if (IS_ERR(fsl_chan->clk))
+			return PTR_ERR(fsl_chan->clk);
 
 		fsl_chan->vchan.desc_free = fsl_edma3_free_desc;
 		vchan_init(&fsl_chan->vchan, &fsl_edma3->dma_dev);
-		fsl_chan->used = false;
+
+		INIT_WORK(&fsl_chan->issue_worker,
+				fsl_edma3_issue_work);
 	}
 
 	mutex_init(&fsl_edma3->fsl_edma3_mutex);
@@ -936,6 +1192,7 @@ static int fsl_edma3_probe(struct platform_device *pdev)
 	dma_cap_set(DMA_PRIVATE, fsl_edma3->dma_dev.cap_mask);
 	dma_cap_set(DMA_SLAVE, fsl_edma3->dma_dev.cap_mask);
 	dma_cap_set(DMA_CYCLIC, fsl_edma3->dma_dev.cap_mask);
+	dma_cap_set(DMA_MEMCPY, fsl_edma3->dma_dev.cap_mask);
 
 	fsl_edma3->dma_dev.dev = &pdev->dev;
 	fsl_edma3->dma_dev.device_alloc_chan_resources
@@ -944,6 +1201,7 @@ static int fsl_edma3_probe(struct platform_device *pdev)
 		= fsl_edma3_free_chan_resources;
 	fsl_edma3->dma_dev.device_tx_status = fsl_edma3_tx_status;
 	fsl_edma3->dma_dev.device_prep_slave_sg = fsl_edma3_prep_slave_sg;
+	fsl_edma3->dma_dev.device_prep_dma_memcpy = fsl_edma3_prep_memcpy;
 	fsl_edma3->dma_dev.device_prep_dma_cyclic = fsl_edma3_prep_dma_cyclic;
 	fsl_edma3->dma_dev.device_config = fsl_edma3_slave_config;
 	fsl_edma3->dma_dev.device_pause = fsl_edma3_pause;
@@ -958,6 +1216,10 @@ static int fsl_edma3_probe(struct platform_device *pdev)
 					BIT(DMA_MEM_TO_DEV) |
 					BIT(DMA_DEV_TO_DEV);
 
+	fsl_edma3->dma_dev.copy_align = DMAENGINE_ALIGN_64_BYTES;
+	/* Per worst case 'nbytes = 1' take CITER as the max_seg_size */
+	dma_set_max_seg_size(fsl_edma3->dma_dev.dev, 0x3fff);
+
 	platform_set_drvdata(pdev, fsl_edma3);
 
 	ret = dma_async_device_register(&fsl_edma3->dma_dev);
@@ -965,6 +1227,28 @@ static int fsl_edma3_probe(struct platform_device *pdev)
 		dev_err(&pdev->dev, "Can't register Freescale eDMA engine.\n");
 		return ret;
 	}
+	/* Attach power domains from dts for each dma chanel device */
+	if (fsl_edma3->drvdata->has_pd) {
+		for (i = 0; i < fsl_edma3->n_chans; i++) {
+			struct fsl_edma3_chan *fsl_chan = &fsl_edma3->chans[i];
+			struct device *dev;
+
+			dev = fsl_edma3_attach_pd(&pdev->dev, np, i);
+			if (!dev) {
+				dev_err(dev, "edma channel attach failed.\n");
+				return -EINVAL;
+			}
+
+			fsl_chan->dev = dev;
+			/* clear meaningless pending irq anyway */
+			writel(1, fsl_chan->membase + EDMA_CH_INT);
+
+			pm_runtime_use_autosuspend(fsl_chan->dev);
+			pm_runtime_set_autosuspend_delay(fsl_chan->dev, 200);
+			pm_runtime_set_active(fsl_chan->dev);
+			pm_runtime_put_sync_suspend(fsl_chan->dev);
+		}
+	}
 
 	ret = of_dma_controller_register(np, fsl_edma3_xlate, fsl_edma3);
 	if (ret) {
@@ -984,9 +1268,12 @@ static int fsl_edma3_remove(struct platform_device *pdev)
 	of_dma_controller_free(np);
 	dma_async_device_unregister(&fsl_edma3->dma_dev);
 
+	clk_disable_unprepare(fsl_edma3->clk_mp);
+
 	return 0;
 }
 
+#ifdef CONFIG_PM_SLEEP
 static int fsl_edma3_suspend_late(struct device *dev)
 {
 	struct fsl_edma3_engine *fsl_edma = dev_get_drvdata(dev);
@@ -999,8 +1286,14 @@ static int fsl_edma3_suspend_late(struct device *dev)
 		fsl_chan = &fsl_edma->chans[i];
 		addr = fsl_chan->membase;
 
-		if (!fsl_chan->used)
+		if ((fsl_chan->edma3->drvdata->has_pd &&
+		    pm_runtime_status_suspended(fsl_chan->dev)) ||
+		    (!fsl_chan->edma3->drvdata->has_pd && !fsl_chan->srcid))
 			continue;
+
+		if (fsl_chan->edma3->drvdata->has_pd)
+			pm_runtime_get_sync(fsl_chan->dev);
+
 		spin_lock_irqsave(&fsl_chan->vchan.lock, flags);
 		fsl_edma->edma_regs[i].csr = readl(addr + EDMA_CH_CSR);
 		fsl_edma->edma_regs[i].sbr = readl(addr + EDMA_CH_SBR);
@@ -1009,8 +1302,10 @@ static int fsl_edma3_suspend_late(struct device *dev)
 			dev_warn(dev, "WARN: There is non-idle channel.");
 			fsl_edma3_disable_request(fsl_chan);
 		}
-		fsl_chan->pm_state = SUSPENDED;
 		spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
+
+		if (fsl_chan->edma3->drvdata->has_pd)
+			pm_runtime_put_sync_suspend(fsl_chan->dev);
 	}
 
 	return 0;
@@ -1028,7 +1323,9 @@ static int fsl_edma3_resume_early(struct device *dev)
 		fsl_chan = &fsl_edma->chans[i];
 		addr = fsl_chan->membase;
 
-		if (!fsl_chan->used)
+		if ((fsl_chan->edma3->drvdata->has_pd &&
+		    pm_runtime_status_suspended(fsl_chan->dev)) ||
+		    (!fsl_chan->edma3->drvdata->has_pd && !fsl_chan->srcid))
 			continue;
 
 		spin_lock_irqsave(&fsl_chan->vchan.lock, flags);
@@ -1038,25 +1335,18 @@ static int fsl_edma3_resume_early(struct device *dev)
 		if (fsl_chan->edesc)
 			fsl_edma3_set_tcd_regs(fsl_chan,
 						fsl_chan->edesc->tcd[0].vtcd);
-		fsl_chan->pm_state = RUNNING;
 		spin_unlock_irqrestore(&fsl_chan->vchan.lock, flags);
 	}
 
 	return 0;
 }
+#endif
 
 static const struct dev_pm_ops fsl_edma3_pm_ops = {
-	.suspend_late   = fsl_edma3_suspend_late,
-	.resume_early   = fsl_edma3_resume_early,
+	SET_LATE_SYSTEM_SLEEP_PM_OPS(fsl_edma3_suspend_late,
+				     fsl_edma3_resume_early)
 };
 
-static const struct of_device_id fsl_edma3_dt_ids[] = {
-	{ .compatible = "fsl,imx8qm-edma", },
-	{ .compatible = "fsl,imx8qm-adma", },
-	{ /* sentinel */ }
-};
-MODULE_DEVICE_TABLE(of, fsl_edma3_dt_ids);
-
 static struct platform_driver fsl_edma3_driver = {
 	.driver		= {
 		.name	= "fsl-edma-v3",
@@ -1071,7 +1361,7 @@ static int __init fsl_edma3_init(void)
 {
 	return platform_driver_register(&fsl_edma3_driver);
 }
-subsys_initcall(fsl_edma3_init);
+fs_initcall(fsl_edma3_init);
 
 static void __exit fsl_edma3_exit(void)
 {
