@@ -1,7 +1,7 @@
 /*
  * Samsung MIPI DSIM Bridge
  *
- * Copyright 2018-2020 NXP
+ * Copyright 2018-2022 NXP
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,14 +18,17 @@
 #include <linux/clk.h>
 #include <linux/completion.h>
 #include <linux/delay.h>
+#include <linux/gcd.h>
+#include <linux/log2.h>
 #include <linux/module.h>
 #include <linux/of_graph.h>
+#include <linux/pm_runtime.h>
 #include <drm/bridge/sec_mipi_dsim.h>
-#include <drm/drmP.h>
+#include <drm/drm_vblank.h>
 #include <drm/drm_atomic_helper.h>
 #include <drm/drm_bridge.h>
 #include <drm/drm_connector.h>
-#include <drm/drm_crtc_helper.h>
+#include <drm/drm_probe_helper.h>
 #include <drm/drm_encoder.h>
 #include <drm/drm_fourcc.h>
 #include <drm/drm_mipi_dsi.h>
@@ -224,9 +227,6 @@
 #define dsim_read(dsim, reg)		readl(dsim->base + reg)
 #define dsim_write(dsim, val, reg)	writel(val, dsim->base + reg)
 
-/* fixed phy ref clk rate */
-#define PHY_REF_CLK		27000000
-
 #define MAX_MAIN_HRESOL		2047
 #define MAX_MAIN_VRESOL		2047
 #define MAX_SUB_HRESOL		1024
@@ -265,19 +265,14 @@
 
 #define MIPI_FIFO_TIMEOUT	msecs_to_jiffies(250)
 
+#define MIPI_HFP_PKT_OVERHEAD	6
+#define MIPI_HBP_PKT_OVERHEAD	6
+#define MIPI_HSA_PKT_OVERHEAD	6
+
 #define to_sec_mipi_dsim(dsi) container_of(dsi, struct sec_mipi_dsim, dsi_host)
 #define conn_to_sec_mipi_dsim(conn)		\
 	container_of(conn, struct sec_mipi_dsim, connector)
 
-/* DSIM PLL configuration from spec:
- *
- * Fout(DDR) = (M * Fin) / (P * 2^S), so Fout / Fin = M / (P * 2^S)
- * Fin_pll   = Fin / P     (6 ~ 12 MHz)
- * S: [2:0], M: [12:3], P: [18:13], so
- * TODO: 'S' is in [0 ~ 3], 'M' is in, 'P' is in [1 ~ 33]
- *
- */
-
 /* used for CEA standard modes */
 struct dsim_hblank_par {
 	char *name;		/* drm display mode name */
@@ -289,10 +284,11 @@ struct dsim_hblank_par {
 };
 
 struct dsim_pll_pms {
-	uint64_t bit_clk;	/* kHz */
+	uint32_t bit_clk;	/* kHz */
 	uint32_t p;
 	uint32_t m;
 	uint32_t s;
+	uint32_t k;
 };
 
 struct sec_mipi_dsim {
@@ -305,15 +301,11 @@ struct sec_mipi_dsim {
 	struct device *dev;
 
 	void __iomem *base;
-	int irq;
-
-	struct clk *clk_cfg;
-	struct clk *clk_pllref;
-	struct clk *pclk;			/* pixel clock */
 
 	/* kHz clocks */
-	uint64_t pix_clk;
-	uint64_t bit_clk;
+	uint32_t pix_clk;
+	uint32_t bit_clk;
+	uint32_t pref_clk;			/* phy ref clock rate in KHz */
 
 	unsigned int lanes;
 	unsigned int channel;			/* virtual channel */
@@ -327,6 +319,7 @@ struct sec_mipi_dsim {
 	unsigned long long lp_data_rate;
 	unsigned long long hs_data_rate;
 	struct videomode vmode;
+	bool enabled;
 
 	struct completion pll_stable;
 	struct completion ph_tx_done;
@@ -380,22 +373,7 @@ static const struct dsim_hblank_par hblank_2lanes[] = {
 	/* {  12,  68, 64 } */
 	{ DSIM_HBLANK_PARAM("720x576"  , 50,  12,  96,  72, 2), },
 	/* {  16,  48, 96 } */
-	{ DSIM_HBLANK_PARAM("640x480"  , 60,  18,  58, 128, 2), },
-};
-
-static const struct dsim_pll_pms pll_pms[] = {
-	{ DSIM_PLL_PMS(891000, 1, 66, 1), },
-	{ DSIM_PLL_PMS(890112, 1, 66, 1), },
-	{ DSIM_PLL_PMS(594000, 3, 66, 0), },
-	{ DSIM_PLL_PMS(593408, 3, 66, 0), },
-	{ DSIM_PLL_PMS(445500, 1, 66, 2), },
-	{ DSIM_PLL_PMS(445056, 1, 66, 2), },
-	{ DSIM_PLL_PMS(324000, 3, 72, 1), },
-	{ DSIM_PLL_PMS(324324, 3, 72, 1), },
-	{ DSIM_PLL_PMS(302400, 5, 112, 1), },
-	{ DSIM_PLL_PMS(302100, 5, 112, 1), },
-	{ DSIM_PLL_PMS(162000, 3, 72, 2), },
-	{ DSIM_PLL_PMS(162162, 3, 72, 2), },
+	{ DSIM_HBLANK_PARAM("640x480"  , 60,  18,  66, 138, 2), },
 };
 
 static const struct dsim_hblank_par *sec_mipi_dsim_get_hblank_par(const char *name,
@@ -438,19 +416,21 @@ static const struct dsim_hblank_par *sec_mipi_dsim_get_hblank_par(const char *na
 	return NULL;
 }
 
-static const struct dsim_pll_pms *sec_mipi_dsim_get_pms(uint64_t bit_clk)
+static int sec_mipi_dsim_set_pref_rate(struct sec_mipi_dsim *dsim)
 {
-	int i;
-	const struct dsim_pll_pms *pms;
+	int ret;
+	uint32_t rate;
+	const struct sec_mipi_dsim_plat_data *pdata = dsim->pdata;
+	const struct sec_mipi_dsim_pll *dpll = pdata->dphy_pll;
+	const struct sec_mipi_dsim_range *fin_range = &dpll->fin;
 
-	for (i = 0; i < ARRAY_SIZE(pll_pms); i++) {
-		pms = &pll_pms[i];
+	ret = pdata->determine_pll_ref_rate(&rate, fin_range->min, fin_range->max);
+	if (ret)
+		return ret;
 
-		if (bit_clk == pms->bit_clk)
-			return pms;
-	}
+	dsim->pref_clk = rate;
 
-	return NULL;
+	return 0;
 }
 
 static void sec_mipi_dsim_irq_init(struct sec_mipi_dsim *dsim);
@@ -472,7 +452,7 @@ static int sec_mipi_dsim_host_attach(struct mipi_dsi_host *host,
 	if (dsim->channel)
 		return -EINVAL;
 
-	if (!(dsi->mode_flags & MIPI_DSI_MODE_VIDEO)		||
+	if ((dsi->mode_flags & MIPI_DSI_MODE_VIDEO)		&&
 	    !((dsi->mode_flags & MIPI_DSI_MODE_VIDEO_BURST)	||
 	      (dsi->mode_flags & MIPI_DSI_MODE_VIDEO_SYNC_PULSE))) {
 		dev_err(dev, "unsupported dsi mode\n");
@@ -541,6 +521,9 @@ static int sec_mipi_dsim_host_detach(struct mipi_dsi_host *host,
 	dsim->format	 = 0;
 	dsim->mode_flags = 0;
 
+	/* detached panel should be NULL */
+	dsim->panel = NULL;
+
 	return 0;
 }
 
@@ -556,12 +539,6 @@ static void sec_mipi_dsim_config_cmd_lpm(struct sec_mipi_dsim *dsim,
 	else
 		escmode &= ~ESCMODE_CMDLPDT;
 
-	/* force BTA at the end of packet transfer
-	 * to receive the acknowledgment from dsi
-	 * peripheral for this transfer
-	 */
-	escmode |= ESCMODE_FORCEBTA;
-
 	dsim_write(dsim, escmode, DSIM_ESCMODE);
 }
 
@@ -585,8 +562,10 @@ static void sec_mipi_dsim_write_pl_to_sfr_fifo(struct sec_mipi_dsim *dsim,
 	switch (length) {
 	case 3:
 		pl_data |= ((u8 *)payload)[2] << 16;
+		fallthrough;
 	case 2:
 		pl_data |= ((u8 *)payload)[1] << 8;
+		fallthrough;
 	case 1:
 		pl_data |= ((u8 *)payload)[0];
 		dsim_write(dsim, pl_data, DSIM_PAYLOAD);
@@ -600,9 +579,6 @@ static void sec_mipi_dsim_write_ph_to_sfr_fifo(struct sec_mipi_dsim *dsim,
 {
 	uint32_t pkthdr;
 
-	/* config LPM for CMD TX */
-	sec_mipi_dsim_config_cmd_lpm(dsim, use_lpm);
-
 	pkthdr = PKTHDR_SET_DATA1(((u8 *)header)[2])	| /* WC MSB  */
 		 PKTHDR_SET_DATA0(((u8 *)header)[1])	| /* WC LSB  */
 		 PKTHDR_SET_DI(((u8 *)header)[0]);	  /* Data ID */
@@ -636,7 +612,7 @@ static int sec_mipi_dsim_read_pl_from_sfr_fifo(struct sec_mipi_dsim *dsim,
 			((u8 *)payload)[1] = PKTHDR_GET_DATA1(ph);
 			word_count++;
 		}
-		/* fall through */
+		fallthrough;
 	case MIPI_DSI_RX_DCS_SHORT_READ_RESPONSE_1BYTE:
 	case MIPI_DSI_RX_GENERIC_SHORT_READ_RESPONSE_1BYTE:
 		((u8 *)payload)[0] = PKTHDR_GET_DATA0(ph);
@@ -669,8 +645,10 @@ static int sec_mipi_dsim_read_pl_from_sfr_fifo(struct sec_mipi_dsim *dsim,
 			switch (word_count) {
 			case 3:
 				((u8 *)payload)[2] = (pl >> 16) & 0xff;
+				fallthrough;
 			case 2:
 				((u8 *)payload)[1] = (pl >> 8) & 0xff;
+				fallthrough;
 			case 1:
 				((u8 *)payload)[0] = pl & 0xff;
 				break;
@@ -689,6 +667,7 @@ static ssize_t sec_mipi_dsim_host_transfer(struct mipi_dsi_host *host,
 					   const struct mipi_dsi_msg *msg)
 {
 	int ret;
+	ssize_t bytes = 0;
 	bool use_lpm;
 	struct mipi_dsi_packet packet;
 	struct sec_mipi_dsim *dsim = to_sec_mipi_dsim(host);
@@ -706,7 +685,9 @@ static ssize_t sec_mipi_dsim_host_transfer(struct mipi_dsi_host *host,
 	if (unlikely(msg->rx_buf))
 		reinit_completion(&dsim->rx_done);
 
+	/* config LPM for CMD TX */
 	use_lpm = msg->flags & MIPI_DSI_MSG_USE_LPM ? true : false;
+	sec_mipi_dsim_config_cmd_lpm(dsim, use_lpm);
 
 	if (packet.payload_length) {		/* Long Packet case */
 		reinit_completion(&dsim->pl_tx_done);
@@ -757,9 +738,13 @@ static ssize_t sec_mipi_dsim_host_transfer(struct mipi_dsi_host *host,
 							  msg->rx_len);
 		if (ret < 0)
 			return ret;
+
+		bytes = msg->rx_len;
+	} else {
+		bytes = packet.size;
 	}
 
-	return 0;
+	return bytes;
 }
 
 static const struct mipi_dsi_host_ops sec_mipi_dsim_host_ops = {
@@ -768,14 +753,16 @@ static const struct mipi_dsi_host_ops sec_mipi_dsim_host_ops = {
 	.transfer = sec_mipi_dsim_host_transfer,
 };
 
-static int sec_mipi_dsim_bridge_attach(struct drm_bridge *bridge)
+static int sec_mipi_dsim_bridge_attach(struct drm_bridge *bridge,
+					enum drm_bridge_attach_flags flags)
 {
 	int ret;
+	bool attach_bridge = false;
 	struct sec_mipi_dsim *dsim = bridge->driver_private;
 	struct device *dev = dsim->dev;
 	struct device_node *np = dev->of_node;
-	struct device_node *endpoint, *remote;
-	struct drm_bridge *next = NULL;
+	struct device_node *endpoint, *remote = NULL;
+	struct drm_bridge *next = ERR_PTR(-ENODEV);
 	struct drm_encoder *encoder = dsim->encoder;
 
 	/* TODO: All bridges and planes should have already been added */
@@ -790,7 +777,14 @@ static int sec_mipi_dsim_bridge_attach(struct drm_bridge *bridge)
 	if (!endpoint)
 		return -ENODEV;
 
-	while(endpoint && !next) {
+	while (endpoint) {
+		/* check the endpoint can attach bridge or not */
+		attach_bridge = of_property_read_bool(endpoint, "attach-bridge");
+		if (!attach_bridge) {
+			endpoint = of_graph_get_next_endpoint(np, endpoint);
+			continue;
+		}
+
 		remote = of_graph_get_remote_port_parent(endpoint);
 
 		if (!remote || !of_device_is_available(remote)) {
@@ -809,16 +803,24 @@ static int sec_mipi_dsim_bridge_attach(struct drm_bridge *bridge)
 		endpoint = of_graph_get_next_endpoint(np, endpoint);
 	}
 
-	/* No valid dsi device attached */
+	/* No workable bridge exists */
+	if (IS_ERR(next))
+		return PTR_ERR(next);
+
+	/* For the panel driver loading is after dsim bridge,
+	 * defer bridge binding to wait for panel driver ready.
+	 * The disadvantage of probe defer is endless probing
+	 * in some cases.
+	 */
 	if (!next)
-		return -ENODEV;
+		return -EPROBE_DEFER;
 
 	/* duplicate bridges or next bridge exists */
-	WARN_ON(bridge == next || bridge->next || dsim->next);
+	WARN_ON(bridge == next || drm_bridge_get_next_bridge(bridge) || dsim->next);
 
 	dsim->next = next;
 	next->encoder = encoder;
-	ret = drm_bridge_attach(encoder, next, bridge);
+	ret = drm_bridge_attach(encoder, next, bridge, flags);
 	if (ret) {
 		dev_err(dev, "Unable to attach bridge %s: %d\n",
 			remote->name, ret);
@@ -826,9 +828,6 @@ static int sec_mipi_dsim_bridge_attach(struct drm_bridge *bridge)
 		return ret;
 	}
 
-	/* bridge chains */
-	bridge->next = next;
-
 	return 0;
 }
 
@@ -872,7 +871,7 @@ static int sec_mipi_dsim_config_pll(struct sec_mipi_dsim *dsim)
 
 static void sec_mipi_dsim_set_main_mode(struct sec_mipi_dsim *dsim)
 {
-	uint32_t bpp, hfp_wc, hbp_wc, hsa_wc;
+	uint32_t bpp, hfp_wc, hbp_wc, hsa_wc, wc;
 	uint32_t mdresol = 0, mvporch = 0, mhporch = 0, msync = 0;
 	struct videomode *vmode = &dsim->vmode;
 
@@ -888,9 +887,15 @@ static void sec_mipi_dsim_set_main_mode(struct sec_mipi_dsim *dsim)
 	bpp = mipi_dsi_pixel_format_to_bpp(dsim->format);
 
 	/* calculate hfp & hbp word counts */
-	if (dsim->panel || !dsim->hpar) {
-		hfp_wc = vmode->hfront_porch * (bpp >> 3);
-		hbp_wc = vmode->hback_porch * (bpp >> 3);
+	if (!dsim->hpar) {
+		wc = DIV_ROUND_UP(vmode->hfront_porch * (bpp >> 3),
+				  dsim->lanes);
+		hfp_wc = wc > MIPI_HFP_PKT_OVERHEAD ?
+			 wc - MIPI_HFP_PKT_OVERHEAD : vmode->hfront_porch;
+		wc = DIV_ROUND_UP(vmode->hback_porch * (bpp >> 3),
+				  dsim->lanes);
+		hbp_wc = wc > MIPI_HBP_PKT_OVERHEAD ?
+			 wc - MIPI_HBP_PKT_OVERHEAD : vmode->hback_porch;
 	} else {
 		hfp_wc = dsim->hpar->hfp_wc;
 		hbp_wc = dsim->hpar->hbp_wc;
@@ -902,9 +907,12 @@ static void sec_mipi_dsim_set_main_mode(struct sec_mipi_dsim *dsim)
 	dsim_write(dsim, mhporch, DSIM_MHPORCH);
 
 	/* calculate hsa word counts */
-	if (dsim->panel || !dsim->hpar)
-		hsa_wc = vmode->hsync_len * (bpp >> 3);
-	else
+	if (!dsim->hpar) {
+		wc = DIV_ROUND_UP(vmode->hsync_len * (bpp >> 3),
+				  dsim->lanes);
+		hsa_wc = wc > MIPI_HSA_PKT_OVERHEAD ?
+			 wc - MIPI_HSA_PKT_OVERHEAD : vmode->hsync_len;
+	} else
 		hsa_wc = dsim->hpar->hsa_wc;
 
 	msync |= MSYNC_SET_MAINVSA(vmode->vsync_len) |
@@ -926,14 +934,15 @@ static void sec_mipi_dsim_config_dpi(struct sec_mipi_dsim *dsim)
 
 	if (dsim->mode_flags & MIPI_DSI_CLOCK_NON_CONTINUOUS) {
 		config |= CONFIG_NON_CONTINUOUS_CLOCK_LANE;
-		config |= CONFIG_CLKLANE_STOP_START;
+		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO)
+			config |= CONFIG_CLKLANE_STOP_START;
 	}
 
 	if (dsim->mode_flags & MIPI_DSI_MODE_VSYNC_FLUSH)
 		config |= CONFIG_MFLUSH_VS;
 
 	/* disable EoT packets in HS mode */
-	if (dsim->mode_flags & MIPI_DSI_MODE_EOT_PACKET)
+	if (dsim->mode_flags & MIPI_DSI_MODE_NO_EOT_PACKET)
 		config |= CONFIG_EOT_R03;
 
 	if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO) {
@@ -951,36 +960,37 @@ static void sec_mipi_dsim_config_dpi(struct sec_mipi_dsim *dsim)
 		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_HSE)
 			config |= CONFIG_HSEDISABLEMODE;
 
-		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_HFP)
+		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_NO_HFP)
 			config |= CONFIG_HFPDISABLEMODE;
 
-		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_HBP)
+		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_NO_HBP)
 			config |= CONFIG_HBPDISABLEMODE;
 
-		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_HSA)
+		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_NO_HSA)
 			config |= CONFIG_HSADISABLEMODE;
 	}
 
 	config |= CONFIG_SET_MAINVC(dsim->channel);
 
-	if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO) {
-		switch (dsim->format) {
-		case MIPI_DSI_FMT_RGB565:
-			config |= CONFIG_SET_MAINPIXFORMAT(0x4);
-			break;
-		case MIPI_DSI_FMT_RGB666_PACKED:
+	switch (dsim->format) {
+	case MIPI_DSI_FMT_RGB565:
+		config |= dsim->mode_flags & MIPI_DSI_MODE_VIDEO ?
+			  CONFIG_SET_MAINPIXFORMAT(0x4) :
+			  CONFIG_SET_MAINPIXFORMAT(0x3);
+		break;
+	case MIPI_DSI_FMT_RGB666_PACKED:
+		if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO)
 			config |= CONFIG_SET_MAINPIXFORMAT(0x5);
-			break;
-		case MIPI_DSI_FMT_RGB666:
-			config |= CONFIG_SET_MAINPIXFORMAT(0x6);
-			break;
-		case MIPI_DSI_FMT_RGB888:
-			config |= CONFIG_SET_MAINPIXFORMAT(0x7);
-			break;
-		default:
-			config |= CONFIG_SET_MAINPIXFORMAT(0x7);
-			break;
-		}
+		break;
+	case MIPI_DSI_FMT_RGB666:
+		config |= CONFIG_SET_MAINPIXFORMAT(0x6);
+		break;
+	case MIPI_DSI_FMT_RGB888:
+		config |= CONFIG_SET_MAINPIXFORMAT(0x7);
+		break;
+	default:
+		config |= CONFIG_SET_MAINPIXFORMAT(0x7);
+		break;
 	}
 
 	/* config data lanes number and enable lanes */
@@ -1070,7 +1080,7 @@ static void sec_mipi_dsim_init_fifo_pointers(struct sec_mipi_dsim *dsim)
 static void sec_mipi_dsim_config_clkctrl(struct sec_mipi_dsim *dsim)
 {
 	uint32_t clkctrl = 0, data_lanes_en;
-	uint64_t byte_clk, esc_prescaler;
+	uint32_t byte_clk, esc_prescaler;
 
 	clkctrl |= CLKCTRL_TXREQUESTHSCLK;
 
@@ -1092,7 +1102,7 @@ static void sec_mipi_dsim_config_clkctrl(struct sec_mipi_dsim *dsim)
 	 * EscClk = ByteClk / EscPrescaler;
 	 */
 	byte_clk = dsim->bit_clk >> 3;
-	esc_prescaler = DIV_ROUND_UP_ULL(byte_clk, MAX_ESC_CLK_FREQ);
+	esc_prescaler = DIV_ROUND_UP(byte_clk, MAX_ESC_CLK_FREQ);
 	clkctrl |= CLKCTRL_SET_ESCPRESCALER(esc_prescaler);
 
 	dsim_write(dsim, clkctrl, DSIM_CLKCTRL);
@@ -1113,72 +1123,229 @@ static void sec_mipi_dsim_set_standby(struct sec_mipi_dsim *dsim,
 	dsim_write(dsim, mdresol, DSIM_MDRESOL);
 }
 
+struct dsim_pll_pms *sec_mipi_dsim_calc_pmsk(struct sec_mipi_dsim *dsim)
+{
+	uint32_t p, m, s;
+	uint32_t best_p = 0, best_m = 0, best_s = 0;
+	uint32_t fin, fout;
+	uint32_t s_pow_2, raw_s;
+	uint64_t mfin, pfvco, pfout, psfout;
+	uint32_t delta, best_delta = ~0U;
+	struct dsim_pll_pms *pll_pms;
+	struct device *dev = dsim->dev;
+	const struct sec_mipi_dsim_plat_data *pdata = dsim->pdata;
+	struct sec_mipi_dsim_pll dpll = *pdata->dphy_pll;
+	struct sec_mipi_dsim_range *prange = &dpll.p;
+	struct sec_mipi_dsim_range *mrange = &dpll.m;
+	struct sec_mipi_dsim_range *srange = &dpll.s;
+	struct sec_mipi_dsim_range *krange = &dpll.k;
+	struct sec_mipi_dsim_range *fvco_range  = &dpll.fvco;
+	struct sec_mipi_dsim_range *fpref_range = &dpll.fpref;
+	struct sec_mipi_dsim_range pr_new = *prange;
+	struct sec_mipi_dsim_range sr_new = *srange;
+
+	pll_pms = devm_kzalloc(dev, sizeof(*pll_pms), GFP_KERNEL);
+	if (!pll_pms) {
+		dev_err(dev, "Unable to allocate 'pll_pms'\n");
+		return ERR_PTR(-ENOMEM);
+	}
+
+	fout = dsim->bit_clk;
+	fin  = dsim->pref_clk;
+
+	/* TODO: ignore 'k' for PMS calculation,
+	 * only use 'p', 'm' and 's' to generate
+	 * the requested PLL output clock.
+	 */
+	krange->min = 0;
+	krange->max = 0;
+
+	/* narrow 'p' range via 'Fpref' limitation:
+	 * Fpref : [2MHz ~ 30MHz] (Fpref = Fin / p)
+	 */
+	prange->min = max(prange->min, DIV_ROUND_UP(fin, fpref_range->max));
+	prange->max = min(prange->max, fin / fpref_range->min);
+
+	/* narrow 'm' range via 'Fvco' limitation:
+	 * Fvco: [1050MHz ~ 2100MHz] (Fvco = ((m + k / 65536) * Fin) / p)
+	 * So, m = Fvco * p / Fin and Fvco > Fin;
+	 */
+	pfvco = (uint64_t)fvco_range->min * prange->min;
+	mrange->min = max_t(uint32_t, mrange->min,
+			    DIV_ROUND_UP_ULL(pfvco, fin));
+	pfvco = (uint64_t)fvco_range->max * prange->max;
+	mrange->max = min_t(uint32_t, mrange->max,
+			    DIV_ROUND_UP_ULL(pfvco, fin));
+
+	dev_dbg(dev, "p: min = %u, max = %u, "
+		     "m: min = %u, max = %u, "
+		     "s: min = %u, max = %u\n",
+		prange->min, prange->max, mrange->min,
+		mrange->max, srange->min, srange->max);
+
+	/* first determine 'm', then can determine 'p', last determine 's' */
+	for (m = mrange->min; m <= mrange->max; m++) {
+		/* p = m * Fin / Fvco */
+		mfin = (uint64_t)m * fin;
+		pr_new.min = max_t(uint32_t, prange->min,
+				   DIV_ROUND_UP_ULL(mfin, fvco_range->max));
+		pr_new.max = min_t(uint32_t, prange->max,
+				   (mfin / fvco_range->min));
+
+		if (pr_new.max < pr_new.min || pr_new.min < prange->min)
+			continue;
+
+		for (p = pr_new.min; p <= pr_new.max; p++) {
+			/* s = order_pow_of_two((m * Fin) / (p * Fout)) */
+			pfout = (uint64_t)p * fout;
+			raw_s = DIV_ROUND_CLOSEST_ULL(mfin, pfout);
+
+			s_pow_2 = rounddown_pow_of_two(raw_s);
+			sr_new.min = max_t(uint32_t, srange->min,
+					   order_base_2(s_pow_2));
+
+			s_pow_2 = roundup_pow_of_two(DIV_ROUND_CLOSEST_ULL(mfin, pfout));
+			sr_new.max = min_t(uint32_t, srange->max,
+					   order_base_2(s_pow_2));
+
+			if (sr_new.max < sr_new.min || sr_new.min < srange->min)
+				continue;
+
+			for (s = sr_new.min; s <= sr_new.max; s++) {
+				/* fout = m * Fin / (p * 2^s) */
+				psfout = pfout * (1 << s);
+				delta = abs(psfout - mfin);
+				if (delta < best_delta) {
+					best_p = p;
+					best_m = m;
+					best_s = s;
+					best_delta = delta;
+				}
+			}
+		}
+	}
+
+	if (best_delta == ~0U) {
+		devm_kfree(dev, pll_pms);
+		return ERR_PTR(-EINVAL);
+	}
+
+	pll_pms->p = best_p;
+	pll_pms->m = best_m;
+	pll_pms->s = best_s;
+
+	dev_dbg(dev, "fout = %u, fin = %u, m = %u, "
+		     "p = %u, s = %u, best_delta = %u\n",
+		fout, fin, pll_pms->m, pll_pms->p, pll_pms->s, best_delta);
+
+	return pll_pms;
+}
+
 int sec_mipi_dsim_check_pll_out(void *driver_private,
 				const struct drm_display_mode *mode)
 {
 	int bpp;
-	uint64_t pix_clk, bit_clk, ref_clk;
+	uint32_t pix_clk, bit_clk;
 	struct sec_mipi_dsim *dsim = driver_private;
 	const struct sec_mipi_dsim_plat_data *pdata = dsim->pdata;
 	const struct dsim_hblank_par *hpar;
-	const struct dsim_pll_pms *pms;
+	const struct dsim_pll_pms *pmsk;
 
 	bpp = mipi_dsi_pixel_format_to_bpp(dsim->format);
 	if (bpp < 0)
 		return -EINVAL;
 
-	pix_clk = mode->clock * 1000;
-	bit_clk = DIV_ROUND_UP_ULL(pix_clk * bpp, dsim->lanes);
+	pix_clk = mode->clock;
+	bit_clk = DIV_ROUND_UP(pix_clk * bpp, dsim->lanes);
 
-	if (bit_clk > pdata->max_data_rate) {
+	if (bit_clk * 1000 > pdata->max_data_rate) {
 		dev_err(dsim->dev,
 			"reuest bit clk freq exceeds lane's maximum value\n");
 		return -EINVAL;
 	}
 
-	dsim->pix_clk = DIV_ROUND_UP_ULL(pix_clk, 1000);
-	dsim->bit_clk = DIV_ROUND_UP_ULL(bit_clk, 1000);
-
-	dsim->pms = 0x4210;
+	dsim->pix_clk = pix_clk;
+	dsim->bit_clk = bit_clk;
 	dsim->hpar = NULL;
-	if (dsim->panel)
-		return 0;
+
+	pmsk = sec_mipi_dsim_calc_pmsk(dsim);
+	if (IS_ERR(pmsk)) {
+		dev_err(dsim->dev,
+			"failed to get pmsk for: fin = %u, fout = %u\n",
+			dsim->pref_clk, dsim->bit_clk);
+		return -EINVAL;
+	}
+
+	dsim->pms = PLLCTRL_SET_P(pmsk->p) |
+		    PLLCTRL_SET_M(pmsk->m) |
+		    PLLCTRL_SET_S(pmsk->s);
+
+	/* free 'dsim_pll_pms' structure data which is
+	 * allocated in 'sec_mipi_dsim_calc_pmsk()'.
+	 */
+	devm_kfree(dsim->dev, (void *)pmsk);
 
 	if (dsim->mode_flags & MIPI_DSI_MODE_VIDEO_SYNC_PULSE) {
 		hpar = sec_mipi_dsim_get_hblank_par(mode->name,
-						    mode->vrefresh,
+						    drm_mode_vrefresh(mode),
 						    dsim->lanes);
-		if (!hpar)
-			return -EINVAL;
 		dsim->hpar = hpar;
-
-		pms = sec_mipi_dsim_get_pms(dsim->bit_clk);
-		if (WARN_ON(!pms))
-			return -EINVAL;
-
-		ref_clk = PHY_REF_CLK / 1000;
-		/* TODO: add PMS calculate and check
-		 * Only support '1080p@60Hz' for now,
-		 * add other modes support later
-		 */
-		dsim->pms = PLLCTRL_SET_P(pms->p) |
-			    PLLCTRL_SET_M(pms->m) |
-			    PLLCTRL_SET_S(pms->s);
+		if (!hpar)
+			dev_dbg(dsim->dev, "no pre-exist hpar can be used\n");
 	}
 
 	return 0;
 }
 EXPORT_SYMBOL(sec_mipi_dsim_check_pll_out);
 
-static void sec_mipi_dsim_bridge_enable(struct drm_bridge *bridge)
+static
+struct drm_crtc *sec_mipi_dsim_get_new_crtc(struct sec_mipi_dsim *dsim,
+					    struct drm_atomic_state *state)
+{
+	struct drm_encoder *encoder = dsim->encoder;
+	struct drm_connector *connector;
+	struct drm_connector_state *conn_state;
+
+	connector = drm_atomic_get_new_connector_for_encoder(state, encoder);
+	if (!connector)
+		return NULL;
+
+	conn_state = drm_atomic_get_new_connector_state(state, connector);
+	if (!conn_state)
+		return NULL;
+
+	return conn_state->crtc;
+}
+
+static void
+sec_mipi_dsim_bridge_atomic_enable(struct drm_bridge *bridge,
+				   struct drm_bridge_state *old_bridge_state)
 {
 	int ret;
 	struct sec_mipi_dsim *dsim = bridge->driver_private;
+	struct drm_atomic_state *old_state = old_bridge_state->base.state;
+	struct drm_crtc *crtc;
+	struct drm_crtc_state *old_crtc_state;
 
 	/* At this moment, the dsim bridge's preceding encoder has
 	 * already been enabled. So the dsim can be configed here
 	 */
 
+	crtc = sec_mipi_dsim_get_new_crtc(dsim, old_state);
+	if (!crtc) {
+		dev_err(dsim->dev, "bridge is enabling without CRTC\n");
+		return;
+	}
+
+	old_crtc_state = drm_atomic_get_old_crtc_state(old_state, crtc);
+	/* Don't do enablement operation if we're coming back from PSR. */
+	if (old_crtc_state && old_crtc_state->self_refresh_active &&
+	    dsim->enabled)
+		return;
+
+	if (dsim->enabled)
+		return;
+
 	/* config main display mode */
 	sec_mipi_dsim_set_main_mode(dsim);
 
@@ -1222,6 +1389,8 @@ static void sec_mipi_dsim_bridge_enable(struct drm_bridge *bridge)
 	/* enable data transfer of dsim */
 	sec_mipi_dsim_set_standby(dsim, true);
 
+	dsim->enabled = true;
+
 	return;
 
 panel_unprepare:
@@ -1256,10 +1425,29 @@ static void sec_mipi_dsim_disable_pll(struct sec_mipi_dsim *dsim)
 	dsim_write(dsim, pllctrl, DSIM_PLLCTRL);
 }
 
-static void sec_mipi_dsim_bridge_disable(struct drm_bridge *bridge)
+static void
+sec_mipi_dsim_bridge_atomic_disable(struct drm_bridge *bridge,
+				    struct drm_bridge_state *old_bridge_state)
 {
 	int ret;
 	struct sec_mipi_dsim *dsim = bridge->driver_private;
+	struct drm_atomic_state *old_state = old_bridge_state->base.state;
+	struct drm_crtc *crtc;
+	struct drm_crtc_state *new_crtc_state;
+
+	crtc = sec_mipi_dsim_get_new_crtc(dsim, old_state);
+	/* No CRTC means we're doing a full shutdown. */
+	if (!crtc)
+		goto disable;
+
+	new_crtc_state = drm_atomic_get_new_crtc_state(old_state, crtc);
+	/* Don't do disablement operation if we're entering PSR. */
+	if (!new_crtc_state || new_crtc_state->self_refresh_active)
+		return;
+
+disable:
+	if (!dsim->enabled)
+		return;
 
 	/* disable panel if exists */
 	if (dsim->panel) {
@@ -1283,64 +1471,13 @@ static void sec_mipi_dsim_bridge_disable(struct drm_bridge *bridge)
 		if (unlikely(ret))
 			dev_err(dsim->dev, "panel unprepare failed: %d\n", ret);
 	}
-}
 
-static bool sec_mipi_dsim_bridge_mode_fixup(struct drm_bridge *bridge,
-					    const struct drm_display_mode *mode,
-					    struct drm_display_mode *adjusted_mode)
-{
-	int private_flags;
-	struct sec_mipi_dsim *dsim = bridge->driver_private;
-
-	/* Since mipi dsi cannot do color conversion,
-	 * so the pixel format output by mipi dsi should
-	 * be the same with the pixel format recieved by
-	 * mipi dsi. And the pixel format information needs
-	 * to be passed to CRTC to be checked with the CRTC
-	 * attached plane fb pixel format.
-	 */
-	switch (dsim->format) {
-	case MIPI_DSI_FMT_RGB888:
-		private_flags = MEDIA_BUS_FMT_RGB888_1X24;
-		break;
-	case MIPI_DSI_FMT_RGB666:
-		private_flags = MEDIA_BUS_FMT_RGB666_1X24_CPADHI;
-		break;
-	case MIPI_DSI_FMT_RGB666_PACKED:
-		private_flags = MEDIA_BUS_FMT_RGB666_1X18;
-		break;
-	case MIPI_DSI_FMT_RGB565:
-		private_flags = MEDIA_BUS_FMT_RGB565_1X16;
-		break;
-	default:
-		return false;
-	}
-
-	adjusted_mode->private_flags = private_flags;
-
-	/* the 'bus_flags' in connector's display_info is useless
-	 * for mipi dsim, since dsim only sends packets with no
-	 * polarities information in the packets. But the dsim
-	 * host has some polarities requirements for the CRTC:
-	 * dsim only can accpet active high Vsync, Hsync and DE
-	 * signals.
-	 */
-	if (adjusted_mode->flags & DRM_MODE_FLAG_NHSYNC) {
-		adjusted_mode->flags &= ~DRM_MODE_FLAG_NHSYNC;
-		adjusted_mode->flags |= DRM_MODE_FLAG_PHSYNC;
-	}
-
-	if (adjusted_mode->flags & DRM_MODE_FLAG_NVSYNC) {
-		adjusted_mode->flags &= ~DRM_MODE_FLAG_NVSYNC;
-		adjusted_mode->flags |= DRM_MODE_FLAG_PVSYNC;
-	}
-
-	return true;
+	dsim->enabled = false;
 }
 
 static void sec_mipi_dsim_bridge_mode_set(struct drm_bridge *bridge,
-					  struct drm_display_mode *mode,
-					  struct drm_display_mode *adjusted_mode)
+					  const struct drm_display_mode *mode,
+					  const struct drm_display_mode *adjusted_mode)
 {
 	struct sec_mipi_dsim *dsim = bridge->driver_private;
 
@@ -1352,7 +1489,134 @@ static void sec_mipi_dsim_bridge_mode_set(struct drm_bridge *bridge,
 	 * so it is called not every time atomic commit.
 	 */
 
-	/* workaround for CEA standard mode "1280x720@60"
+	drm_display_mode_to_videomode(adjusted_mode, &dsim->vmode);
+}
+
+static u32 *sec_mipi_dsim_atomic_get_input_bus_fmts(struct drm_bridge *bridge,
+						    struct drm_bridge_state *bridge_state,
+						    struct drm_crtc_state *crtc_state,
+						    struct drm_connector_state *conn_state,
+						    u32 output_fmt,
+						    unsigned int *num_input_fmts)
+{
+	u32 *input_fmts;
+	struct sec_mipi_dsim *dsim = bridge->driver_private;
+
+	/* use dsi format to determine output bus format
+	 * if it is passed with MEDIA_BUS_FMT_FIXED
+	 */
+	if (output_fmt == MEDIA_BUS_FMT_FIXED) {
+		switch (dsim->format) {
+		case MIPI_DSI_FMT_RGB888:
+			output_fmt = MEDIA_BUS_FMT_RGB888_1X24;
+			break;
+		case MIPI_DSI_FMT_RGB666:
+			output_fmt = MEDIA_BUS_FMT_RGB666_1X24_CPADHI;
+			break;
+		case MIPI_DSI_FMT_RGB666_PACKED:
+			output_fmt = MEDIA_BUS_FMT_RGB666_1X18;
+			break;
+		case MIPI_DSI_FMT_RGB565:
+			output_fmt = MEDIA_BUS_FMT_RGB565_1X16;
+			break;
+		default:
+			return NULL;
+		}
+	} else {
+		/* check if the output format matches with
+		 * DSI device requested format
+		 */
+		switch (dsim->format) {
+		case MIPI_DSI_FMT_RGB888:
+			if (output_fmt != MEDIA_BUS_FMT_RGB888_1X24)
+				return NULL;
+			break;
+		case MIPI_DSI_FMT_RGB666:
+			if (output_fmt != MEDIA_BUS_FMT_RGB666_1X24_CPADHI)
+				return NULL;
+			break;
+		case MIPI_DSI_FMT_RGB666_PACKED:
+			if (output_fmt != MEDIA_BUS_FMT_RGB666_1X18)
+				return NULL;
+			break;
+		case MIPI_DSI_FMT_RGB565:
+			if (output_fmt != MEDIA_BUS_FMT_RGB565_1X16)
+				return NULL;
+			break;
+		default:
+			return NULL;
+		}
+	}
+
+	/* Since dsim cannot do any color conversion, so the
+	 * bus format output by mipi dsi should just be
+	 * propagated to the bus format recieved by mipi dsi
+	 * directly.
+	 */
+	input_fmts = drm_atomic_helper_bridge_propagate_bus_fmt(bridge,
+								bridge_state,
+								crtc_state,
+								conn_state,
+								output_fmt,
+								num_input_fmts);
+
+	return input_fmts;
+}
+
+static int sec_mipi_dsim_bridge_atomic_check(struct drm_bridge *bridge,
+					     struct drm_bridge_state *bridge_state,
+					     struct drm_crtc_state *crtc_state,
+					     struct drm_connector_state *conn_state)
+{
+	struct drm_display_mode *adjusted_mode = &crtc_state->adjusted_mode;
+	struct drm_bus_cfg *input_bus_cfg = &bridge_state->input_bus_cfg;
+	struct sec_mipi_dsim *dsim = bridge->driver_private;
+
+	/*
+	 * If the DSI mode is not video mode, it implies
+	 * that the connector is self refresh aware.
+	 */
+	if (!(dsim->mode_flags & MIPI_DSI_MODE_VIDEO))
+		conn_state->self_refresh_aware = true;
+
+	/* seems unnecessary to check the input bus format
+	 * again, since during the negotiation process, it
+	 * has already been checked
+	 */
+	if (bridge_state->output_bus_cfg.format == MEDIA_BUS_FMT_FIXED)
+		bridge_state->output_bus_cfg.format = bridge_state->input_bus_cfg.format;
+
+	/* adjust Hsync and Vsync polarities for drm display mode,
+	 * since DSIM can only accept active high Hsync and Vsync
+	 * signals
+	 */
+	if (adjusted_mode->flags & DRM_MODE_FLAG_NHSYNC) {
+		adjusted_mode->flags &= ~DRM_MODE_FLAG_NHSYNC;
+		adjusted_mode->flags |= DRM_MODE_FLAG_PHSYNC;
+	}
+
+	if (adjusted_mode->flags & DRM_MODE_FLAG_NVSYNC) {
+		adjusted_mode->flags &= ~DRM_MODE_FLAG_NVSYNC;
+		adjusted_mode->flags |= DRM_MODE_FLAG_PVSYNC;
+	}
+
+	/* adjust input DE and pixel data sample polarities for
+	 * input bus_flags, since DSIM can only accept active
+	 * high DE and sample pixel data at clock postive edge
+	 */
+	if (input_bus_cfg->flags & DRM_BUS_FLAG_DE_LOW ||
+	    !(input_bus_cfg->flags & DRM_BUS_FLAG_DE_HIGH)) {
+		input_bus_cfg->flags &= ~DRM_BUS_FLAG_DE_LOW;
+		input_bus_cfg->flags |= DRM_BUS_FLAG_DE_HIGH;
+	}
+
+	if (input_bus_cfg->flags & DRM_BUS_FLAG_PIXDATA_SAMPLE_NEGEDGE ||
+	    !(input_bus_cfg->flags & DRM_BUS_FLAG_PIXDATA_SAMPLE_POSEDGE)) {
+		input_bus_cfg->flags &= ~DRM_BUS_FLAG_PIXDATA_SAMPLE_NEGEDGE;
+		input_bus_cfg->flags |= DRM_BUS_FLAG_PIXDATA_SAMPLE_POSEDGE;
+	}
+
+	/* workaround for CEA standard mode "1280x720@60" "1920x1080p24"
 	 * display on 4 data lanes with Non-burst with sync
 	 * pulse DSI mode, since use the standard horizontal
 	 * timings cannot display correctly. And this code
@@ -1360,8 +1624,8 @@ static void sec_mipi_dsim_bridge_mode_set(struct drm_bridge *bridge,
 	 * since the DSI device lane number change always
 	 * happens after that.
 	 */
-	if (!strcmp(mode->name, "1280x720") &&
-	    mode->vrefresh == 60	    &&
+	if (!strcmp(adjusted_mode->name, "1280x720") &&
+	    drm_mode_vrefresh(adjusted_mode) == 60   &&
 	    dsim->lanes == 4		    &&
 	    dsim->mode_flags & MIPI_DSI_MODE_VIDEO_SYNC_PULSE) {
 		adjusted_mode->hsync_start += 2;
@@ -1369,26 +1633,33 @@ static void sec_mipi_dsim_bridge_mode_set(struct drm_bridge *bridge,
 		adjusted_mode->htotal      += 2;
 	}
 
-	drm_display_mode_to_videomode(adjusted_mode, &dsim->vmode);
+	if (!strcmp(adjusted_mode->name, "1920x1080") &&
+	    drm_mode_vrefresh(adjusted_mode) == 24 &&
+	    dsim->lanes == 4		    &&
+	    dsim->mode_flags & MIPI_DSI_MODE_VIDEO_SYNC_PULSE) {
+		adjusted_mode->hsync_start += 2;
+		adjusted_mode->hsync_end   += 2;
+		adjusted_mode->htotal      += 2;
+	}
+
+	return 0;
 }
 
 static const struct drm_bridge_funcs sec_mipi_dsim_bridge_funcs = {
+	.atomic_check = sec_mipi_dsim_bridge_atomic_check,
+	.atomic_reset = drm_atomic_helper_bridge_reset,
+	.atomic_duplicate_state = drm_atomic_helper_bridge_duplicate_state,
+	.atomic_destroy_state = drm_atomic_helper_bridge_destroy_state,
+	.atomic_get_input_bus_fmts = sec_mipi_dsim_atomic_get_input_bus_fmts,
 	.attach     = sec_mipi_dsim_bridge_attach,
-	.enable     = sec_mipi_dsim_bridge_enable,
-	.disable    = sec_mipi_dsim_bridge_disable,
+	.atomic_enable	= sec_mipi_dsim_bridge_atomic_enable,
+	.atomic_disable	= sec_mipi_dsim_bridge_atomic_disable,
 	.mode_set   = sec_mipi_dsim_bridge_mode_set,
-	.mode_fixup = sec_mipi_dsim_bridge_mode_fixup,
 };
 
 void sec_mipi_dsim_suspend(struct device *dev)
 {
-	struct sec_mipi_dsim *dsim = dev_get_drvdata(dev);
-
 	/* TODO: add dsim reset */
-
-	clk_disable_unprepare(dsim->clk_cfg);
-
-	clk_disable_unprepare(dsim->clk_pllref);
 }
 EXPORT_SYMBOL(sec_mipi_dsim_suspend);
 
@@ -1396,10 +1667,6 @@ void sec_mipi_dsim_resume(struct device *dev)
 {
 	struct sec_mipi_dsim *dsim = dev_get_drvdata(dev);
 
-	clk_prepare_enable(dsim->clk_pllref);
-
-	clk_prepare_enable(dsim->clk_cfg);
-
 	sec_mipi_dsim_irq_init(dsim);
 
 	/* TODO: add dsim de-reset */
@@ -1633,7 +1900,7 @@ static int sec_mipi_dsim_connector_get_modes(struct drm_connector *connector)
 	if (WARN_ON(!dsim->panel))
 		return -ENODEV;
 
-	return drm_panel_get_modes(dsim->panel);
+	return drm_panel_get_modes(dsim->panel, connector);
 }
 
 static const struct drm_connector_helper_funcs
@@ -1660,7 +1927,7 @@ static const struct drm_connector_funcs sec_mipi_dsim_connector_funcs = {
 };
 
 int sec_mipi_dsim_bind(struct device *dev, struct device *master, void *data,
-		       struct drm_encoder *encoder, struct resource *res,
+		       struct drm_encoder *encoder, void __iomem *base,
 		       int irq, const struct sec_mipi_dsim_plat_data *pdata)
 {
 	int ret, version;
@@ -1668,6 +1935,7 @@ int sec_mipi_dsim_bind(struct device *dev, struct device *master, void *data,
 	struct drm_bridge *bridge;
 	struct drm_connector *connector;
 	struct sec_mipi_dsim *dsim;
+	struct device_node *node = NULL;
 
 	dev_dbg(dev, "sec-dsim bridge bind begin\n");
 
@@ -1678,47 +1946,30 @@ int sec_mipi_dsim_bind(struct device *dev, struct device *master, void *data,
 	}
 
 	dsim->dev = dev;
-	dsim->irq = irq;
+	dsim->base = base;
 	dsim->pdata = pdata;
 	dsim->encoder = encoder;
 
 	dsim->dsi_host.ops = &sec_mipi_dsim_host_ops;
 	dsim->dsi_host.dev = dev;
 
-	dsim->base = devm_ioremap_resource(dev, res);
-	if (IS_ERR(dsim->base))
-		return PTR_ERR(dsim->base);
+	dev_set_drvdata(dev, dsim);
 
-	dsim->clk_pllref = devm_clk_get(dev, "pll-ref");
-	if (IS_ERR(dsim->clk_pllref)) {
-		ret = PTR_ERR(dsim->clk_pllref);
-		dev_err(dev, "Unable to get phy pll reference clock: %d\n", ret);
-		return ret;
-	}
-
-	dsim->clk_cfg = devm_clk_get(dev, "cfg");
-	if (IS_ERR(dsim->clk_cfg)) {
-		ret = PTR_ERR(dsim->clk_cfg);
-		dev_err(dev, "Unable to get configuration clock: %d\n", ret);
-		return ret;
-	}
-
-	clk_prepare_enable(dsim->clk_cfg);
+	pm_runtime_get_sync(dev);
 	version = dsim_read(dsim, DSIM_VERSION);
 	WARN_ON(version != pdata->version);
-	clk_disable_unprepare(dsim->clk_cfg);
+	pm_runtime_put_sync(dev);
 
 	dev_info(dev, "version number is %#x\n", version);
 
-	/* TODO: set pll ref clock rate to be fixed with 27MHz */
-	ret = clk_set_rate(dsim->clk_pllref, PHY_REF_CLK);
+	/* set suitable rate for phy ref clock */
+	ret = sec_mipi_dsim_set_pref_rate(dsim);
 	if (ret) {
 		dev_err(dev, "failed to set pll ref clock rate\n");
 		return ret;
 	}
 
-	ret = devm_request_irq(dev, dsim->irq,
-			       sec_mipi_dsim_irq_handler,
+	ret = devm_request_irq(dev, irq, sec_mipi_dsim_irq_handler,
 			       0, dev_name(dev), dsim);
 	if (ret) {
 		dev_err(dev, "failed to request dsim irq: %d\n", ret);
@@ -1759,18 +2010,35 @@ int sec_mipi_dsim_bind(struct device *dev, struct device *master, void *data,
 	bridge->funcs = &sec_mipi_dsim_bridge_funcs;
 	bridge->of_node = dev->of_node;
 	bridge->encoder = encoder;
-	encoder->bridge = bridge;
-
-	dev_set_drvdata(dev, dsim);
 
 	/* attach sec dsim bridge and its next bridge if exists */
-	ret = drm_bridge_attach(encoder, bridge, NULL);
+	ret = drm_bridge_attach(encoder, bridge, NULL, 0);
 	if (ret) {
 		dev_err(dev, "Failed to attach bridge: %s\n", dev_name(dev));
+
+		/* no bridge exists, so defer probe to wait
+		 * panel driver loading
+		 */
+		if (ret != -EPROBE_DEFER) {
+			for_each_available_child_of_node(dev->of_node, node) {
+				/* skip nodes without reg property */
+				if (!of_find_property(node, "reg", NULL))
+					continue;
+
+				/* error codes only ENODEV or EPROBE_DEFER */
+				dsim->panel = of_drm_find_panel(node);
+				if (!IS_ERR(dsim->panel))
+					goto panel;
+
+				ret = PTR_ERR(dsim->panel);
+			}
+		}
+
 		mipi_dsi_host_unregister(&dsim->dsi_host);
 		return ret;
 	}
 
+panel:
 	if (dsim->panel) {
 		/* A panel has been attached */
 		connector = &dsim->connector;
@@ -1786,11 +2054,7 @@ int sec_mipi_dsim_bind(struct device *dev, struct device *master, void *data,
 		/* TODO */
 		connector->dpms = DRM_MODE_DPMS_OFF;
 
-		ret = drm_mode_connector_attach_encoder(connector, encoder);
-		if (ret)
-			goto cleanup_connector;
-
-		ret = drm_panel_attach(dsim->panel, connector);
+		ret = drm_connector_attach_encoder(connector, encoder);
 		if (ret)
 			goto cleanup_connector;
 	}
@@ -1811,11 +2075,8 @@ void sec_mipi_dsim_unbind(struct device *dev, struct device *master, void *data)
 {
 	struct sec_mipi_dsim *dsim = dev_get_drvdata(dev);
 
-	if (dsim->panel) {
-		drm_panel_detach(dsim->panel);
+	if (dsim->panel)
 		drm_connector_cleanup(&dsim->connector);
-		dsim->panel = NULL;
-	}
 
 	mipi_dsi_host_unregister(&dsim->dsi_host);
 }
