@@ -1,5 +1,5 @@
 /*
- * Copyright 2017-2018 NXP
+ * Copyright 2017-2019 NXP
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,22 +12,24 @@
  * for more details.
  */
 
-#include <drm/drmP.h>
+#include <drm/drm_vblank.h>
 #include <drm/drm_atomic.h>
 #include <drm/drm_atomic_helper.h>
+#include <drm/drm_blend.h>
+#include <drm/drm_color_mgmt.h>
 #include <drm/drm_fb_cma_helper.h>
+#include <drm/drm_fourcc.h>
+#include <drm/drm_gem_atomic_helper.h>
 #include <drm/drm_gem_cma_helper.h>
 #include <drm/drm_plane_helper.h>
 #include <video/dpu.h>
 #include <video/imx8-prefetch.h>
 #include "dpu-plane.h"
-#include "imx-drm.h"
+#include "../imx-drm.h"
 
-/*
- * RGB and packed/2planar YUV formats
- * are widely supported by many fetch units.
- */
-static const uint32_t dpu_primary_formats[] = {
+#define FRAC_16_16(mult, div)    (((mult) << 16) / (div))
+
+static const uint32_t dpu_formats[] = {
 	DRM_FORMAT_ARGB8888,
 	DRM_FORMAT_XRGB8888,
 	DRM_FORMAT_ABGR8888,
@@ -44,19 +46,6 @@ static const uint32_t dpu_primary_formats[] = {
 	DRM_FORMAT_NV21,
 };
 
-static const uint32_t dpu_overlay_formats[] = {
-	DRM_FORMAT_XRGB8888,
-	DRM_FORMAT_XBGR8888,
-	DRM_FORMAT_RGBX8888,
-	DRM_FORMAT_BGRX8888,
-	DRM_FORMAT_RGB565,
-
-	DRM_FORMAT_YUYV,
-	DRM_FORMAT_UYVY,
-	DRM_FORMAT_NV12,
-	DRM_FORMAT_NV21,
-};
-
 static const uint64_t dpu_format_modifiers[] = {
 	DRM_FORMAT_MOD_VIVANTE_TILED,
 	DRM_FORMAT_MOD_VIVANTE_SUPER_TILED,
@@ -65,6 +54,16 @@ static const uint64_t dpu_format_modifiers[] = {
 	DRM_FORMAT_MOD_INVALID,
 };
 
+static unsigned int dpu_plane_get_default_zpos(enum drm_plane_type type)
+{
+	if (type == DRM_PLANE_TYPE_PRIMARY)
+		return 0;
+	else if (type == DRM_PLANE_TYPE_OVERLAY)
+		return 1;
+
+	return 0;
+}
+
 static void dpu_plane_destroy(struct drm_plane *plane)
 {
 	struct dpu_plane *dpu_plane = to_dpu_plane(plane);
@@ -87,11 +86,11 @@ static void dpu_plane_reset(struct drm_plane *plane)
 	if (!state)
 		return;
 
-	state->base.zpos = plane->type == DRM_PLANE_TYPE_PRIMARY ? 0 : 1;
+	__drm_atomic_helper_plane_reset(plane, &state->base);
 
-	plane->state = &state->base;
-	plane->state->plane = plane;
-	plane->state->rotation = DRM_MODE_ROTATE_0;
+	plane->state->zpos = dpu_plane_get_default_zpos(plane->type);
+	plane->state->color_encoding = DRM_COLOR_YCBCR_BT601;
+	plane->state->color_range = DRM_COLOR_YCBCR_FULL_RANGE;
 }
 
 static struct drm_plane_state *
@@ -114,39 +113,22 @@ dpu_drm_atomic_plane_duplicate_state(struct drm_plane *plane)
 	copy->aux_stage = state->aux_stage;
 	copy->aux_source = state->aux_source;
 	copy->aux_blend = state->aux_blend;
-	copy->layer_x = state->layer_x;
-	copy->layer_y = state->layer_y;
-	copy->base_x = state->base_x;
-	copy->base_y = state->base_y;
-	copy->base_w = state->base_w;
-	copy->base_h = state->base_h;
 	copy->is_top = state->is_top;
 	copy->use_prefetch = state->use_prefetch;
 	copy->use_aux_prefetch = state->use_aux_prefetch;
 	copy->need_aux_source = state->need_aux_source;
-	copy->left_layer_x = state->left_layer_x;
-	copy->left_base_x = state->left_base_x;
-	copy->left_base_w = state->left_base_w;
 	copy->left_src_w = state->left_src_w;
 	copy->left_crtc_w = state->left_crtc_w;
-	copy->right_layer_x = state->right_layer_x;
-	copy->right_base_x = state->right_base_x;
-	copy->right_base_w = state->right_base_w;
+	copy->left_crtc_x = state->left_crtc_x;
 	copy->right_src_w = state->right_src_w;
 	copy->right_crtc_w = state->right_crtc_w;
+	copy->right_crtc_x = state->right_crtc_x;
 	copy->is_left_top = state->is_left_top;
 	copy->is_right_top = state->is_right_top;
 
 	return &copy->base;
 }
 
-static void dpu_drm_atomic_plane_destroy_state(struct drm_plane *plane,
-					       struct drm_plane_state *state)
-{
-	__drm_atomic_helper_plane_destroy_state(state);
-	kfree(to_dpu_plane_state(state));
-}
-
 static bool dpu_drm_plane_format_mod_supported(struct drm_plane *plane,
 					       uint32_t format,
 					       uint64_t modifier)
@@ -179,6 +161,13 @@ static bool dpu_drm_plane_format_mod_supported(struct drm_plane *plane,
 	}
 }
 
+static void dpu_drm_atomic_plane_destroy_state(struct drm_plane *plane,
+					       struct drm_plane_state *state)
+{
+	__drm_atomic_helper_plane_destroy_state(state);
+	kfree(to_dpu_plane_state(state));
+}
+
 static const struct drm_plane_funcs dpu_plane_funcs = {
 	.update_plane	= drm_atomic_helper_update_plane,
 	.disable_plane	= drm_atomic_helper_disable_plane,
@@ -195,9 +184,9 @@ drm_plane_state_to_baseaddr(struct drm_plane_state *state, bool aux_source)
 	struct drm_framebuffer *fb = state->fb;
 	struct drm_gem_cma_object *cma_obj;
 	struct dpu_plane_state *dpstate = to_dpu_plane_state(state);
-	unsigned int x = (state->src_x >> 16) +
+	unsigned int x = (state->src.x1 >> 16) +
 				(aux_source ? dpstate->left_src_w : 0);
-	unsigned int y = state->src_y >> 16;
+	unsigned int y = state->src.y1 >> 16;
 
 	cma_obj = drm_fb_cma_get_gem_obj(fb, 0);
 	BUG_ON(!cma_obj);
@@ -209,7 +198,7 @@ drm_plane_state_to_baseaddr(struct drm_plane_state *state, bool aux_source)
 		y /= 2;
 
 	return cma_obj->paddr + fb->offsets[0] + fb->pitches[0] * y +
-	       drm_format_plane_cpp(fb->format->format, 0) * x;
+	       fb->format->cpp[0] * x;
 }
 
 static inline dma_addr_t
@@ -218,8 +207,8 @@ drm_plane_state_to_uvbaseaddr(struct drm_plane_state *state, bool aux_source)
 	struct drm_framebuffer *fb = state->fb;
 	struct drm_gem_cma_object *cma_obj;
 	struct dpu_plane_state *dpstate = to_dpu_plane_state(state);
-	int x = (state->src_x >> 16) + (aux_source ? dpstate->left_src_w : 0);
-	int y = state->src_y >> 16;
+	int x = (state->src.x1 >> 16) + (aux_source ? dpstate->left_src_w : 0);
+	int y = state->src.y1 >> 16;
 
 	cma_obj = drm_fb_cma_get_gem_obj(fb, 1);
 	BUG_ON(!cma_obj);
@@ -227,41 +216,41 @@ drm_plane_state_to_uvbaseaddr(struct drm_plane_state *state, bool aux_source)
 	if (fb->modifier)
 		return cma_obj->paddr + fb->offsets[1];
 
-	x /= drm_format_horz_chroma_subsampling(fb->format->format);
-	y /= drm_format_vert_chroma_subsampling(fb->format->format);
+	x /= fb->format->hsub;
+	y /= fb->format->vsub;
 
 	if (fb->flags & DRM_MODE_FB_INTERLACED)
 		y /= 2;
 
 	return cma_obj->paddr + fb->offsets[1] + fb->pitches[1] * y +
-	       drm_format_plane_cpp(fb->format->format, 1) * x;
+	       fb->format->cpp[1] * x;
+}
+
+static inline bool dpu_plane_fb_format_is_yuv(u32 fmt)
+{
+	return fmt == DRM_FORMAT_YUYV || fmt == DRM_FORMAT_UYVY ||
+	       fmt == DRM_FORMAT_NV12 || fmt == DRM_FORMAT_NV21;
 }
 
 static int dpu_plane_atomic_check(struct drm_plane *plane,
-				  struct drm_plane_state *state)
+				  struct drm_atomic_state *state)
 {
 	struct dpu_plane *dplane = to_dpu_plane(plane);
-	struct dpu_plane_state *dpstate = to_dpu_plane_state(state);
-	struct dpu_plane_state *old_dpstate = to_dpu_plane_state(plane->state);
+	struct drm_plane_state *new_plane_state = drm_atomic_get_new_plane_state(state,
+										 plane);
+	struct dpu_plane_state *dpstate = to_dpu_plane_state(new_plane_state);
 	struct dpu_plane_res *res = &dplane->grp->res;
 	struct drm_crtc_state *crtc_state;
-	struct drm_framebuffer *fb = state->fb;
+	struct drm_framebuffer *fb = new_plane_state->fb;
 	struct dpu_fetchunit *fu;
 	struct dprc *dprc;
 	dma_addr_t baseaddr, uv_baseaddr = 0;
-	u32 src_w = state->src_w >> 16, src_h = state->src_h >> 16,
-	    src_x = state->src_x >> 16, src_y = state->src_y >> 16;
+	u32 src_w, src_h, src_x, src_y;
 	unsigned int frame_width;
-	int bpp;
+	int min_scale, bpp, ret;
 	bool fb_is_interlaced;
 	bool check_aux_source = false;
 
-	/* pure software check */
-	if (plane->type != DRM_PLANE_TYPE_PRIMARY)
-		if (WARN_ON(dpstate->base_x || dpstate->base_y ||
-			    dpstate->base_w || dpstate->base_h))
-			return -EINVAL;
-
 	/* ok to disable */
 	if (!fb) {
 		dpstate->stage = LB_PRIM_SEL__DISABLE;
@@ -270,121 +259,156 @@ static int dpu_plane_atomic_check(struct drm_plane *plane,
 		dpstate->aux_stage = LB_PRIM_SEL__DISABLE;
 		dpstate->aux_source = LB_SEC_SEL__DISABLE;
 		dpstate->aux_blend = ID_NONE;
-		dpstate->layer_x = 0;
-		dpstate->layer_y = 0;
-		dpstate->base_x = 0;
-		dpstate->base_y = 0;
-		dpstate->base_w = 0;
-		dpstate->base_h = 0;
 		dpstate->is_top = false;
 		dpstate->use_prefetch = false;
 		dpstate->use_aux_prefetch = false;
 		dpstate->need_aux_source = false;
-		dpstate->left_layer_x = 0;
-		dpstate->left_base_x = 0;
-		dpstate->left_base_w = 0;
 		dpstate->left_src_w = 0;
 		dpstate->left_crtc_w = 0;
-		dpstate->right_layer_x = 0;
-		dpstate->right_base_x = 0;
-		dpstate->right_base_w = 0;
+		dpstate->left_crtc_x = 0;
 		dpstate->right_src_w = 0;
 		dpstate->right_crtc_w = 0;
+		dpstate->right_crtc_x = 0;
 		dpstate->is_left_top = false;
 		dpstate->is_right_top = false;
 		return 0;
 	}
 
-	if (!state->crtc)
+	if (!new_plane_state->crtc) {
+		DRM_DEBUG_KMS("[PLANE:%d:%s] has no CRTC in plane state\n",
+				plane->base.id, plane->name);
 		return -EINVAL;
+	}
+
+	src_w = drm_rect_width(&new_plane_state->src) >> 16;
+	src_h = drm_rect_height(&new_plane_state->src) >> 16;
+	src_x = new_plane_state->src.x1 >> 16;
+	src_y = new_plane_state->src.y1 >> 16;
 
 	fb_is_interlaced = !!(fb->flags & DRM_MODE_FB_INTERLACED);
 
 	if (fb->modifier &&
 	    fb->modifier != DRM_FORMAT_MOD_AMPHION_TILED &&
 	    fb->modifier != DRM_FORMAT_MOD_VIVANTE_TILED &&
-	    fb->modifier != DRM_FORMAT_MOD_VIVANTE_SUPER_TILED)
+	    fb->modifier != DRM_FORMAT_MOD_VIVANTE_SUPER_TILED) {
+		DRM_DEBUG_KMS("[PLANE:%d:%s] unsupported fb modifier\n",
+				plane->base.id, plane->name);
 		return -EINVAL;
-
-	if (dplane->grp->has_vproc) {
-		/* no down scaling */
-		if (src_w > state->crtc_w || src_h > state->crtc_h)
-			return -EINVAL;
-	} else {
-		/* no scaling */
-		if (src_w != state->crtc_w || src_h != state->crtc_h)
-			return -EINVAL;
 	}
 
-	/* no off screen */
-	if (state->crtc_x < 0 || state->crtc_y < 0)
-		return -EINVAL;
-
-	crtc_state =
-		drm_atomic_get_existing_crtc_state(state->state, state->crtc);
+	crtc_state = drm_atomic_get_existing_crtc_state(state,
+							new_plane_state->crtc);
 	if (WARN_ON(!crtc_state))
 		return -EINVAL;
 
-	/* mode set is needed when base x/y is changed */
-	if (plane->type == DRM_PLANE_TYPE_PRIMARY)
-		if ((dpstate->base_x != old_dpstate->base_x) ||
-		    (dpstate->base_y != old_dpstate->base_y))
-			crtc_state->mode_changed = true;
+	min_scale = dplane->grp->has_vproc ?
+				FRAC_16_16(min(src_w, src_h), 8192) :
+				DRM_PLANE_HELPER_NO_SCALING;
+	ret = drm_atomic_helper_check_plane_state(new_plane_state, crtc_state,
+						  min_scale,
+						  DRM_PLANE_HELPER_NO_SCALING,
+						  true, false);
+	if (ret) {
+		DRM_DEBUG_KMS("[PLANE:%d:%s] failed to check plane state\n",
+				plane->base.id, plane->name);
+		return ret;
+	}
 
-	if (state->crtc_x + state->crtc_w >
-	    crtc_state->adjusted_mode.hdisplay)
-		return -EINVAL;
-	if (state->crtc_y + state->crtc_h >
-	    crtc_state->adjusted_mode.vdisplay)
+	/* no off screen */
+	if (new_plane_state->dst.x1 < 0 || new_plane_state->dst.y1 < 0 ||
+	    (new_plane_state->dst.x2 > crtc_state->adjusted_mode.hdisplay) ||
+	    (new_plane_state->dst.y2 > crtc_state->adjusted_mode.vdisplay)) {
+		DRM_DEBUG_KMS("[PLANE:%d:%s] no off screen\n",
+				plane->base.id, plane->name);
 		return -EINVAL;
+	}
 
 	/* pixel/line count and position parameters check */
-	if (drm_format_horz_chroma_subsampling(fb->format->format) == 2) {
+	if (fb->format->hsub == 2) {
 		if (dpstate->left_src_w || dpstate->right_src_w) {
 			if ((dpstate->left_src_w % 2) ||
-			    (dpstate->right_src_w % 2) || (src_x % 2))
+			    (dpstate->right_src_w % 2) || (src_x % 2)) {
+				DRM_DEBUG_KMS("[PLANE:%d:%s] bad left/right uv width or xoffset\n",
+						plane->base.id, plane->name);
 				return -EINVAL;
+			}
 		} else {
-			if ((src_w % 2) || (src_x % 2))
+			if ((src_w % 2) || (src_x % 2)) {
+				DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv width or xoffset\n",
+						plane->base.id, plane->name);
 				return -EINVAL;
+			}
 		}
 	}
-	if (drm_format_vert_chroma_subsampling(fb->format->format) == 2) {
-		if (src_h % (fb_is_interlaced ? 4 : 2))
+	if (fb->format->vsub == 2) {
+		if (src_h % (fb_is_interlaced ? 4 : 2)) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv height\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
-		if (src_y % (fb_is_interlaced ? 4 : 2))
+		}
+		if (src_y % (fb_is_interlaced ? 4 : 2)) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv yoffset\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 	}
 
 	/* for tile formats, framebuffer has to be tile aligned */
 	switch (fb->modifier) {
 	case DRM_FORMAT_MOD_AMPHION_TILED:
-		if (fb->width % 8)
+		if (fb->width % 8) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad fb width for AMPHION tile\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
-		if (fb->height % 256)
+		}
+		if (fb->height % 256) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad fb height for AMPHION tile\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 		break;
 	case DRM_FORMAT_MOD_VIVANTE_TILED:
-		if (fb->width % 4)
+		if (fb->width % 4) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad fb width for VIVANTE tile\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
-		if (fb->height % 4)
+		}
+		if (fb->height % 4) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad fb height for VIVANTE tile\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 		break;
 	case DRM_FORMAT_MOD_VIVANTE_SUPER_TILED:
-		if (fb->width % 64)
+		if (fb->width % 64) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad fb width for VIVANTE super tile\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
-		if (fb->height % 64)
+		}
+		if (fb->height % 64) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad fb height for VIVANTE super tile\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 		break;
 	default:
 		break;
 	}
 
+	/* do not support BT709 full range */
+	if (dpu_plane_fb_format_is_yuv(fb->format->format) &&
+	    new_plane_state->color_encoding == DRM_COLOR_YCBCR_BT709 &&
+	    new_plane_state->color_range == DRM_COLOR_YCBCR_FULL_RANGE)
+		return -EINVAL;
+
 again:
 	fu = source_to_fu(res,
 		check_aux_source ? dpstate->aux_source : dpstate->source);
-	if (!fu)
+	if (!fu) {
+		DRM_DEBUG_KMS("[PLANE:%d:%s] cannot get fetch unit\n",
+				plane->base.id, plane->name);
 		return -EINVAL;
+	}
 
 	dprc = fu->dprc;
 
@@ -397,7 +421,7 @@ again:
 	if (dprc &&
 	    dprc_format_supported(dprc, fb->format->format, fb->modifier) &&
 	    dprc_stride_supported(dprc, fb->pitches[0], fb->pitches[1],
-					frame_width, fb->format->format)) {
+				  frame_width, fb->format->format)) {
 		if (check_aux_source)
 			dpstate->use_aux_prefetch = true;
 		else
@@ -410,20 +434,20 @@ again:
 	}
 
 	if (fb->modifier) {
-		if (check_aux_source && !dpstate->use_aux_prefetch)
+		if (check_aux_source && !dpstate->use_aux_prefetch) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] cannot do tile resolving wo prefetch\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
-		else if (!check_aux_source && !dpstate->use_prefetch)
+		} else if (!check_aux_source && !dpstate->use_prefetch) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] cannot do tile resolving wo prefetch\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 	}
 
-	/*
-	 * base address alignment check
-	 *
-	 * The (uv) base address offset introduced by PRG x/y
-	 * offset(for tile formats) would not impact the alignment
-	 * check, so we don't take the offset into consideration.
-	 */
-	baseaddr = drm_plane_state_to_baseaddr(state, check_aux_source);
+	/* base address alignment check */
+	baseaddr = drm_plane_state_to_baseaddr(new_plane_state,
+					       check_aux_source);
 	switch (fb->format->format) {
 	case DRM_FORMAT_YUYV:
 	case DRM_FORMAT_UYVY:
@@ -434,56 +458,83 @@ again:
 		bpp = 8;
 		break;
 	default:
-		bpp = drm_format_plane_cpp(fb->format->format, 0) * 8;
+		bpp = fb->format->cpp[0] * 8;
 		break;
 	}
 	switch (bpp) {
 	case 32:
-		if (baseaddr & 0x3)
+		if (baseaddr & 0x3) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] 32bpp fb bad baddr alignment\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 		break;
 	case 16:
 		if (fb->modifier) {
-			if (baseaddr & 0x1)
+			if (baseaddr & 0x1) {
+				DRM_DEBUG_KMS("[PLANE:%d:%s] 16bpp tile fb bad baddr alignment\n",
+						plane->base.id, plane->name);
 				return -EINVAL;
+			}
 		} else {
 			if (check_aux_source) {
 				if (baseaddr &
-				    (dpstate->use_aux_prefetch ? 0x7 : 0x1))
+				    (dpstate->use_aux_prefetch ? 0x7 : 0x1)) {
+					DRM_DEBUG_KMS("[PLANE:%d:%s] 16bpp fb bad baddr alignment\n",
+							plane->base.id, plane->name);
 					return -EINVAL;
+				}
 			} else {
 				if (baseaddr &
-				    (dpstate->use_prefetch ? 0x7 : 0x1))
+				    (dpstate->use_prefetch ? 0x7 : 0x1)) {
+					DRM_DEBUG_KMS("[PLANE:%d:%s] 16bpp fb bad baddr alignment\n",
+							plane->base.id, plane->name);
 					return -EINVAL;
+				}
 			}
 		}
 		break;
 	}
 
-	if (fb->pitches[0] > 0x10000)
+	if (fb->pitches[0] > 0x10000) {
+		DRM_DEBUG_KMS("[PLANE:%d:%s] fb pitch[0] is too big\n",
+				plane->base.id, plane->name);
 		return -EINVAL;
+	}
 
 	/* UV base address alignment check, assuming 16bpp */
-	if (drm_format_num_planes(fb->format->format) > 1) {
-		uv_baseaddr = drm_plane_state_to_uvbaseaddr(state,
+	if (fb->format->num_planes > 1) {
+		uv_baseaddr = drm_plane_state_to_uvbaseaddr(new_plane_state,
 							check_aux_source);
 		if (fb->modifier) {
-			if (uv_baseaddr & 0x1)
+			if (uv_baseaddr & 0x1) {
+				DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv baddr alignment for tile fb\n",
+						plane->base.id, plane->name);
 				return -EINVAL;
+			}
 		} else {
 			if (check_aux_source) {
 				if (uv_baseaddr &
-				    (dpstate->use_aux_prefetch ? 0x7 : 0x1))
+				    (dpstate->use_aux_prefetch ? 0x7 : 0x1)) {
+					DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv baddr alignment\n",
+							plane->base.id, plane->name);
 					return -EINVAL;
+				}
 			} else {
 				if (uv_baseaddr &
-				    (dpstate->use_prefetch ? 0x7 : 0x1))
+				    (dpstate->use_prefetch ? 0x7 : 0x1)) {
+					DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv baddr alignment\n",
+							plane->base.id, plane->name);
 					return -EINVAL;
+				}
 			}
 		}
 
-		if (fb->pitches[1] > 0x10000)
+		if (fb->pitches[1] > 0x10000) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] fb pitch[1] is too big\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 	}
 
 	if (!check_aux_source && dpstate->use_prefetch &&
@@ -491,14 +542,23 @@ again:
 					fb->format->format,
 					fb->modifier,
 					baseaddr, uv_baseaddr)) {
-		if (fb->modifier)
+		if (fb->modifier) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad pitch\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 
-		if (bpp == 16 && (baseaddr & 0x1))
+		if (bpp == 16 && (baseaddr & 0x1)) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad baddr alignment\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 
-		if (uv_baseaddr & 0x1)
+		if (uv_baseaddr & 0x1) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv baddr alignment\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 
 		dpstate->use_prefetch = false;
 	} else if (check_aux_source && dpstate->use_aux_prefetch &&
@@ -506,14 +566,23 @@ again:
 					fb->format->format,
 					fb->modifier,
 					baseaddr, uv_baseaddr)) {
-		if (fb->modifier)
+		if (fb->modifier) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad pitch\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 
-		if (bpp == 16 && (baseaddr & 0x1))
+		if (bpp == 16 && (baseaddr & 0x1)) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad baddr alignment\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 
-		if (uv_baseaddr & 0x1)
+		if (uv_baseaddr & 0x1) {
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bad uv baddr alignment\n",
+					plane->base.id, plane->name);
 			return -EINVAL;
+		}
 
 		dpstate->use_aux_prefetch = false;
 	}
@@ -527,12 +596,13 @@ again:
 }
 
 static void dpu_plane_atomic_update(struct drm_plane *plane,
-				    struct drm_plane_state *old_state)
+				    struct drm_atomic_state *state)
 {
 	struct dpu_plane *dplane = to_dpu_plane(plane);
-	struct drm_plane_state *state = plane->state;
-	struct dpu_plane_state *dpstate = to_dpu_plane_state(state);
-	struct drm_framebuffer *fb = state->fb;
+	struct drm_plane_state *new_plane_state = drm_atomic_get_new_plane_state(state,
+										 plane);
+	struct dpu_plane_state *dpstate = to_dpu_plane_state(new_plane_state);
+	struct drm_framebuffer *fb = new_plane_state->fb;
 	struct dpu_plane_res *res = &dplane->grp->res;
 	struct dpu_fetchunit *fu;
 	struct dpu_fetchunit *fe = NULL;
@@ -540,17 +610,15 @@ static void dpu_plane_atomic_update(struct drm_plane *plane,
 	struct dpu_hscaler *hs = NULL;
 	struct dpu_vscaler *vs = NULL;
 	struct dpu_layerblend *lb;
-	struct dpu_constframe *cf, *aux_cf;
 	struct dpu_extdst *ed;
-	struct dpu_framegen *fg, *aux_fg;
-	struct device *dev = plane->dev->dev;
+	struct dpu_framegen *fg;
 	dma_addr_t baseaddr, uv_baseaddr = 0;
 	dpu_block_id_t blend, fe_id, vs_id = ID_NONE, hs_id;
 	lb_sec_sel_t source;
 	lb_prim_sel_t stage;
 	unsigned int stream_id;
-	unsigned int src_w, src_h, src_x, src_y;
-	unsigned int layer_x;
+	unsigned int src_w, src_h, src_x, src_y, dst_w, dst_h;
+	unsigned int crtc_x;
 	unsigned int mt_w = 0, mt_h = 0;	/* w/h in a micro-tile */
 	int bpp, lb_id;
 	bool need_fetcheco, need_hscaler = false, need_vscaler = false;
@@ -559,7 +627,6 @@ static void dpu_plane_atomic_update(struct drm_plane *plane,
 	bool update_aux_source = false;
 	bool use_prefetch;
 	bool need_modeset;
-	bool is_overlay = plane->type == DRM_PLANE_TYPE_OVERLAY;
 	bool fb_is_interlaced;
 
 	/*
@@ -569,7 +636,8 @@ static void dpu_plane_atomic_update(struct drm_plane *plane,
 	if (!fb)
 		return;
 
-	need_modeset = drm_atomic_crtc_needs_modeset(state->crtc->state);
+	need_modeset =
+		drm_atomic_crtc_needs_modeset(new_plane_state->crtc->state);
 	fb_is_interlaced = !!(fb->flags & DRM_MODE_FB_INTERLACED);
 
 again:
@@ -586,15 +654,15 @@ again:
 	if (crtc_use_pc) {
 		if (update_aux_source) {
 			stream_id = 1;
-			layer_x = dpstate->right_layer_x;
+			crtc_x = dpstate->right_crtc_x;
 		} else {
 			stream_id = dpstate->left_src_w ? 0 : 1;
-			layer_x = dpstate->left_src_w ?
-				dpstate->left_layer_x : dpstate->right_layer_x;
+			crtc_x = dpstate->left_src_w ?
+				 dpstate->left_crtc_x : dpstate->right_crtc_x;
 		}
 	} else {
 		stream_id = dplane->stream_id;
-		layer_x = dpstate->layer_x;
+		crtc_x = new_plane_state->crtc_x;
 	}
 
 	fg = res->fg[stream_id];
@@ -617,18 +685,21 @@ again:
 		else
 			src_w = dpstate->left_src_w;
 	} else {
-		src_w = state->src_w >> 16;
+		src_w = drm_rect_width(&new_plane_state->src) >> 16;
 	}
-	src_h = state->src_h >> 16;
+	src_h = drm_rect_height(&new_plane_state->src) >> 16;
 	if (crtc_use_pc && update_aux_source) {
 		if (fb->modifier)
-			src_x = (state->src_x >> 16) + dpstate->left_src_w;
+			src_x = (new_plane_state->src_x >> 16) +
+				dpstate->left_src_w;
 		else
 			src_x = 0;
 	} else {
-		src_x = fb->modifier ? (state->src_x >> 16) : 0;
+		src_x = fb->modifier ? (new_plane_state->src_x >> 16) : 0;
 	}
-	src_y = fb->modifier ? (state->src_y >> 16) : 0;
+	src_y = fb->modifier ? (new_plane_state->src_y >> 16) : 0;
+	dst_w = drm_rect_width(&new_plane_state->dst);
+	dst_h = drm_rect_height(&new_plane_state->dst);
 
 	if (fetchunit_is_fetchdecode(fu)) {
 		if (fetchdecode_need_fetcheco(fu, fb->format->format)) {
@@ -639,14 +710,14 @@ again:
 		}
 
 		/* assume pixel combiner is unused */
-		if ((src_w != state->crtc_w) && !crtc_use_pc) {
+		if ((src_w != dst_w) && !crtc_use_pc) {
 			need_hscaler = true;
 			hs = fetchdecode_get_hscaler(fu);
 			if (IS_ERR(hs))
 				return;
 		}
 
-		if ((src_h != state->crtc_h) || fb_is_interlaced) {
+		if ((src_h != dst_h) || fb_is_interlaced) {
 			need_vscaler = true;
 			vs = fetchdecode_get_vscaler(fu);
 			if (IS_ERR(vs))
@@ -664,7 +735,7 @@ again:
 		bpp = 8;
 		break;
 	default:
-		bpp = drm_format_plane_cpp(fb->format->format, 0) * 8;
+		bpp = fb->format->cpp[0] * 8;
 		break;
 	}
 
@@ -682,9 +753,10 @@ again:
 		break;
 	}
 
-	baseaddr = drm_plane_state_to_baseaddr(state, update_aux_source);
+	baseaddr = drm_plane_state_to_baseaddr(new_plane_state,
+					       update_aux_source);
 	if (need_fetcheco)
-		uv_baseaddr = drm_plane_state_to_uvbaseaddr(state,
+		uv_baseaddr = drm_plane_state_to_uvbaseaddr(new_plane_state,
 							update_aux_source);
 
 	if (use_prefetch &&
@@ -694,10 +766,15 @@ again:
 
 	fu->ops->set_burstlength(fu, src_x, mt_w, bpp, baseaddr, use_prefetch);
 	fu->ops->set_src_bpp(fu, bpp);
-	fu->ops->set_src_stride(fu, src_w, src_x, mt_w, bpp, fb->pitches[0],
+	fu->ops->set_src_stride(fu, src_w, src_w, mt_w, bpp, fb->pitches[0],
 				baseaddr, use_prefetch);
 	fu->ops->set_src_buf_dimensions(fu, src_w, src_h, 0, fb_is_interlaced);
-	fu->ops->set_fmt(fu, fb->format->format, fb_is_interlaced);
+	fu->ops->set_pixel_blend_mode(fu, new_plane_state->pixel_blend_mode,
+				      new_plane_state->alpha,
+				      fb->format->format);
+	fu->ops->set_fmt(fu, fb->format->format,
+			 new_plane_state->color_encoding,
+			 new_plane_state->color_range, fb_is_interlaced);
 	fu->ops->enable_src_buf(fu);
 	fu->ops->set_framedimensions(fu, src_w, src_h, fb_is_interlaced);
 	fu->ops->set_baseaddress(fu, src_w, src_x, src_y, mt_w, mt_h, bpp,
@@ -705,9 +782,8 @@ again:
 	fu->ops->set_stream_id(fu, stream_id ?
 					DPU_PLANE_SRC_TO_DISP_STREAM1 :
 					DPU_PLANE_SRC_TO_DISP_STREAM0);
-	fu->ops->unpin_off(fu);
 
-	dev_dbg(dev, "[PLANE:%d:%s] %s-0x%02x\n",
+	DRM_DEBUG_KMS("[PLANE:%d:%s] %s-0x%02x\n",
 				plane->base.id, plane->name, fu->name, fu->id);
 
 	if (need_fetcheco) {
@@ -722,13 +798,16 @@ again:
 
 		fetchdecode_pixengcfg_dynamic_src_sel(fu,
 						(fd_dynamic_src_sel_t)fe_id);
-		fe->ops->set_burstlength(fe, src_x, mt_w, bpp, uv_baseaddr,
+		fe->ops->set_burstlength(fe, src_w, mt_w, bpp, uv_baseaddr,
 					 use_prefetch);
 		fe->ops->set_src_bpp(fe, 16);
 		fe->ops->set_src_stride(fe, src_w, src_x, mt_w, bpp,
 					fb->pitches[1],
 					uv_baseaddr, use_prefetch);
-		fe->ops->set_fmt(fe, fb->format->format, fb_is_interlaced);
+		fe->ops->set_fmt(fe, fb->format->format,
+				 new_plane_state->color_encoding,
+				 new_plane_state->color_range,
+				 fb_is_interlaced);
 		fe->ops->set_src_buf_dimensions(fe, src_w, src_h,
 						fb->format->format,
 						fb_is_interlaced);
@@ -740,9 +819,8 @@ again:
 		fe->ops->set_stream_id(fe, stream_id ?
 					DPU_PLANE_SRC_TO_DISP_STREAM1 :
 					DPU_PLANE_SRC_TO_DISP_STREAM0);
-		fe->ops->unpin_off(fe);
 
-		dev_dbg(dev, "[PLANE:%d:%s] %s-0x%02x\n",
+		DRM_DEBUG_KMS("[PLANE:%d:%s] %s-0x%02x\n",
 				plane->base.id, plane->name, fe->name, fe_id);
 	} else {
 		if (fetchunit_is_fetchdecode(fu))
@@ -758,10 +836,11 @@ again:
 
 		vscaler_pixengcfg_dynamic_src_sel(vs, (vs_src_sel_t)source);
 		vscaler_pixengcfg_clken(vs, CLKEN__AUTOMATIC);
-		vscaler_setup1(vs, src_h, state->crtc_h, fb_is_interlaced);
+		vscaler_setup1(vs, src_h, new_plane_state->crtc_h,
+			       fb_is_interlaced);
 		vscaler_setup2(vs, fb_is_interlaced);
 		vscaler_setup3(vs, fb_is_interlaced);
-		vscaler_output_size(vs, state->crtc_h);
+		vscaler_output_size(vs, dst_h);
 		vscaler_field_mode(vs, fb_is_interlaced ?
 						SCALER_ALWAYS0 : SCALER_INPUT);
 		vscaler_filter_mode(vs, SCALER_LINEAR);
@@ -773,7 +852,7 @@ again:
 
 		source = (lb_sec_sel_t)vs_id;
 
-		dev_dbg(dev, "[PLANE:%d:%s] vscaler-0x%02x\n",
+		DRM_DEBUG_KMS("[PLANE:%d:%s] vscaler-0x%02x\n",
 					plane->base.id, plane->name, vs_id);
 	}
 
@@ -787,8 +866,8 @@ again:
 							(hs_src_sel_t)vs_id :
 							(hs_src_sel_t)source);
 		hscaler_pixengcfg_clken(hs, CLKEN__AUTOMATIC);
-		hscaler_setup1(hs, src_w, state->crtc_w);
-		hscaler_output_size(hs, state->crtc_w);
+		hscaler_setup1(hs, src_w, dst_w);
+		hscaler_output_size(hs, dst_w);
 		hscaler_filter_mode(hs, SCALER_LINEAR);
 		hscaler_scale_mode(hs, SCALER_UPSCALE);
 		hscaler_mode(hs, SCALER_ACTIVE);
@@ -798,7 +877,7 @@ again:
 
 		source = (lb_sec_sel_t)hs_id;
 
-		dev_dbg(dev, "[PLANE:%d:%s] hscaler-0x%02x\n",
+		DRM_DEBUG_KMS("[PLANE:%d:%s] hscaler-0x%02x\n",
 					plane->base.id, plane->name, hs_id);
 	}
 
@@ -809,75 +888,44 @@ again:
 			       fb->modifier, baseaddr, uv_baseaddr,
 			       prefetch_start, uv_prefetch_start,
 			       fb_is_interlaced);
-		if (prefetch_start || uv_prefetch_start)
-			dprc_enable(dprc);
+
+		dprc_enable(dprc);
 
 		dprc_reg_update(dprc);
 
 		if (prefetch_start || uv_prefetch_start) {
 			dprc_first_frame_handle(dprc);
 
-			if (!need_modeset && is_overlay)
+			if (!need_modeset &&
+			    new_plane_state->normalized_zpos != 0)
 				framegen_wait_for_frame_counter_moving(fg);
 		}
 
 		if (update_aux_source)
-			dev_dbg(dev, "[PLANE:%d:%s] use aux prefetch\n",
+			DRM_DEBUG_KMS("[PLANE:%d:%s] use aux prefetch\n",
 						plane->base.id, plane->name);
 		else
-			dev_dbg(dev, "[PLANE:%d:%s] use prefetch\n",
+			DRM_DEBUG_KMS("[PLANE:%d:%s] use prefetch\n",
 						plane->base.id, plane->name);
 	} else if (dprc) {
 		dprc_disable(dprc);
 
 		if (update_aux_source)
-			dev_dbg(dev, "[PLANE:%d:%s] bypass aux prefetch\n",
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bypass aux prefetch\n",
 						plane->base.id, plane->name);
 		else
-			dev_dbg(dev, "[PLANE:%d:%s] bypass prefetch\n",
+			DRM_DEBUG_KMS("[PLANE:%d:%s] bypass prefetch\n",
 						plane->base.id, plane->name);
 	}
 
 	layerblend_pixengcfg_dynamic_prim_sel(lb, stage);
 	layerblend_pixengcfg_dynamic_sec_sel(lb, source);
 	layerblend_control(lb, LB_BLEND);
-	layerblend_blendcontrol(lb, need_hscaler || need_vscaler);
+	layerblend_blendcontrol(lb, new_plane_state->normalized_zpos,
+				new_plane_state->pixel_blend_mode,
+				new_plane_state->alpha);
 	layerblend_pixengcfg_clken(lb, CLKEN__AUTOMATIC);
-	layerblend_position(lb, layer_x, dpstate->layer_y);
-
-	if (plane->type == DRM_PLANE_TYPE_PRIMARY) {
-		unsigned int base_w, base_x;
-
-		cf = res->cf[stream_id];
-
-		if (crtc_use_pc) {
-			if (update_aux_source || !dpstate->left_crtc_w) {
-				base_w = dpstate->right_base_w;
-				base_x = dpstate->right_base_x;
-			} else {
-				base_w = dpstate->left_base_w;
-				base_x = dpstate->left_base_x;
-			}
-
-			if (!dpstate->left_crtc_w || !dpstate->right_crtc_w) {
-				aux_cf = dpu_aux_cf_peek(cf);
-				aux_fg = dpu_aux_fg_peek(fg);
-
-				constframe_framedimensions_copy_prim(aux_cf);
-				constframe_constantcolor(aux_cf, 0, 0, 0, 0);
-
-				framegen_sacfg(aux_fg, 0, 0);
-			}
-		} else {
-			base_w = dpstate->base_w;
-			base_x = dpstate->base_x;
-		}
-
-		constframe_framedimensions(cf, base_w, dpstate->base_h);
-		constframe_constantcolor(cf, 0, 0, 0, 0);
-
-		framegen_sacfg(fg, base_x, dpstate->base_y);
-	}
+	layerblend_position(lb, crtc_x, new_plane_state->crtc_y);
 
 	if (crtc_use_pc) {
 		if ((!stream_id && dpstate->is_left_top) ||
@@ -893,13 +941,13 @@ again:
 	}
 
 	if (update_aux_source)
-		dev_dbg(dev, "[PLANE:%d:%s] *aux* source-0x%02x stage-0x%02x blend-0x%02x\n",
+		DRM_DEBUG_KMS("[PLANE:%d:%s] *aux* source-0x%02x stage-0x%02x blend-0x%02x\n",
 				plane->base.id, plane->name,
-				source, stage, blend);
+				source, dpstate->stage, dpstate->blend);
 	else
-		dev_dbg(dev, "[PLANE:%d:%s] source-0x%02x stage-0x%02x blend-0x%02x\n",
+		DRM_DEBUG_KMS("[PLANE:%d:%s] source-0x%02x stage-0x%02x blend-0x%02x\n",
 				plane->base.id, plane->name,
-				source, stage, blend);
+				source, dpstate->stage, dpstate->blend);
 
 	if (dpstate->need_aux_source && !update_aux_source) {
 		update_aux_source = true;
@@ -908,20 +956,20 @@ again:
 }
 
 static const struct drm_plane_helper_funcs dpu_plane_helper_funcs = {
-	.prepare_fb = drm_fb_cma_prepare_fb,
+	.prepare_fb = drm_gem_plane_helper_prepare_fb,
 	.atomic_check = dpu_plane_atomic_check,
 	.atomic_update = dpu_plane_atomic_update,
 };
 
-struct dpu_plane *dpu_plane_init(struct drm_device *drm,
-				 unsigned int possible_crtcs,
-				 unsigned int stream_id,
-				 struct dpu_plane_grp *grp,
-				 enum drm_plane_type type)
+struct dpu_plane *dpu_plane_create(struct drm_device *drm,
+				   unsigned int possible_crtcs,
+				   unsigned int stream_id,
+				   struct dpu_plane_grp *grp,
+				   enum drm_plane_type type)
 {
 	struct dpu_plane *dpu_plane;
 	struct drm_plane *plane;
-	unsigned int ov_num;
+	unsigned int zpos = dpu_plane_get_default_zpos(type);
 	int ret;
 
 	dpu_plane = kzalloc(sizeof(*dpu_plane), GFP_KERNEL);
@@ -933,45 +981,44 @@ struct dpu_plane *dpu_plane_init(struct drm_device *drm,
 
 	plane = &dpu_plane->base;
 
-	if (type == DRM_PLANE_TYPE_PRIMARY)
-		ret = drm_universal_plane_init(drm, plane, possible_crtcs,
-					       &dpu_plane_funcs,
-					       dpu_primary_formats,
-					       ARRAY_SIZE(dpu_primary_formats),
-					       dpu_format_modifiers,
-					       type, NULL);
-	else
-		ret = drm_universal_plane_init(drm, plane, possible_crtcs,
-					       &dpu_plane_funcs,
-					       dpu_overlay_formats,
-					       ARRAY_SIZE(dpu_overlay_formats),
-					       dpu_format_modifiers,
-					       type, NULL);
-	if (ret) {
-		kfree(dpu_plane);
-		return ERR_PTR(ret);
-	}
+	ret = drm_universal_plane_init(drm, plane, possible_crtcs,
+				       &dpu_plane_funcs,
+				       dpu_formats, ARRAY_SIZE(dpu_formats),
+				       dpu_format_modifiers, type, NULL);
+	if (ret)
+		goto err;
 
 	drm_plane_helper_add(plane, &dpu_plane_helper_funcs);
 
-	switch (type) {
-	case DRM_PLANE_TYPE_PRIMARY:
-		ret = drm_plane_create_zpos_immutable_property(plane, 0);
-		break;
-	case DRM_PLANE_TYPE_OVERLAY:
-		/* filter out the primary plane */
-		ov_num = grp->hw_plane_num - 1;
+	ret = drm_plane_create_zpos_property(plane,
+					     zpos, 0, grp->hw_plane_num - 1);
+	if (ret)
+		goto err;
 
-		ret = drm_plane_create_zpos_property(plane, 1, 1, ov_num);
-		break;
-	default:
-		ret = -EINVAL;
-	}
+	ret = drm_plane_create_alpha_property(plane);
+	if (ret)
+		goto err;
 
-	if (ret) {
-		kfree(dpu_plane);
-		return ERR_PTR(ret);
-	}
+	ret = drm_plane_create_blend_mode_property(plane,
+					BIT(DRM_MODE_BLEND_PIXEL_NONE) |
+					BIT(DRM_MODE_BLEND_PREMULTI)   |
+					BIT(DRM_MODE_BLEND_COVERAGE));
+	if (ret)
+		goto err;
+
+	ret = drm_plane_create_color_properties(plane,
+					BIT(DRM_COLOR_YCBCR_BT601) |
+					BIT(DRM_COLOR_YCBCR_BT709),
+					BIT(DRM_COLOR_YCBCR_LIMITED_RANGE) |
+					BIT(DRM_COLOR_YCBCR_FULL_RANGE),
+					DRM_COLOR_YCBCR_BT601,
+					DRM_COLOR_YCBCR_FULL_RANGE);
+	if (ret)
+		goto err;
 
 	return dpu_plane;
+
+err:
+	kfree(dpu_plane);
+	return ERR_PTR(ret);
 }
