@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016 Freescale Semiconductor, Inc.
- * Copyright 2017-2018 NXP
+ * Copyright 2017-2020 NXP
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 #include <linux/mutex.h>
 #include <linux/platform_device.h>
 #include <linux/types.h>
-#include <soc/imx8/sc/sci.h>
+#include <drm/drm_mode.h>
 #include <video/dpu.h>
 #include "dpu-prv.h"
 
@@ -79,7 +79,9 @@
 #define FRAMEINDEX_SHIFT	14
 #define FGCHSTAT		0x78
 #define SECSYNCSTAT		BIT(24)
+#define SFIFOEMPTY		BIT(16)
 #define FGCHSTATCLR		0x7C
+#define CLRSECSTAT		BIT(16)
 #define FGSKEWMON		0x80
 #define FGSFIFOMIN		0x84
 #define FGSFIFOMAX		0x88
@@ -94,13 +96,13 @@ struct dpu_framegen {
 	void __iomem *base;
 	struct clk *clk_pll;
 	struct clk *clk_bypass;
-	struct clk *clk_disp_sel;
 	struct clk *clk_disp;
+	struct clk *clk_disp_lpcg;
 	struct mutex mutex;
 	int id;
+	unsigned int encoder_type;
 	bool inuse;
 	bool use_bypass_clk;
-	bool encoder_type_has_lvds;
 	bool side_by_side;
 	struct dpu_soc *dpu;
 };
@@ -110,209 +112,77 @@ static inline u32 dpu_fg_read(struct dpu_framegen *fg, unsigned int offset)
 	return readl(fg->base + offset);
 }
 
-static inline void dpu_fg_write(struct dpu_framegen *fg, u32 value,
-				unsigned int offset)
+static inline void dpu_fg_write(struct dpu_framegen *fg,
+				unsigned int offset, u32 value)
 {
 	writel(value, fg->base + offset);
 }
 
-/* FIXME: enable pixel link in a proper manner */
-static void dpu_pixel_link_enable(int dpu_id, int stream_id)
-{
-	sc_err_t sciErr;
-	sc_ipc_t ipcHndl = 0;
-	u32 mu_id;
-
-	sciErr = sc_ipc_getMuID(&mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("Cannot obtain MU ID\n");
-		return;
-	}
-
-	sciErr = sc_ipc_open(&ipcHndl, mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("sc_ipc_open failed! (sciError = %d)\n", sciErr);
-		return;
-	}
-
-	if (dpu_id == 0) {
-		sciErr = sc_misc_set_control(ipcHndl, SC_R_DC_0,
-			stream_id ? SC_C_PXL_LINK_MST2_ENB : SC_C_PXL_LINK_MST1_ENB, 1);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_0:SC_C_PXL_LINK_MST%d_ENB sc_misc_set_control failed! (sciError = %d)\n", stream_id + 1, sciErr);
-	} else if (dpu_id == 1) {
-		sciErr = sc_misc_set_control(ipcHndl, SC_R_DC_1,
-			stream_id ? SC_C_PXL_LINK_MST2_ENB : SC_C_PXL_LINK_MST1_ENB, 1);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_1:SC_C_PXL_LINK_MST%d_ENB sc_misc_set_control failed! (sciError = %d)\n", stream_id + 1, sciErr);
-	}
-
-	sc_ipc_close(mu_id);
-}
-
-/* FIXME: disable pixel link in a proper manner */
-static void dpu_pixel_link_disable(int dpu_id, int stream_id)
-{
-	sc_err_t sciErr;
-	sc_ipc_t ipcHndl = 0;
-	u32 mu_id;
-
-	sciErr = sc_ipc_getMuID(&mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("Cannot obtain MU ID\n");
-		return;
-	}
-
-	sciErr = sc_ipc_open(&ipcHndl, mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("sc_ipc_open failed! (sciError = %d)\n", sciErr);
-		return;
-	}
-
-	if (dpu_id == 0) {
-		sciErr = sc_misc_set_control(ipcHndl, SC_R_DC_0,
-			stream_id ? SC_C_PXL_LINK_MST2_ENB : SC_C_PXL_LINK_MST1_ENB, 0);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_0:SC_C_PXL_LINK_MST%d_ENB sc_misc_set_control failed! (sciError = %d)\n", stream_id + 1, sciErr);
-	} else if (dpu_id == 1) {
-		sciErr = sc_misc_set_control(ipcHndl, SC_R_DC_1,
-			stream_id ? SC_C_PXL_LINK_MST2_ENB : SC_C_PXL_LINK_MST1_ENB, 0);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_1:SC_C_PXL_LINK_MST%d_ENB sc_misc_set_control failed! (sciError = %d)\n", stream_id + 1, sciErr);
-	}
-
-	sc_ipc_close(mu_id);
-}
-
-/* FIXME: set MST address for pixel link in a proper manner */
-static void dpu_pixel_link_set_mst_addr(int dpu_id, int stream_id, int mst_addr)
-{
-	sc_err_t sciErr;
-	sc_ipc_t ipcHndl = 0;
-	u32 mu_id;
-
-	sciErr = sc_ipc_getMuID(&mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("Cannot obtain MU ID\n");
-		return;
-	}
-
-	sciErr = sc_ipc_open(&ipcHndl, mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("sc_ipc_open failed! (sciError = %d)\n", sciErr);
-		return;
-	}
-
-	if (dpu_id == 0) {
-		sciErr = sc_misc_set_control(ipcHndl, SC_R_DC_0, stream_id ?
-			SC_C_PXL_LINK_MST2_ADDR : SC_C_PXL_LINK_MST1_ADDR,
-								mst_addr);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_0:SC_C_PXL_LINK_MST%d_ADDR sc_misc_set_control failed! (sciError = %d)\n", stream_id + 1, sciErr);
-	} else if (dpu_id == 1) {
-		sciErr = sc_misc_set_control(ipcHndl, SC_R_DC_1, stream_id ?
-			SC_C_PXL_LINK_MST2_ADDR : SC_C_PXL_LINK_MST1_ADDR,
-								mst_addr);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_1:SC_C_PXL_LINK_MST%d_ADDR sc_misc_set_control failed! (sciError = %d)\n", stream_id + 1, sciErr);
-	}
-
-	sc_ipc_close(mu_id);
-}
-
-/* FIXME: set dc sync mode for pixel link in a proper manner */
-static void dpu_pixel_link_set_dc_sync_mode(int dpu_id, bool enable)
-{
-	sc_err_t sciErr;
-	sc_ipc_t ipcHndl = 0;
-	u32 mu_id;
-
-	sciErr = sc_ipc_getMuID(&mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("Cannot obtain MU ID\n");
-		return;
-	}
-
-	sciErr = sc_ipc_open(&ipcHndl, mu_id);
-	if (sciErr != SC_ERR_NONE) {
-		pr_err("sc_ipc_open failed! (sciError = %d)\n", sciErr);
-		return;
-	}
-
-	if (dpu_id == 0) {
-		sciErr = sc_misc_set_control(ipcHndl,
-						SC_R_DC_0, SC_C_MODE, enable);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_0:SC_C_MODE sc_misc_set_control failed! (sciError = %d)\n", sciErr);
-	} else if (dpu_id == 1) {
-		sciErr = sc_misc_set_control(ipcHndl,
-						SC_R_DC_1, SC_C_MODE, enable);
-		if (sciErr != SC_ERR_NONE)
-			pr_err("SC_R_DC_1:SC_C_MODE sc_misc_set_control failed! (sciError = %d)\n", sciErr);
-	}
-
-	sc_ipc_close(mu_id);
-}
-
 void framegen_enable(struct dpu_framegen *fg)
 {
-	struct dpu_soc *dpu = fg->dpu;
-	const struct dpu_devtype *devtype = dpu->devtype;
-
-	mutex_lock(&fg->mutex);
-	dpu_fg_write(fg, FGEN, FGENABLE);
-	mutex_unlock(&fg->mutex);
-
-	if (!(devtype->has_dual_ldb && fg->encoder_type_has_lvds))
-		dpu_pixel_link_enable(dpu->id, fg->id);
+	dpu_fg_write(fg, FGENABLE, FGEN);
 }
 EXPORT_SYMBOL_GPL(framegen_enable);
 
 void framegen_disable(struct dpu_framegen *fg)
 {
-	struct dpu_soc *dpu = fg->dpu;
-	const struct dpu_devtype *devtype = dpu->devtype;
-
-	if (!(devtype->has_dual_ldb && fg->encoder_type_has_lvds))
-		dpu_pixel_link_disable(dpu->id, fg->id);
-
-	mutex_lock(&fg->mutex);
-	dpu_fg_write(fg, 0, FGENABLE);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, FGENABLE, 0);
 }
 EXPORT_SYMBOL_GPL(framegen_disable);
 
+void framegen_enable_pixel_link(struct dpu_framegen *fg)
+{
+	struct dpu_soc *dpu = fg->dpu;
+	const struct dpu_data *data = dpu->data;
+
+	if (!(data->has_dual_ldb && fg->encoder_type == DRM_MODE_ENCODER_LVDS))
+		dpu_pxlink_set_mst_enable(fg->dpu, fg->id, true);
+
+	if (fg->encoder_type == DRM_MODE_ENCODER_DPI) {
+		dpu_pxlink_set_mst_valid(fg->dpu, fg->id, true);
+		dpu_pxlink_set_sync_ctrl(fg->dpu, fg->id, true);
+	}
+}
+EXPORT_SYMBOL_GPL(framegen_enable_pixel_link);
+
+void framegen_disable_pixel_link(struct dpu_framegen *fg)
+{
+	struct dpu_soc *dpu = fg->dpu;
+	const struct dpu_data *data = dpu->data;
+
+	if (fg->encoder_type == DRM_MODE_ENCODER_DPI) {
+		dpu_pxlink_set_mst_valid(fg->dpu, fg->id, false);
+		dpu_pxlink_set_sync_ctrl(fg->dpu, fg->id, false);
+	}
+
+	if (!(data->has_dual_ldb && fg->encoder_type == DRM_MODE_ENCODER_LVDS))
+		dpu_pxlink_set_mst_enable(fg->dpu, fg->id, false);
+}
+EXPORT_SYMBOL_GPL(framegen_disable_pixel_link);
+
 void framegen_shdtokgen(struct dpu_framegen *fg)
 {
-	mutex_lock(&fg->mutex);
-	dpu_fg_write(fg, SHDTOKGEN, FGSLR);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, FGSLR, SHDTOKGEN);
 }
 EXPORT_SYMBOL_GPL(framegen_shdtokgen);
 
 void framegen_syncmode(struct dpu_framegen *fg, fgsyncmode_t mode)
 {
-	struct dpu_soc *dpu = fg->dpu;
 	u32 val;
 
-	mutex_lock(&fg->mutex);
 	val = dpu_fg_read(fg, FGSTCTRL);
 	val &= ~FGSYNCMODE_MASK;
 	val |= mode;
-	dpu_fg_write(fg, val, FGSTCTRL);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, FGSTCTRL, val);
 
-	dpu_pixel_link_set_dc_sync_mode(dpu->id, mode != FGSYNCMODE__OFF);
+	dpu_pxlink_set_dc_sync_mode(fg->dpu, mode != FGSYNCMODE__OFF);
 }
 EXPORT_SYMBOL_GPL(framegen_syncmode);
 
-void
-framegen_cfg_videomode(struct dpu_framegen *fg,
-		       struct drm_display_mode *m, bool side_by_side,
-		       bool encoder_type_has_tmds, bool encoder_type_has_lvds)
+void framegen_cfg_videomode(struct dpu_framegen *fg, struct drm_display_mode *m,
+			    bool side_by_side, unsigned int encoder_type)
 {
 	struct dpu_soc *dpu = fg->dpu;
-	const struct dpu_devtype *devtype = dpu->devtype;
 	u32 hact, htotal, hsync, hsbp;
 	u32 vact, vtotal, vsync, vsbp;
 	u32 kick_row, kick_col;
@@ -321,7 +191,7 @@ framegen_cfg_videomode(struct dpu_framegen *fg,
 	int div = 0;
 
 	fg->side_by_side = side_by_side;
-	fg->encoder_type_has_lvds = encoder_type_has_lvds;
+	fg->encoder_type = encoder_type;
 
 	hact = m->crtc_hdisplay;
 	htotal = m->crtc_htotal;
@@ -340,12 +210,11 @@ framegen_cfg_videomode(struct dpu_framegen *fg,
 	vsync = m->crtc_vsync_end - m->crtc_vsync_start;
 	vsbp = m->crtc_vtotal - m->crtc_vsync_start;
 
-	mutex_lock(&fg->mutex);
 	/* video mode */
-	dpu_fg_write(fg, HACT(hact) | HTOTAL(htotal), HTCFG1);
-	dpu_fg_write(fg, HSYNC(hsync) | HSBP(hsbp) | HSEN, HTCFG2);
-	dpu_fg_write(fg, VACT(vact) | VTOTAL(vtotal), VTCFG1);
-	dpu_fg_write(fg, VSYNC(vsync) | VSBP(vsbp) | VSEN, VTCFG2);
+	dpu_fg_write(fg, HTCFG1, HACT(hact)   | HTOTAL(htotal));
+	dpu_fg_write(fg, HTCFG2, HSYNC(hsync) | HSBP(hsbp) | HSEN);
+	dpu_fg_write(fg, VTCFG1, VACT(vact)   | VTOTAL(vtotal));
+	dpu_fg_write(fg, VTCFG2, VSYNC(vsync) | VSBP(vsbp) | VSEN);
 
 	kick_col = hact + 1;
 	kick_row = vact;
@@ -353,53 +222,47 @@ framegen_cfg_videomode(struct dpu_framegen *fg,
 	 * FrameGen as slave needs to be kicked later for
 	 * one line comparing to the master.
 	 */
-	if (side_by_side && framegen_is_slave(fg) &&
-	    devtype->has_syncmode_fixup)
+	if (side_by_side && framegen_is_slave(fg))
 		kick_row++;
 
 	/* pkickconfig */
-	dpu_fg_write(fg, COL(kick_col) | ROW(kick_row) | EN, PKICKCONFIG);
+	dpu_fg_write(fg, PKICKCONFIG, COL(kick_col) | ROW(kick_row) | EN);
 
 	/* skikconfig */
-	dpu_fg_write(fg, COL(kick_col) | ROW(kick_row) | EN, SKICKCONFIG);
+	dpu_fg_write(fg, SKICKCONFIG, COL(kick_col) | ROW(kick_row) | EN);
 
-	/* primary position config */
-	dpu_fg_write(fg, STARTX(0) | STARTY(0), PACFG);
+	/* primary and secondary area position config */
+	dpu_fg_write(fg, PACFG, STARTX(0) | STARTY(0));
+	dpu_fg_write(fg, SACFG, STARTX(0) | STARTY(0));
 
 	/* alpha */
 	val = dpu_fg_read(fg, FGINCTRL);
 	val &= ~(ENPRIMALPHA | ENSECALPHA);
-	dpu_fg_write(fg, val, FGINCTRL);
+	dpu_fg_write(fg, FGINCTRL, val);
 
 	val = dpu_fg_read(fg, FGINCTRLPANIC);
 	val &= ~(ENPRIMALPHA | ENSECALPHA);
-	dpu_fg_write(fg, val, FGINCTRLPANIC);
+	dpu_fg_write(fg, FGINCTRLPANIC, val);
 
 	/* constant color */
-	dpu_fg_write(fg, 0, FGCCR);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, FGCCR, 0);
 
-	disp_clock_rate = m->clock * 1000;
+	disp_clock_rate = m->crtc_clock * 1000;
 
-	/*
-	 * To workaround setting clock rate failure issue
-	 * when the system resumes back from PM sleep mode,
-	 * we need to get the clock rates before setting
-	 * their rates, otherwise, setting the clock rates
-	 * will fail.
-	 */
-	if (devtype->has_disp_sel_clk && encoder_type_has_tmds) {
+	if (encoder_type == DRM_MODE_ENCODER_TMDS) {
 		if (side_by_side)
-			dpu_pixel_link_set_mst_addr(dpu->id, fg->id,
-							fg->id ? 2 : 1);
+			dpu_pxlink_set_mst_addr(dpu, fg->id, fg->id ? 2 : 1);
 		else
-			dpu_pixel_link_set_mst_addr(dpu->id, fg->id, 1);
+			dpu_pxlink_set_mst_addr(dpu, fg->id, 1);
 
-		clk_set_parent(fg->clk_disp_sel, fg->clk_bypass);
+		clk_set_parent(fg->clk_disp, fg->clk_bypass);
 
 		fg->use_bypass_clk = true;
 	} else {
-		dpu_pixel_link_set_mst_addr(dpu->id, fg->id, 0);
+		dpu_pxlink_set_mst_addr(dpu, fg->id,
+				encoder_type == DRM_MODE_ENCODER_DPI ? 1 : 0);
+
+		clk_set_parent(fg->clk_disp, fg->clk_pll);
 
 		/* find an even divisor for PLL */
 		do {
@@ -407,11 +270,6 @@ framegen_cfg_videomode(struct dpu_framegen *fg,
 			pll_clock_rate = disp_clock_rate * div;
 		} while (pll_clock_rate < PLL_MIN_FREQ_HZ);
 
-		if (devtype->has_disp_sel_clk)
-			clk_set_parent(fg->clk_disp_sel, fg->clk_pll);
-
-		clk_get_rate(fg->clk_pll);
-		clk_get_rate(fg->clk_disp);
 		clk_set_rate(fg->clk_pll, pll_clock_rate);
 		clk_set_rate(fg->clk_disp, disp_clock_rate);
 
@@ -424,54 +282,36 @@ void framegen_pkickconfig(struct dpu_framegen *fg, bool enable)
 {
 	u32 val;
 
-	mutex_lock(&fg->mutex);
 	val = dpu_fg_read(fg, PKICKCONFIG);
 	if (enable)
 		val |= EN;
 	else
 		val &= ~EN;
-	dpu_fg_write(fg, val, PKICKCONFIG);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, PKICKCONFIG, val);
 }
 EXPORT_SYMBOL_GPL(framegen_pkickconfig);
 
 void framegen_syncmode_fixup(struct dpu_framegen *fg, bool enable)
 {
-	struct dpu_soc *dpu = fg->dpu;
 	u32 val;
 
-	if (!dpu->devtype->has_syncmode_fixup)
-		return;
-
-	mutex_lock(&fg->mutex);
 	val = dpu_fg_read(fg, SECSTATCONFIG);
 	if (enable)
 		val |= BIT(7);
 	else
 		val &= ~BIT(7);
-	dpu_fg_write(fg, val, SECSTATCONFIG);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, SECSTATCONFIG, val);
 }
 EXPORT_SYMBOL_GPL(framegen_syncmode_fixup);
 
-void framegen_sacfg(struct dpu_framegen *fg, unsigned int x, unsigned int y)
-{
-	mutex_lock(&fg->mutex);
-	dpu_fg_write(fg, STARTX(x) | STARTY(y), SACFG);
-	mutex_unlock(&fg->mutex);
-}
-EXPORT_SYMBOL_GPL(framegen_sacfg);
-
 void framegen_displaymode(struct dpu_framegen *fg, fgdm_t mode)
 {
 	u32 val;
 
-	mutex_lock(&fg->mutex);
 	val = dpu_fg_read(fg, FGINCTRL);
 	val &= ~FGDM_MASK;
 	val |= mode;
-	dpu_fg_write(fg, val, FGINCTRL);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, FGINCTRL, val);
 }
 EXPORT_SYMBOL_GPL(framegen_displaymode);
 
@@ -479,12 +319,10 @@ void framegen_panic_displaymode(struct dpu_framegen *fg, fgdm_t mode)
 {
 	u32 val;
 
-	mutex_lock(&fg->mutex);
 	val = dpu_fg_read(fg, FGINCTRLPANIC);
 	val &= ~FGDM_MASK;
 	val |= mode;
-	dpu_fg_write(fg, val, FGINCTRLPANIC);
-	mutex_unlock(&fg->mutex);
+	dpu_fg_write(fg, FGINCTRLPANIC, val);
 }
 EXPORT_SYMBOL_GPL(framegen_panic_displaymode);
 
@@ -499,10 +337,6 @@ void framegen_wait_done(struct dpu_framegen *fg, struct drm_display_mode *m)
 	if (dotclock == 0) {
 		/* fall back to display mode's clock */
 		dotclock = m->crtc_clock;
-
-		if (!(fg->side_by_side && fg->id == 1))
-			dev_warn(fg->dpu->dev,
-				"pixel clock for FrameGen%d is zero\n", fg->id);
 	}
 
 	/*
@@ -521,11 +355,9 @@ void framegen_wait_done(struct dpu_framegen *fg, struct drm_display_mode *m)
 	}
 	timeout = jiffies + pending_framedur_jiffies;
 
-	mutex_lock(&fg->mutex);
 	do {
 		val = dpu_fg_read(fg, FGENSTS);
 	} while ((val & ENSTS) && time_before(jiffies, timeout));
-	mutex_unlock(&fg->mutex);
 
 	dev_dbg(fg->dpu->dev, "FrameGen%d pending frame duration is %ums\n",
 			 fg->id, jiffies_to_msecs(pending_framedur_jiffies));
@@ -551,18 +383,16 @@ void framegen_read_timestamp(struct dpu_framegen *fg,
 {
 	u32 stamp;
 
-	mutex_lock(&fg->mutex);
 	stamp = dpu_fg_read(fg, FGTIMESTAMP);
 	*frame_index = framegen_frame_index(stamp);
 	*line_index = framegen_line_index(stamp);
-	mutex_unlock(&fg->mutex);
 }
 EXPORT_SYMBOL_GPL(framegen_read_timestamp);
 
 void framegen_wait_for_frame_counter_moving(struct dpu_framegen *fg)
 {
 	u32 frame_index, line_index, last_frame_index;
-	unsigned long timeout = jiffies + msecs_to_jiffies(50);
+	unsigned long timeout = jiffies + msecs_to_jiffies(100);
 
 	framegen_read_timestamp(fg, &frame_index, &line_index);
 	do {
@@ -582,13 +412,33 @@ void framegen_wait_for_frame_counter_moving(struct dpu_framegen *fg)
 }
 EXPORT_SYMBOL_GPL(framegen_wait_for_frame_counter_moving);
 
-bool framegen_secondary_is_syncup(struct dpu_framegen *fg)
+bool framegen_secondary_requests_to_read_empty_fifo(struct dpu_framegen *fg)
 {
 	u32 val;
+	bool empty;
 
-	mutex_lock(&fg->mutex);
 	val = dpu_fg_read(fg, FGCHSTAT);
-	mutex_unlock(&fg->mutex);
+
+	empty = !!(val & SFIFOEMPTY);
+
+	if (empty)
+		dev_dbg(fg->dpu->dev,
+			"FrameGen%d secondary requests to read empty FIFO\n",
+			fg->id);
+
+	return empty;
+}
+EXPORT_SYMBOL_GPL(framegen_secondary_requests_to_read_empty_fifo);
+
+void framegen_secondary_clear_channel_status(struct dpu_framegen *fg)
+{
+	dpu_fg_write(fg, FGCHSTATCLR, CLRSECSTAT);
+}
+EXPORT_SYMBOL_GPL(framegen_secondary_clear_channel_status);
+
+bool framegen_secondary_is_syncup(struct dpu_framegen *fg)
+{
+	u32 val = dpu_fg_read(fg, FGCHSTAT);
 
 	return val & SECSYNCSTAT;
 }
@@ -596,7 +446,7 @@ EXPORT_SYMBOL_GPL(framegen_secondary_is_syncup);
 
 void framegen_wait_for_secondary_syncup(struct dpu_framegen *fg)
 {
-	unsigned long timeout = jiffies + msecs_to_jiffies(50);
+	unsigned long timeout = jiffies + msecs_to_jiffies(100);
 	bool syncup;
 
 	do {
@@ -617,6 +467,7 @@ void framegen_enable_clock(struct dpu_framegen *fg)
 	if (!fg->use_bypass_clk)
 		clk_prepare_enable(fg->clk_pll);
 	clk_prepare_enable(fg->clk_disp);
+	clk_prepare_enable(fg->clk_disp_lpcg);
 }
 EXPORT_SYMBOL_GPL(framegen_enable_clock);
 
@@ -625,14 +476,15 @@ void framegen_disable_clock(struct dpu_framegen *fg)
 	if (!fg->use_bypass_clk)
 		clk_disable_unprepare(fg->clk_pll);
 	clk_disable_unprepare(fg->clk_disp);
+	clk_disable_unprepare(fg->clk_disp_lpcg);
 }
 EXPORT_SYMBOL_GPL(framegen_disable_clock);
 
 bool framegen_is_master(struct dpu_framegen *fg)
 {
-	const struct dpu_devtype *devtype = fg->dpu->devtype;
+	const struct dpu_data *data = fg->dpu->data;
 
-	return fg->id == devtype->master_stream_id;
+	return fg->id == data->master_stream_id;
 }
 EXPORT_SYMBOL_GPL(framegen_is_master);
 
@@ -723,21 +575,18 @@ int dpu_fg_init(struct dpu_soc *dpu, unsigned int id,
 	if (IS_ERR(fg->clk_pll))
 		return PTR_ERR(fg->clk_pll);
 
-	if (dpu->devtype->has_disp_sel_clk) {
-		fg->clk_bypass = devm_clk_get(dpu->dev, "bypass0");
-		if (IS_ERR(fg->clk_bypass))
-			return PTR_ERR(fg->clk_bypass);
-
-		fg->clk_disp_sel = devm_clk_get(dpu->dev,
-					id ? "disp1_sel" : "disp0_sel");
-			if (IS_ERR(fg->clk_disp_sel))
-				return PTR_ERR(fg->clk_disp_sel);
-	}
+	fg->clk_bypass = devm_clk_get(dpu->dev, "bypass0");
+	if (IS_ERR(fg->clk_bypass))
+		return PTR_ERR(fg->clk_bypass);
 
 	fg->clk_disp = devm_clk_get(dpu->dev, id ? "disp1" : "disp0");
 	if (IS_ERR(fg->clk_disp))
 		return PTR_ERR(fg->clk_disp);
 
+	fg->clk_disp_lpcg = devm_clk_get(dpu->dev, id ? "disp1_lpcg" : "disp0_lpcg");
+	if (IS_ERR(fg->clk_disp_lpcg))
+		return PTR_ERR(fg->clk_disp_lpcg);
+
 	fg->dpu = dpu;
 	fg->id = id;
 	mutex_init(&fg->mutex);
