@@ -1,5 +1,5 @@
 /*
- * Copyright 2018 NXP
+ * Copyright 2018,2021-2022 NXP
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include <linux/of_graph.h>
 #include <linux/platform_device.h>
 #include <linux/pm_runtime.h>
-#include <linux/regmap.h>
+#include <linux/reset.h>
 #include <linux/types.h>
 #include <drm/drm_fourcc.h>
 #include <video/imx-lcdif.h>
@@ -32,24 +32,13 @@
 
 #define DRIVER_NAME "imx-lcdif"
 
-/* TODO: add this to platform data later */
-#define DISP_MIX_SFT_RSTN_CSR		0x00
-#define DISP_MIX_CLK_EN_CSR		0x04
-
-/* 'DISP_MIX_SFT_RSTN_CSR' bit fields */
-#define BUS_RSTN_BLK_SYNC_SFT_EN	BIT(6)
-
-/* 'DISP_MIX_CLK_EN_CSR' bit fields */
-#define BUS_BLK_CLK_SFT_EN		BIT(12)
-#define LCDIF_PIXEL_CLK_SFT_EN		BIT(7)
-#define LCDIF_APB_CLK_SFT_EN		BIT(6)
-
 struct lcdif_soc {
 	struct device *dev;
 
 	int irq;
 	void __iomem *base;
-	struct regmap *gpr;
+	struct reset_control *soft_resetn;
+	struct reset_control *clk_enable;
 	atomic_t rpm_suspended;
 
 	struct clk *clk_pix;
@@ -83,9 +72,10 @@ struct lcdif_soc_pdata imx8mm_pdata = {
 
 static const struct of_device_id imx_lcdif_dt_ids[] = {
 	{ .compatible = "fsl,imx8mm-lcdif", .data = &imx8mm_pdata, },
+	{ .compatible = "fsl,imx8mn-lcdif", .data = &imx8mm_pdata, },
 	{ /* sentinel */ }
 };
-MODULE_DEVICE_TABLE(of, lcdif_dt_ids);
+MODULE_DEVICE_TABLE(of, imx_lcdif_dt_ids);
 
 #ifdef CONFIG_PM
 static int imx_lcdif_runtime_suspend(struct device *dev);
@@ -101,32 +91,17 @@ static int imx_lcdif_runtime_resume(struct device *dev)
 }
 #endif
 
-void disp_mix_bus_rstn_reset(struct regmap *gpr, bool reset)
+static int lcdif_rstc_reset(struct reset_control *rstc, bool assert)
 {
-	if (!reset)
-		/* release reset */
-		regmap_update_bits(gpr, DISP_MIX_SFT_RSTN_CSR,
-				   BUS_RSTN_BLK_SYNC_SFT_EN,
-				   BUS_RSTN_BLK_SYNC_SFT_EN);
-	else
-		/* hold reset */
-		regmap_update_bits(gpr, DISP_MIX_SFT_RSTN_CSR,
-				   BUS_RSTN_BLK_SYNC_SFT_EN,
-				   0x0);
-}
+	int ret;
 
-void disp_mix_lcdif_clks_enable(struct regmap *gpr, bool enable)
-{
-	if (enable)
-		/* enable lcdif clks */
-		regmap_update_bits(gpr, DISP_MIX_CLK_EN_CSR,
-				   LCDIF_PIXEL_CLK_SFT_EN | LCDIF_APB_CLK_SFT_EN,
-				   LCDIF_PIXEL_CLK_SFT_EN | LCDIF_APB_CLK_SFT_EN);
-	else
-		/* disable lcdif clks */
-		regmap_update_bits(gpr, DISP_MIX_CLK_EN_CSR,
-				   LCDIF_PIXEL_CLK_SFT_EN | LCDIF_APB_CLK_SFT_EN,
-				   0x0);
+	if (!rstc)
+		return 0;
+
+	ret = assert ? reset_control_assert(rstc)	:
+		       reset_control_deassert(rstc);
+
+	return ret;
 }
 
 static int lcdif_enable_clocks(struct lcdif_soc *lcdif)
@@ -279,7 +254,6 @@ EXPORT_SYMBOL(lcdif_get_bus_fmt_from_pix_fmt);
 
 int lcdif_set_pix_fmt(struct lcdif_soc *lcdif, u32 format)
 {
-	struct drm_format_name_buf format_name;
 	u32 ctrl = 0, ctrl1 = 0;
 
 	/* TODO: lcdif should be disabled to set pixel format */
@@ -350,8 +324,8 @@ int lcdif_set_pix_fmt(struct lcdif_soc *lcdif, u32 format)
 			       lcdif->base + LCDIF_CTRL2 + REG_SET);
 		break;
 	default:
-		dev_err(lcdif->dev, "unsupported pixel format: %s\n",
-			drm_get_format_name(format, &format_name));
+		dev_err(lcdif->dev, "unsupported pixel format: %p4cc\n",
+			&format);
 		return -EINVAL;
 	}
 
@@ -386,12 +360,15 @@ void lcdif_set_bus_fmt(struct lcdif_soc *lcdif, u32 bus_format)
 }
 EXPORT_SYMBOL(lcdif_set_bus_fmt);
 
-void lcdif_set_fb_addr(struct lcdif_soc *lcdif, int id, u32 addr)
+void lcdif_set_fb_addr(struct lcdif_soc *lcdif, int id, u32 addr, bool use_i80)
 {
 	switch (id) {
 	case 0:
 		/* primary plane */
-		writel(addr, lcdif->base + LCDIF_NEXT_BUF);
+		if (use_i80)
+			writel(addr, lcdif->base + LCDIF_CUR_BUF);
+		else
+			writel(addr, lcdif->base + LCDIF_NEXT_BUF);
 		break;
 	default:
 		/* TODO: add overlay support */
@@ -403,8 +380,8 @@ EXPORT_SYMBOL(lcdif_set_fb_addr);
 void lcdif_set_fb_hcrop(struct lcdif_soc *lcdif, u32 src_w,
 			u32 fb_w, bool crop)
 {
-	u32 mask_cnt, htotal, hcount;
-	u32 vdctrl2, vdctrl3, vdctrl4, transfer_count;
+	u32 mask_cnt;
+	u32 vdctrl3, vdctrl4, transfer_count;
 	u32 pigeon_12_0, pigeon_12_1, pigeon_12_2;
 
 	if (!crop) {
@@ -414,24 +391,15 @@ void lcdif_set_fb_hcrop(struct lcdif_soc *lcdif, u32 src_w,
 		return;
 	}
 
-	/* transfer_count's hcount, vdctrl2's htotal and vdctrl4's
-	 * H_VALID_DATA_CNT should use fb width instead of hactive
-	 * when requires cropping.
-	 * */
+	/*
+	 * transfer_count's hcount and vdctrl4's H_VALID_DATA_CNT
+	 * should use fb width instead of hactive when requires cropping.
+	 */
 	transfer_count = readl(lcdif->base + LCDIF_TRANSFER_COUNT);
-	hcount = TRANSFER_COUNT_GET_HCOUNT(transfer_count);
-
 	transfer_count &= ~TRANSFER_COUNT_SET_HCOUNT(0xffff);
 	transfer_count |= TRANSFER_COUNT_SET_HCOUNT(fb_w);
 	writel(transfer_count, lcdif->base + LCDIF_TRANSFER_COUNT);
 
-	vdctrl2 = readl(lcdif->base + LCDIF_VDCTRL2);
-	htotal  = VDCTRL2_GET_HSYNC_PERIOD(vdctrl2);
-	htotal  += fb_w - hcount;
-	vdctrl2 &= ~VDCTRL2_SET_HSYNC_PERIOD(0x3ffff);
-	vdctrl2 |= VDCTRL2_SET_HSYNC_PERIOD(htotal);
-	writel(vdctrl2, lcdif->base + LCDIF_VDCTRL2);
-
 	vdctrl4 = readl(lcdif->base + LCDIF_VDCTRL4);
 	vdctrl4 &= ~SET_DOTCLK_H_VALID_DATA_CNT(0x3ffff);
 	vdctrl4 |= SET_DOTCLK_H_VALID_DATA_CNT(fb_w);
@@ -458,13 +426,18 @@ void lcdif_set_fb_hcrop(struct lcdif_soc *lcdif, u32 src_w,
 EXPORT_SYMBOL(lcdif_set_fb_hcrop);
 
 
-void lcdif_set_mode(struct lcdif_soc *lcdif, struct videomode *vmode)
+void lcdif_set_mode(struct lcdif_soc *lcdif, struct videomode *vmode,
+		    bool use_i80)
 {
 	const struct of_device_id *of_id =
 			of_match_device(imx_lcdif_dt_ids, lcdif->dev);
-	const struct lcdif_soc_pdata *soc_pdata = of_id->data;
+	const struct lcdif_soc_pdata *soc_pdata;
 	u32 vdctrl0, vdctrl1, vdctrl2, vdctrl3, vdctrl4, htotal;
 
+	if (unlikely(!of_id))
+		return;
+	soc_pdata = of_id->data;
+
 	/* Clear the FIFO */
 	writel(CTRL1_FIFO_CLEAR, lcdif->base + LCDIF_CTRL1 + REG_SET);
 	writel(CTRL1_FIFO_CLEAR, lcdif->base + LCDIF_CTRL1 + REG_CLR);
@@ -479,6 +452,12 @@ void lcdif_set_mode(struct lcdif_soc *lcdif, struct videomode *vmode)
 	       TRANSFER_COUNT_SET_HCOUNT(vmode->hactive),
 	       lcdif->base + LCDIF_TRANSFER_COUNT);
 
+	if (use_i80) {
+		/* use MPU 8080 mode */
+		writel(CTRL1_MODE86, lcdif->base + LCDIF_CTRL1 + REG_CLR);
+		return;
+	}
+
 	vdctrl0 = VDCTRL0_ENABLE_PRESENT		|
 		  VDCTRL0_VSYNC_PERIOD_UNIT 		|
 		  VDCTRL0_VSYNC_PULSE_WIDTH_UNIT	|
@@ -536,20 +515,36 @@ void lcdif_set_mode(struct lcdif_soc *lcdif, struct videomode *vmode)
 }
 EXPORT_SYMBOL(lcdif_set_mode);
 
-void lcdif_enable_controller(struct lcdif_soc *lcdif)
+void lcdif_enable_controller(struct lcdif_soc *lcdif, bool use_i80)
 {
-	u32 ctrl2, vdctrl4;
+	u32 ctrl2, vdctrl4, timing;
 
 	ctrl2	= readl(lcdif->base + LCDIF_CTRL2);
 	vdctrl4 = readl(lcdif->base + LCDIF_VDCTRL4);
 
 	ctrl2 &= ~CTRL2_OUTSTANDING_REQS(0x7);
-	ctrl2 |= CTRL2_OUTSTANDING_REQS(REQ_16);
+	ctrl2 |= CTRL2_OUTSTANDING_REQS(use_i80 ? REQ_8 : REQ_16);
 	writel(ctrl2, lcdif->base + LCDIF_CTRL2);
 
-	/* Continous dotclock mode */
-	writel(CTRL_BYPASS_COUNT | CTRL_DOTCLK_MODE,
-	       lcdif->base + LCDIF_CTRL + REG_SET);
+	if (use_i80) {
+		/* MPU 8080 write mode */
+		writel(CTRL_DATA_SELECT, lcdif->base + LCDIF_CTRL + REG_SET);
+		writel(CTRL_READ_WRITEB, lcdif->base + LCDIF_CTRL + REG_CLR);
+		writel(CTRL_BYPASS_COUNT, lcdif->base + LCDIF_CTRL + REG_CLR);
+		writel(CTRL_DVI_MODE | CTRL_VSYNC_MODE | CTRL_DOTCLK_MODE,
+		       lcdif->base + LCDIF_CTRL + REG_CLR);
+
+		writel(CTRL1_COMBINE_MPU_WR_STRB,
+		       lcdif->base + LCDIF_CTRL1 + REG_CLR);
+
+		timing = TIMING_CMD_HOLD(2)  | TIMING_CMD_SETUP(1) |
+			 TIMING_DATA_HOLD(2) | TIMING_DATA_SETUP(1);
+		writel(timing, lcdif->base + LCDIF_TIMING);
+	} else {
+		/* Continuous dotclock mode */
+		writel(CTRL_BYPASS_COUNT | CTRL_DOTCLK_MODE,
+		       lcdif->base + LCDIF_CTRL + REG_SET);
+	}
 
 	/* enable the SYNC signals first, then the DMA engine */
 	vdctrl4 |= VDCTRL4_SYNC_SIGNALS_ON;
@@ -565,18 +560,21 @@ void lcdif_enable_controller(struct lcdif_soc *lcdif)
 }
 EXPORT_SYMBOL(lcdif_enable_controller);
 
-void lcdif_disable_controller(struct lcdif_soc *lcdif)
+void lcdif_disable_controller(struct lcdif_soc *lcdif, bool use_i80)
 {
 	int ret;
 	u32 ctrl, vdctrl4;
 
 	writel(CTRL_RUN, lcdif->base + LCDIF_CTRL + REG_CLR);
-	writel(CTRL_DOTCLK_MODE, lcdif->base + LCDIF_CTRL + REG_CLR);
 
-	ret = readl_poll_timeout(lcdif->base + LCDIF_CTRL, ctrl,
-				 !(ctrl & CTRL_RUN), 0, 1000);
-	if (WARN_ON(ret))
-		dev_err(lcdif->dev, "disable lcdif run timeout\n");
+	if (!use_i80) {
+		writel(CTRL_DOTCLK_MODE, lcdif->base + LCDIF_CTRL + REG_CLR);
+
+		ret = readl_poll_timeout(lcdif->base + LCDIF_CTRL, ctrl,
+					 !(ctrl & CTRL_RUN), 0, 1000);
+		if (WARN_ON(ret))
+			dev_err(lcdif->dev, "disable lcdif run timeout\n");
+	}
 
 	writel(CTRL_MASTER, lcdif->base + LCDIF_CTRL + REG_CLR);
 
@@ -586,6 +584,16 @@ void lcdif_disable_controller(struct lcdif_soc *lcdif)
 }
 EXPORT_SYMBOL(lcdif_disable_controller);
 
+long lcdif_pix_clk_round_rate(struct lcdif_soc *lcdif,
+			      unsigned long rate)
+{
+	if (unlikely(!rate))
+		return -EINVAL;
+
+	return clk_round_rate(lcdif->clk_pix, rate);
+}
+EXPORT_SYMBOL(lcdif_pix_clk_round_rate);
+
 static int platform_remove_device_fn(struct device *dev, void *data)
 {
 	struct platform_device *pdev = to_platform_device(dev);
@@ -648,10 +656,56 @@ err_register:
 	return ret;
 }
 
+static int lcdif_of_parse_resets(struct lcdif_soc *lcdif)
+{
+	int ret;
+	struct device *dev = lcdif->dev;
+	struct device_node *np = dev->of_node;
+	struct device_node *parent, *child;
+	struct of_phandle_args args;
+	struct reset_control *rstc;
+	const char *compat;
+	uint32_t len, rstc_num = 0;
+
+	ret = of_parse_phandle_with_args(np, "resets", "#reset-cells",
+					 0, &args);
+	if (ret)
+		return ret;
+
+	parent = args.np;
+	for_each_child_of_node(parent, child) {
+		compat = of_get_property(child, "compatible", NULL);
+		if (!compat)
+			continue;
+
+		rstc = of_reset_control_array_get(child, false, false, true);
+		if (IS_ERR(rstc))
+			continue;
+
+		len = strlen(compat);
+		if (!of_compat_cmp("lcdif,soft-resetn", compat, len)) {
+			lcdif->soft_resetn = rstc;
+			rstc_num++;
+		} else if (!of_compat_cmp("lcdif,clk-enable", compat, len)) {
+			lcdif->clk_enable = rstc;
+			rstc_num++;
+		}
+		else
+			dev_warn(dev, "invalid lcdif reset node: %s\n", compat);
+	}
+
+	if (!rstc_num) {
+		dev_err(dev, "no invalid reset control exists\n");
+		return -EINVAL;
+	}
+
+	return 0;
+}
+
 static int imx_lcdif_probe(struct platform_device *pdev)
 {
+	int ret;
 	struct device *dev = &pdev->dev;
-	struct device_node *np = dev->of_node;
 	struct lcdif_soc *lcdif;
 	struct resource *res;
 
@@ -687,11 +741,11 @@ static int imx_lcdif_probe(struct platform_device *pdev)
 	if (IS_ERR(lcdif->base))
 		return PTR_ERR(lcdif->base);
 
-	lcdif->gpr = syscon_regmap_lookup_by_phandle(np, "lcdif-gpr");
-	if (IS_ERR(lcdif->gpr))
-		return PTR_ERR(lcdif->gpr);
-
 	lcdif->dev = dev;
+	ret = lcdif_of_parse_resets(lcdif);
+	if (ret)
+		return ret;
+
 	platform_set_drvdata(pdev, lcdif);
 
 	atomic_set(&lcdif->rpm_suspended, 0);
@@ -720,15 +774,6 @@ static int imx_lcdif_resume(struct device *dev)
 {
 	return imx_lcdif_runtime_resume(dev);
 }
-#else
-static int imx_lcdif_suspend(struct device *dev)
-{
-	return 0;
-}
-static int imx_lcdif_resume(struct device *dev)
-{
-	return 0;
-}
 #endif
 
 #ifdef CONFIG_PM
@@ -767,8 +812,17 @@ static int imx_lcdif_runtime_resume(struct device *dev)
 		return ret;
 	}
 
-	disp_mix_bus_rstn_reset(lcdif->gpr, false);
-	disp_mix_lcdif_clks_enable(lcdif->gpr, true);
+	ret = lcdif_rstc_reset(lcdif->soft_resetn, false);
+	if (ret) {
+		dev_err(dev, "deassert soft_resetn failed\n");
+		return ret;
+	}
+
+	ret = lcdif_rstc_reset(lcdif->clk_enable, true);
+	if (ret) {
+		dev_err(dev, "assert clk_enable failed\n");
+		return ret;
+	}
 
 	/* Pull LCDIF out of reset */
 	writel(0x0, lcdif->base + LCDIF_CTRL);
