@@ -54,6 +54,10 @@ const struct SIRCS_CODE_TABLE {
     {SIRCS_CODE_DEF(0x3B, 0x0D10),  KEY_FN_F7,          "NEWS"      },
     {SIRCS_CODE_DEF(0x50, 0xB0),    KEY_F16,            "BASS+"     },
     {SIRCS_CODE_DEF(0x51, 0xB0),    KEY_F17,            "BASS-"     },
+    {SIRCS_CODE_DEF(0x62, 0x0510),  KEY_BRL_DOT4,       "AUTO VOLUME"               },
+    {SIRCS_CODE_DEF(0x74, 0x0d10),  KEY_BRL_DOT3,       "REAR ON/OFF"               },
+    {SIRCS_CODE_DEF(0x4e, 0x90),    KEY_F3,             "REAR+ DEMO"                },
+    {SIRCS_CODE_DEF(0x4f, 0x90),    KEY_F4,             "REAR- DEMO"                },
     /* MAIN MODE */
     {SIRCS_CODE_DEF(0x15, 0x0179),  KEY_ESC,            "ZONE3 I/O POWER ON/OFF"    },
     {SIRCS_CODE_DEF(0x15, 0x79),    KEY_MINUS,          "ZONE2 I/O POWER ON/OFF"    },
@@ -165,11 +169,6 @@ const struct SIRCS_CODE_TABLE {
     {SIRCS_CODE_DEF(0x33, 0x79),    KEY_SPELLCHECK,     "FR ZONE2"              },
     {SIRCS_CODE_DEF(0x31, 0x79),    KEY_VOICEMAIL,      "NEXT ZONE2"            },
     {SIRCS_CODE_DEF(0x34, 0x79),    KEY_LOGOFF,         "FF ZONE2"              },
-    {SIRCS_CODE_DEF(0x32, 0x79),    KEY_MEDIA_REPEAT,   "PLAY ZONE2"            },
-    {SIRCS_CODE_DEF(0x3d, 0x79),    KEY_MEDIA_REPEAT,   "PLAY ZONE2"            },
-    {SIRCS_CODE_DEF(0x38, 0x79),    KEY_10CHANNELSUP,   "STOP ZONE2"            },
-    {SIRCS_CODE_DEF(0x39, 0x79),    KEY_IMAGES,         "PLAY/PAUSE ZONE2"      },
-    {SIRCS_CODE_DEF(0x18, 0x79),    KEY_BRL_DOT5,       "FM ZONE2"              },
     /* ZONE3 MODE */                
     /* Power key is the same as Main mode */
     {SIRCS_CODE_DEF(0x78, 0x0179),  KEY_KATAKANAHIRAGANA,   "GAME ZONE3"            },
@@ -195,11 +194,6 @@ const struct SIRCS_CODE_TABLE {
     {SIRCS_CODE_DEF(0x33, 0x0179),  KEY_DISPLAYTOGGLE,      "FR ZONE3"              },
     {SIRCS_CODE_DEF(0x31, 0x0179),  KEY_LAST,               "NEXT ZONE3"            },
     {SIRCS_CODE_DEF(0x34, 0x0179),  KEY_MESSENGER,          "FF ZONE3"              },
-    {SIRCS_CODE_DEF(0x32, 0x0179),  KEY_DEL_EOL,            "PLAY ZONE3"            },
-    {SIRCS_CODE_DEF(0x3d, 0x0179),  KEY_DEL_EOL,            "PLAY ZONE3"            },
-    {SIRCS_CODE_DEF(0x38, 0x0179),  KEY_DEL_EOS,            "STOP ZONE3"            },
-    {SIRCS_CODE_DEF(0x39, 0x0179),  KEY_DEL_LINE,           "PLAY/PAUSE ZONE3"      },
-    {SIRCS_CODE_DEF(0x18, 0x0179),  KEY_BRL_DOT6,           "FM ZONE3"              },
 
     /* DEALER MODE(MAIN) */
     {SIRCS_CODE_DEF(0x2e, 0x0179),  KEY_LIST,       "POWER ON(ZONE3)"           },
