@@ -1,16 +1,9 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * Freescale i.MX7 SoC series MIPI-CSI V3.3 receiver driver
  *
  * Copyright (C) 2015-2016 Freescale Semiconductor, Inc. All Rights Reserved.
- */
-
-/*
- * The code contained herein is licensed under the GNU General Public
- * License. You may obtain a copy of the GNU General Public License
- * Version 2 or later at the following locations:
- *
- * http://www.opensource.org/licenses/gpl-license.html
- * http://www.gnu.org/copyleft/gpl.html
+ * Copyright 2019 NXP
  */
 /*
  * Samsung S5P/EXYNOS SoC series MIPI-CSI receiver driver
@@ -39,6 +32,7 @@
 #include <linux/pm_runtime.h>
 #include <linux/regmap.h>
 #include <linux/regulator/consumer.h>
+#include <linux/reset.h>
 #include <linux/slab.h>
 #include <linux/spinlock.h>
 #include <linux/videodev2.h>
@@ -190,9 +184,6 @@ MODULE_PARM_DESC(debug, "Debug level (0-2)");
 #define MIPI_CSIS_PKTDATA_EVEN		0x3000
 #define MIPI_CSIS_PKTDATA_SIZE		SZ_4K
 
-#define GPR_MIPI_RESET			0x08
-#define GPR_MIPI_S_RESETN		BIT(16)
-
 #define DEFAULT_SCLK_CSIS_FREQ	166000000UL
 
 enum {
@@ -295,9 +286,8 @@ struct csi_state {
 	struct csis_pktbuf pkt_buf;
 	struct mipi_csis_event events[MIPI_CSIS_NUM_EVENTS];
 
-	struct v4l2_async_subdev    asd;
+	struct fwnode_handle *fwnode;
 	struct v4l2_async_notifier  subdev_notifier;
-	struct v4l2_async_subdev    *async_subdevs[2];
 
 	struct csis_hw_reset hw_reset;
 	struct regulator     *mipi_phy_regulator;
@@ -393,23 +383,19 @@ static int mipi_csis_phy_init(struct csi_state *state)
 
 static int mipi_csis_phy_reset_mx8mm(struct csi_state *state)
 {
-	struct device_node *np = state->dev->of_node;
-	struct regmap *gpr;
+	struct reset_control *phy_reset;
 
-	gpr = syscon_regmap_lookup_by_phandle(np, "csi-gpr");
-	if (IS_ERR(gpr))
-		return PTR_ERR(gpr);
+	phy_reset = devm_reset_control_get_exclusive(state->dev, "csi,mipi_rst");
+	if (IS_ERR(phy_reset))
+		return PTR_ERR(phy_reset);
 
-	regmap_update_bits(gpr, GPR_MIPI_RESET,
-			   GPR_MIPI_S_RESETN,
-			   0x0);
+	reset_control_assert(phy_reset);
 	usleep_range(10, 20);
-	regmap_update_bits(gpr, GPR_MIPI_RESET,
-			   GPR_MIPI_S_RESETN,
-			   GPR_MIPI_S_RESETN);
+	reset_control_deassert(phy_reset);
 	usleep_range(10, 20);
 
 	return 0;
+
 }
 
 static int mipi_csis_phy_reset(struct csi_state *state)
@@ -730,7 +716,7 @@ unlock:
 }
 
 static int mipi_csis_enum_mbus_code(struct v4l2_subdev *mipi_sd,
-				    struct v4l2_subdev_pad_config *cfg,
+				    struct v4l2_subdev_state *sd_state,
 				    struct v4l2_subdev_mbus_code_enum *code)
 {
 	struct csi_state *state = mipi_sd_to_csi_state(mipi_sd);
@@ -752,7 +738,7 @@ static int mipi_csis_enum_mbus_code(struct v4l2_subdev *mipi_sd,
 }
 
 static int mipi_csis_set_fmt(struct v4l2_subdev *mipi_sd,
-			     struct v4l2_subdev_pad_config *cfg,
+			     struct v4l2_subdev_state *sd_state,
 			     struct v4l2_subdev_format *format)
 {
 	struct csi_state *state = mipi_sd_to_csi_state(mipi_sd);
@@ -790,7 +776,7 @@ static int mipi_csis_set_fmt(struct v4l2_subdev *mipi_sd,
 }
 
 static int mipi_csis_get_fmt(struct v4l2_subdev *mipi_sd,
-			     struct v4l2_subdev_pad_config *cfg,
+			     struct v4l2_subdev_state *sd_state,
 			     struct v4l2_subdev_format *format)
 {
 	struct csi_state *state = mipi_sd_to_csi_state(mipi_sd);
@@ -835,7 +821,7 @@ static int mipi_csis_g_parm(struct v4l2_subdev *mipi_sd, struct v4l2_streamparm
 }
 
 static int mipi_csis_enum_framesizes(struct v4l2_subdev *mipi_sd,
-		struct v4l2_subdev_pad_config *cfg,
+		struct v4l2_subdev_state *sd_state,
 		struct v4l2_subdev_frame_size_enum *fse)
 {
 	struct csi_state *state = mipi_sd_to_csi_state(mipi_sd);
@@ -845,7 +831,7 @@ static int mipi_csis_enum_framesizes(struct v4l2_subdev *mipi_sd,
 }
 
 static int mipi_csis_enum_frameintervals(struct v4l2_subdev *mipi_sd,
-		struct v4l2_subdev_pad_config *cfg,
+		struct v4l2_subdev_state *sd_state,
 		struct v4l2_subdev_frame_interval_enum *fie)
 {
 	struct csi_state *state = mipi_sd_to_csi_state(mipi_sd);
@@ -943,7 +929,7 @@ static int subdev_notifier_bound(struct v4l2_async_notifier *notifier,
 	struct csi_state *state = notifier_to_mipi_dev(notifier);
 
 	/* Find platform data for this sensor subdev */
-	if (state->asd.match.fwnode.fwnode == dev_fwnode(subdev->dev))
+	if (state->fwnode == dev_fwnode(subdev->dev))
 		state->sensor_sd = subdev;
 
 	if (subdev == NULL)
@@ -993,13 +979,20 @@ static int mipi_csis_parse_dt(struct platform_device *pdev,
 static int mipi_csis_pm_resume(struct device *dev, bool runtime);
 static const struct of_device_id mipi_csis_of_match[];
 
+static const struct v4l2_async_notifier_operations mxc_mipi_csi_subdev_ops = {
+	.bound = subdev_notifier_bound,
+};
+
 /* register parent dev */
 static int mipi_csis_subdev_host(struct csi_state *state)
 {
 	struct device_node *parent = state->dev->of_node;
 	struct device_node *node, *port, *rem;
+	struct v4l2_async_subdev *asd;
 	int ret;
 
+	v4l2_async_notifier_init(&state->subdev_notifier);
+
 	/* Attach sensors linked to csi receivers */
 	for_each_available_child_of_node(parent, node) {
 		if (of_node_cmp(node->name, "port"))
@@ -1018,17 +1011,24 @@ static int mipi_csis_subdev_host(struct csi_state *state)
 			return -1;
 		}
 
-		state->asd.match_type = V4L2_ASYNC_MATCH_FWNODE;
-		state->asd.match.fwnode.fwnode = of_fwnode_handle(rem);
-		state->async_subdevs[0] = &state->asd;
+		state->fwnode = of_fwnode_handle(rem);
+		asd = v4l2_async_notifier_add_fwnode_subdev(
+						&state->subdev_notifier,
+						state->fwnode,
+						struct v4l2_async_subdev);
+		if (IS_ERR(asd)) {
+			of_node_put(rem);
+			dev_err(state->dev, "failed to add subdev to a notifier\n");
+			return PTR_ERR(asd);
+		}
 
 		of_node_put(rem);
 		break;
 	}
 
-	state->subdev_notifier.subdevs = state->async_subdevs;
-	state->subdev_notifier.num_subdevs = 1;
-	state->subdev_notifier.bound = subdev_notifier_bound;
+
+	state->subdev_notifier.v4l2_dev = &state->v4l2_dev;
+	state->subdev_notifier.ops = &mxc_mipi_csi_subdev_ops;
 
 	ret = v4l2_async_notifier_register(&state->v4l2_dev,
 					&state->subdev_notifier);
@@ -1163,6 +1163,7 @@ static int mipi_csis_probe(struct platform_device *pdev)
 			goto e_sd_host;
 	}
 
+	mipi_csis_clk_disable(state);
 	dev_info(&pdev->dev,
 			"lanes: %d, hs_settle: %d, clk_settle: %d, wclk: %d, freq: %u\n",
 		 state->num_lanes, state->hs_settle, state->clk_settle,
@@ -1272,12 +1273,12 @@ static int mipi_csis_remove(struct platform_device *pdev)
 	struct csi_state *state = platform_get_drvdata(pdev);
 
 	v4l2_async_unregister_subdev(&state->mipi_sd);
+	v4l2_async_notifier_cleanup(&state->subdev_notifier);
 	v4l2_async_notifier_unregister(&state->subdev_notifier);
 	v4l2_device_unregister(&state->v4l2_dev);
 
 	pm_runtime_disable(&pdev->dev);
 	mipi_csis_pm_suspend(&pdev->dev, true);
-	mipi_csis_clk_disable(state);
 	pm_runtime_set_suspended(&pdev->dev);
 
 	return 0;
