@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2018 MediaTek Inc.
+ * Copyright 2022 Sony Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -11,7 +12,7 @@
  * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
  */
 
-#include <audio_ipi_dma.h>
+#include <soc/mediatek/hifi4dsp/audio_ipi_dma.h>
 
 #include <linux/io.h>
 #include <linux/genalloc.h>
@@ -20,22 +21,22 @@
 #include <linux/uaccess.h>      /* needed by copy_to_user */
 
 #ifdef CONFIG_MTK_AUDIODSP_SUPPORT
-#include <adsp_helper.h>
+#include <soc/mediatek/hifi4dsp/adsp_helper.h>
 #endif
 
 #ifdef CONFIG_MTK_HIFI4DSP_SUPPORT
-#include "audio_memory.h"
+#include <soc/mediatek/hifi4dsp/audio_memory.h>
 #endif
 
-#include <audio_log.h>
-#include <audio_assert.h>
-#include <audio_ipi_platform.h>
+#include <soc/mediatek/hifi4dsp/audio_log.h>
+#include <soc/mediatek/hifi4dsp/audio_assert.h>
+#include <soc/mediatek/hifi4dsp/audio_ipi_platform.h>
 
-#include <audio_task.h>
-#include <audio_controller_msg_id.h>
-#include <audio_messenger_ipi.h>
+#include <soc/mediatek/hifi4dsp/audio_task.h>
+#include <soc/mediatek/hifi4dsp/audio_controller_msg_id.h>
+#include <soc/mediatek/hifi4dsp/audio_messenger_ipi.h>
 
-#include <audio_ringbuf.h>
+#include <soc/mediatek/hifi4dsp/audio_ringbuf.h>
 
 
 
