@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2018 MediaTek Inc.
+ * Copyright 2022 Sony Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -11,7 +12,7 @@
  * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
  */
 
-#include "audio_ipi_queue.h"
+#include <soc/mediatek/hifi4dsp/audio_ipi_queue.h>
 
 #include <linux/slab.h>         /* needed by kmalloc */
 
@@ -21,13 +22,13 @@
 
 #include <linux/delay.h>
 
-#include "audio_log.h"
-#include "audio_assert.h"
+#include <soc/mediatek/hifi4dsp/audio_log.h>
+#include <soc/mediatek/hifi4dsp/audio_assert.h>
 
-#include "audio_ipi_platform.h"
+#include <soc/mediatek/hifi4dsp/audio_ipi_platform.h>
 
-#include "audio_messenger_ipi.h"
-#include "audio_task.h"
+#include <soc/mediatek/hifi4dsp/audio_messenger_ipi.h>
+#include <soc/mediatek/hifi4dsp/audio_task.h>
 
 
 
