@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2017 MediaTek Inc.
+ * Copyright 2022 Sony Corporation
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -11,7 +12,7 @@
  * GNU General Public License for more details.
  */
 
-#include <adsp_ipi_queue.h>
+#include <soc/mediatek/hifi4dsp/adsp_ipi_queue.h>
 
 #include <linux/types.h>
 #include <linux/errno.h>
