@@ -24,9 +24,9 @@
 
 #define HIFI4DSP_RESET
 
-#include "adsp_ipi.h"
-#include "adsp_ipi_queue.h"
-#include "adsp_ipi_platform.h"
+#include <soc/mediatek/hifi4dsp/adsp_ipi.h>
+#include <soc/mediatek/hifi4dsp/adsp_ipi_queue.h>
+#include <soc/mediatek/hifi4dsp/adsp_ipi_platform.h>
 
 /*
  * =============================================================================
@@ -86,7 +86,7 @@ struct adsp_share_obj *adsp_rcv_obj[ADSP_CORE_TOTAL];
 struct mutex adsp_ipi_mutex[ADSP_CORE_TOTAL];
 
 #define IPI_WAKEUP_TIME 1000
-struct wakeup_source ipi_wakeup_lock;
+//struct wakeup_source ipi_wakeup_lock;
 
 /*
  * =============================================================================
@@ -575,7 +575,7 @@ static int __init mt8570_ipi_probe(struct platform_device *pdev)
 	enable_irq_wake(irq_num);
 #endif
 
-	wakeup_source_init(&ipi_wakeup_lock, "ipi_wakeup_lock");
+//	wakeup_source_init(&ipi_wakeup_lock, "ipi_wakeup_lock");
 #endif
 
 	mt8570_ipi_platform_init(pdev);
@@ -611,7 +611,7 @@ unsigned int is_from_suspend;
 static int audio_ipi_pm_resume(struct device *device)
 {
 	pr_notice("audio_ipi_pm_resume is resume!\n");
-	__pm_wakeup_event(&ipi_wakeup_lock, IPI_WAKEUP_TIME);
+//	__pm_wakeup_event(&ipi_wakeup_lock, IPI_WAKEUP_TIME);
 #ifdef HIFI4DSP_RESET
 	atomic_set(&is_from_suspend, 0);
 #endif
