@@ -15,7 +15,7 @@
 #define pr_fmt(fmt) HIFI4DSP_LOAD_DRV_NAME ": " fmt
 
 #include <linux/clk.h>
-#include <adsp_ipi.h>
+#include <soc/mediatek/hifi4dsp/adsp_ipi.h>
 #include <linux/debugfs.h>
 #include <linux/delay.h>
 #include <linux/device.h>
@@ -34,7 +34,11 @@
 #include <linux/of_device.h>
 #include <linux/platform_device.h>
 #include <linux/uaccess.h>
-#include <hifi4dsp_load/hifi4dsp_load.h>
+#include <linux/regulator/consumer.h>
+#include <soc/mediatek/hifi4dsp/hifi4dsp_load.h>
+#include <linux/atomic.h>
+#include <linux/workqueue.h>
+#include <linux/jiffies.h>
 
 #define DSP_LOAD_UNIT_TEST	0	
 //#define SARD		//uncomment for SARD debug logs
@@ -78,6 +82,12 @@
 #define GPR_LOG_BUF_ADDR		   (DSP0_GPR1d)
 #define GPR_DSP1_REBOOT			   (DSP1_GPR1d)
 
+
+static void hifi4dsp_stop_dsp(void);
+
+struct delayed_work uevent_issue_work;
+struct workqueue_struct *uevent_issue_queue;
+
 /*
  * hifi4dsp image package and format.
  *
@@ -126,12 +136,10 @@ struct hifi4dsp_callback {
 
 struct hifi4dsp_load_dev {
 	struct device *dev;
-	int boot_done;
+	atomic_t boot_done;
 	//int rst_gpio; //added by SARD
 	struct gpio_desc *gpiod;  //added by SARD
-	struct gpio_desc *gpiod_pvc;  //added by SCO
-	struct gpio_desc *gpiod_p18;  //added by SCO
-	struct gpio_desc *gpiod_p33;  //added by SCO
+	struct regulator *reg_p33v;  //added by SCO
 	struct hifi4dsp_callback callback;
 	struct hifi4dsp_image_info img_info;
 	struct dentry *dent;
@@ -381,6 +389,31 @@ out_err:
 	return -ENOMEM;
 }
 
+static void hifi4dsp_load_uevent(struct work_struct *work)
+{
+	int ret;
+	char *envp[] = {
+		"HIFI4DSP_LOAD_READY=true",
+		NULL
+	};
+
+	ret = 0;
+
+	ret = kobject_uevent_env(&(hifi4dsp_load->dev->kobj), KOBJ_CHANGE, envp);
+	if (ret == 0)
+		pr_info("send uevent : HIFI4DSP_LOAD_READY=true\n");
+	else
+		pr_err("%s: failed to send uevent\n", __func__);
+}
+
+static void delayed_work_hifi4dsp_load_uevent(void)
+{
+	queue_delayed_work(uevent_issue_queue,
+			&uevent_issue_work,
+			msecs_to_jiffies(30));
+	pr_info("Apply uevent issue to delayed work queue\n");
+}
+
 static int check_image_header_info(u8 *data, int size)
 {
 	int ret = 0;
@@ -700,30 +733,28 @@ static void fixup_hifi4dsp_early_setting(void)
 #ifdef MULTI_DSP_LOAD
 static void add_hifi4dsp_run_status(void)
 {
-	hifi4dsp_load->boot_done += 1;
+	atomic_inc(&(hifi4dsp_load->boot_done));
 	pr_notice("load bin done and start to run now.\n");
 }
 #else
 static void set_hifi4dsp_run_status(void)
 {
-	hifi4dsp_load->boot_done = 1;
+	atomic_set(&(hifi4dsp_load->boot_done), 1);
 	pr_notice("load bin done and start to run now.\n");
 }
 #endif /* MULTI_DSP_LOAD */
 
-#ifdef HIFI4DSP_RESET
 static void reset_hifi4dsp_run_status(void)
 {
-	hifi4dsp_load->boot_done = 0;
+	atomic_set(&(hifi4dsp_load->boot_done), 0);
 }
-#endif /* HIFI4DSP_RESET */
 
 /*
  * HIFI4DSP has boot done or not.
  */
 int hifi4dsp_run_status(void)
 {
-	return hifi4dsp_load->boot_done;
+	return atomic_read(&(hifi4dsp_load->boot_done));
 }
 EXPORT_SYMBOL(hifi4dsp_run_status);
 
@@ -792,6 +823,14 @@ void load_image_to_dsp_and_run(
 	/* callback function for user */
 	if (hifi4dsp_load->callback.cb)
 		hifi4dsp_load->callback.cb(hifi4dsp_load->callback.args);
+
+	/* send uevent */
+#ifdef MULTI_DSP_LOAD
+	if (num_of_dsp == hifi4dsp_run_status())
+		delayed_work_hifi4dsp_load_uevent();
+#else
+	delayed_work_hifi4dsp_load_uevent();
+#endif /* MULTI_DSP_LOAD */
 }
 
 /*
@@ -939,6 +978,14 @@ int load_hifi4dsp_bin_and_run()
 #endif /* MULTI_DSP_LOAD */
 	}
 
+	/* send uevent */
+#ifdef MULTI_DSP_LOAD
+	if (num_of_dsp == hifi4dsp_run_status())
+		delayed_work_hifi4dsp_load_uevent();
+#else
+	delayed_work_hifi4dsp_load_uevent();
+#endif /* MULTI_DSP_LOAD */
+
 	return ret;
 }
 EXPORT_SYMBOL(load_hifi4dsp_bin_and_run);
@@ -951,8 +998,13 @@ int breed_hifi4dsp(void *data)
 	pr_info("%s() enter.\n", __func__);
 	
 	/* sleep() or wait_mutex() */
-	while (!hifi4dsp_spi_get_status())
+	while (!hifi4dsp_spi_get_status()) {
+		if (kthread_should_stop()) {
+			/* Some one abort me. */
+			return -EINTR;
+		}
 		msleep(500);
+	}
 
 	pr_info("Calling async_load_hifi4dsp_bin_and_run.\n");
 	ret = async_load_hifi4dsp_bin_and_run(NULL, NULL);
@@ -968,50 +1020,39 @@ int breed_hifi4dsp(void *data)
 static int hifi4dsp_load_pm_suspend(struct device *dev)
 {
 #ifdef HIFI4DSP_RESET
+	int ret;
+
 	if(hifi4dsp_load->gpiod) {
 		gpiod_set_value_cansleep(hifi4dsp_load->gpiod, 0);
 		usleep_range(2500, 3000);
 	}
 
-	if(hifi4dsp_load->gpiod_p33) {
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_p33, 0);
-		usleep_range(2500, 3000);
-	}
-
-	if(hifi4dsp_load->gpiod_p18) {
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_p18, 0);
-		usleep_range(1500, 2000);
-	}
-
-	if(hifi4dsp_load->gpiod_pvc) {
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_pvc, 0);
+	if(hifi4dsp_load->reg_p33v) {
+		ret = regulator_disable(hifi4dsp_load->reg_p33v);
 	}
 
 	reset_hifi4dsp_run_status();
 #endif //HIFI4DSP_RESET
+
+	cancel_delayed_work_sync(&uevent_issue_work);
+
 	return 0;
 }
 
 static int hifi4dsp_load_pm_resume(struct device *dev)
 {
 #ifdef HIFI4DSP_RESET
-	if(hifi4dsp_load->gpiod_pvc) {
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_pvc, 1);
-		usleep_range(1500, 2000);
-	}
+	int ret;
 
-	if(hifi4dsp_load->gpiod_p18) {
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_p18, 1);
-		usleep_range(2500, 3000);
-	}
-
-	if(hifi4dsp_load->gpiod_p33) {
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_p33, 1);
-		usleep_range(6500, 7000);
+	if(hifi4dsp_load->reg_p33v) {
+		ret = regulator_enable(hifi4dsp_load->reg_p33v);
 	}
 
 	if(hifi4dsp_load->gpiod) {
+		gpiod_set_value_cansleep(hifi4dsp_load->gpiod, 0);
+		usleep_range(55000, 60000);//Minimum Reset delay is 50ms as per schematics
 		gpiod_set_value_cansleep(hifi4dsp_load->gpiod, 1);
+		usleep_range(55000, 60000);
 	}
 #endif //HIFI4DSP_RESET
 	return 0;
@@ -1022,6 +1063,37 @@ static const struct dev_pm_ops hifi4dsp_load_pm_ops = {
 	.resume   = hifi4dsp_load_pm_resume,
 };
 
+static ssize_t load_status_show(
+	struct device *dev,
+	struct device_attribute *attr,
+	char *buf)
+{
+	ssize_t result = 0;
+
+#ifdef MULTI_DSP_LOAD
+	if (num_of_dsp == hifi4dsp_run_status())
+		result += snprintf(buf, PAGE_SIZE, "1\n");
+	else
+		result += snprintf(buf, PAGE_SIZE, "0\n");
+#else
+	result += snprintf(buf, PAGE_SIZE, "%.d\n", hifi4dsp_run_status());
+#endif /* MULTI_DSP_LOAD */
+
+	return result;
+}
+
+static DEVICE_ATTR_RO(load_status);
+
+static struct attribute *hifi4dsp_attributes[] = {
+	&dev_attr_load_status.attr,
+	NULL,
+};
+
+static const struct attribute_group hifi4dsp_attr_group = {
+	.name = "hifi4dsp-load",
+	.attrs = hifi4dsp_attributes,
+};
+
 static int hifi4dsp_load_probe(struct platform_device *pdev)
 {
 	int ret = 0;
@@ -1036,35 +1108,32 @@ static int hifi4dsp_load_probe(struct platform_device *pdev)
 	hifi4dsp_load = devm_kzalloc(&pdev->dev,
 			sizeof(struct hifi4dsp_load_dev), GFP_KERNEL);
 			
-	if (!hifi4dsp_load)
-		return -ENOMEM;
+	if (!hifi4dsp_load) {
+		ret = -ENOMEM;
+		goto error_malloc;
+	}
 	
 	hifi4dsp_load->dev = &pdev->dev;
+	reset_hifi4dsp_run_status();
 	hifi4dsp_debugfs_init();
 	//pr_info("hifi4dsp_load->dev = 0x%x \n",hifi4dsp_load->dev);
 
 	of_property_read_u32(np, "hifi4dsp,num-of-dsp", &num_of_dsp);
 
 #ifdef HIFI4DSP_RESET
-	hifi4dsp_load->gpiod_pvc = devm_gpiod_get(&pdev->dev, "mtk8570,pcontvc", GPIOD_OUT_LOW);
-	if (IS_ERR(hifi4dsp_load->gpiod_pvc)) {
-		ret = PTR_ERR(hifi4dsp_load->gpiod_pvc);
-		pr_notice("error getting gpio PCONT_VCORE(%d)\n", ret);
-		return ret;
-	}
-
-	hifi4dsp_load->gpiod_p18 = devm_gpiod_get(&pdev->dev, "mtk8570,pcont18", GPIOD_OUT_LOW);
-	if (IS_ERR(hifi4dsp_load->gpiod_p18)) {
-		ret = PTR_ERR(hifi4dsp_load->gpiod_p18);
-		pr_notice("error getting gpio PCONT_1.8V(%d)\n", ret);
-		return ret;
-	}
-
-	hifi4dsp_load->gpiod_p33 = devm_gpiod_get(&pdev->dev, "mtk8570,pcont33", GPIOD_OUT_LOW);
-	if (IS_ERR(hifi4dsp_load->gpiod_p33)) {
-		ret = PTR_ERR(hifi4dsp_load->gpiod_p33);
-		pr_notice("error getting gpio PCONT_3.3V(%d)\n", ret);
-		return ret;
+	hifi4dsp_load->reg_p33v = devm_regulator_get(&pdev->dev, "p33v");
+	if (!IS_ERR(hifi4dsp_load->reg_p33v)) {
+		ret = regulator_enable(hifi4dsp_load->reg_p33v);
+		if (ret) {
+			pr_notice("Failed to enable PCONT_3.3V regulator: %d\n", ret);
+			goto error_pcont;
+		}
+	} else {
+		if (PTR_ERR(hifi4dsp_load->reg_p33v) == -EPROBE_DEFER) {
+			ret = -EPROBE_DEFER;
+			goto error_pcont;
+		}
+		hifi4dsp_load->reg_p33v = NULL;
 	}
 
 	hifi4dsp_load->gpiod = devm_gpiod_get(&pdev->dev, "mtk8570,reset", GPIOD_OUT_LOW);
@@ -1073,37 +1142,15 @@ static int hifi4dsp_load_probe(struct platform_device *pdev)
 	if (IS_ERR(hifi4dsp_load->gpiod)) {
 		ret = PTR_ERR(hifi4dsp_load->gpiod);
 		pr_notice("error getting gpio (%d)\n", ret);
-		return ret;
+		goto error_gpio;
 	}
 	ret = gpiod_direction_output(hifi4dsp_load->gpiod, 1);
-	ret |= gpiod_direction_output(hifi4dsp_load->gpiod_pvc, 1);
-	ret |= gpiod_direction_output(hifi4dsp_load->gpiod_p18, 1);
-	ret |= gpiod_direction_output(hifi4dsp_load->gpiod_p33, 1);
 /*	------        -------
 *		 |  50ms  |
 *        |________|
 */
-	if(hifi4dsp_load->gpiod_pvc)
-	{
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_pvc, 1);
-		usleep_range(1500, 2000);
-	}
-
-	if(hifi4dsp_load->gpiod_p18)
-	{
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_p18, 1);
-		usleep_range(2500, 3000);
-	}
-
-	if(hifi4dsp_load->gpiod_p33)
-	{
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod_p33, 1);
-		usleep_range(7000, 8000);
-	}
-
 	if(hifi4dsp_load->gpiod)
 	{
-		gpiod_set_value_cansleep(hifi4dsp_load->gpiod, 0);
 		usleep_range(55000, 60000);//Minimum Reset delay is 50ms as per schematics
 		gpiod_set_value_cansleep(hifi4dsp_load->gpiod, 1);
 		usleep_range(55000, 60000);
@@ -1117,12 +1164,56 @@ static int hifi4dsp_load_probe(struct platform_device *pdev)
 	if (IS_ERR(dsp_task)) {
 		pr_notice("Couldn't create kthread for breed_hifi4dsp\n");
 		ret = PTR_ERR(dsp_task);
-		return ret;
+		goto error_kthread_run;
 	}
 	pr_info("Start to run kthread [breed_hifi4dsp]\n");
 #endif
 
+	/* create workqueue for uevent */
+	INIT_DELAYED_WORK(&uevent_issue_work, hifi4dsp_load_uevent);
+
+	uevent_issue_queue = create_singlethread_workqueue("hifi4dsp uevent issue");
+	if (uevent_issue_queue == NULL) {
+		pr_err("%s: failed create singlethread workqueue\n", __func__);
+		ret = -ENOMEM;
+		goto error_create_workqueue;
+	}
+
+	/* create sysfs */
+	ret = sysfs_create_group(&dev->kobj, &hifi4dsp_attr_group);
+	if (ret != 0)
+		goto error_create_sysfs;
+
+	return 0;
+
+error_create_sysfs:
+	cancel_delayed_work_sync(&uevent_issue_work);
+	destroy_workqueue(uevent_issue_queue);
+
+error_create_workqueue:
+#if DSP_LOAD_UNIT_TEST
+	while (kthread_stop(dsp_task))
+		schedule_timeout(3 * HZ);
+#endif
+
+
+#if DSP_LOAD_UNIT_TEST
+error_kthread_run:
+#endif
+
+#ifdef HIFI4DSP_RESET
+error_gpio:
+error_pcont:
+	hifi4dsp_stop_dsp();
+#endif
+
+	debugfs_remove_recursive(hifi4dsp_load->dent);
+
+error_malloc:
+	devm_kfree(&pdev->dev, hifi4dsp_load);
+
 	return ret;
+
 }
 
 static void hifi4dsp_stop_dsp(void)
@@ -1151,7 +1242,12 @@ static void hifi4dsp_stop_dsp(void)
 
 static int hifi4dsp_load_remove(struct platform_device *pdev)
 {
+	int ret = 0;
+
 	hifi4dsp_stop_dsp();
+	if(hifi4dsp_load->reg_p33v) {
+		ret = regulator_disable(hifi4dsp_load->reg_p33v);
+	}
 	debugfs_remove_recursive(hifi4dsp_load->dent);
 /*	devm_gpio_free(hifi4dsp_load->dev, hifi4dsp_load->gpiod);*/
 	devm_kfree(&pdev->dev, hifi4dsp_load);
@@ -1160,7 +1256,19 @@ static int hifi4dsp_load_remove(struct platform_device *pdev)
 
 static void hifi4dsp_load_shutdown(struct platform_device *pdev)
 {
+	int ret = 0;
+
+	/* remove sysfs */
+	sysfs_remove_group(&pdev->dev.kobj, &hifi4dsp_attr_group);
+
+	/* destruct workqueue for uevent */
+	cancel_delayed_work_sync(&uevent_issue_work);
+	destroy_workqueue(uevent_issue_queue);
+
 	hifi4dsp_stop_dsp();
+	if(hifi4dsp_load->reg_p33v) {
+		ret = regulator_disable(hifi4dsp_load->reg_p33v);
+	}
 	debugfs_remove_recursive(hifi4dsp_load->dent);
 	devm_kfree(&pdev->dev, hifi4dsp_load);
 }
