@@ -34,7 +34,7 @@
 #include <linux/spi/spi.h>
 #include <linux/spi/spidev.h>
 #include <linux/platform_data/spi-mt65xx.h>
-#include <hifi4dsp_spi/hifi4dsp_spi.h>
+#include <soc/mediatek/hifi4dsp/hifi4dsp_spi.h>
 
 //#define SARD_DBG
 
@@ -823,12 +823,12 @@ static int hifi4dsp_spi_probe(struct spi_device *spi)
 	 * Structure filled with mtk-spi crtical values.
 	 */
 	spi->bits_per_word = 8;
-	data->rx_mlsb = 0;
+	/*data->rx_mlsb = 0;
 	data->tx_mlsb = 0;
-	/*data->command_cnt = 1;
+	data->command_cnt = 1;
 	data->dummy_cnt = 0;
-	data->get_tick_dly = tick_delay;*/
-	data->cs_pol = 0;
+	data->get_tick_dly = tick_delay;
+	data->cs_pol = 0;*/
 	data->sample_sel = 0;
 	spi->controller_data = (void *)data;
 #if 0	
