@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2018 MediaTek Inc.
+ * Copyright 2022 Sony Corporation
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -33,7 +34,7 @@
 #include <linux/spi/spi.h>
 #include <linux/spi/spidev.h>
 #include <linux/platform_data/spi-mt65xx.h>
-#include <hifi4dsp_spi/hifi4dsp_spi.h>
+#include <soc/mediatek/hifi4dsp/hifi4dsp_spi.h>
 
 /*
  * uncomment for SARD debug logs
