@@ -3,6 +3,7 @@
  * sony_sw_pll.c : Sony software pll driver
  *
  *  Copyright 2020,2021 Sony Home Entertainment & Sound Products Inc.
+ *  Copyright 2022,2023 Sony Corporation
  */
 
 #include <linux/module.h>
@@ -133,6 +134,7 @@ struct sony_spdif_priv {
 	uint32_t  spdif_irq_stat;
 	uint32_t  spdif_event_log;
 	struct mutex sw_pll_workque_lock;
+	struct mutex enable_lock;
 };
 static struct sony_spdif_priv *sw_pll_priv;
 static SW_PLL_PARA Para;
@@ -187,7 +189,7 @@ static int sw_pll_set_mute_ack(uint8_t mute_ack)
 	return 0;
 }
 
-EXPORT_SYMBOL_GPL(sw_pll_set_mute_ack);
+//EXPORT_SYMBOL_GPL(sw_pll_set_mute_ack);
 
 int sw_pll_get_dsm_lock(void)
 {
@@ -242,6 +244,28 @@ int sw_pll_set_mode(uint8_t opt_i2s_mode)
 
 EXPORT_SYMBOL_GPL(sw_pll_set_mode);
 
+
+static int sw_pll_kthread(void *arg);
+int sw_pll_set_task_enable(uint8_t enable)
+{
+	mutex_lock(&sw_pll_priv->enable_lock);
+	if (enable && !sw_pll_priv->kthread_tsk) {
+		sw_pll_priv->kthread_tsk = kthread_run(sw_pll_kthread, sw_pll_priv, "sw_pll_kthread");
+		sw_pll_priv->is_enable = 1;
+	}
+	else if (!enable && sw_pll_priv->kthread_tsk) {
+		kthread_stop(sw_pll_priv->kthread_tsk);
+		sw_pll_priv->kthread_tsk = NULL;
+		sw_pll_priv->is_enable = 0;
+	}
+	mutex_unlock(&sw_pll_priv->enable_lock);
+
+	return 0;
+}
+
+EXPORT_SYMBOL_GPL(sw_pll_set_task_enable);
+
+
 static void PkAve1(uint32_t BCK, uint32_t* Pk, uint32_t* Cent, int Init)
 {
 	static uint32_t  fMin=0, fMax=0, Pending=0;
@@ -1618,6 +1642,11 @@ static void sw_pll_kthread_main(void)
 	}
 	mutex_unlock(&sw_pll_priv->fixed_lock);
 
+	//spdif:unlock,make swpll into unlock
+	if (sw_pll_priv->opt_i2s_mode == SW_PLL_MODE_OPT)
+		if (sw_pll_priv->spdif_dpll_lock != 1)
+			Para.RBCTR = 0;
+
 	SW_PLL_Core(&Para, &Stat);
 
 	if (Para.Fixed_fs == 0) {
@@ -1845,14 +1874,10 @@ static void sw_pll_detected_wq(struct work_struct *w)
 
 static ssize_t enable_store(struct kobject *kobj, struct kobj_attribute *attr, const char *buf, size_t count)
 {
-	sscanf(buf, "%d", &sw_pll_priv->is_enable);
-	if (sw_pll_priv->is_enable && !sw_pll_priv->kthread_tsk) {
-		sw_pll_priv->kthread_tsk = kthread_run(sw_pll_kthread, sw_pll_priv, "sw_pll_kthread");
-	}
-	else if (!sw_pll_priv->is_enable && sw_pll_priv->kthread_tsk) {
-		kthread_stop(sw_pll_priv->kthread_tsk);
-		sw_pll_priv->kthread_tsk = NULL;
-	}
+	int is_enable;
+
+	sscanf(buf, "%d", &is_enable);
+	sw_pll_set_task_enable(is_enable);
 
 	return count;
 }
@@ -1909,7 +1934,7 @@ static ssize_t data_show(struct kobject *kobj, struct kobj_attribute *attr, char
 			sw_pll_priv->logdata[i].spdif_srpc_lock, sw_pll_priv->logdata[i].opt_i2s_mode, sw_pll_priv->logdata[i].spdif_open_stat,
 			sw_pll_priv->logdata[i].spdif_irq_stat, sw_pll_priv->logdata[i].spdif_event_log);
 #endif
-		strncat(total_buf, tmp_buf, sizeof(tmp_buf));
+		strncat(total_buf, tmp_buf, sizeof(total_buf));
 	}
 	if (sw_pll_priv->log_read_counter + sw_pll_priv->log_read_size <= sw_pll_priv->log_counter)
 		sw_pll_priv->log_read_counter += sw_pll_priv->log_read_size;
@@ -2008,13 +2033,14 @@ static int sony_sw_pll_probe(struct platform_device *pdev)
 	sw_pll_priv->log_read_counter = 0;
 
 	sw_pll_priv->spdif_event_log = 0;
-	sw_pll_priv->sai6_base = ioremap_nocache(SAI6_BASE_ADDR,0x100);
-	sw_pll_priv->ccm_base = ioremap_nocache(ANATOP_BASE_ADDR,0x1c);
+	sw_pll_priv->sai6_base = ioremap(SAI6_BASE_ADDR,0x100);
+	sw_pll_priv->ccm_base = ioremap(ANATOP_BASE_ADDR,0x1c);
 	spin_lock_init(&sw_pll_priv->sw_pll_lock);
 	mutex_init(&sw_pll_priv->dsm_lock);
 	mutex_init(&sw_pll_priv->fixed_lock);
 	mutex_init(&sw_pll_priv->transport_lock);
 	mutex_init(&sw_pll_priv->sw_pll_workque_lock);
+	mutex_init(&sw_pll_priv->enable_lock);
 
 	sw_pll_priv->sw_pll_kobj = kobject_create_and_add("sw_pll", kernel_kobj);
 	if (!sw_pll_priv->sw_pll_kobj)
@@ -2081,18 +2107,12 @@ static int sony_sw_pll_probe(struct platform_device *pdev)
 	Para.Cond_Flag = 0;
 	Para.Fixed_fs = 1;
 
-	//start in boot
-	sw_pll_priv->kthread_tsk = kthread_run(sw_pll_kthread, sw_pll_priv, "sw_pll_kthread");
-	sw_pll_priv->is_enable = true;
 	return 0;
 }
 
 static int sony_sw_pll_remove(struct platform_device *pdev)
 {
-	if (sw_pll_priv->kthread_tsk != NULL) {
-		kthread_stop(sw_pll_priv->kthread_tsk);
-		sw_pll_priv->kthread_tsk = NULL;
-	}
+	sw_pll_set_task_enable(0);
 	if (sw_pll_priv->logdata != NULL) {
 		kfree(sw_pll_priv->logdata);
 		sw_pll_priv->logdata = NULL;
@@ -2103,28 +2123,6 @@ static int sony_sw_pll_remove(struct platform_device *pdev)
 	return 0;
 }
 
-static int sw_pll_suspend(struct device *dev)
-{
-	if (sw_pll_priv->kthread_tsk != NULL) {
-		kthread_stop(sw_pll_priv->kthread_tsk);
-		sw_pll_priv->kthread_tsk = NULL;
-	}
-	return 0;
-}
-
-static int sw_pll_resume(struct device *dev)
-{
-	if (sw_pll_priv->is_enable && !sw_pll_priv->kthread_tsk) {
-		sw_pll_priv->kthread_tsk = kthread_run(sw_pll_kthread, sw_pll_priv, "sw_pll_kthread");
-	}
-	return 0;
-}
-
-static const struct dev_pm_ops sw_pll_dev_pmops = {
-	.suspend = sw_pll_suspend,
-	.resume  = sw_pll_resume,
-};
-
 static const struct of_device_id sony_sw_pll_dt_ids[] = {
 	{
 	.compatible = "sony,sw_pll",
@@ -2139,7 +2137,6 @@ static struct platform_driver sony_sw_pll_driver = {
 		.name = DRV_NAME,
 		.owner = THIS_MODULE,
 		.of_match_table = sony_sw_pll_dt_ids,
-		.pm    = &sw_pll_dev_pmops,
 	},
 };
 
