@@ -2,7 +2,7 @@
 *
 *    The MIT License (MIT)
 *
-*    Copyright (c) 2014 - 2018 Vivante Corporation
+*    Copyright (c) 2014 - 2020 Vivante Corporation
 *
 *    Permission is hereby granted, free of charge, to any person obtaining a
 *    copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 *
 *    The GPL License (GPL)
 *
-*    Copyright (C) 2014 - 2018 Vivante Corporation
+*    Copyright (C) 2014 - 2020 Vivante Corporation
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
@@ -121,7 +121,7 @@ struct _gckHEAP
     gctUINT64                   timeStamp;
 #endif
 
-#if VIVANTE_PROFILER || gcmIS_DEBUG(gcdDEBUG_CODE)
+#if VIVANTE_PROFILER_SYSTEM_MEMORY || gcmIS_DEBUG(gcdDEBUG_CODE)
     /* Profile information. */
     gctUINT32                   allocCount;
     gctUINT64                   allocBytes;
@@ -198,7 +198,7 @@ _CompactKernelHeap(
     gctPOINTER p;
     gcskHEAP_PTR freeList = gcvNULL;
 
-    gcmkHEADER_ARG("Heap=0x%x", Heap);
+    gcmkHEADER_ARG("Heap=%p", Heap);
 
     /* Walk all the heaps. */
     for (heap = Heap->heap; heap != gcvNULL; heap = next)
@@ -284,7 +284,7 @@ _CompactKernelHeap(
                 heap->next->prev = heap->prev;
             }
 
-#if VIVANTE_PROFILER || gcmIS_DEBUG(gcdDEBUG_CODE)
+#if VIVANTE_PROFILER_SYSTEM_MEMORY || gcmIS_DEBUG(gcdDEBUG_CODE)
             /* Update profiling. */
             Heap->heapCount  -= 1;
             Heap->heapMemory -= heap->size + gcmSIZEOF(gcskHEAP);
@@ -310,7 +310,7 @@ _CompactKernelHeap(
 
             /* Free the heap. */
             gcmkTRACE_ZONE(gcvLEVEL_INFO, gcvZONE_HEAP,
-                           "Freeing heap 0x%x (%lu bytes)",
+                           "Freeing heap %p (%lu bytes)",
                            heap, heap->size + gcmSIZEOF(gcskHEAP));
             gcmkVERIFY_OK(gckOS_FreeMemory(Heap->os, heap));
         }
@@ -360,7 +360,7 @@ gckHEAP_Construct(
     gckHEAP heap = gcvNULL;
     gctPOINTER pointer = gcvNULL;
 
-    gcmkHEADER_ARG("Os=0x%x AllocationSize=%lu", Os, AllocationSize);
+    gcmkHEADER_ARG("Os=%p AllocationSize=%lu", Os, AllocationSize);
 
     /* Verify the arguments. */
     gcmkVERIFY_OBJECT(Os, gcvOBJ_OS);
@@ -382,7 +382,7 @@ gckHEAP_Construct(
     heap->timeStamp      = 0;
 #endif
 
-#if VIVANTE_PROFILER || gcmIS_DEBUG(gcdDEBUG_CODE)
+#if VIVANTE_PROFILER_SYSTEM_MEMORY || gcmIS_DEBUG(gcdDEBUG_CODE)
     /* Zero the counters. */
     heap->allocCount      = 0;
     heap->allocBytes      = 0;
@@ -401,7 +401,7 @@ gckHEAP_Construct(
     *Heap = heap;
 
     /* Success. */
-    gcmkFOOTER_ARG("*Heap=0x%x", *Heap);
+    gcmkFOOTER_ARG("*Heap=%p", *Heap);
     return gcvSTATUS_OK;
 
 OnError:
@@ -442,7 +442,7 @@ gckHEAP_Destroy(
     gctSIZE_T leaked = 0;
 #endif
 
-    gcmkHEADER_ARG("Heap=0x%x", Heap);
+    gcmkHEADER_ARG("Heap=%p", Heap);
 
     for (heap = Heap->heap; heap != gcvNULL; heap = Heap->heap)
     {
@@ -507,7 +507,7 @@ gckHEAP_Allocate(
     gcskNODE_PTR node, used, prevFree = gcvNULL;
     gctPOINTER memory = gcvNULL;
 
-    gcmkHEADER_ARG("Heap=0x%x Bytes=%lu", Heap, Bytes);
+    gcmkHEADER_ARG("Heap=%p Bytes=%lu", Heap, Bytes);
 
     /* Verify the arguments. */
     gcmkVERIFY_OBJECT(Heap, gcvOBJ_HEAP);
@@ -616,7 +616,7 @@ gckHEAP_Allocate(
                              &memory));
 
     gcmkTRACE_ZONE(gcvLEVEL_INFO, gcvZONE_HEAP,
-                   "Allocated heap 0x%x (%lu bytes)",
+                   "Allocated heap %p (%lu bytes)",
                    memory, Heap->allocationSize);
 
     /* Acquire the mutex. */
@@ -658,7 +658,7 @@ gckHEAP_Allocate(
     /* No previous free. */
     prevFree = gcvNULL;
 
-#if VIVANTE_PROFILER || gcmIS_DEBUG(gcdDEBUG_CODE)
+#if VIVANTE_PROFILER_SYSTEM_MEMORY || gcmIS_DEBUG(gcdDEBUG_CODE)
     /* Update profiling. */
     Heap->heapCount  += 1;
     Heap->heapMemory += Heap->allocationSize;
@@ -730,7 +730,7 @@ UseNode:
     used->timeStamp = ++Heap->timeStamp;
 #endif
 
-#if VIVANTE_PROFILER || gcmIS_DEBUG(gcdDEBUG_CODE)
+#if VIVANTE_PROFILER_SYSTEM_MEMORY || gcmIS_DEBUG(gcdDEBUG_CODE)
     /* Update profile counters. */
     Heap->allocCount      += 1;
     Heap->allocBytes      += bytes;
@@ -746,7 +746,7 @@ UseNode:
     *Memory = used + 1;
 
     /* Success. */
-    gcmkFOOTER_ARG("*Memory=0x%x", *Memory);
+    gcmkFOOTER_ARG("*Memory=%p", *Memory);
     return gcvSTATUS_OK;
 
 OnError:
@@ -795,7 +795,7 @@ gckHEAP_Free(
     gcskNODE_PTR node;
     gceSTATUS status;
 
-    gcmkHEADER_ARG("Heap=0x%x Memory=0x%x", Heap, Memory);
+    gcmkHEADER_ARG("Heap=%p Memory=%p", Heap, Memory);
 
     /* Verify the arguments. */
     gcmkVERIFY_OBJECT(Heap, gcvOBJ_HEAP);
@@ -811,7 +811,7 @@ gckHEAP_Free(
     /* Mark the node as freed. */
     node->next = gcvNULL;
 
-#if VIVANTE_PROFILER || gcmIS_DEBUG(gcdDEBUG_CODE)
+#if VIVANTE_PROFILER_SYSTEM_MEMORY || gcmIS_DEBUG(gcdDEBUG_CODE)
     /* Update profile counters. */
     Heap->allocBytes -= node->bytes;
 #endif
@@ -830,13 +830,13 @@ OnError:
     return status;
 }
 
-#if VIVANTE_PROFILER
+#if VIVANTE_PROFILER_SYSTEM_MEMORY
 gceSTATUS
 gckHEAP_ProfileStart(
     IN gckHEAP Heap
     )
 {
-    gcmkHEADER_ARG("Heap=0x%x", Heap);
+    gcmkHEADER_ARG("Heap=%p", Heap);
 
     /* Verify the arguments. */
     gcmkVERIFY_OBJECT(Heap, gcvOBJ_HEAP);
@@ -862,13 +862,13 @@ gckHEAP_ProfileEnd(
     IN gctCONST_STRING Title
     )
 {
-    gcmkHEADER_ARG("Heap=0x%x Title=0x%x", Heap, Title);
+    gcmkHEADER_ARG("Heap=%p Title=%p", Heap, Title);
 
     /* Verify the arguments. */
     gcmkVERIFY_OBJECT(Heap, gcvOBJ_HEAP);
     gcmkVERIFY_ARGUMENT(Title != gcvNULL);
 
-    gcmkPRINT("");
+    gcmkPRINT("\n");
     gcmkPRINT("=====[ HEAP - %s ]=====", Title);
     gcmkPRINT("Number of allocations           : %12u",   Heap->allocCount);
     gcmkPRINT("Number of bytes allocated       : %12llu", Heap->allocBytes);
@@ -884,9 +884,5 @@ gckHEAP_ProfileEnd(
     gcmkFOOTER_NO();
     return gcvSTATUS_OK;
 }
-#endif /* VIVANTE_PROFILER */
-
-/*******************************************************************************
-***** Test Code ****************************************************************
-*******************************************************************************/
+#endif /* VIVANTE_PROFILER_SYSTEM_MEMORY */
 
