@@ -2,7 +2,7 @@
 *
 *    The MIT License (MIT)
 *
-*    Copyright (c) 2014 - 2018 Vivante Corporation
+*    Copyright (c) 2014 - 2020 Vivante Corporation
 *
 *    Permission is hereby granted, free of charge, to any person obtaining a
 *    copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 *
 *    The GPL License (GPL)
 *
-*    Copyright (C) 2014 - 2018 Vivante Corporation
+*    Copyright (C) 2014 - 2020 Vivante Corporation
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
@@ -125,32 +125,6 @@ typedef struct _gcoBUFOBJ *             gcoBUFOBJ;
 #define gcdATTRIBUTE_COUNT              32
 #define gcdVERTEXARRAY_POOL_CAPACITY    32
 
-typedef enum _gcePROGRAM_STAGE
-{
-    gcvPROGRAM_STAGE_VERTEX   = 0x0,
-    gcvPROGRAM_STAGE_TCS      = 0x1,
-    gcvPROGRAM_STAGE_TES      = 0x2,
-    gcvPROGRAM_STAGE_GEOMETRY = 0x3,
-    gcvPROGRAM_STAGE_FRAGMENT = 0x4,
-    gcvPROGRAM_STAGE_COMPUTE  = 0x5,
-    gcvPROGRAM_STAGE_OPENCL   = 0x6,
-    gcvPROGRAM_STAGE_LAST
-}
-gcePROGRAM_STAGE;
-
-typedef enum _gcePROGRAM_STAGE_BIT
-{
-    gcvPROGRAM_STAGE_VERTEX_BIT   = 1 << gcvPROGRAM_STAGE_VERTEX,
-    gcvPROGRAM_STAGE_TCS_BIT      = 1 << gcvPROGRAM_STAGE_TCS,
-    gcvPROGRAM_STAGE_TES_BIT      = 1 << gcvPROGRAM_STAGE_TES,
-    gcvPROGRAM_STAGE_GEOMETRY_BIT = 1 << gcvPROGRAM_STAGE_GEOMETRY,
-    gcvPROGRAM_STAGE_FRAGMENT_BIT = 1 << gcvPROGRAM_STAGE_FRAGMENT,
-    gcvPROGRAM_STAGE_COMPUTE_BIT  = 1 << gcvPROGRAM_STAGE_COMPUTE,
-    gcvPROGRAM_STAGE_OPENCL_BIT   = 1 << gcvPROGRAM_STAGE_OPENCL,
-}
-gcePROGRAM_STAGE_BIT;
-
-
 #define gcvPORGRAM_STAGE_GPIPE (gcvPROGRAM_STAGE_VERTEX_BIT | \
                                 gcvPROGRAM_STAGE_TCS_BIT    | \
                                 gcvPROGRAM_STAGE_TES_BIT    | \
@@ -223,12 +197,6 @@ gcoHAL_QueryStreamCaps(
 
 /*----------------------------------------------------------------------------*/
 /*--------------------------------- gcoSURF 3D --------------------------------*/
-typedef enum _gceBLIT_FLAG
-{
-    gcvBLIT_FLAG_SKIP_DEPTH_WRITE   = 1 << 0,
-    gcvBLIT_FLAG_SKIP_STENCIL_WRITE = 1 << 1,
-} gceBLIT_FLAG;
-
 typedef struct _gcsSURF_BLIT_ARGS
 {
     gcoSURF     srcSurface;
@@ -247,22 +215,6 @@ typedef struct _gcsSURF_BLIT_ARGS
 }
 gcsSURF_BLIT_ARGS;
 
-
-
-
-/* Clear flags. */
-typedef enum _gceCLEAR
-{
-    gcvCLEAR_COLOR              = 0x1,
-    gcvCLEAR_DEPTH              = 0x2,
-    gcvCLEAR_STENCIL            = 0x4,
-    gcvCLEAR_HZ                 = 0x8,
-    gcvCLEAR_WITH_GPU_ONLY      = 0x100,
-    gcvCLEAR_WITH_CPU_ONLY      = 0x200,
-    gcvCLEAR_MULTI_SLICES       = 0x400,
-}
-gceCLEAR;
-
 typedef struct _gcsSURF_CLEAR_ARGS
 {
     /*
@@ -305,33 +257,6 @@ typedef struct _gscSURF_BLITDRAW_BLIT
     gcsRECT  scissor;
 }gscSURF_BLITDRAW_BLIT;
 
-
-typedef enum _gceBLITDRAW_TYPE
-{
-    gcvBLITDRAW_CLEAR = 0,
-    gcvBLITDRAW_BLIT  = 1,
-
-    /* last number, not a real type */
-    gcvBLITDRAW_NUM_TYPE
- }
-gceBLITDRAW_TYPE;
-
-typedef enum _gceSPLIT_DRAW_TYPE
-{
-    gcvSPLIT_DRAW_UNKNOWN      = 0x0,
-    gcvSPLIT_DRAW_1,
-    gcvSPLIT_DRAW_2,
-    gcvSPLIT_DRAW_3,
-    gcvSPLIT_DRAW_4,
-    gcvSPLIT_DRAW_XFB,
-    gcvSPLIT_DRAW_INDEX_FETCH,
-    gcvSPLIT_DRAW_TCS,
-    gcvSPLIT_DRAW_WIDE_LINE,
-    gcvSPLIT_DRAW_STIPPLE,
-    gcvSPLIT_DRAW_LAST
-}
-gceSPLIT_DRAW_TYPE;
-
 typedef gceSTATUS (* gctSPLIT_DRAW_FUNC_PTR)(
     IN gctPOINTER gc,
     IN gctPOINTER instantDraw,
@@ -377,7 +302,7 @@ typedef struct _gscSURF_BLITDRAW_ARGS
        struct _gscSURF_BLITDRAW_ARG_v1
        {
             /* Whether it's clear or blit operation, can be extended. */
-            gceBLITDRAW_TYPE type;
+            gceBLIT_TYPE type;
 
             union _gscSURF_BLITDRAW_UNION
             {
@@ -471,7 +396,8 @@ depr_gcoSURF_ResolveRect(
 gceSTATUS
 gcoSURF_Resample(
     IN gcoSURF SrcSurf,
-    IN gcoSURF DstSurf
+    IN gcoSURF DstSurf,
+    IN gctBOOL sRGBDecode
     );
 
 /* Resolve rectangular area of a surface. */
@@ -573,6 +499,19 @@ gcoSURF_DrawBlit(
     gscSURF_BLITDRAW_BLIT *Args
     );
 
+gceSTATUS
+gcoSURF_DrawBlitDepth(
+    gcsSURF_VIEW *SrcView,
+    gcsSURF_VIEW *DstView,
+    gscSURF_BLITDRAW_BLIT *Args
+);
+
+gceSTATUS
+gcoSURF_ComputeBlit(
+    gcsSURF_VIEW* SrcView,
+    gcsSURF_VIEW* DstView,
+    gscSURF_BLITDRAW_BLIT* Args
+);
 
 /******************************************************************************\
 ******************************** gcoINDEX Object *******************************
@@ -682,7 +621,7 @@ gceSTATUS
 gcoINDEX_GetIndexRange(
     IN gcoINDEX Index,
     IN gceINDEX_TYPE Type,
-    IN gctUINT32 Offset,
+    IN gctSIZE_T Offset,
     IN gctUINT32 Count,
     OUT gctUINT32 * MinimumIndex,
     OUT gctUINT32 * MaximumIndex
@@ -702,14 +641,6 @@ gcoCLHardware_Construct(void);
 ********************************** gco3D Object *********************************
 \******************************************************************************/
 
-/* Blending targets. */
-typedef enum _gceBLEND_UNIT
-{
-    gcvBLEND_SOURCE,
-    gcvBLEND_TARGET,
-}
-gceBLEND_UNIT;
-
 /* Construct a new gco3D object. */
 gceSTATUS
 gco3D_Construct(
@@ -1487,59 +1418,13 @@ gco3D_SetLogicOp(
     IN gctUINT8 Rop
     );
 
-typedef enum _gceXfbCmd
-{
-    gcvXFBCMD_BEGIN           = 0,
-    gcvXFBCMD_PAUSE           = 1,
-    gcvXFBCMD_RESUME          = 2,
-    gcvXFBCMD_END             = 3,
-    gcvXFBCMD_PAUSE_INCOMMIT  = 4,
-    gcvXFBCMD_RESUME_INCOMMIT = 5,
-    gcvXFBCMD_INVALID         = 6,
-}
-gceXfbCmd;
-
-typedef enum _gceXfbStatus
-{
-    gcvXFB_Disabled = 0,
-    gcvXFB_Paused,
-    gcvXFB_Enabled,
-}
-gceXfbStatus;
-
-typedef enum _gceQueryStatus
-{
-    gcvQUERY_Disabled = 0,
-    gcvQUERY_Paused   = 1,
-    gcvQUERY_Enabled  = 2,
-}
-gceQueryStatus;
-
-typedef enum _gceQueryCmd
-{
-    gcvQUERYCMD_BEGIN   = 0,
-    gcvQUERYCMD_PAUSE   = 1,
-    gcvQUERYCMD_RESUME  = 2,
-    gcvQUERYCMD_END     = 3,
-    gcvQUERYCMD_INVALID = 4,
-}
-gceQueryCmd;
-
-typedef enum _gceQueryType
-{
-    gcvQUERY_OCCLUSION = 0,
-    gcvQUERY_XFB_WRITTEN = 1,
-    gcvQUERY_PRIM_GENERATED = 2,
-    gcvQUERY_MAX_NUM = 3,
-}
-gceQueryType;
-
 gceSTATUS
 gco3D_SetQuery(
     IN gco3D Engine,
     IN gctUINT32 QueryHeader,
     IN gceQueryType Type,
-    IN gctBOOL Enable
+    IN gctBOOL Enable,
+    IN gctUINT32 Index
     );
 
 gceSTATUS
@@ -1549,6 +1434,7 @@ gco3D_GetQuery(
     IN gcsSURF_NODE_PTR Node,
     IN gctUINT32    Size,
     IN gctPOINTER   Locked,
+    IN gctUINT32    IndexedId,
     OUT gctINT32 * Index
     );
 
@@ -1645,10 +1531,11 @@ typedef struct _gcsTHREAD_WALKER_INFO
 
     gctUINT32   threadAllocation;
     gctBOOL     barrierUsed;
-
+    gctUINT32   memoryAccessFlag; /* same as gceMEMORY_ACCESS_FLAG */
     gctBOOL     indirect;
     gctUINT32   groupNumberUniformIdx;
     gctUINT32   baseAddress;
+    gctBOOL     bDual16;
 }
 gcsTHREAD_WALKER_INFO;
 
@@ -1982,19 +1869,6 @@ gco3D_SetAlphaTextureFunction(
 ******************************* gcoTEXTURE Object *******************************
 \******************************************************************************/
 
-/* Cube faces. */
-typedef enum _gceTEXTURE_FACE
-{
-    gcvFACE_NONE = 0,
-    gcvFACE_POSITIVE_X,
-    gcvFACE_NEGATIVE_X,
-    gcvFACE_POSITIVE_Y,
-    gcvFACE_NEGATIVE_Y,
-    gcvFACE_POSITIVE_Z,
-    gcvFACE_NEGATIVE_Z,
-}
-gceTEXTURE_FACE;
-
 typedef struct _gcsTEXTURE
 {
     /* Addressing modes. */
@@ -2026,12 +1900,18 @@ typedef struct _gcsTEXTURE
     gceTEXTURE_COMPARE_MODE     compareMode;
     gceCOMPARE                  compareFunc;
 
+    gceTEXTURE_DS_TEX_MODE      dsTextureMode;
+
     gceTEXTURE_DS_MODE          dsMode;
 
     /* sRGB decode */
     gceTEXTURE_SRGBDECODE       sRGB;
 
     gcuVALUE                    borderColor[4];
+    gctBOOL                     descDirty;
+
+    /* texture stage */
+    gctINT                      stage;
 }
 gcsTEXTURE, * gcsTEXTURE_PTR;
 
@@ -2248,6 +2128,12 @@ gcoTEXTURE_Disable(
     IN gctBOOL DefaultInteger
     );
 
+gceSTATUS
+gcoTEXTURE_Clear(
+    IN gcoTEXTURE Texture,
+    IN gctINT MipMap
+    );
+
 gceSTATUS
 gcoTEXTURE_Flush(
     IN gcoTEXTURE Texture
@@ -2391,41 +2277,6 @@ gcoTEXTURE_GenerateMipMap(
 ******************************* gcoSTREAM Object ******************************
 \******************************************************************************/
 
-typedef enum _gceVERTEX_FORMAT
-{
-    gcvVERTEX_BYTE,
-    gcvVERTEX_UNSIGNED_BYTE,
-    gcvVERTEX_SHORT,
-    gcvVERTEX_UNSIGNED_SHORT,
-    gcvVERTEX_INT,
-    gcvVERTEX_UNSIGNED_INT,
-    gcvVERTEX_FIXED,
-    gcvVERTEX_HALF,
-    gcvVERTEX_FLOAT,
-    gcvVERTEX_UNSIGNED_INT_10_10_10_2,
-    gcvVERTEX_INT_10_10_10_2,
-    gcvVERTEX_UNSIGNED_INT_2_10_10_10_REV,
-    gcvVERTEX_INT_2_10_10_10_REV,
-    /* integer format */
-    gcvVERTEX_INT8,
-    gcvVERTEX_INT16,
-    gcvVERTEX_INT32,
-}
-gceVERTEX_FORMAT;
-
-/* What the SW converting scheme to create temp attrib */
-typedef enum _gceATTRIB_SCHEME
-{
-    gcvATTRIB_SCHEME_KEEP = 0,
-    gcvATTRIB_SCHEME_2_10_10_10_REV_TO_FLOAT,
-    gcvATTRIB_SCHEME_BYTE_TO_IVEC4,
-    gcvATTRIB_SCHEME_SHORT_TO_IVEC4,
-    gcvATTRIB_SCHEME_INT_TO_IVEC4,
-    gcvATTRIB_SCHEME_UBYTE_TO_UVEC4,
-    gcvATTRIB_SCHEME_USHORT_TO_UVEC4,
-    gcvATTRIB_SCHEME_UINT_TO_UVEC4,
-} gceATTRIB_SCHEME;
-
 gceSTATUS
 gcoSTREAM_Construct(
     IN gcoHAL Hal,
@@ -2710,6 +2561,7 @@ typedef struct _gcsVERTEXARRAY_INDEX_INFO
     gctSIZE_T        count;
     gceINDEX_TYPE    indexType;
     gctPOINTER       indexMemory;
+    gctUINT          restartElement;
 
     union _gcsVERTEXARRAY_INDEX_INFO_UNION
     {
@@ -2807,32 +2659,6 @@ gcoHAL_DumpGPUProfile(
 /******************************************************************************
 **********************gcoBUFOBJ object*****************************************
 *******************************************************************************/
-typedef enum _gceBUFOBJ_TYPE
-{
-    gcvBUFOBJ_TYPE_ARRAY_BUFFER = 1,
-    gcvBUFOBJ_TYPE_ELEMENT_ARRAY_BUFFER  = 2,
-    gcvBUFOBJ_TYPE_GENERIC_BUFFER = 100
-
-} gceBUFOBJ_TYPE;
-
-typedef enum _gceBUFOBJ_USAGE
-{
-    gcvBUFOBJ_USAGE_STREAM_DRAW = 1,
-    gcvBUFOBJ_USAGE_STREAM_READ,
-    gcvBUFOBJ_USAGE_STREAM_COPY,
-    gcvBUFOBJ_USAGE_STATIC_DRAW,
-    gcvBUFOBJ_USAGE_STATIC_READ,
-    gcvBUFOBJ_USAGE_STATIC_COPY,
-    gcvBUFOBJ_USAGE_DYNAMIC_DRAW,
-    gcvBUFOBJ_USAGE_DYNAMIC_READ,
-    gcvBUFOBJ_USAGE_DYNAMIC_COPY,
-
-    /* special patch for optimaize performance,
-    ** no fence and duplicate stream to ensure data correct
-    */
-    gcvBUFOBJ_USAGE_DISABLE_FENCE_DYNAMIC_STREAM = 256
-} gceBUFOBJ_USAGE;
-
 /* Construct a new gcoBUFOBJ object. */
 gceSTATUS
 gcoBUFOBJ_Construct(
@@ -2890,8 +2716,9 @@ gceSTATUS
 gcoBUFOBJ_IndexBind (
     IN gcoBUFOBJ Index,
     IN gceINDEX_TYPE Type,
-    IN gctUINT32 Offset,
-    IN gctSIZE_T Count
+    IN gctSIZE_T Offset,
+    IN gctSIZE_T Count,
+    IN gctUINT   RestartElement
     );
 
 /* Find min and max index for the index buffer */
@@ -2899,7 +2726,7 @@ gceSTATUS
 gcoBUFOBJ_IndexGetRange(
     IN gcoBUFOBJ Index,
     IN gceINDEX_TYPE Type,
-    IN gctUINT32 Offset,
+    IN gctSIZE_T Offset,
     IN gctUINT32 Count,
     OUT gctUINT32 * MinimumIndex,
     OUT gctUINT32 * MaximumIndex
@@ -2908,6 +2735,12 @@ gcoBUFOBJ_IndexGetRange(
 /*  Sets a buffer object as dirty */
 gceSTATUS
 gcoBUFOBJ_SetDirty(
+    IN gcoBUFOBJ BufObj,
+    IN gctBOOL Dirty
+    );
+
+gctBOOL
+gcoBUFOBJ_IsDirty(
     IN gcoBUFOBJ BufObj
     );
 
@@ -2956,8 +2789,9 @@ gcoBUFOBJ_ReAllocBufNode(
 
 /* Handle GPU cache operations */
 gceSTATUS
-gcoBUFOBJ_GPUCacheOperation(
-    gcoBUFOBJ BufObj
+gcoBUFOBJ_SetCPUWrite(
+    gcoBUFOBJ BufObj,
+    gctBOOL Value
     );
 
 /* Dump buffer. */
