@@ -2,7 +2,7 @@
 *
 *    The MIT License (MIT)
 *
-*    Copyright (c) 2014 - 2018 Vivante Corporation
+*    Copyright (c) 2014 - 2020 Vivante Corporation
 *
 *    Permission is hereby granted, free of charge, to any person obtaining a
 *    copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 *
 *    The GPL License (GPL)
 *
-*    Copyright (C) 2014 - 2018 Vivante Corporation
+*    Copyright (C) 2014 - 2020 Vivante Corporation
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
@@ -61,7 +61,9 @@
 #include <asm/atomic.h>
 #include <linux/dma-mapping.h>
 #include <linux/slab.h>
-
+#if defined(CONFIG_X86) && (LINUX_VERSION_CODE >= KERNEL_VERSION(4,12,0))
+#include <asm/set_memory.h>
+#endif
 #include "gc_hal_kernel_platform.h"
 
 #define _GC_OBJ_ZONE    gcvZONE_OS
@@ -101,6 +103,10 @@ struct gfp_mdl_priv
         {
             /* Pointer to a array of pointers to page. */
             struct page **nonContiguousPages;
+
+            struct page **Pages1M;
+            int numPages1M;
+            int *isExact;
             struct sg_table sgt;
         };
     };
@@ -166,12 +172,12 @@ _GFPAllocatorDebugfsCleanup(
 static void
 _NonContiguousFree(
     IN struct page ** Pages,
-    IN gctUINT32 NumPages
+    IN gctSIZE_T NumPages
     )
 {
-    gctINT i;
+    gctSIZE_T i;
 
-    gcmkHEADER_ARG("Pages=%p, NumPages=%u", Pages, NumPages);
+    gcmkHEADER_ARG("Pages=%p, NumPages=%zx", Pages, NumPages);
 
     gcmkASSERT(Pages != gcvNULL);
 
@@ -192,26 +198,29 @@ _NonContiguousFree(
     gcmkFOOTER_NO();
 }
 
-static struct page **
+static gceSTATUS
 _NonContiguousAlloc(
-    IN gctUINT32 NumPages,
+    IN struct gfp_mdl_priv *MdlPriv,
+    IN gctSIZE_T NumPages,
     IN gctUINT32 Gfp
     )
 {
     struct page ** pages;
     struct page *p;
-    gctINT i, size;
+    gctSIZE_T i, size;
 
-    gcmkHEADER_ARG("NumPages=%u", NumPages);
+    gcmkHEADER_ARG("NumPages=%zx", NumPages);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 32)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)
+    if (NumPages > totalram_pages())
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 32)
     if (NumPages > totalram_pages)
 #else
     if (NumPages > num_physpages)
 #endif
     {
         gcmkFOOTER_NO();
-        return gcvNULL;
+        return gcvSTATUS_INVALID_ARGUMENT;
     }
 
     size = NumPages * sizeof(struct page *);
@@ -225,7 +234,7 @@ _NonContiguousAlloc(
         if (!pages)
         {
             gcmkFOOTER_NO();
-            return gcvNULL;
+            return gcvSTATUS_OUT_OF_MEMORY;
         }
     }
 
@@ -237,7 +246,7 @@ _NonContiguousAlloc(
         {
             _NonContiguousFree(pages, i);
             gcmkFOOTER_NO();
-            return gcvNULL;
+            return gcvSTATUS_OUT_OF_MEMORY;
         }
 
 #if gcdDISCRETE_PAGES
@@ -258,7 +267,7 @@ _NonContiguousAlloc(
                 {
                     _NonContiguousFree(pages, i);
                     gcmkFOOTER_NO();
-                    return gcvNULL;
+                    return gcvSTATUS_OUT_OF_MEMORY;
                 }
             }
         }
@@ -267,8 +276,188 @@ _NonContiguousAlloc(
         pages[i] = p;
     }
 
-    gcmkFOOTER_ARG("pages=0x%X", pages);
-    return pages;
+    MdlPriv->nonContiguousPages = pages;
+
+    gcmkFOOTER_ARG("pages=%p", pages);
+    return gcvSTATUS_OK;
+}
+
+static void
+_NonContiguous1MPagesFree(
+    IN struct gfp_mdl_priv *MdlPriv,
+    IN gctUINT32 NumPages1M
+    )
+{
+    gctINT i;
+
+    if (MdlPriv->Pages1M && MdlPriv->isExact)
+    {
+        for (i = 0; i < NumPages1M && MdlPriv->Pages1M[i]; i++)
+        {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+            if (MdlPriv->isExact[i] == gcvTRUE)
+            {
+                free_pages_exact(page_address(MdlPriv->Pages1M[i]), gcd1M_PAGE_SIZE);
+            }
+            else
+#endif
+            {
+                __free_pages(MdlPriv->Pages1M[i], get_order(gcd1M_PAGE_SIZE));
+            }
+        }
+    }
+
+    if (MdlPriv->Pages1M)
+    {
+        if (is_vmalloc_addr(MdlPriv->Pages1M))
+        {
+            vfree(MdlPriv->Pages1M);
+        }
+        else
+        {
+            kfree(MdlPriv->Pages1M);
+        }
+        MdlPriv->Pages1M = gcvNULL;
+    }
+
+    if (MdlPriv->isExact)
+    {
+         if (is_vmalloc_addr(MdlPriv->isExact))
+        {
+            vfree(MdlPriv->isExact);
+        }
+        else
+        {
+            kfree(MdlPriv->isExact);
+        }
+    }
+
+    if (MdlPriv->nonContiguousPages)
+    {
+        if (is_vmalloc_addr(MdlPriv->nonContiguousPages))
+        {
+            vfree(MdlPriv->nonContiguousPages);
+        }
+        else
+        {
+            kfree(MdlPriv->nonContiguousPages);
+        }
+        MdlPriv->nonContiguousPages = gcvNULL;
+    }
+}
+
+static gceSTATUS
+_NonContiguous1MPagesAlloc(
+    IN struct gfp_mdl_priv *MdlPriv,
+    IN gctSIZE_T *NumPages,
+    IN gctUINT32 Gfp
+    )
+{
+    gceSTATUS status;
+    size_t numPages1M, num, size;
+    struct page **pages;
+    struct page *page;
+    void *addr = NULL;
+    gctINT i, j;
+
+    MdlPriv->numPages1M = 0;
+
+    numPages1M = ((*NumPages << PAGE_SHIFT) + (gcd1M_PAGE_SIZE - 1)) >> gcd1M_PAGE_SHIFT;
+
+    *NumPages = (numPages1M << gcd1M_PAGE_SHIFT) >> PAGE_SHIFT;
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)
+        if (*NumPages > totalram_pages())
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 32)
+        if (*NumPages > totalram_pages)
+#else
+        if (*NumPages > num_physpages)
+#endif
+    {
+        return gcvSTATUS_INVALID_ARGUMENT;
+    }
+
+    num = gcd1M_PAGE_SIZE / PAGE_SIZE;
+
+    size = numPages1M * sizeof(struct page *);
+    MdlPriv->Pages1M = kmalloc(size, GFP_KERNEL | gcdNOWARN);
+    if (!MdlPriv->Pages1M)
+    {
+        MdlPriv->Pages1M = vmalloc(size);
+
+        if (!MdlPriv->Pages1M)
+        {
+            gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
+        }
+    }
+
+    size = numPages1M * sizeof(int);
+    MdlPriv->isExact = kmalloc(size, GFP_KERNEL | gcdNOWARN);
+    if (!MdlPriv->isExact)
+    {
+        MdlPriv->isExact = vmalloc(size);
+        if (!MdlPriv->isExact)
+        {
+            gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
+        }
+    }
+    memset(MdlPriv->isExact, 0, size);
+
+    size = *NumPages * sizeof(struct page *);
+    pages = kmalloc(size, GFP_KERNEL | gcdNOWARN);
+    if (!pages)
+    {
+        pages = vmalloc(size);
+        if (!pages)
+        {
+            gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
+        }
+    }
+    MdlPriv->nonContiguousPages = pages;
+
+    for (i = 0; i < numPages1M; i++)
+    {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+        addr = alloc_pages_exact(gcd1M_PAGE_SIZE, (Gfp & ~__GFP_HIGHMEM) | __GFP_NORETRY);
+
+        MdlPriv->Pages1M[i] = addr ? virt_to_page(addr) : gcvNULL;
+        if (MdlPriv->Pages1M[i])
+        {
+            MdlPriv->isExact[i] = gcvTRUE;
+        }
+#endif
+
+        if (MdlPriv->Pages1M[i] == gcvNULL)
+        {
+            int order = get_order(gcd1M_PAGE_SIZE);
+
+            if (order >= MAX_ORDER)
+            {
+                gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
+            }
+
+            MdlPriv->Pages1M[i] = alloc_pages(Gfp, order);
+        }
+
+        if (MdlPriv->Pages1M[i] == gcvNULL)
+        {
+            gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
+        }
+
+        MdlPriv->numPages1M += 1;
+
+        for (j = 0; j < num; j++)
+        {
+            page = nth_page(MdlPriv->Pages1M[i], j);
+            pages[i * num + j] = page;
+        }
+    }
+
+    return gcvSTATUS_OK;
+OnError:
+    _NonContiguous1MPagesFree(MdlPriv, MdlPriv->numPages1M);
+
+    return status;
 }
 
 /***************************************************************************\
@@ -283,9 +472,14 @@ _GFPAlloc(
     )
 {
     gceSTATUS status;
-    gctUINT i;
+    gctSIZE_T i = 0;
     gctBOOL contiguous = Flags & gcvALLOC_FLAG_CONTIGUOUS;
-    u32 gfp = (contiguous ? (__GFP_HIGH | __GFP_ATOMIC) : GFP_KERNEL) | __GFP_HIGHMEM | gcdNOWARN;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 4, 0)
+    u32 normal_gfp = __GFP_HIGH | __GFP_ATOMIC | __GFP_NORETRY | gcdNOWARN;
+#else
+    u32 normal_gfp = __GFP_HIGH | __GFP_NORETRY | gcdNOWARN;
+#endif
+    u32 gfp = (contiguous ? normal_gfp : GFP_KERNEL) | __GFP_HIGHMEM | gcdNOWARN;
 
     struct gfp_alloc *priv = (struct gfp_alloc *)Allocator->privateData;
     struct gfp_mdl_priv *mdlPriv = gcvNULL;
@@ -296,12 +490,22 @@ _GFPAlloc(
     gcmkHEADER_ARG("Allocator=%p Mdl=%p NumPages=%zu Flags=0x%x", Allocator, Mdl, NumPages, Flags);
 
 #ifdef gcdSYS_FREE_MEMORY_LIMIT
-    if (Flags & gcvALLOC_FLAG_MEMLIMIT)
+    if ((Flags & gcvALLOC_FLAG_MEMLIMIT) && !contiguous)
     {
+        long freeram = 0;
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 14, 0)
+        freeram = global_zone_page_state(NR_FREE_PAGES);
+#ifdef CONFIG_CMA
+        freeram -= global_zone_page_state(NR_FREE_CMA_PAGES);
+#endif
+#else
         struct sysinfo temsysinfo;
         si_meminfo(&temsysinfo);
+        freeram = temsysinfo.freeram;
+#endif
 
-        if ((temsysinfo.freeram < NumPages) || ((temsysinfo.freeram-NumPages) < gcdSYS_FREE_MEMORY_LIMIT))
+        if ((freeram < NumPages) || ((freeram - NumPages) < gcdSYS_FREE_MEMORY_LIMIT))
         {
             gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
         }
@@ -332,6 +536,15 @@ _GFPAlloc(
 
 #endif
 
+    if ((Flags & gcvALLOC_FLAG_NON_CONTIGUOUS) && (Flags & gcvALLOC_FLAG_1M_PAGES))
+    {
+        Mdl->pageUnit1M = gcvTRUE;
+    }
+    else
+    {
+        Mdl->pageUnit1M = gcvFALSE;
+    }
+
     if (contiguous)
     {
         size_t bytes = NumPages << PAGE_SHIFT;
@@ -360,7 +573,6 @@ _GFPAlloc(
             }
 
             mdlPriv->contiguousPages = alloc_pages(gfp, order);
-
         }
 
         if (mdlPriv->contiguousPages == gcvNULL)
@@ -372,7 +584,7 @@ _GFPAlloc(
                 mdlPriv->contiguousPages, 0, NumPages * PAGE_SIZE,
                 DMA_FROM_DEVICE);
 
-        if (!mdlPriv->dma_addr)
+        if (dma_mapping_error(galcore_device, mdlPriv->dma_addr))
         {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
             if (mdlPriv->exact)
@@ -400,18 +612,23 @@ _GFPAlloc(
     }
     else
     {
-        mdlPriv->nonContiguousPages = _NonContiguousAlloc(NumPages, gfp);
-
-        if (mdlPriv->nonContiguousPages == gcvNULL)
+        if (Mdl->pageUnit1M)
         {
-            gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
+            gcmkONERROR(_NonContiguous1MPagesAlloc(mdlPriv, &NumPages, gfp));
         }
+        else
+        {
+            status = _NonContiguousAlloc(mdlPriv, NumPages, normal_gfp);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (3,6,0) \
-    && (defined(ARCH_HAS_SG_CHAIN) || defined(CONFIG_ARCH_HAS_SG_CHAIN))
+            if (gcmIS_ERROR(status))
+            {
+                gcmkONERROR(_NonContiguousAlloc(mdlPriv, NumPages, gfp));
+            }
+        }
+#if gcdUSE_Linux_SG_TABLE_API
         result = sg_alloc_table_from_pages(&mdlPriv->sgt,
                     mdlPriv->nonContiguousPages, NumPages, 0,
-                    NumPages << PAGE_SHIFT, GFP_KERNEL);
+                    NumPages << PAGE_SHIFT, normal_gfp);
 
 #else
         result = alloc_sg_list_from_pages(&mdlPriv->sgt.sgl,
@@ -422,7 +639,15 @@ _GFPAlloc(
 #endif
         if (result < 0)
         {
-            _NonContiguousFree(mdlPriv->nonContiguousPages, NumPages);
+            if (Mdl->pageUnit1M)
+            {
+                _NonContiguous1MPagesFree(mdlPriv, mdlPriv->numPages1M);
+            }
+            else
+            {
+                _NonContiguousFree(mdlPriv->nonContiguousPages, NumPages);
+            }
+
             gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
         }
 
@@ -431,10 +656,16 @@ _GFPAlloc(
 
         if (result != mdlPriv->sgt.nents)
         {
-            _NonContiguousFree(mdlPriv->nonContiguousPages, NumPages);
+            if (Mdl->pageUnit1M)
+            {
+                _NonContiguous1MPagesFree(mdlPriv, mdlPriv->numPages1M);
+            }
+            else
+            {
+                _NonContiguousFree(mdlPriv->nonContiguousPages, NumPages);
+            }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (3,6,0) \
-    && (defined (ARCH_HAS_SG_CHAIN) || defined (CONFIG_ARCH_HAS_SG_CHAIN))
+#if gcdUSE_Linux_SG_TABLE_API
             sg_free_table(&mdlPriv->sgt);
 #else
             kfree(mdlPriv->sgt.sgl);
@@ -486,6 +717,7 @@ _GFPAlloc(
     atomic_add(high, &priv->high);
 
     Mdl->priv = mdlPriv;
+    Mdl->numPages = NumPages;
 
     gcmkFOOTER_NO();
     return gcvSTATUS_OK;
@@ -517,13 +749,13 @@ _GFPGetSGT(
 
     gceSTATUS status = gcvSTATUS_OK;
     gctSIZE_T offset = Offset & ~PAGE_MASK; /* Offset to the first page */
-    gctINT skipPages = Offset >> PAGE_SHIFT;     /* skipped pages */
-    gctINT numPages = (PAGE_ALIGN(Offset + Bytes) >> PAGE_SHIFT) - skipPages;
-    gctINT i;
+    gctSIZE_T skipPages = Offset >> PAGE_SHIFT;     /* skipped pages */
+    gctSIZE_T numPages = (PAGE_ALIGN(Offset + Bytes) >> PAGE_SHIFT) - skipPages;
+    gctSIZE_T i;
 
     gcmkASSERT(Offset + Bytes <= Mdl->numPages << PAGE_SHIFT);
 
-    if (Mdl->contiguous)
+    if (mdlPriv->contiguous)
     {
         pages = tmpPages = kmalloc(sizeof(struct page*) * numPages, GFP_KERNEL | gcdNOWARN);
         if (!pages)
@@ -577,25 +809,24 @@ _GFPFree(
     IN OUT PLINUX_MDL Mdl
     )
 {
-    gctINT i;
+    gctSIZE_T i;
     struct page * page;
     struct gfp_alloc *priv = (struct gfp_alloc *)Allocator->privateData;
     struct gfp_mdl_priv *mdlPriv = Mdl->priv;
     int low  = 0;
     int high = 0;
 
-    if (Mdl->contiguous)
+    if (mdlPriv->contiguous)
     {
         dma_unmap_page(galcore_device, mdlPriv->dma_addr,
-                Mdl->numPages << PAGE_SHIFT, DMA_TO_DEVICE);
+                Mdl->numPages << PAGE_SHIFT, DMA_FROM_DEVICE);
     }
     else
     {
         dma_unmap_sg(galcore_device, mdlPriv->sgt.sgl, mdlPriv->sgt.nents,
-                DMA_TO_DEVICE);
+                DMA_FROM_DEVICE);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (3,6,0) \
-    && (defined (ARCH_HAS_SG_CHAIN) || defined (CONFIG_ARCH_HAS_SG_CHAIN))
+#if gcdUSE_Linux_SG_TABLE_API
         sg_free_table(&mdlPriv->sgt);
 #else
         kfree(mdlPriv->sgt.sgl);
@@ -604,7 +835,7 @@ _GFPFree(
 
     for (i = 0; i < Mdl->numPages; i++)
     {
-        if (Mdl->contiguous)
+        if (mdlPriv->contiguous)
         {
             page = nth_page(mdlPriv->contiguousPages, i);
         }
@@ -628,7 +859,7 @@ _GFPFree(
     atomic_sub(low, &priv->low);
     atomic_sub(high, &priv->high);
 
-    if (Mdl->contiguous)
+    if (mdlPriv->contiguous)
     {
 #if defined(CONFIG_X86)
         if (!PageHighMem(mdlPriv->contiguousPages))
@@ -654,7 +885,14 @@ _GFPFree(
         set_pages_array_wb(mdlPriv->nonContiguousPages, Mdl->numPages);
 #endif
 
-        _NonContiguousFree(mdlPriv->nonContiguousPages, Mdl->numPages);
+        if (Mdl->pageUnit1M)
+        {
+            _NonContiguous1MPagesFree(mdlPriv, mdlPriv->numPages1M);
+        }
+        else
+        {
+            _NonContiguousFree(mdlPriv->nonContiguousPages, Mdl->numPages);
+        }
     }
 
     kfree(Mdl->priv);
@@ -716,7 +954,7 @@ _GFPMmap(
     }
     else
     {
-        gctUINT i;
+        gctSIZE_T i;
         unsigned long start = vma->vm_start;
 
         for (i = 0; i < numPages; ++i)
@@ -773,7 +1011,7 @@ _GFPUnmapUser(
                 );
     }
 #else
-    down_write(&current->mm->mmap_sem);
+    down_write(&current_mm_mmap_sem);
     if (do_munmap(current->mm, (unsigned long)MdlMap->vmaAddr, Size) < 0)
     {
         gcmkTRACE_ZONE(
@@ -782,7 +1020,7 @@ _GFPUnmapUser(
                 __FUNCTION__, __LINE__
                 );
     }
-    up_write(&current->mm->mmap_sem);
+    up_write(&current_mm_mmap_sem);
 #endif
 
     MdlMap->vma = NULL;
@@ -809,14 +1047,14 @@ _GFPMapUser(
                     MAP_SHARED | MAP_NORESERVE,
                     0);
 #else
-    down_write(&current->mm->mmap_sem);
+    down_write(&current_mm_mmap_sem);
     userLogical = (gctPOINTER)do_mmap_pgoff(NULL,
                     0L,
                     Mdl->numPages * PAGE_SIZE,
                     PROT_READ | PROT_WRITE,
                     MAP_SHARED,
                     0);
-    up_write(&current->mm->mmap_sem);
+    up_write(&current_mm_mmap_sem);
 #endif
 
     gcmkTRACE_ZONE(
@@ -840,7 +1078,8 @@ _GFPMapUser(
         gcmkONERROR(gcvSTATUS_OUT_OF_MEMORY);
     }
 
-    down_write(&current->mm->mmap_sem);
+    down_write(&current_mm_mmap_sem);
+
     do
     {
         struct vm_area_struct *vma = find_vma(current->mm, (unsigned long)userLogical);
@@ -860,7 +1099,8 @@ _GFPMapUser(
         MdlMap->vma = vma;
     }
     while (gcvFALSE);
-    up_write(&current->mm->mmap_sem);
+
+    up_write(&current_mm_mmap_sem);
 
     if (gcmIS_SUCCESS(status))
     {
@@ -881,20 +1121,30 @@ static gceSTATUS
 _GFPMapKernel(
     IN gckALLOCATOR Allocator,
     IN PLINUX_MDL Mdl,
+    IN gctSIZE_T Offset,
+    IN gctSIZE_T Bytes,
     OUT gctPOINTER *Logical
     )
 {
     void *addr = 0;
-    gctINT numPages = Mdl->numPages;
+    gctSIZE_T numPages = Mdl->numPages;
     struct gfp_mdl_priv *mdlPriv = Mdl->priv;
-
+    unsigned long pgoff = (Offset >> PAGE_SHIFT);
     struct page ** pages;
     gctBOOL free = gcvFALSE;
     pgprot_t pgprot;
-    gctINT i;
 
-    if (Mdl->contiguous)
+    if (Offset + Bytes > (numPages << PAGE_SHIFT))
     {
+        return gcvSTATUS_INVALID_ARGUMENT;
+    }
+
+    numPages = ((Offset & ~PAGE_MASK) + Bytes + (PAGE_SIZE - 1)) >> PAGE_SHIFT;
+
+    if (mdlPriv->contiguous)
+    {
+        gctSIZE_T i;
+
         pages = kmalloc(sizeof(struct page *) * numPages, GFP_KERNEL | gcdNOWARN);
 
         if (!pages)
@@ -904,14 +1154,14 @@ _GFPMapKernel(
 
         for (i = 0; i < numPages; i++)
         {
-            pages[i] = nth_page(mdlPriv->contiguousPages, i);
+            pages[i] = nth_page(mdlPriv->contiguousPages, i + pgoff);
         }
 
         free = gcvTRUE;
     }
     else
     {
-        pages = mdlPriv->nonContiguousPages;
+        pages = &mdlPriv->nonContiguousPages[pgoff];
     }
 
     /* ioremap() can't work on system memory since 2.6.38. */
@@ -937,7 +1187,8 @@ _GFPMapKernel(
 
     if (addr)
     {
-        *Logical = addr;
+        /* Append offset in page. */
+        *Logical = (uint8_t *)addr + (Offset & ~PAGE_MASK);
         return gcvSTATUS_OK;
     }
     else
@@ -953,7 +1204,7 @@ _GFPUnmapKernel(
     IN gctPOINTER Logical
     )
 {
-    vunmap(Logical);
+    vunmap((void *)((uintptr_t)Logical & PAGE_MASK));
 
     return gcvSTATUS_OK;
 }
@@ -964,12 +1215,15 @@ _GFPCache(
     IN PLINUX_MDL Mdl,
     IN gctSIZE_T Offset,
     IN gctPOINTER Logical,
-    IN gctUINT32 Bytes,
+    IN gctSIZE_T Bytes,
     IN gceCACHEOPERATION Operation
     )
 {
     struct gfp_mdl_priv *mdlPriv = Mdl->priv;
     enum dma_data_direction dir;
+    dma_addr_t dma_addr = (mdlPriv->dma_addr + Offset) & PAGE_MASK;
+    gctSIZE_T bytes = (mdlPriv->dma_addr + Offset + Bytes) - dma_addr;
+    gctINT numPages = GetPageCount(bytes, 0);
 
     switch (Operation)
     {
@@ -979,7 +1233,7 @@ _GFPCache(
         if (mdlPriv->contiguous)
         {
             dma_sync_single_for_device(galcore_device,
-                    mdlPriv->dma_addr, Mdl->numPages << PAGE_SHIFT, dir);
+                    dma_addr, numPages << PAGE_SHIFT, dir);
         }
         else
         {
@@ -994,7 +1248,7 @@ _GFPCache(
         if (mdlPriv->contiguous)
         {
             dma_sync_single_for_device(galcore_device,
-                    mdlPriv->dma_addr, Mdl->numPages << PAGE_SHIFT, dir);
+                    dma_addr, numPages << PAGE_SHIFT, dir);
         }
         else
         {
@@ -1007,7 +1261,7 @@ _GFPCache(
         if (mdlPriv->contiguous)
         {
             dma_sync_single_for_cpu(galcore_device,
-                    mdlPriv->dma_addr, Mdl->numPages << PAGE_SHIFT, dir);
+                    dma_addr, numPages << PAGE_SHIFT, dir);
         }
         else
         {
@@ -1022,7 +1276,7 @@ _GFPCache(
         if (mdlPriv->contiguous)
         {
             dma_sync_single_for_cpu(galcore_device,
-                    mdlPriv->dma_addr, Mdl->numPages << PAGE_SHIFT, dir);
+                    dma_addr, numPages << PAGE_SHIFT, dir);
         }
         else
         {
@@ -1050,7 +1304,7 @@ _GFPPhysical(
     gctUINT32 offsetInPage = Offset & ~PAGE_MASK;
     gctUINT32 index = Offset / PAGE_SIZE;
 
-    if (Mdl->contiguous)
+    if (mdlPriv->contiguous)
     {
         *Physical = page_to_phys(nth_page(mdlPriv->contiguousPages, index));
     }
@@ -1130,9 +1384,10 @@ _GFPAlloctorInit(
                           | gcvALLOC_FLAG_MEMLIMIT
                           | gcvALLOC_FLAG_ALLOC_ON_FAULT
                           | gcvALLOC_FLAG_DMABUF_EXPORTABLE
-#if defined(CONFIG_ZONE_DMA32) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)
+#if (defined(CONFIG_ZONE_DMA32) || defined(CONFIG_ZONE_DMA)) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)
                           | gcvALLOC_FLAG_4GB_ADDR
 #endif
+                          | gcvALLOC_FLAG_1M_PAGES
                           ;
 
 #if defined(gcdEMULATE_SECURE_ALLOCATOR)
