@@ -2,7 +2,7 @@
 *
 *    The MIT License (MIT)
 *
-*    Copyright (c) 2014 - 2018 Vivante Corporation
+*    Copyright (c) 2014 - 2020 Vivante Corporation
 *
 *    Permission is hereby granted, free of charge, to any person obtaining a
 *    copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 *
 *    The GPL License (GPL)
 *
-*    Copyright (C) 2014 - 2018 Vivante Corporation
+*    Copyright (C) 2014 - 2020 Vivante Corporation
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
@@ -59,6 +59,7 @@
 
 #include <linux/slab.h>
 #include <linux/pagemap.h>
+#include <linux/cache.h>
 
 #define _GC_OBJ_ZONE gcvZONE_ALLOCATOR
 
@@ -92,7 +93,6 @@ struct um_desc
         struct
         {
             struct page **pages;
-            struct sg_table sgt;
         };
 
         /* UM_PFN_MAP. */
@@ -103,6 +103,8 @@ struct um_desc
         };
     };
 
+    struct sg_table sgt;
+
     /* contiguous chunks, does not include padding pages. */
     int chunk_count;
 
@@ -130,11 +132,17 @@ static int import_page_map(struct um_desc *um,
     int result;
     struct page **pages;
 
+    if ((addr & (cache_line_size() - 1)) || (size & (cache_line_size() - 1)))
+    {
+        /* Not cpu cacheline size aligned, can not support. */
+        return -EINVAL;
+    }
+
     pages = kzalloc(page_count * sizeof(void *), GFP_KERNEL | gcdNOWARN);
     if (!pages)
         return -ENOMEM;
 
-    down_read(&current->mm->mmap_sem);
+    down_read(&current_mm_mmap_sem);
 
     result = get_user_pages(
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 6, 0)
@@ -152,7 +160,7 @@ static int import_page_map(struct um_desc *um,
             pages,
             NULL);
 
-    up_read(&current->mm->mmap_sem);
+    up_read(&current_mm_mmap_sem);
 
     if (result < page_count)
     {
@@ -177,8 +185,7 @@ static int import_page_map(struct um_desc *um,
         }
     }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (3,6,0) \
-    && (defined(ARCH_HAS_SG_CHAIN) || defined(CONFIG_ARCH_HAS_SG_CHAIN))
+#if gcdUSE_Linux_SG_TABLE_API
     result = sg_alloc_table_from_pages(&um->sgt, pages, page_count,
                     addr & ~PAGE_MASK, size, GFP_KERNEL | gcdNOWARN);
 
@@ -207,8 +214,7 @@ static int import_page_map(struct um_desc *um,
     return 0;
 
 error:
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (3,6,0) \
-    && (defined(ARCH_HAS_SG_CHAIN) || defined(CONFIG_ARCH_HAS_SG_CHAIN))
+#if gcdUSE_Linux_SG_TABLE_API
     sg_free_table(&um->sgt);
 #else
     kfree(um->sgt.sgl);
@@ -229,13 +235,16 @@ static int import_pfn_map(struct um_desc *um,
     struct vm_area_struct *vma;
     unsigned long *pfns;
     int *refs;
+    struct page **pages = gcvNULL;
+    int result = 0;
+    size_t pageCount = 0;
 
     if (!current->mm)
         return -ENOTTY;
 
-    down_read(&current->mm->mmap_sem);
+    down_read(&current_mm_mmap_sem);
     vma = find_vma(current->mm, addr);
-    up_read(&current->mm->mmap_sem);
+    up_read(&current_mm_mmap_sem);
 
     if (!vma)
         return -ENOTTY;
@@ -253,10 +262,21 @@ static int import_pfn_map(struct um_desc *um,
         return -ENOMEM;
     }
 
+    pages = kzalloc(pfn_count * sizeof(void *), GFP_KERNEL | gcdNOWARN);
+    if (!pages)
+    {
+        kfree(pfns);
+        kfree(refs);
+        return -ENOMEM;
+    }
+
     for (i = 0; i < pfn_count; i++)
     {
         spinlock_t *ptl;
         pgd_t *pgd;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION (5,9,0)
+        p4d_t *p4d;
+#endif
         pud_t *pud;
         pmd_t *pmd;
         pte_t *pte;
@@ -265,7 +285,23 @@ static int import_pfn_map(struct um_desc *um,
         if (pgd_none(*pgd) || pgd_bad(*pgd))
             goto err;
 
+#if (defined(CONFIG_CPU_CSKYV2) || defined(CONFIG_X86)) \
+    && LINUX_VERSION_CODE >= KERNEL_VERSION (4,12,0)
+        pud = pud_offset((p4d_t*)pgd, addr);
+#elif (defined(CONFIG_CPU_CSKYV2)) \
+    && LINUX_VERSION_CODE >= KERNEL_VERSION (4,11,0)
+        pud = pud_offset((p4d_t*)pgd, addr);
+#else
+#if LINUX_VERSION_CODE >= KERNEL_VERSION (5,9,0)
+        p4d = p4d_offset(pgd, addr);
+        if (p4d_none(READ_ONCE(*p4d)))
+            goto err;
+
+        pud = pud_offset(p4d, addr);
+#else
         pud = pud_offset(pgd, addr);
+#endif
+#endif
         if (pud_none(*pud) || pud_bad(*pud))
             goto err;
 
@@ -274,11 +310,6 @@ static int import_pfn_map(struct um_desc *um,
             goto err;
 
         pte = pte_offset_map_lock(current->mm, pmd, addr, &ptl);
-        if (!pte)
-        {
-            spin_unlock(ptl);
-            goto err;
-        }
 
         if (!pte_present(*pte))
         {
@@ -299,6 +330,8 @@ static int import_pfn_map(struct um_desc *um,
         {
             struct page *page = pfn_to_page(pfns[i]);
             refs[i] = get_page_unless_zero(page);
+            pages[i] = page;
+            pageCount++;
         }
     }
 
@@ -311,6 +344,41 @@ static int import_pfn_map(struct um_desc *um,
         }
     }
 
+    if (pageCount == pfn_count)
+    {
+#if gcdUSE_Linux_SG_TABLE_API
+        result = sg_alloc_table_from_pages(&um->sgt, pages, pfn_count,
+                        addr & ~PAGE_MASK, pfn_count * PAGE_SIZE, GFP_KERNEL | gcdNOWARN);
+
+#else
+        result = alloc_sg_list_from_pages(&um->sgt.sgl, pages, pfn_count,
+                        addr & ~PAGE_MASK, pfn_count * PAGE_SIZE, &um->sgt.nents);
+
+        um->sgt.orig_nents = um->sgt.nents;
+#endif
+        if (unlikely(result < 0))
+        {
+            printk("[galcore]: %s: sg_alloc_table_from_pages failed\n", __FUNCTION__);
+            goto err;
+        }
+
+        result = dma_map_sg(galcore_device, um->sgt.sgl, um->sgt.nents, DMA_TO_DEVICE);
+
+        if (unlikely(result != um->sgt.nents))
+        {
+#if gcdUSE_Linux_SG_TABLE_API
+            sg_free_table(&um->sgt);
+#else
+            kfree(um->sgt.sgl);
+#endif
+            printk("[galcore]: %s: dma_map_sg failed\n", __FUNCTION__);
+            goto err;
+        }
+    }
+
+    kfree(pages);
+    pages = gcvNULL;
+
     um->type = UM_PFN_MAP;
     um->pfns = pfns;
     um->refs = refs;
@@ -323,6 +391,9 @@ err:
     if (refs)
         kfree(refs);
 
+    if (pages)
+        kfree(pages);
+
     return -ENOTTY;
 }
 
@@ -330,7 +401,7 @@ static gceSTATUS
 _Import(
     IN gckOS Os,
     IN gctPOINTER Memory,
-    IN gctUINT32 Physical,
+    IN gctPHYS_ADDR_T Physical,
     IN gctSIZE_T Size,
     IN struct um_desc * UserMemory
     )
@@ -344,14 +415,14 @@ _Import(
     gctSIZE_T extraPage;
     gctSIZE_T pageCount, i;
 
-    gcmkHEADER_ARG("Os=0x%p Memory=%p Physical=0x%x Size=%lu", Os, Memory, Physical, Size);
+    gcmkHEADER_ARG("Os=%p Memory=%p Physical=0x%llx Size=%lu", Os, Memory, Physical, Size);
 
     /* Verify the arguments. */
     gcmkVERIFY_OBJECT(Os, gcvOBJ_OS);
-    gcmkVERIFY_ARGUMENT(Memory != gcvNULL || Physical != ~0U);
+    gcmkVERIFY_ARGUMENT(Memory != gcvNULL || Physical != ~0ULL);
     gcmkVERIFY_ARGUMENT(Size > 0);
 
-    memory = (unsigned long)Memory;
+    memory = untagged_addr((unsigned long)Memory);
 
     /* Get the number of required pages. */
     end = (memory + Size + PAGE_SIZE - 1) >> PAGE_SHIFT;
@@ -385,7 +456,7 @@ _Import(
 
         for (i = 0; i < pageCount; i++)
         {
-            u32 data;
+            u32 data = 0;
 
             get_user(data, (u32 *)vaddr);
             put_user(data, (u32 *)vaddr);
@@ -398,7 +469,9 @@ _Import(
             }
         }
 
+        down_read(&current_mm_mmap_sem);
         vma = find_vma(current->mm, memory);
+        up_read(&current_mm_mmap_sem);
 
         if (!vma)
         {
@@ -414,6 +487,7 @@ _Import(
         vm_flags = vma->vm_flags;
         vaddr = vma->vm_end;
 
+        down_read(&current_mm_mmap_sem);
         while (vaddr < memory + Size)
         {
             vma = find_vma(current->mm, vaddr);
@@ -421,17 +495,20 @@ _Import(
             if (!vma)
             {
                 /* No such memory. */
+                up_read(&current_mm_mmap_sem);
                 gcmkONERROR(gcvSTATUS_INVALID_ARGUMENT);
             }
 
             if ((vma->vm_flags & VM_PFNMAP) != (vm_flags & VM_PFNMAP))
             {
                 /* Can not support different map type: both PFN and PAGE detected. */
+                up_read(&current_mm_mmap_sem);
                 gcmkONERROR(gcvSTATUS_NOT_SUPPORTED);
             }
 
             vaddr = vma->vm_end;
         }
+        up_read(&current_mm_mmap_sem);
     }
 
     if (Physical != gcvINVALID_PHYSICAL_ADDRESS)
@@ -469,7 +546,7 @@ _Import(
 
         if (Physical != gcvINVALID_PHYSICAL_ADDRESS)
         {
-            if(Physical >0xFFFFFFFFu || Physical + Size > 0xFFFFFFFFu )
+            if(Physical > 0xFFFFFFFFu || Physical + Size > 0xFFFFFFFFu )
             {
                 gcmkONERROR(gcvSTATUS_INVALID_ARGUMENT);
             }
@@ -575,11 +652,15 @@ static void release_physical_map(struct um_desc *um)
 static void release_page_map(struct um_desc *um)
 {
     int i;
+    dma_sync_sg_for_device(galcore_device,
+                    um->sgt.sgl, um->sgt.nents, DMA_TO_DEVICE);
 
-    dma_unmap_sg(galcore_device, um->sgt.sgl, um->sgt.nents, DMA_TO_DEVICE);
+    dma_sync_sg_for_cpu(galcore_device,
+                    um->sgt.sgl, um->sgt.nents, DMA_FROM_DEVICE);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION (3,6,0) \
-    && (defined(ARCH_HAS_SG_CHAIN) || defined(CONFIG_ARCH_HAS_SG_CHAIN))
+    dma_unmap_sg(galcore_device, um->sgt.sgl, um->sgt.nents, DMA_FROM_DEVICE);
+
+#if gcdUSE_Linux_SG_TABLE_API
     sg_free_table(&um->sgt);
 #else
     kfree(um->sgt.sgl);
@@ -687,6 +768,8 @@ static gceSTATUS
 _UserMemoryMapKernel(
     IN gckALLOCATOR Allocator,
     IN PLINUX_MDL Mdl,
+    IN gctSIZE_T Offset,
+    IN gctSIZE_T Bytes,
     OUT gctPOINTER *Logical
     )
 {
@@ -711,14 +794,14 @@ _UserMemoryCache(
     IN PLINUX_MDL Mdl,
     IN gctSIZE_T Offset,
     IN gctPOINTER Logical,
-    IN gctUINT32 Bytes,
+    IN gctSIZE_T Bytes,
     IN gceCACHEOPERATION Operation
     )
 {
     struct um_desc *um = Mdl->priv;
     enum dma_data_direction dir;
 
-    if (um->type != UM_PAGE_MAP)
+    if (um->type == UM_PHYSICAL_MAP)
     {
         _MemoryBarrier();
         return gcvSTATUS_OK;
@@ -784,13 +867,13 @@ _UserMemoryPhysical(
         switch (userMemory->type)
         {
         case UM_PHYSICAL_MAP:
-            *Physical = userMemory->physical + index * PAGE_SIZE;
+            *Physical = userMemory->physical + (gctPHYS_ADDR_T)index * PAGE_SIZE;
             break;
         case UM_PAGE_MAP:
             *Physical = page_to_phys(userMemory->pages[index]);
             break;
         case UM_PFN_MAP:
-            *Physical = userMemory->pfns[index] << PAGE_SHIFT;
+            *Physical = (gctPHYS_ADDR_T)userMemory->pfns[index] << PAGE_SHIFT;
             break;
         }
     }
