@@ -2,7 +2,7 @@
 *
 *    The MIT License (MIT)
 *
-*    Copyright (c) 2014 - 2018 Vivante Corporation
+*    Copyright (c) 2014 - 2020 Vivante Corporation
 *
 *    Permission is hereby granted, free of charge, to any person obtaining a
 *    copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,7 @@
 *
 *    The GPL License (GPL)
 *
-*    Copyright (C) 2014 - 2018 Vivante Corporation
+*    Copyright (C) 2014 - 2020 Vivante Corporation
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
@@ -55,7 +55,14 @@
 
 #if gcdENABLE_DRM
 
+#include <linux/version.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,9,0)
+#include <drm/drm_drv.h>
+#include <drm/drm_file.h>
+#include <drm/drm_ioctl.h>
+#else
 #include <drm/drmP.h>
+#endif
 #include <drm/drm_gem.h>
 #include <linux/dma-buf.h>
 #include "gc_hal_kernel_linux.h"
@@ -75,10 +82,27 @@ struct viv_gem_object {
     gctBOOL               cacheable;
 };
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,11,0)
+void viv_gem_free_object(struct drm_gem_object *gem_obj);
+struct dma_buf *viv_gem_prime_export(struct drm_gem_object *gem_obj, int flags);
+
+static const struct drm_gem_object_funcs viv_gem_object_funcs = {
+    .free = viv_gem_free_object,
+    .export = viv_gem_prime_export,
+};
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,4,0)
+struct dma_buf *viv_gem_prime_export(struct drm_gem_object *gem_obj,
+                int flags)
+{
+    struct drm_device *drm = gem_obj->dev;
+#else
 struct dma_buf *viv_gem_prime_export(struct drm_device *drm,
                 struct drm_gem_object *gem_obj,
                 int flags)
 {
+#endif
     struct viv_gem_object *viv_obj = container_of(gem_obj, struct viv_gem_object, base);
     struct dma_buf *dmabuf = gcvNULL;
     gckGALDEVICE gal_dev = (gckGALDEVICE)drm->dev_private;
@@ -86,7 +110,7 @@ struct dma_buf *viv_gem_prime_export(struct drm_device *drm,
     if (gal_dev)
     {
         gckKERNEL kernel = gal_dev->device->map[gal_dev->device->defaultHwType].kernels[0];
-        gcmkVERIFY_OK(gckVIDMEM_NODE_Export(kernel, viv_obj->node_handle, flags,
+        gcmkVERIFY_OK(gckVIDMEM_NODE_Export(kernel, viv_obj->node_object, flags,
                                             (gctPOINTER*)&dmabuf, gcvNULL));
     }
 
@@ -127,6 +151,9 @@ struct drm_gem_object *viv_gem_prime_import(struct drm_device *drm,
     /* ioctl output */
     gem_obj = kzalloc(sizeof(struct viv_gem_object), GFP_KERNEL);
     drm_gem_private_object_init(drm, gem_obj, dmabuf->size);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,11,0)
+    gem_obj->funcs = &viv_gem_object_funcs;
+#endif
     viv_obj = container_of(gem_obj, struct viv_gem_object, base);
     viv_obj->node_handle = iface.u.WrapUserMemory.node;
     viv_obj->node_object = nodeObject;
@@ -168,6 +195,7 @@ static int viv_ioctl_gem_create(struct drm_device *drm, void *data,
     gckVIDMEM_NODE nodeObject;
     gctUINT32 flags = gcvALLOC_FLAG_DMABUF_EXPORTABLE;
     gceSTATUS status = gcvSTATUS_OK;
+    gctUINT64 alignSize = PAGE_ALIGN(args->size);
 
     gal_dev = (gckGALDEVICE)drm->dev_private;
     if (!gal_dev)
@@ -195,9 +223,9 @@ static int viv_ioctl_gem_create(struct drm_device *drm, void *data,
     gckOS_ZeroMemory(&iface, sizeof(iface));
     iface.command = gcvHAL_ALLOCATE_LINEAR_VIDEO_MEMORY;
     iface.hardwareType = gal_dev->device->defaultHwType;
-    iface.u.AllocateLinearVideoMemory.bytes = PAGE_ALIGN(args->size);
+    iface.u.AllocateLinearVideoMemory.bytes = alignSize;
     iface.u.AllocateLinearVideoMemory.alignment = 256;
-    iface.u.AllocateLinearVideoMemory.type = gcvSURF_RENDER_TARGET; /* should be general */
+    iface.u.AllocateLinearVideoMemory.type = gcvVIDMEM_TYPE_GENERIC;
     iface.u.AllocateLinearVideoMemory.flag = flags;
     iface.u.AllocateLinearVideoMemory.pool = gcvPOOL_DEFAULT;
     gcmkONERROR(gckDEVICE_Dispatch(gal_dev->device, &iface));
@@ -208,7 +236,10 @@ static int viv_ioctl_gem_create(struct drm_device *drm, void *data,
 
     /* ioctl output */
     gem_obj = kzalloc(sizeof(struct viv_gem_object), GFP_KERNEL);
-    drm_gem_private_object_init(drm, gem_obj, iface.u.AllocateLinearVideoMemory.bytes);
+    drm_gem_private_object_init(drm, gem_obj, (size_t)alignSize);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,11,0)
+    gem_obj->funcs = &viv_gem_object_funcs;
+#endif
     ret = drm_gem_handle_create(file, gem_obj, &args->handle);
 
     viv_obj = container_of(gem_obj, struct viv_gem_object, base);
@@ -292,14 +323,14 @@ static int viv_ioctl_gem_unlock(struct drm_device *drm, void *data,
     iface.command = gcvHAL_UNLOCK_VIDEO_MEMORY;
     iface.hardwareType = gal_dev->device->defaultHwType;
     iface.u.UnlockVideoMemory.node = (gctUINT64)viv_obj->node_handle;
-    iface.u.UnlockVideoMemory.type = gcvSURF_TYPE_UNKNOWN;
+    iface.u.UnlockVideoMemory.type = gcvVIDMEM_TYPE_GENERIC;
     gcmkONERROR(gckDEVICE_Dispatch(gal_dev->device, &iface));
 
     memset(&iface, 0, sizeof(iface));
     iface.command = gcvHAL_BOTTOM_HALF_UNLOCK_VIDEO_MEMORY;
     iface.hardwareType = gal_dev->device->defaultHwType;
     iface.u.BottomHalfUnlockVideoMemory.node = (gctUINT64)viv_obj->node_handle;
-    iface.u.BottomHalfUnlockVideoMemory.type = gcvSURF_TYPE_UNKNOWN;
+    iface.u.BottomHalfUnlockVideoMemory.type = gcvVIDMEM_TYPE_GENERIC;
     gcmkONERROR(gckDEVICE_Dispatch(gal_dev->device, &iface));
 
 OnError:
@@ -470,9 +501,10 @@ static int viv_ioctl_gem_set_tiling(struct drm_device *drm, void *data,
     }
     viv_obj = container_of(gem_obj, struct viv_gem_object, base);
 
-    viv_obj->node_object->tilingMode = args->tiling_mode;
-    viv_obj->node_object->tsMode     = args->ts_mode;
-    viv_obj->node_object->clearValue = args->clear_value;
+    viv_obj->node_object->tilingMode    = args->tiling_mode;
+    viv_obj->node_object->tsMode        = args->ts_mode;
+    viv_obj->node_object->tsCacheMode   = args->ts_cache_mode;
+    viv_obj->node_object->clearValue    = args->clear_value;
 
 OnError:
     if (gem_obj)
@@ -505,9 +537,10 @@ static int viv_ioctl_gem_get_tiling(struct drm_device *drm, void *data,
     }
     viv_obj = container_of(gem_obj, struct viv_gem_object, base);
 
-    args->tiling_mode = viv_obj->node_object->tilingMode;
-    args->ts_mode     = viv_obj->node_object->tsMode;
-    args->clear_value = viv_obj->node_object->clearValue;
+    args->tiling_mode   = viv_obj->node_object->tilingMode;
+    args->ts_mode       = viv_obj->node_object->tsMode;
+    args->ts_cache_mode = viv_obj->node_object->tsCacheMode;
+    args->clear_value   = viv_obj->node_object->clearValue;
 
 OnError:
     if (gem_obj)
@@ -558,6 +591,8 @@ static int viv_ioctl_gem_attach_aux(struct drm_device *drm, void *data,
         gctBOOL is2BitPerTile = is128BTILE ? gcvFALSE : gckHARDWARE_IsFeatureAvailable(kernel->hardware , gcvFEATURE_TILE_STATUS_2BITS);
         gctBOOL isCompressionDEC400 = gckHARDWARE_IsFeatureAvailable(kernel->hardware , gcvFEATURE_COMPRESSION_DEC400);
         gctPOINTER entry = gcvNULL;
+        gckVIDMEM_NODE ObjNode = gcvNULL;
+        gctUINT32 processID = 0;
         gctUINT32 tileStatusFiller = (isCompressionDEC400 || ((kernel->hardware->identity.chipModel == gcv500) && (kernel->hardware->identity.chipRevision > 2)))
                                   ? 0xFFFFFFFF
                                   : is2BitPerTile ? 0x55555555 : 0x11111111;
@@ -581,10 +616,13 @@ static int viv_ioctl_gem_attach_aux(struct drm_device *drm, void *data,
         iface.u.LockVideoMemory.cacheable = viv_ts_obj->cacheable;
         gcmkONERROR(gckDEVICE_Dispatch(gal_dev->device, &iface));
 
+        gcmkONERROR(gckOS_GetProcessID(&processID));
+        gcmkONERROR(gckVIDMEM_HANDLE_Lookup(kernel, processID, viv_ts_obj->node_handle, &ObjNode));
+        gcmkONERROR(gckVIDMEM_NODE_LockCPU(kernel, ObjNode, gcvFALSE, gcvFALSE, &entry));
+
         /* Fill tile status node with tileStatusFiller. */
-        gcmkONERROR(gckVIDMEM_NODE_LockCPU(kernel, viv_ts_obj->node_handle, &entry));
         memset(entry , tileStatusFiller , (__u64)gem_ts_obj->size);
-        gcmkONERROR(gckVIDMEM_NODE_UnlockCPU(kernel, viv_ts_obj->node_handle, entry));
+        gcmkONERROR(gckVIDMEM_NODE_UnlockCPU(kernel, ObjNode, 0, gcvFALSE, gcvFALSE));
 
         /* UnLock tile status node. */
         memset(&iface, 0, sizeof(iface));
@@ -627,8 +665,10 @@ static int viv_ioctl_gem_ref_node(struct drm_device *drm, void *data,
     gckKERNEL kernel = gcvNULL;
     gctUINT32 processID;
     gckVIDMEM_NODE nodeObj;
+    gceDATABASE_TYPE type;
     gctUINT32 nodeHandle = 0, tsNodeHandle = 0;
     gctBOOL refered = gcvFALSE;
+    gctBOOL isContiguous = gcvFALSE;
     int ret = 0;
 
     gal_dev = (gckGALDEVICE)drm->dev_private;
@@ -648,21 +688,40 @@ static int viv_ioctl_gem_ref_node(struct drm_device *drm, void *data,
 
     gcmkONERROR(gckOS_GetProcessID(&processID));
     gcmkONERROR(gckVIDMEM_HANDLE_Allocate(kernel, nodeObj, &nodeHandle));
+
+    type = gcvDB_VIDEO_MEMORY
+         | (nodeObj->type << gcdDB_VIDEO_MEMORY_TYPE_SHIFT)
+         | (nodeObj->pool << gcdDB_VIDEO_MEMORY_POOL_SHIFT);
+
+    gcmkONERROR(gckVIDMEM_NODE_IsContiguous(kernel, nodeObj, &isContiguous));
+    if (isContiguous)
+    {
+        type |= (gcvDB_CONTIGUOUS << gcdDB_VIDEO_MEMORY_DBTYPE_SHIFT);
+    }
     gcmkONERROR(
         gckKERNEL_AddProcessDB(kernel,
-                               processID, gcvDB_VIDEO_MEMORY,
+                               processID, type,
                                gcmINT2PTR(nodeHandle),
                                gcvNULL,
                                0));
     gcmkONERROR(gckVIDMEM_NODE_Reference(kernel, nodeObj));
     refered = gcvTRUE;
-
     if (nodeObj->tsNode)
     {
+        type = gcvDB_VIDEO_MEMORY
+             | (nodeObj->tsNode->type << gcdDB_VIDEO_MEMORY_TYPE_SHIFT)
+             | (nodeObj->tsNode->pool << gcdDB_VIDEO_MEMORY_POOL_SHIFT);
+
+        gcmkONERROR(gckVIDMEM_NODE_IsContiguous(kernel, nodeObj->tsNode, &isContiguous));
+        if (isContiguous)
+        {
+            type |= (gcvDB_CONTIGUOUS << gcdDB_VIDEO_MEMORY_DBTYPE_SHIFT);
+        }
+
         gcmkONERROR(gckVIDMEM_HANDLE_Allocate(kernel, nodeObj->tsNode, &tsNodeHandle));
         gcmkONERROR(
             gckKERNEL_AddProcessDB(kernel,
-                                   processID, gcvDB_VIDEO_MEMORY,
+                                   processID, type,
                                    gcmINT2PTR(tsNodeHandle),
                                    gcvNULL,
                                    0));
@@ -770,17 +829,25 @@ static const struct file_operations viv_drm_fops = {
 };
 
 static struct drm_driver viv_drm_driver = {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,4,0)
+    .driver_features    = DRIVER_GEM | DRIVER_RENDER,
+#else
     .driver_features    = DRIVER_GEM | DRIVER_PRIME | DRIVER_RENDER,
+#endif
     .open = viv_drm_open,
     .postclose = viv_drm_postclose,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5,11,0)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,7,0)
     .gem_free_object_unlocked = viv_gem_free_object,
 #else
     .gem_free_object    = viv_gem_free_object,
+#endif
 #endif
     .prime_handle_to_fd = drm_gem_prime_handle_to_fd,
     .prime_fd_to_handle = drm_gem_prime_fd_to_handle,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5,11,0)
     .gem_prime_export   = viv_gem_prime_export,
+#endif
     .gem_prime_import   = viv_gem_prime_import,
     .ioctls             = viv_ioctls,
     .num_ioctls         = DRM_VIV_NUM_IOCTLS,
