@@ -2,8 +2,7 @@
 *
 *    The MIT License (MIT)
 *
-*    Copyright (c) 2014 - 2018 Vivante Corporation
-*    Copyright 2019 NXP
+*    Copyright (c) 2014 - 2020 Vivante Corporation
 *
 *    Permission is hereby granted, free of charge, to any person obtaining a
 *    copy of this software and associated documentation files (the "Software"),
@@ -27,8 +26,7 @@
 *
 *    The GPL License (GPL)
 *
-*    Copyright (C) 2014 - 2018 Vivante Corporation
-*    Copyright 2019 NXP
+*    Copyright (C) 2014 - 2020 Vivante Corporation
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
@@ -60,6 +58,7 @@
 #include "gc_hal_kernel_device.h"
 #include "gc_hal_driver.h"
 #include <linux/slab.h>
+#include <linux/pm_qos.h>
 
 #if defined(CONFIG_PM_OPP)
 #include <linux/pm_opp.h>
@@ -75,7 +74,7 @@
 #   include <linux/platform_device.h>
 #endif
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4,9,0)) || defined(IMX8_SCU_CONTROL)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,1,0)
 #  define IMX_GPU_SUBSYSTEM   1
 #  include <linux/component.h>
 #endif
@@ -90,15 +89,20 @@
 #    include <linux/busfreq-imx6.h>
 #    include <linux/reset.h>
 #  else
-#    include <linux/busfreq-imx.h>
+#      include <linux/busfreq-imx.h>
 #    include <linux/reset.h>
 #  endif
 #endif
 
 #include <linux/clk.h>
 
-#if defined(IMX8_SCU_CONTROL)
-#  include <soc/imx8/sc/sci.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,0,0)
+#include <linux/firmware/imx/ipc.h>
+#include <linux/firmware/imx/svc/misc.h>
+#include <dt-bindings/firmware/imx/rsrc.h>
+static struct imx_sc_ipc *gpu_ipcHandle;
+#elif defined(IMX8_SCU_CONTROL)
+#include <soc/imx8/sc/sci.h>
 static sc_ipc_t gpu_ipcHandle;
 #endif
 
@@ -109,7 +113,7 @@ static sc_ipc_t gpu_ipcHandle;
 #include <linux/pm_runtime.h>
 #include <linux/regulator/consumer.h>
 
-#ifdef CONFIG_DEVICE_THERMAL
+#if IS_ENABLED(CONFIG_DEVICE_THERMAL)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
 #    include <linux/device_cooling.h>
 #    define REG_THERMAL_NOTIFIER(a) register_devfreq_cooling_notifier(a);
@@ -138,93 +142,109 @@ struct platform_device *pdevice;
 #  include <linux/sched.h>
 #  include <linux/profile.h>
 
-static unsigned long timeout;
-struct task_struct *almostfail;
+struct task_struct *oom_crashpending;
 
 static int
-notif_func(struct notifier_block *self, unsigned long val, void *data)
+task_notify_func(struct notifier_block *self, unsigned long val, void *data)
 {
     struct task_struct *task = data;
 
-    if (task == almostfail)
-        almostfail = NULL;
+    if (task == oom_crashpending) {
+        oom_crashpending = NULL;
+    }
 
     return NOTIFY_DONE;
 }
 
 static struct notifier_block task_nb = {
-    .notifier_call  = notif_func,
+    .notifier_call  = task_notify_func,
 };
 
-static int force_shrink_mem(IN gckKERNEL Kernel)
-{
-    struct task_struct *p = NULL;
-    struct task_struct *selected = NULL;
-    int cur_size = 0;
-    int set_size = 0;
-    int oom_val = 0;
-    int mem_adj = 0;
-    int retVal = -1;
+extern struct task_struct *oom_crashpending;
+static unsigned long oom_crashpending_timeout;
 
-    if (almostfail && time_before_eq(jiffies, timeout))
+static int force_contiguous_lowmem_shrink(IN gckKERNEL Kernel)
+{
+    struct task_struct *p;
+    struct task_struct *selected = NULL;
+    int tasksize;
+        int ret = -1;
+    int min_adj = 0;
+    int selected_tasksize = 0;
+    int selected_oom_adj;
+
+    /* Return if we already have a oom crash pending
+     */
+    if (oom_crashpending &&
+        time_before_eq(jiffies, oom_crashpending_timeout)) {
         return 0;
+    }
+    selected_oom_adj = min_adj;
 
     rcu_read_lock();
 
     for_each_process(p) {
-        gcuDATABASE_INFO info;
         struct mm_struct *mm;
         struct signal_struct *sig;
-
-        cur_size = 0;
+                gcuDATABASE_INFO info;
+        int oom_adj;
 
         task_lock(p);
-        sig = p->signal;
         mm = p->mm;
-        if (!sig || !mm) {
+        sig = p->signal;
+
+        if (!mm || !sig) {
             task_unlock(p);
             continue;
         }
-        oom_val = sig->oom_score_adj;
-        if (oom_val < 0) {
+
+        oom_adj = sig->oom_score_adj;
+
+        if (oom_adj < min_adj) {
             task_unlock(p);
             continue;
         }
+
+        tasksize = 0;
         task_unlock(p);
 
         rcu_read_unlock();
-        if (gckKERNEL_QueryProcessDB(Kernel, p->pid, gcvFALSE, gcvDB_VIDEO_MEMORY, &info) == gcvSTATUS_OK){
-            cur_size += info.counters.bytes / PAGE_SIZE;
-        }
-        if (gckKERNEL_QueryProcessDB(Kernel, p->pid, gcvFALSE, gcvDB_CONTIGUOUS, &info) == gcvSTATUS_OK){
-            cur_size += info.counters.bytes / PAGE_SIZE;
+
+        if (gckKERNEL_QueryProcessDB(Kernel, p->pid, gcvFALSE, gcvDB_VIDEO_MEMORY, &info) == gcvSTATUS_OK) {
+            tasksize += info.counters.bytes / PAGE_SIZE;
         }
+
         rcu_read_lock();
 
-        if (cur_size <= 0) continue;
+        if (tasksize <= 0)
+            continue;
 
-        printk("<gpu> pid %d (%s), adj %d, size %d\n", p->pid, p->comm, oom_val, cur_size);
+        printk("<gpu> pid %d (%s), adj %d, size %d \n", p->pid, p->comm, oom_adj, tasksize);
 
         if (selected) {
-            if ((oom_val < mem_adj) || (oom_val == mem_adj && cur_size <= set_size)) continue;
+            if (oom_adj < selected_oom_adj)
+                continue;
+            if (oom_adj == selected_oom_adj &&
+                tasksize <= selected_tasksize)
+                continue;
         }
-        set_size = cur_size;
-        mem_adj = oom_val;
         selected = p;
+        selected_tasksize = tasksize;
+        selected_oom_adj = oom_adj;
     }
 
-    if (selected && mem_adj > 0) {
+    if (selected && selected_oom_adj > 0) {
         printk("<gpu> send sigkill to %d (%s), adj %d, size %d\n",
-                 selected->pid, selected->comm, mem_adj, set_size);
-        almostfail = selected;
-        timeout = jiffies + HZ;
+                 selected->pid, selected->comm,
+                 selected_oom_adj, selected_tasksize);
+        oom_crashpending = selected;
+        oom_crashpending_timeout = jiffies + HZ;
         force_sig(SIGKILL, selected);
-        retVal = 0;
+        ret = 0;
     }
 
     rcu_read_unlock();
-
-    return retVal;
+    return ret;
 }
 
 extern gckKERNEL
@@ -250,7 +270,7 @@ _ShrinkMemory(
 
     if (kernel != gcvNULL)
     {
-        if (force_shrink_mem(kernel) != 0)
+        if (force_contiguous_lowmem_shrink(kernel) != 0)
             status = gcvSTATUS_OUT_OF_MEMORY;
     }
     else
@@ -271,6 +291,8 @@ static int thermal_hot_pm_notify(struct notifier_block *nb, unsigned long event,
     static gctBOOL bAlreadyTooHot = gcvFALSE;
     gckHARDWARE hardware;
     gckGALDEVICE galDevice;
+    gctUINT FscaleVal = orgFscale;
+    gctUINT core = gcvCORE_MAJOR;
 
     galDevice = platform_get_drvdata(pdevice);
     if (!galDevice)
@@ -293,14 +315,19 @@ static int thermal_hot_pm_notify(struct notifier_block *nb, unsigned long event,
 
     if (event && !bAlreadyTooHot) {
         gckHARDWARE_GetFscaleValue(hardware,&orgFscale,&minFscale, &maxFscale);
-        gckHARDWARE_SetFscaleValue(hardware, minFscale);
+        FscaleVal = minFscale;
         bAlreadyTooHot = gcvTRUE;
         printk("System is too hot. GPU3D will work at %d/64 clock.\n", minFscale);
     } else if (!event && bAlreadyTooHot) {
-        gckHARDWARE_SetFscaleValue(hardware, orgFscale);
         printk("Hot alarm is canceled. GPU3D clock will return to %d/64\n", orgFscale);
         bAlreadyTooHot = gcvFALSE;
     }
+
+    while (galDevice->kernels[core] && core <= gcvCORE_3D_MAX)
+    {
+        gckHARDWARE_SetFscaleValue(galDevice->kernels[core++]->hardware, FscaleVal, ~0U);
+    }
+
     return NOTIFY_OK;
 }
 
@@ -308,7 +335,7 @@ static struct notifier_block thermal_hot_pm_notifier =
 {
     .notifier_call = thermal_hot_pm_notify,
 };
-#endif
+#endif /* ifndef CONFIG_MXC_GPU_VIV_DISABLE_THERMAL */
 
 static ssize_t gpu3DMinClock_show(struct device_driver *dev, char *buf)
 {
@@ -317,7 +344,6 @@ static ssize_t gpu3DMinClock_show(struct device_driver *dev, char *buf)
 
     galDevice = platform_get_drvdata(pdevice);
 
-    minf = 0;
     if (galDevice->kernels[gcvCORE_MAJOR])
     {
          gckHARDWARE_GetFscaleValue(galDevice->kernels[gcvCORE_MAJOR]->hardware,
@@ -334,21 +360,25 @@ static ssize_t gpu3DMinClock_store(struct device_driver *dev, const char *buf, s
     gctINT fields;
     gctUINT MinFscaleValue;
     gckGALDEVICE galDevice;
+    gctUINT core = gcvCORE_MAJOR;
 
     galDevice = platform_get_drvdata(pdevice);
+    if (!galDevice)
+         return -EINVAL;
 
-    if (galDevice->kernels[gcvCORE_MAJOR])
+    fields = sscanf(buf, "%d", &MinFscaleValue);
+
+    if (fields < 1)
+         return -EINVAL;
+
+    while (galDevice->kernels[core] && core <= gcvCORE_3D_MAX)
     {
-         fields = sscanf(buf, "%d", &MinFscaleValue);
-
-         if (fields < 1)
-             return -EINVAL;
-
-         gckHARDWARE_SetMinFscaleValue(galDevice->kernels[gcvCORE_MAJOR]->hardware,MinFscaleValue);
+         gckHARDWARE_SetMinFscaleValue(galDevice->kernels[core++]->hardware,MinFscaleValue);
     }
 
     return count;
 }
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
 static DRIVER_ATTR_RW(gpu3DMinClock);
 #else
@@ -391,7 +421,7 @@ static ssize_t gpu3DClockScale_store(struct device_driver *dev, const char *buf,
 
     while (galDevice->kernels[core] && core <= gcvCORE_3D_MAX)
     {
-         gckHARDWARE_SetFscaleValue(galDevice->kernels[core++]->hardware,FscaleValue);
+         gckHARDWARE_SetFscaleValue(galDevice->kernels[core++]->hardware,FscaleValue,FscaleValue);
     }
 
     return count;
@@ -468,12 +498,15 @@ struct imx_priv
     struct reset_control *rstc[gcdMAX_GPU_COUNT];
 #endif
 
-#if defined(IMX8_SCU_CONTROL)
-    sc_rsrc_t sc_gpu_pid[gcdMAX_GPU_COUNT];
-#endif
+    uint32_t sc_gpu_pid[gcdMAX_GPU_COUNT];
 
     int gpu3dCount;
 
+#ifdef CONFIG_PM
+    int pm_qos_core;
+    struct pm_qos_request pm_qos;
+#endif
+
 #if defined(CONFIG_PM_OPP)
     struct gpu_govern imx_gpu_govern;
 #endif
@@ -493,25 +526,25 @@ static ssize_t gpu_govern_show(struct device_driver *dev, char *buf)
     unsigned long shader_freq;
 
     if (priv->imx_gpu_govern.num_modes == GOVERN_COUNT)
-	    max_modes = priv->imx_gpu_govern.num_modes - 1;
+        max_modes = priv->imx_gpu_govern.num_modes - 1;
     else
-	    max_modes = priv->imx_gpu_govern.num_modes;
+        max_modes = priv->imx_gpu_govern.num_modes;
 
     len = sprintf(buf, "GPU support %d modes\n", priv->imx_gpu_govern.num_modes);
 
 
     for (i = priv->imx_gpu_govern.current_mode; i <= max_modes; i++)
     {
-	    core_freq   = priv->imx_gpu_govern.core_clk_freq[i];
-	    shader_freq = priv->imx_gpu_govern.shader_clk_freq[i];
+        core_freq   = priv->imx_gpu_govern.core_clk_freq[i];
+        shader_freq = priv->imx_gpu_govern.shader_clk_freq[i];
 
-	    len += sprintf(buf + len,
-			    "%s:\tcore_clk frequency: %lu\tshader_clk frequency: %lu\n",
-			    govern_modes[i], core_freq, shader_freq);
+        len += sprintf(buf + len,
+                "%s:\tcore_clk frequency: %lu\tshader_clk frequency: %lu\n",
+                govern_modes[i], core_freq, shader_freq);
     }
 
     len += sprintf(buf + len, "Currently GPU runs on mode %s\n",
-		    govern_modes[priv->imx_gpu_govern.current_mode]);
+        govern_modes[priv->imx_gpu_govern.current_mode]);
 
     return len;
 }
@@ -523,6 +556,14 @@ static ssize_t gpu_govern_store(struct device_driver *dev, const char *buf, size
     struct imx_priv *priv = &imxPriv;
     int core = gcvCORE_MAJOR;
     int i;
+    gckGALDEVICE device = platform_get_drvdata(pdevice);
+    gctBOOL mutex_acquired = gcvFALSE;
+    gceSTATUS status;
+
+    if (!device)
+    {
+        return count;
+    }
 
     for (i = 0; i < GOVERN_COUNT; i++)
     {
@@ -537,6 +578,14 @@ static ssize_t gpu_govern_store(struct device_driver *dev, const char *buf, size
         return count;
     }
 
+    /* Get the device commit mutex. */
+    gcmkONERROR(gckOS_AcquireMutex(device->os, device->device->commitMutex,
+            gcvINFINITE));
+    mutex_acquired = gcvTRUE;
+
+    /* Suspend the GPU. */
+    gcmkONERROR(gckGALDEVICE_Suspend(device, gcvPOWER_SUSPEND));
+
     core_freq   = priv->imx_gpu_govern.core_clk_freq[i];
     shader_freq = priv->imx_gpu_govern.shader_clk_freq[i];
     priv->imx_gpu_govern.current_mode = i;
@@ -549,13 +598,31 @@ static ssize_t gpu_govern_store(struct device_driver *dev, const char *buf, size
         if (clk_core != NULL && clk_shader != NULL &&
             core_freq != 0 && shader_freq != 0)
         {
+#ifdef CONFIG_PM
+            pm_runtime_get_sync(priv->pmdev[core]);
+#endif
             clk_set_rate(clk_core, core_freq);
             clk_set_rate(clk_shader, shader_freq);
+#ifdef CONFIG_PM
+            pm_runtime_put_sync(priv->pmdev[core]);
+#endif
         }
     }
 
+    /* Resume GPU to previous state. */
+    gcmkVERIFY_OK(gckGALDEVICE_Resume(device));
+
+OnError:
+    /* Release the commit mutex. */
+    if (mutex_acquired)
+    {
+        gcmkVERIFY_OK(gckOS_ReleaseMutex(device->os,
+                device->device->commitMutex));
+    }
+
     return count;
 }
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
 static DRIVER_ATTR_RW(gpu_govern);
 #else
@@ -569,14 +636,8 @@ int init_gpu_opp_table(struct device *dev)
     int nr;
     int ret = 0;
     int i, p;
-    int core = gcvCORE_MAJOR;
     struct imx_priv *priv = &imxPriv;
 
-    struct clk *clk_core;
-    struct clk *clk_shader;
-
-    unsigned long core_freq, shader_freq;
-
     priv->imx_gpu_govern.num_modes = 0;
 
     prop = of_find_property(dev->of_node, "operating-points", NULL);
@@ -585,7 +646,7 @@ int init_gpu_opp_table(struct device *dev)
     }
 
     if (!prop->value) {
-	dev_err(dev, "operating-points invalid. Frequency scaling will not work\n");
+        dev_err(dev, "operating-points invalid. Frequency scaling will not work\n");
         return -ENODATA;
     }
 
@@ -607,11 +668,11 @@ int init_gpu_opp_table(struct device *dev)
      * are divisible by 2 (X Y) hence there's no need to test for odd values.
      */
     if (nr < 6)
-	    priv->imx_gpu_govern.current_mode = UNDERDRIVE;
+        priv->imx_gpu_govern.current_mode = UNDERDRIVE;
     else if (nr == 6 || nr == 8)
-	    priv->imx_gpu_govern.current_mode = NOMINAL;
+        priv->imx_gpu_govern.current_mode = NOMINAL;
     else
-	    priv->imx_gpu_govern.current_mode = OVERDRIVE;
+        priv->imx_gpu_govern.current_mode = OVERDRIVE;
 
     val = prop->value;
 
@@ -645,7 +706,7 @@ int init_gpu_opp_table(struct device *dev)
         priv->imx_gpu_govern.shader_clk_freq[i] = shader_freq;
 
         p++;
-	i++;
+    i++;
     }
 
     priv->imx_gpu_govern.num_modes = p;
@@ -656,32 +717,8 @@ int init_gpu_opp_table(struct device *dev)
         ret = driver_create_file(dev->driver, &driver_attr_gpu_govern);
         if (ret) {
             dev_err(dev, "create gpu_govern attr failed (%d)\n", ret);
-	    return ret;
-	}
-
-	/*
-	 * This could be redundant, but it is useful for testing DTS with
-	 * different OPPs that have assigned-clock rates different than the
-	 * ones specified in OPP tuple array. Otherwise we will display
-	 * different clock values when the driver is loaded. Further
-	 * modifications of the governor will display correctly but not when
-	 * the driver has been loaded.
-	 */
-	core_freq = priv->imx_gpu_govern.core_clk_freq[priv->imx_gpu_govern.current_mode];
-	shader_freq = priv->imx_gpu_govern.shader_clk_freq[priv->imx_gpu_govern.current_mode];
-
-	if (core_freq && shader_freq) {
-		for (; core <= gcvCORE_3D_MAX; core++) {
-			clk_core = priv->imx_gpu_clks[core].clk_core;
-			clk_shader = priv->imx_gpu_clks[core].clk_shader;
-
-			if (clk_core != NULL && clk_shader != NULL) {
-				clk_set_rate(clk_core, core_freq);
-				clk_set_rate(clk_shader, shader_freq);
-			}
-		}
-	}
-
+        return ret;
+    }
     }
 
     return ret;
@@ -695,13 +732,13 @@ int remove_gpu_opp_table(void)
     int max_modes;
 
     if (priv->imx_gpu_govern.num_modes == GOVERN_COUNT)
-	    max_modes = priv->imx_gpu_govern.num_modes - 1;
+        max_modes = priv->imx_gpu_govern.num_modes - 1;
     else
-	    max_modes = priv->imx_gpu_govern.num_modes;
+        max_modes = priv->imx_gpu_govern.num_modes;
 
     /* if we don't have any modes available we don't have OPP */
     if (max_modes == 0)
-	    return 0;
+        return 0;
 
     for (i = priv->imx_gpu_govern.current_mode; i <= max_modes; i++)
     {
@@ -731,8 +768,7 @@ static int mxc_gpu_sub_bind(struct device *dev,
     return 0;
 }
 
-static void mxc_gpu_sub_unbind(struct device *dev, struct device *master,
-    void *data)
+static void mxc_gpu_sub_unbind(struct device *dev, struct device *master, void *data)
 {
 }
 
@@ -789,11 +825,13 @@ static int patch_param_imx8_subsystem(struct platform_device *pdev,
 {
     int i = 0;
     struct resource* res;
+    struct imx_priv *priv = &imxPriv;
     struct device_node *node = pdev->dev.of_node;
     struct device_node *core_node;
     int core = gcvCORE_MAJOR;
 
-    while ((core_node = of_parse_phandle(node, "cores", i++)) != NULL) {
+    while ((core_node = of_parse_phandle(node, "cores", i++)) != NULL &&
+            core < gcvCORE_COUNT) {
         struct platform_device *pdev_gpu;
         int irqLine = -1;
 
@@ -807,6 +845,12 @@ static int patch_param_imx8_subsystem(struct platform_device *pdev,
         if (!pdev_gpu)
             break;
 
+#ifdef CONFIG_PM
+        if(of_device_is_compatible(core_node,"fsl,imx8-vipsi")) {
+            priv->pm_qos_core = core;
+        }
+#endif
+
         irqLine = platform_get_irq(pdev_gpu, 0);
 
         if (irqLine < 0)
@@ -817,6 +861,9 @@ static int patch_param_imx8_subsystem(struct platform_device *pdev,
         if (!res)
             break;
 
+        while(!priv->pmdev[core] && core < (gcvCORE_COUNT-1))
+            core++;
+
         args->irqs[core] = irqLine;
         args->registerBases[core] = res->start;
         args->registerSizes[core] = res->end - res->start + 1;
@@ -837,6 +884,7 @@ static inline int get_power_imx8_subsystem(struct device *pdev)
     struct clk *clk_core = NULL;
     struct clk *clk_shader = NULL;
     struct clk *clk_axi = NULL;
+    struct clk *clk_ahb = NULL;
 
     /* Initialize the clock structure */
     int i = 0;
@@ -844,30 +892,12 @@ static inline int get_power_imx8_subsystem(struct device *pdev)
     struct device_node *core_node;
     int core = gcvCORE_MAJOR;
 
-#if defined(IMX8_SCU_CONTROL)
-    sc_err_t sciErr;
-    uint32_t mu_id;
-
-    sciErr = sc_ipc_getMuID(&mu_id);
-
-    if (sciErr != SC_ERR_NONE) {
-        printk("galcore; cannot obtain mu id\n");
-        return -EINVAL;
-    }
-
-    sciErr = sc_ipc_open(&gpu_ipcHandle, mu_id);
-
-    if (sciErr != SC_ERR_NONE) {
-        printk("galcore: cannot open MU channel to SCU\n");
-        return -EINVAL;
-    }
-#endif
-
     while ((core_node = of_parse_phandle(node, "cores", i++)) != NULL) {
         struct platform_device *pdev_gpu = NULL;
         clk_shader = NULL;
         clk_core = NULL;
         clk_axi = NULL;
+        clk_ahb = NULL;
 
         if (!of_device_is_available(core_node)) {
             of_node_put(core_node);
@@ -886,20 +916,25 @@ static inline int get_power_imx8_subsystem(struct device *pdev)
             break;
         }
 
-        clk_axi = clk_get(&pdev_gpu->dev, "bus");
+        clk_axi = clk_get(&pdev_gpu->dev, "axi");
 
         if (IS_ERR(clk_axi))
             clk_axi = NULL;
 
+        clk_ahb = clk_get(&pdev_gpu->dev, "ahb");
+
+        if (IS_ERR(clk_ahb))
+            clk_ahb = NULL;
+
         clk_shader = clk_get(&pdev_gpu->dev, "shader");
 
         if (IS_ERR(clk_shader)) {
-            printk("galcore: clk_get clk_3d_shader failed\n");
-            continue;
+            priv->gpu3dCount = core;
+            core = gcvCORE_2D;
+            clk_shader = NULL;
         }
 
 #if defined(CONFIG_ANDROID) && LINUX_VERSION_CODE < KERNEL_VERSION(4,9,0)
-        /* TODO: freescale BSP issue in some platform like imx8dv. */
         clk_prepare(clk_core);
         clk_set_rate(clk_core, 800000000);
         clk_unprepare(clk_core);
@@ -912,11 +947,37 @@ static inline int get_power_imx8_subsystem(struct device *pdev)
         priv->imx_gpu_clks[core].clk_shader = clk_shader;
         priv->imx_gpu_clks[core].clk_core   = clk_core;
         priv->imx_gpu_clks[core].clk_axi    = clk_axi;
+        priv->imx_gpu_clks[core].clk_ahb    = clk_ahb;
 
-#if defined(IMX8_SCU_CONTROL)
         if (of_property_read_u32(core_node, "fsl,sc_gpu_pid", &priv->sc_gpu_pid[core])) {
             priv->sc_gpu_pid[core] = 0;
         }
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,0,0)
+        else if (!gpu_ipcHandle) {
+            uint32_t ret = imx_scu_get_handle(&gpu_ipcHandle);
+            if (ret != 0) {
+                printk("galcore: cannot open MU channel to SCU\n");
+                return ret;
+            }
+    }
+#elif defined(IMX8_SCU_CONTROL)
+        else if (!gpu_ipcHandle) {
+            sc_err_t sciErr;
+            uint32_t mu_id;
+            sciErr = sc_ipc_getMuID(&mu_id);
+
+            if (sciErr != SC_ERR_NONE) {
+                printk("galcore; cannot obtain mu id\n");
+                return -EINVAL;
+            }
+
+            sciErr = sc_ipc_open(&gpu_ipcHandle, mu_id);
+
+            if (sciErr != SC_ERR_NONE) {
+                printk("galcore: cannot open MU channel to SCU\n");
+                return -EINVAL;
+            }
+        }
 #endif
 
 #ifdef CONFIG_PM
@@ -930,7 +991,8 @@ static inline int get_power_imx8_subsystem(struct device *pdev)
         ++core;
     }
 
-    priv->gpu3dCount = core;
+    if (core <= gcvCORE_3D_MAX)
+        priv->gpu3dCount = core;
 
     if (core_node)
         of_node_put(core_node);
@@ -948,48 +1010,83 @@ static int patch_param_imx6(struct platform_device *pdev,
     res = platform_get_resource_byname(pdev, IORESOURCE_IRQ, "irq_3d");
 
     if (res)
-        args->irqLine = res->start;
+        args->irqs[gcvCORE_MAJOR] = res->start;
 
     res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "iobase_3d");
 
     if (res) {
-        args->registerMemBase = res->start;
-        args->registerMemSize = res->end - res->start + 1;
+        args->registerBases[gcvCORE_MAJOR] = res->start;
+        args->registerSizes[gcvCORE_MAJOR] = res->end - res->start + 1;
     }
 
     res = platform_get_resource_byname(pdev, IORESOURCE_IRQ, "irq_2d");
 
     if (res)
-        args->irqLine2D = res->start;
+        args->irqs[gcvCORE_2D] = res->start;
 
     res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "iobase_2d");
 
     if (res) {
-        args->registerMemBase2D = res->start;
-        args->registerMemSize2D = res->end - res->start + 1;
+        args->registerBases[gcvCORE_2D] = res->start;
+        args->registerSizes[gcvCORE_2D] = res->end - res->start + 1;
     }
 
     res = platform_get_resource_byname(pdev, IORESOURCE_IRQ, "irq_vg");
 
     if (res)
-        args->irqLineVG = res->start;
+        args->irqs[gcvCORE_VG] = res->start;
 
     res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "iobase_vg");
 
     if (res) {
-        args->registerMemBaseVG = res->start;
-        args->registerMemSizeVG = res->end - res->start + 1;
+        args->registerBases[gcvCORE_VG] = res->start;
+        args->registerSizes[gcvCORE_VG] = res->end - res->start + 1;
     }
 
     return 0;
 }
 
+static bool is_layerscape;
+
+static int patch_param_ls(struct platform_device *pdev,
+        gcsMODULE_PARAMETERS *args)
+{
+    struct device_node *node = pdev->dev.of_node;
+    struct resource *res;
+    int core = gcvCORE_MAJOR;
+    struct platform_device *pdev_gpu;
+    int irqLine = -1;
+
+    pdev_gpu = of_find_device_by_node(node);
+    if (!pdev_gpu)
+        return gcvSTATUS_DEVICE;
+
+    of_node_put(node);
+
+    irqLine = platform_get_irq(pdev_gpu, 0);
+    if (irqLine < 0)
+        return gcvSTATUS_NOT_FOUND;
+
+    res = platform_get_resource(pdev_gpu, IORESOURCE_MEM, 0);
+    if (!res)
+        return gcvSTATUS_NOT_FOUND;
+
+    args->irqs[core] = irqLine;
+    args->registerBases[core] = res->start;
+    args->registerSizes[core] = res->end - res->start + 1;
+
+    return 0;
+}
+
 static int patch_param(struct platform_device *pdev,
                 gcsMODULE_PARAMETERS *args)
 {
     struct resource* res;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,1,0)
+    struct device_node *node;
+    struct resource res_mem;
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
     struct device_node *dn =pdev->dev.of_node;
     const u32 *prop;
@@ -999,18 +1096,22 @@ static int patch_param(struct platform_device *pdev,
 
     pdevice = pdev;
 
+    if (is_layerscape) {
+        patch_param_ls(pdev, args);
+    } else {
 #ifdef IMX_GPU_SUBSYSTEM
-    if (pdev->dev.of_node && use_imx_gpu_subsystem)
-        patch_param_imx8_subsystem(pdev, args);
-    else
+        if (pdev->dev.of_node && use_imx_gpu_subsystem)
+            patch_param_imx8_subsystem(pdev, args);
+        else
 #endif
-        patch_param_imx6(pdev, args);
+            patch_param_imx6(pdev, args);
+    }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
-    if(args->compression == -1)
+    if(args->compression == gcvCOMPRESSION_OPTION_DEFAULT)
     {
         const u32 *property;
-        args->compression = gcvCOMPRESSION_OPTION_DEFAULT;
+
         property = of_get_property(pdev->dev.of_node, "depth-compression", NULL);
         if (property && *property == 0)
         {
@@ -1018,6 +1119,7 @@ static int patch_param(struct platform_device *pdev,
         }
     }
 #endif
+
     res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "phys_baseaddr");
 
     if (res && !args->baseAddress && !args->physSize) {
@@ -1026,7 +1128,21 @@ static int patch_param(struct platform_device *pdev,
     }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4,1,0)
-    res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "contiguous_mem");
+    node = of_parse_phandle(pdev->dev.of_node, "memory-region", 0);
+
+    if (node && !of_address_to_resource(node, 0, &res_mem))
+    {
+        res = &res_mem;
+    }
+    else
+    {
+        res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "contiguous_mem");
+    }
+
+    if (node)
+    {
+        of_node_put(node);
+    }
 
     if (res) {
         if (args->contiguousBase == 0)
@@ -1072,7 +1188,7 @@ static int patch_param(struct platform_device *pdev,
 
 #if defined(IMX8_PHYS_SIZE)
         args->physSize = IMX8_PHYS_SIZE;
-#else
+#elif defined(CONFIG_SOC_IMX6Q)
         args->physSize = 0x80000000;
 #endif
     }
@@ -1084,6 +1200,10 @@ int init_priv(void)
 {
     memset(&imxPriv, 0, sizeof(imxPriv));
 
+#ifdef CONFIG_PM
+    imxPriv.pm_qos_core = -1;
+#endif
+
 #ifdef CONFIG_GPU_LOW_MEMORY_KILLER
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4,1,0)
     task_free_register(&task_nb);
@@ -1108,6 +1228,7 @@ free_priv(void)
 }
 
 static int set_clock(int gpu, int enable);
+static int set_power(int gpu, int enable);
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
 static void imx6sx_optimize_qosc_for_GPU(void)
@@ -1122,6 +1243,7 @@ static void imx6sx_optimize_qosc_for_GPU(void)
     src_base = of_iomap(np, 0);
     WARN_ON(!src_base);
 
+    set_power(gcvCORE_MAJOR, 1);
     set_clock(gcvCORE_MAJOR, 1);
 
     writel_relaxed(0, src_base); /* Disable clkgate & soft_rst */
@@ -1131,6 +1253,8 @@ static void imx6sx_optimize_qosc_for_GPU(void)
     writel_relaxed(0x0f000822, src_base+0x1400+0xe0); /* Set Read QoS 8 for gpu */
 
     set_clock(gcvCORE_MAJOR, 0);
+    set_power(gcvCORE_MAJOR, 0);
+
     return;
 }
 #endif
@@ -1265,7 +1389,7 @@ static inline int get_power(struct device *pdev)
 #if gcdENABLE_FSCALE_VAL_ADJUST && (defined(CONFIG_DEVICE_THERMAL) || defined(CONFIG_DEVICE_THERMAL_MODULE))
 #ifndef CONFIG_MXC_GPU_VIV_DISABLE_THERMAL
     REG_THERMAL_NOTIFIER(&thermal_hot_pm_notifier);
-#endif
+#endif /* ifndef CONFIG_MXC_GPU_VIV_DISABLE_THERMAL */
 
     ret = driver_create_file(pdev->driver, &driver_attr_gpu3DMinClock);
 
@@ -1281,7 +1405,7 @@ static inline int get_power(struct device *pdev)
 #if defined(CONFIG_PM_OPP)
     ret = init_gpu_opp_table(pdev);
     if (ret)
-	dev_err(pdev, "OPP init failed!\n");
+        dev_err(pdev, "OPP init failed!\n");
 #endif
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
@@ -1341,7 +1465,7 @@ static inline void put_power(void)
 #if gcdENABLE_FSCALE_VAL_ADJUST && (defined(CONFIG_DEVICE_THERMAL) || defined(CONFIG_DEVICE_THERMAL_MODULE))
 #ifndef CONFIG_MXC_GPU_VIV_DISABLE_THERMAL
     UNREG_THERMAL_NOTIFIER(&thermal_hot_pm_notifier);
-#endif
+#endif /* ifndef CONFIG_MXC_GPU_VIV_DISABLE_THERMAL */
 
     driver_remove_file(pdevice->dev.driver, &driver_attr_gpu3DMinClock);
 
@@ -1352,7 +1476,7 @@ static inline void put_power(void)
     remove_gpu_opp_table();
 #endif
 
-#if defined(IMX8_SCU_CONTROL)
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(5,0,0)) && defined(IMX8_SCU_CONTROL)
     if (gpu_ipcHandle)
         sc_ipc_close(gpu_ipcHandle);
 #endif
@@ -1364,6 +1488,11 @@ static inline int set_power(int gpu, int enable)
     struct imx_priv* priv = &imxPriv;
 #endif
 
+    struct clk *clk_core = priv->imx_gpu_clks[gpu].clk_core;
+    struct clk *clk_shader = priv->imx_gpu_clks[gpu].clk_shader;
+    struct clk *clk_axi = priv->imx_gpu_clks[gpu].clk_axi;
+    struct clk *clk_ahb = priv->imx_gpu_clks[gpu].clk_ahb;
+
     if (enable) {
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,14,0)
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,5,0) || LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
@@ -1380,9 +1509,37 @@ static inline int set_power(int gpu, int enable)
 
 #ifdef CONFIG_PM
         pm_runtime_get_sync(priv->pmdev[gpu]);
+        if(priv->pm_qos_core == gpu) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,7,0)
+            cpu_latency_qos_add_request(&priv->pm_qos, 0);
+#else
+            pm_qos_add_request(&(priv->pm_qos), PM_QOS_CPU_DMA_LATENCY, 0);
+#endif
+        }
 #endif
 
-#if defined(IMX8_SCU_CONTROL)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,0,0)
+        if (priv->sc_gpu_pid[gpu]) {
+            uint32_t ret = imx_sc_misc_set_control(gpu_ipcHandle, priv->sc_gpu_pid[gpu], IMX_SC_C_ID, gpu);
+            if (ret) {
+                printk("galcore: failed to set gpu id for 3d_%d\n", gpu);
+                return ret;
+            }
+            if (priv->gpu3dCount > 1) {
+                ret = imx_sc_misc_set_control(gpu_ipcHandle, priv->sc_gpu_pid[gpu], IMX_SC_C_SINGLE_MODE, 0);
+                if (ret) {
+                    printk("galcore: failed to set gpu dual mode for 3d_%d\n", gpu);
+                    return ret;
+                }
+            } else {
+                ret = imx_sc_misc_set_control(gpu_ipcHandle, priv->sc_gpu_pid[gpu], IMX_SC_C_SINGLE_MODE, 1);
+                if (ret) {
+                printk("galcore: failed to set gpu single mode for 3d_%d\n", gpu);
+                return ret;
+                }
+            }
+        }
+#elif defined(IMX8_SCU_CONTROL)
         if (priv->sc_gpu_pid[gpu]) {
             sc_err_t sciErr = sc_misc_set_control(gpu_ipcHandle, priv->sc_gpu_pid[gpu], SC_C_ID, gpu);
             if (sciErr != SC_ERR_NONE)
@@ -1393,15 +1550,44 @@ static inline int set_power(int gpu, int enable)
                 if (sciErr != SC_ERR_NONE)
                     printk("galcore: failed to set gpu dual mode for 3d_%d\n", gpu);
             } else {
-                 sciErr = sc_misc_set_control(gpu_ipcHandle, priv->sc_gpu_pid[gpu], SC_C_SINGLE_MODE, 1);
-                 if (sciErr != SC_ERR_NONE)
-                     printk("galcore: failed to set gpu single mode for 3d_%d\n", gpu);
+                sciErr = sc_misc_set_control(gpu_ipcHandle, priv->sc_gpu_pid[gpu], SC_C_SINGLE_MODE, 1);
+                if (sciErr != SC_ERR_NONE)
+                    printk("galcore: failed to set gpu single mode for 3d_%d\n", gpu);
             }
         }
 #endif
+        if (clk_core)
+            clk_prepare(clk_core);
+
+        if (clk_shader)
+            clk_prepare(clk_shader);
+
+        if (clk_axi)
+            clk_prepare(clk_axi);
+
+        if (clk_ahb)
+            clk_prepare(clk_ahb);
     } else {
+        if (clk_core)
+            clk_unprepare(clk_core);
+
+        if (clk_shader)
+            clk_unprepare(clk_shader);
+
+        if (clk_axi)
+            clk_unprepare(clk_axi);
+
+        if (clk_ahb)
+            clk_unprepare(clk_ahb);
 #ifdef CONFIG_PM
         pm_runtime_put_sync(priv->pmdev[gpu]);
+        if(priv->pm_qos_core == gpu) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,7,0)
+            cpu_latency_qos_remove_request(&priv->pm_qos);
+#else
+            pm_qos_remove_request(&(priv->pm_qos));
+#endif
+        }
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,14,0)
@@ -1426,19 +1612,6 @@ int set_clock(int gpu, int enable)
     struct clk *clk_ahb = priv->imx_gpu_clks[gpu].clk_ahb;
 
     if (enable) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
-        if (clk_core)
-            clk_prepare(clk_core);
-
-        if (clk_shader)
-            clk_prepare(clk_shader);
-
-        if (clk_axi)
-            clk_prepare(clk_axi);
-
-        if (clk_ahb)
-            clk_prepare(clk_ahb);
-#endif
         if (clk_core)
             clk_enable(clk_core);
 
@@ -1462,20 +1635,6 @@ int set_clock(int gpu, int enable)
 
         if (clk_ahb)
             clk_disable(clk_ahb);
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
-        if (clk_core)
-            clk_unprepare(clk_core);
-
-        if (clk_shader)
-            clk_unprepare(clk_shader);
-
-        if (clk_axi)
-            clk_unprepare(clk_axi);
-
-        if (clk_ahb)
-            clk_unprepare(clk_ahb);
-#endif
     }
 
     return gcvSTATUS_OK;
@@ -1483,19 +1642,19 @@ int set_clock(int gpu, int enable)
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,5,0)
 #ifdef CONFIG_PM
+#ifdef CONFIG_PM_RUNTIME
 static int gpu_runtime_suspend(struct device *dev)
 {
-    dev_dbg(dev, "%s:%d release_bus_freq\n", __func__, __LINE__);
-    release_bus_freq(BUS_FREQ_AUDIO);
+    release_bus_freq(BUS_FREQ_HIGH);
     return 0;
 }
 
 static int gpu_runtime_resume(struct device *dev)
 {
-    dev_dbg(dev, "%s:%d request_bus_freq\n", __func__, __LINE__);
-    request_bus_freq(BUS_FREQ_AUDIO);
+    request_bus_freq(BUS_FREQ_HIGH);
     return 0;
 }
+#endif
 
 static struct dev_pm_ops gpu_pm_ops;
 #endif
@@ -1515,7 +1674,7 @@ static int adjust_platform_driver(struct platform_driver *driver)
     memcpy(&gpu_pm_ops, driver->driver.pm, sizeof(struct dev_pm_ops));
 
     /* Add runtime PM callback. */
-#ifdef CONFIG_PM
+#ifdef CONFIG_PM_RUNTIME
     gpu_pm_ops.runtime_suspend = gpu_runtime_suspend;
     gpu_pm_ops.runtime_resume = gpu_runtime_resume;
     gpu_pm_ops.runtime_idle = NULL;
@@ -1577,8 +1736,13 @@ _AdjustParam(
     patch_param(Platform->device, Args);
 
     if ((of_find_compatible_node(NULL, NULL, "fsl,imx8mq-gpu") ||
-        of_find_compatible_node(NULL, NULL, "fsl,imx8mm-gpu")) &&
+    of_find_compatible_node(NULL, NULL, "fsl,imx8mm-gpu") ||
+    of_find_compatible_node(NULL, NULL, "fsl,imx8mn-gpu")) &&
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,0,0)
+        ((Args->baseAddress + totalram_pages() * PAGE_SIZE) > 0x100000000))
+#else
         ((Args->baseAddress + totalram_pages * PAGE_SIZE) > 0x100000000))
+#endif
     {
         Platform->flagBits |= gcvPLATFORM_FLAG_LIMIT_4G_ADDRESS;
     }
@@ -1653,7 +1817,7 @@ _Reset(
         return gcvSTATUS_INVALID_CONFIG;
 }
 
-struct soc_platform_ops imx_platform_ops =
+struct _gcsPLATFORM_OPERATIONS imx_platform_ops =
 {
     .adjustParam  = _AdjustParam,
     .getPower     = _GetPower,
@@ -1666,14 +1830,28 @@ struct soc_platform_ops imx_platform_ops =
 #endif
 };
 
-static struct soc_platform imx_platform =
+static struct _gcsPLATFORM imx_platform =
 {
     .name = __FILE__,
     .ops  = &imx_platform_ops,
 };
 
-int soc_platform_init(struct platform_driver *pdrv,
-            struct soc_platform **platform)
+static const struct of_device_id gpu_match[] = {
+    { .compatible = "fsl,ls1028a-gpu"},
+    { /* sentinel */ }
+};
+
+struct _gcsPLATFORM_OPERATIONS ls_platform_ops = {
+    .adjustParam  = _AdjustParam,
+};
+
+static struct _gcsPLATFORM ls_platform = {
+    .name = __FILE__,
+    .ops  = &ls_platform_ops,
+};
+
+int gckPLATFORM_Init(struct platform_driver *pdrv,
+            struct _gcsPLATFORM **platform)
 {
 #ifdef IMX_GPU_SUBSYSTEM
     if (of_find_compatible_node(NULL, NULL, "fsl,imx8-gpu-ss")) {
@@ -1686,6 +1864,14 @@ int soc_platform_init(struct platform_driver *pdrv,
     }
 #endif
 
+    if (of_find_compatible_node(NULL, NULL, "fsl,ls1028a-gpu")) {
+        is_layerscape = 1;
+        pdrv->driver.of_match_table = gpu_match;
+        *platform = &ls_platform;
+
+        return 0;
+    }
+
     adjust_platform_driver(pdrv);
     init_priv();
 
@@ -1694,15 +1880,20 @@ int soc_platform_init(struct platform_driver *pdrv,
 #endif
 
     *platform = &imx_platform;
+#ifdef GALCORE_BUILD_BY
+    printk("module built by %s at %s", GALCORE_BUILD_BY, GALCORE_BUILD_TM);
+#endif
     return 0;
 }
 
-int soc_platform_terminate(struct soc_platform *platform)
+int gckPLATFORM_Terminate(struct _gcsPLATFORM *platform)
 {
+    if (is_layerscape)
+        return 0;
+
 #ifdef IMX_GPU_SUBSYSTEM
     unregister_mxc_gpu_sub_driver();
 #endif
-
     free_priv();
     return 0;
 }
