@@ -19,6 +19,7 @@
  *    http://www.opensource.org/licenses/gpl-license.html
  *    http://www.gnu.org/copyleft/gpl.html
  *****************************************************************************/
+
 #include <linux/hantrodec.h>
 #include "dwl_defs.h"
 #include <linux/io.h>
@@ -47,14 +48,14 @@
 #include <linux/platform_device.h>
 #include <linux/pm_runtime.h>
 #include <linux/clk.h>
-#include <linux/busfreq-imx.h>
+#include <linux/compat.h>
+//#include <linux/busfreq-imx.h>
 
 #ifdef CONFIG_DEVICE_THERMAL
-#include <linux/device_cooling.h>
-#define HANTRO_REG_THERMAL_NOTIFIER(a) register_devfreq_cooling_notifier(a)
-#define HANTRO_UNREG_THERMAL_NOTIFIER(a) unregister_devfreq_cooling_notifier(a)
+#include <linux/thermal.h>
 DEFINE_SPINLOCK(thermal_lock);
 /*1:hot, 0: not hot*/
+#define HANTRO_COOLING_MAX_STATE 1
 static int thermal_event;
 static int thermal_cur;
 static int hantro_clock_ratio = 2;
@@ -145,7 +146,7 @@ static struct device *hantro_dev;
 static struct clk *hantro_clk_g1;
 static struct clk *hantro_clk_g2;
 static struct clk *hantro_clk_bus;
-static struct regulator *hantro_regulator;
+//static struct regulator *hantro_regulator;
 
 static int hantro_dbg = -1;
 module_param(hantro_dbg, int, 0644);
@@ -171,6 +172,7 @@ typedef struct {
 	int cores;
 	struct fasync_struct *async_queue_dec;
 	struct fasync_struct *async_queue_pp;
+	struct thermal_cooling_device *cooling;
 } hantrodec_t;
 
 static hantrodec_t hantrodec_data; /* dynamic allocation? */
@@ -224,6 +226,7 @@ DECLARE_WAIT_QUEUE_HEAD(hw_queue);
 static u32 cfg[HXDEC_MAX_CORES];
 static u32 timeout;
 
+#if 0
 static int hantro_update_voltage(struct device *dev)
 {
 	unsigned long new_vol, old_vol;
@@ -251,6 +254,7 @@ static int hantro_update_voltage(struct device *dev)
 	}
 	return 0;
 }
+#endif
 
 static int hantro_clk_enable(struct device *dev)
 {
@@ -286,7 +290,7 @@ static int hantro_ctrlblk_reset(struct device *dev)
 
 	//config G1/G2
 	hantro_clk_enable(dev);
-	iobase = (volatile u8 *)ioremap_nocache(BLK_CTL_BASE, 0x10000);
+	iobase = (volatile u8 *)ioremap(BLK_CTL_BASE, 0x10000);
 	iowrite32(0x3, iobase);  //VPUMIX G1/G2 block soft reset  control
 	iowrite32(0x3, iobase+4); //VPUMIX G1/G2 block clock enable control
 	iowrite32(0xFFFFFFFF, iobase + 0x8); // all G1 fuse dec enable
@@ -327,24 +331,37 @@ static int hantro_thermal_check(struct device *dev)
 	}
 	pr_info("hantro: event(%d), g1, g2, bus clock: %ld, %ld, %ld\n", thermal_cur,
 		clk_get_rate(hantro_clk_g1),	clk_get_rate(hantro_clk_g2), clk_get_rate(hantro_clk_bus));
-	hantro_update_voltage(dev);
+	//hantro_update_voltage(dev);
 	return 0;
 }
 
-static int hantro_thermal_hot_notify(struct notifier_block *nb, unsigned long event, void *dummy)
+static int hantro_cooling_get_max_state(struct thermal_cooling_device *cdev, unsigned long *state)
+{
+	*state = HANTRO_COOLING_MAX_STATE;
+	return 0;
+}
+
+static int hantro_cooling_get_cur_state(struct thermal_cooling_device *cdev, unsigned long *state)
+{
+	*state = thermal_event;
+	return 0;
+}
+
+static int hantro_cooling_set_cur_state(struct thermal_cooling_device *cdev, unsigned long state)
 {
 	unsigned long flags;
 
 	spin_lock_irqsave(&thermal_lock, flags);
-	thermal_event = event;		/*event: 1: hot, 0: cool*/
+	thermal_event = state;		/*event: 1: hot, 0: cool*/
 	spin_unlock_irqrestore(&thermal_lock, flags);
-	pr_info("hantro receive hot notification event: %ld\n", event);
-
-	return NOTIFY_OK;
+	pr_info("hantro receive cooling set state: %ld\n", state);
+	return 0;
 }
 
-static struct notifier_block hantro_thermal_hot_notifier = {
-	.notifier_call = hantro_thermal_hot_notify,
+static struct thermal_cooling_device_ops hantro_cooling_ops = {
+	.get_max_state = hantro_cooling_get_max_state,
+	.get_cur_state = hantro_cooling_get_cur_state,
+	.set_cur_state = hantro_cooling_set_cur_state,
 };
 #endif  //CONFIG_DEVICE_THERMAL
 
@@ -414,7 +431,7 @@ static void ReadCoreConfig(hantrodec_t *dev)
 			cfg[c] |= tmp ? 1 << DWL_CLIENT_TYPE_RV_DEC : 0;
 
 			/* Post-processor configuration */
-			reg = ioread32(dev->hwregs[c] + HANTROPP_SYNTH_CFG * 4);
+			//reg = ioread32(dev->hwregs[c] + HANTROPP_SYNTH_CFG * 4);
 		} else {
 			reg = ioread32(dev->hwregs[c] + HANTRODEC_SYNTH_CFG_2 * 4);
 
@@ -461,7 +478,8 @@ int GetDecCore(long Core, hantrodec_t *dev, struct file *filp)
 }
 
 int GetDecCoreAny(long *Core, hantrodec_t *dev, struct file *filp,
-			unsigned long format) {
+			unsigned long format)
+{
 	int success = 0;
 	long c;
 
@@ -479,7 +497,8 @@ int GetDecCoreAny(long *Core, hantrodec_t *dev, struct file *filp,
 	return success;
 }
 int GetDecCoreID(hantrodec_t *dev, struct file *filp,
-			unsigned long format) {
+			unsigned long format)
+{
 	long c;
 
 	int core_id = -1;
@@ -506,12 +525,10 @@ static int hantrodec_choose_core(int is_g1)
 		return -EBUSY;
 	}
 
-	reg = (volatile u8 *) ioremap_nocache(blk_base, 0x1000);
+	reg = (volatile u8 *) ioremap(blk_base, 0x1000);
 
 	if (reg == NULL) {
 		pr_err("blk_ctl: failed to ioremap HW regs\n");
-		if (reg)
-			iounmap((void *)reg);
 		release_mem_region(blk_base, 0x1000);
 		return -EBUSY;
 	}
@@ -1028,9 +1045,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 	 * "write" is reversed
 	 */
 	if (_IOC_DIR(cmd) & _IOC_READ)
-		err = !access_ok(VERIFY_WRITE, (void *) arg, _IOC_SIZE(cmd));
+		err = !access_ok((void *) arg, _IOC_SIZE(cmd));
 	else if (_IOC_DIR(cmd) & _IOC_WRITE)
-		err = !access_ok(VERIFY_READ, (void *) arg, _IOC_SIZE(cmd));
+		err = !access_ok((void *) arg, _IOC_SIZE(cmd));
 
 	if (err)
 		return -EFAULT;
@@ -1098,6 +1115,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= hantrodec_data.cores)
+			return -EFAULT;
+
 		DecFlushRegs(&hantrodec_data, &Core);
 		break;
 	}
@@ -1111,6 +1131,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= hantrodec_data.cores)
+			return -EFAULT;
+
 		PPFlushRegs(&hantrodec_data, &Core);
 		break;
 	}
@@ -1124,6 +1147,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= hantrodec_data.cores)
+			return -EFAULT;
+
 		return DecRefreshRegs(&hantrodec_data, &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCS_PP_PULL_REG): {
@@ -1136,6 +1162,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= hantrodec_data.cores)
+			return -EFAULT;
+
 		return PPRefreshRegs(&hantrodec_data, &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCH_DEC_RESERVE): {
@@ -1176,6 +1205,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= hantrodec_data.cores)
+			return -EFAULT;
+
 		return WaitDecReadyAndRefreshRegs(&hantrodec_data, &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCX_PP_WAIT): {
@@ -1188,6 +1220,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= hantrodec_data.cores)
+			return -EFAULT;
+
 		return WaitPPReadyAndRefreshRegs(&hantrodec_data, &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCG_CORE_WAIT): {
@@ -1224,6 +1259,7 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			PDEBUG("hantrodec: pp_core[%li]  %s\n",
 					tmp, pp_owner[tmp] == NULL ? "FREE" : "RESERVED");
 		}
+		break;
 	}
 	default:
 		return -ENOTTY;
@@ -1243,7 +1279,7 @@ static int get_hantro_core_desc32(struct core_desc *kp, struct core_desc_32 __us
 {
 	u32 tmp;
 
-	if (!access_ok(VERIFY_READ, up, sizeof(struct core_desc_32)) ||
+	if (!access_ok(up, sizeof(struct core_desc_32)) ||
 				get_user(kp->id, &up->id) ||
 				get_user(kp->size, &up->size) ||
 				get_user(tmp, &up->regs)) {
@@ -1257,7 +1293,7 @@ static int put_hantro_core_desc32(struct core_desc *kp, struct core_desc_32 __us
 {
 	u32 tmp = (u32)((unsigned long)kp->regs);
 
-	if (!access_ok(VERIFY_WRITE, up, sizeof(struct core_desc_32)) ||
+	if (!access_ok(up, sizeof(struct core_desc_32)) ||
 				put_user(kp->id, &up->id) ||
 				put_user(kp->size, &up->size) ||
 				put_user(tmp, &up->regs)) {
@@ -1268,12 +1304,11 @@ static int put_hantro_core_desc32(struct core_desc *kp, struct core_desc_32 __us
 static long hantrodec_ioctl32(struct file *filp, unsigned int cmd, unsigned long arg)
 {
 #define HANTRO_IOCTL32(err, filp, cmd, arg) { \
-		mm_segment_t old_fs = get_fs(); \
-		set_fs(KERNEL_DS); \
+		mm_segment_t old_fs = force_uaccess_begin(); \
 		err = hantrodec_ioctl(filp, cmd, arg); \
 		if (err) \
 			return err; \
-		set_fs(old_fs); \
+		force_uaccess_end(old_fs); \
 	}
 
 	union {
@@ -1595,7 +1630,7 @@ static int ReserveIO(void)
 				return -EBUSY;
 			}
 
-			hantrodec_data.hwregs[i] = (volatile u8 *) ioremap_nocache(multicorebase[i],
+			hantrodec_data.hwregs[i] = (volatile u8 *) ioremap(multicorebase[i],
 			hantrodec_data.iosize[i]);
 
 			if (hantrodec_data.hwregs[i] == NULL) {
@@ -1770,16 +1805,19 @@ static int hantro_dev_probe(struct platform_device *pdev)
 	pr_debug("hantro: g1, g2, bus clock: 0x%lX, 0x%lX, 0x%lX\n", clk_get_rate(hantro_clk_g1),
 				clk_get_rate(hantro_clk_g2), clk_get_rate(hantro_clk_bus));
 
+#if 0
 	hantro_regulator = regulator_get(&pdev->dev, "regulator");
 	if (IS_ERR(hantro_regulator)) {
 		pr_err("hantro: get regulator failed\n");
 		return -ENODEV;
 	}
 	hantro_update_voltage(&pdev->dev);
-
+#endif
 	hantro_clk_enable(&pdev->dev);
 	pm_runtime_enable(&pdev->dev);
+
 	pm_runtime_get_sync(&pdev->dev);
+
 	hantro_ctrlblk_reset(&pdev->dev);
 
 	err = hantrodec_init(pdev);
@@ -1800,7 +1838,10 @@ static int hantro_dev_probe(struct platform_device *pdev)
 	}
 
 #ifdef CONFIG_DEVICE_THERMAL
-	HANTRO_REG_THERMAL_NOTIFIER(&hantro_thermal_hot_notifier);
+	hantrodec_data.cooling = thermal_of_cooling_device_register(pdev->dev.of_node,
+		(char *)dev_name(&pdev->dev), &hantrodec_data, &hantro_cooling_ops);
+	if (IS_ERR(hantrodec_data.cooling))
+		goto err_out_class;
 	thermal_event = 0;
 	thermal_cur = 0;
 	hantro_dynamic_clock = 0;
@@ -1824,6 +1865,9 @@ static int hantro_dev_remove(struct platform_device *pdev)
 	hantro_clk_enable(&pdev->dev);
 	pm_runtime_get_sync(&pdev->dev);
 	if (hantrodec_major > 0) {
+#ifdef CONFIG_DEVICE_THERMAL
+		thermal_cooling_device_unregister(hantrodec_data.cooling);
+#endif
 		device_destroy(hantro_class, MKDEV(hantrodec_major, 0));
 		class_destroy(hantro_class);
 		hantrodec_cleanup();
@@ -1833,9 +1877,12 @@ static int hantro_dev_remove(struct platform_device *pdev)
 	pm_runtime_disable(&pdev->dev);
 	hantro_clk_disable(&pdev->dev);
 
-#ifdef CONFIG_DEVICE_THERMAL
-	HANTRO_UNREG_THERMAL_NOTIFIER(&hantro_thermal_hot_notifier);
-#endif
+	if (!IS_ERR(hantro_clk_g1))
+		clk_put(hantro_clk_g1);
+	if (!IS_ERR(hantro_clk_g2))
+		clk_put(hantro_clk_g2);
+	if (!IS_ERR(hantro_clk_bus))
+		clk_put(hantro_clk_bus);
 
 	return 0;
 }
@@ -1854,13 +1901,13 @@ static int hantro_resume(struct device *dev)
 }
 static int hantro_runtime_suspend(struct device *dev)
 {
-	release_bus_freq(BUS_FREQ_HIGH);
+//	release_bus_freq(BUS_FREQ_HIGH);
 	return 0;
 }
 
 static int hantro_runtime_resume(struct device *dev)
 {
-	request_bus_freq(BUS_FREQ_HIGH);
+//	request_bus_freq(BUS_FREQ_HIGH);
 	hantro_ctrlblk_reset(dev);
 	return 0;
 }
@@ -1899,14 +1946,11 @@ static int __init hantro_init(void)
 
 static void __exit hantro_exit(void)
 {
-	if (!IS_ERR(hantro_clk_g1))
-		clk_put(hantro_clk_g1);
-	if (!IS_ERR(hantro_clk_g2))
-		clk_put(hantro_clk_g2);
-	if (!IS_ERR(hantro_clk_bus))
-		clk_put(hantro_clk_bus);
+
+#if 0
 	if (!IS_ERR(hantro_regulator))
 		regulator_put(hantro_regulator);
+#endif
 	platform_driver_unregister(&mxchantro_driver);
 }
 
