@@ -1,7 +1,7 @@
 /*****************************************************************************
  *    The GPL License (GPL)
  *
- *    Copyright (c) 2015-2018, VeriSilicon Inc.
+ *    Copyright (c) 2015-2020, VeriSilicon Inc.
  *    Copyright (c) 2011-2014, Google Inc.
  *
  *    This program is free software; you can redistribute it and/or
@@ -48,17 +48,15 @@
 #include <linux/pm_runtime.h>
 #include <linux/clk.h>
 #include <linux/busfreq-imx.h>
-
+#include <linux/compat.h>
 #include <linux/delay.h>
 
 //#define CONFIG_DEVICE_THERMAL_HANTRO
 #ifdef CONFIG_DEVICE_THERMAL_HANTRO
-#include <linux/device_cooling.h>
-#define HANTRO_REG_THERMAL_NOTIFIER(a) register_devfreq_cooling_notifier(a)
-#define HANTRO_UNREG_THERMAL_NOTIFIER(a) unregister_devfreq_cooling_notifier(a)
+#include <linux/thermal.h>
 static DEFINE_SPINLOCK(thermal_lock);
 /*1:hot, 0: not hot*/
-static int thermal_event;
+#define HANTRO_COOLING_MAX_STATE 1
 static int hantro_clock_ratio = 2;
 static int hantro_dynamic_clock;
 module_param(hantro_clock_ratio, int, 0644);
@@ -98,6 +96,8 @@ MODULE_PARM_DESC(hantro_dynamic_clock, "enable or disable dynamic clock rate");
 
 #define DEC_IO_SIZE_MAX             (MAX(HANTRO_G2_DEC_REGS, HANTRO_G1_TOTAL_REGS) * 4)
 
+#define HANTRO_CORE_ID_INVALID		(-1)
+
 /********************************************************************
  *                                              PORTING SEGMENT
  * NOTES: customer should modify these configuration if do porting to own platform.
@@ -107,8 +107,6 @@ MODULE_PARM_DESC(hantro_dynamic_clock, "enable or disable dynamic clock rate");
 #define HXDEC_MAX_CORES                 2
 
 /* Logic module base address */
-#define SOCLE_LOGIC_0_BASE              0x38300000
-#define SOCLE_LOGIC_1_BASE              0x38310000
 #define BLK_CTL_BASE                          0x38330000 //0x38320000
 
 #define VEXPRESS_LOGIC_0_BASE           0xFC010000
@@ -132,11 +130,7 @@ static const int DecHwId[] = {
 	0x6732 /* G2 */
 };
 
-static ulong multicorebase[HXDEC_MAX_CORES] = {
-	SOCLE_LOGIC_0_BASE,
-	SOCLE_LOGIC_1_BASE
-};
-
+static ulong multicorebase[HXDEC_MAX_CORES];
 
 static struct class *hantro_class;
 #define DEVICE_NAME		"mxc_hantro"
@@ -161,7 +155,7 @@ MODULE_PARM_DESC(hantro_dbg, "Debug level (0-1)");
 
 
 static int hantrodec_major;
-static int cores = 2;
+static int cores;
 /* here's all the must remember stuff */
 typedef struct {
 	//char *buffer;
@@ -169,6 +163,7 @@ typedef struct {
 	volatile u8 *hwregs;
 	int irq;
 	int hw_id;
+	int hw_active;
 	int core_id;
 	//int cores;
 	//struct fasync_struct *async_queue_dec;
@@ -186,13 +181,15 @@ typedef struct {
 	struct device *dev;
 	struct mutex dev_mutex;
 	int thermal_cur;
+	int thermal_event;
+	struct thermal_cooling_device *cooling;
 } hantrodec_t;
 
 static hantrodec_t hantrodec_data[HXDEC_MAX_CORES]; /* dynamic allocation? */
 
 typedef struct {
-	char inst_id;
-	char core_id;	//1:g1; 2:g2; 3:unknow
+	int inst_id;
+	int core_id;
 } hantrodec_instance;
 static unsigned long instance_mask;
 #define MAX_HANTRODEC_INSTANCE 32
@@ -237,17 +234,14 @@ static DECLARE_WAIT_QUEUE_HEAD(hw_queue);
 
 static int hantro_device_id(struct device *dev)
 {
-	int id;
-
-	if (strcmp("vpu_g1", dev->of_node->name) == 0) {
-		id = 0;
-	} else if (strcmp("vpu_g2", dev->of_node->name) == 0) {
-		id = 1;
-	} else {
-		return id = -1;
+	if (strcmp("vpu_g1", dev->of_node->name) == 0 ||
+	    strcmp("vpu_g2", dev->of_node->name) == 0) {
+		return cores;
 	}
-	return id;
+
+	return HANTRO_CORE_ID_INVALID;
 }
+
 static int hantro_clk_enable(hantrodec_clk *clk)
 {
 	clk_prepare(clk->dec);
@@ -273,8 +267,8 @@ static int hantro_ctrlblk_reset(hantrodec_t *dev)
 
 	//config G1/G2
 	hantro_clk_enable(&dev->clk);
-	iobase = (volatile u8 *)ioremap_nocache(BLK_CTL_BASE, 0x10000);
-	if (dev->core_id == 0) {
+	iobase = (volatile u8 *)ioremap(BLK_CTL_BASE, 0x10000);
+	if (IS_G1(dev->hw_id)) {
 		val = ioread32(iobase);
 		val &= (~0x2);
 		iowrite32(val, iobase);  //assert G1 block soft reset  control
@@ -322,23 +316,26 @@ static int hantro_power_on_disirq(hantrodec_t *hantrodev)
 static int hantro_new_instance(void)
 {
 	int idx;
+	unsigned long flags;
 
-	spin_lock(&owner_lock);
+	spin_lock_irqsave(&owner_lock, flags);
 	if (instance_mask  == ((1UL << MAX_HANTRODEC_INSTANCE) - 1)) {
-		spin_unlock(&owner_lock);
+		spin_unlock_irqrestore(&owner_lock, flags);
 		return -1;
 	}
 	idx = ffz(instance_mask);
 	set_bit(idx, &instance_mask);
-	spin_unlock(&owner_lock);
+	spin_unlock_irqrestore(&owner_lock, flags);
 	return idx;
 }
 
 static int hantro_free_instance(int idx)
 {
-	spin_lock(&owner_lock);
+	unsigned long flags;
+
+	spin_lock_irqsave(&owner_lock, flags);
 	clear_bit(idx, &instance_mask);
-	spin_unlock(&owner_lock);
+	spin_unlock_irqrestore(&owner_lock, flags);
 
 	return 0;
 }
@@ -350,12 +347,12 @@ static int hantro_thermal_check(struct device *dev)
 	unsigned long flags;
 
 	spin_lock_irqsave(&thermal_lock, flags);
-	if (thermal_event == hantrodev->thermal_cur) {
+	if (hantrodev->thermal_event == hantrodev->thermal_cur) {
 		/*nothing to do and return directly*/
 		spin_unlock_irqrestore(&thermal_lock, flags);
 		return 0;
 	}
-	hantrodev->thermal_cur = thermal_event;
+	hantrodev->thermal_cur = hantrodev->thermal_event;
 	spin_unlock_irqrestore(&thermal_lock, flags);
 
 	if (hantrodev->thermal_cur) {
@@ -375,20 +372,36 @@ static int hantro_thermal_check(struct device *dev)
 	return 0;
 }
 
-static int hantro_thermal_hot_notify(struct notifier_block *nb, unsigned long event, void *dummy)
+static int hantro_cooling_get_max_state(struct thermal_cooling_device *cdev, unsigned long *state)
 {
-	unsigned long flags;
-
-	spin_lock_irqsave(&thermal_lock, flags);
-	thermal_event = event;		/*event: 1: hot, 0: cool*/
-	spin_unlock_irqrestore(&thermal_lock, flags);
-	pr_info("hantro receive hot notification event: %ld\n", event);
-
-	return NOTIFY_OK;
+	*state = HANTRO_COOLING_MAX_STATE;
+	return 0;
 }
 
-static struct notifier_block hantro_thermal_hot_notifier = {
-	.notifier_call = hantro_thermal_hot_notify,
+static int hantro_cooling_get_cur_state(struct thermal_cooling_device *cdev, unsigned long *state)
+{
+	hantrodec_t *hantrodev = cdev->devdata;
+
+	*state = hantrodev->thermal_event;
+	return 0;
+}
+
+static int hantro_cooling_set_cur_state(struct thermal_cooling_device *cdev, unsigned long state)
+{
+	unsigned long flags;
+	hantrodec_t *hantrodev = cdev->devdata;
+
+	spin_lock_irqsave(&thermal_lock, flags);
+	hantrodev->thermal_event = state;		/*event: 1: hot, 0: cool*/
+	spin_unlock_irqrestore(&thermal_lock, flags);
+	pr_info("hantro receive cooling set state: %ld\n", state);
+	return 0;
+}
+
+static struct thermal_cooling_device_ops hantro_cooling_ops = {
+	.get_max_state = hantro_cooling_get_max_state,
+	.get_cur_state = hantro_cooling_get_cur_state,
+	.set_cur_state = hantro_cooling_set_cur_state,
 };
 #endif  //CONFIG_DEVICE_THERMAL_HANTRO
 
@@ -458,7 +471,7 @@ static void ReadCoreConfig(hantrodec_t *dev)
 			dev->cfg |= tmp ? 1 << DWL_CLIENT_TYPE_RV_DEC : 0;
 
 			/* Post-processor configuration */
-			reg = ioread32(dev->hwregs + HANTROPP_SYNTH_CFG * 4);
+			//reg = ioread32(dev->hwregs + HANTROPP_SYNTH_CFG * 4);
 		} else {
 			reg = ioread32(dev->hwregs + HANTRODEC_SYNTH_CFG_2 * 4);
 
@@ -505,7 +518,8 @@ static int GetDecCore(hantrodec_t *dev, struct file *filp)
 }
 
 static int GetDecCoreAny(long *Core, hantrodec_t *dev, struct file *filp,
-			unsigned long format) {
+			unsigned long format)
+{
 	int success = 0;
 	//long c;
 
@@ -527,7 +541,7 @@ static int GetDecCoreID(unsigned long format)
 {
 	long c;
 
-	int core_id = -1;
+	int core_id = HANTRO_CORE_ID_INVALID;
 
 	for (c = 0; c < cores; c++) {
 		/* a Core that has format */
@@ -551,7 +565,7 @@ static int hantrodec_choose_core(int is_g1)
 		return -EBUSY;
 	}
 
-	reg = (volatile u8 *) ioremap_nocache(blk_base, 0x1000);
+	reg = (volatile u8 *) ioremap(blk_base, 0x1000);
 
 	if (reg == NULL) {
 		pr_err("blk_ctl: failed to ioremap HW regs\n");
@@ -769,6 +783,8 @@ static long DecFlushRegs(hantrodec_t *dev, struct core_desc *Core)
 			iowrite32(dev->dec_regs[i], dev->hwregs + i*4);
 	}
 
+	if (dev->dec_regs[1] & 0x1)
+		dev->hw_active = 1;
 	/* write the status register, which may start the decoder */
 	iowrite32(dev->dec_regs[1], dev->hwregs + 4);
 
@@ -843,6 +859,49 @@ static long DecRefreshRegs(hantrodec_t *dev, struct core_desc *Core)
 	return 0;
 }
 
+static long DecRestoreRegs(hantrodec_t *dev)
+{
+	long i;
+
+	if (IS_G1(dev->hw_id)) {
+		/* write dec regs to hardware */
+		/* both original and extended regs need to be written */
+		for (i = 1; i <= HANTRO_DEC_ORG_LAST_REG; i++)
+			iowrite32(dev->dec_regs[i], dev->hwregs + i*4);
+#ifdef USE_64BIT_ENV
+		for (i = HANTRO_DEC_EXT_FIRST_REG; i <= HANTRO_DEC_EXT_LAST_REG; i++)
+			iowrite32(dev->dec_regs[i], dev->hwregs + i*4);
+#endif
+	} else {
+		/* write all regs to hardware */
+		for (i = 1; i <= HANTRO_G2_DEC_LAST_REG; i++)
+			iowrite32(dev->dec_regs[i], dev->hwregs + i*4);
+	}
+	return 0;
+}
+
+static long DecStoreRegs(hantrodec_t *dev)
+{
+	long i;
+
+	if (IS_G1(dev->hw_id)) {
+		/* read all registers from hardware */
+		/* both original and extended regs need to be read */
+		for (i = 0; i <= HANTRO_DEC_ORG_LAST_REG; i++)
+			dev->dec_regs[i] = ioread32(dev->hwregs + i*4);
+#ifdef USE_64BIT_ENV
+		for (i = HANTRO_DEC_EXT_FIRST_REG; i <= HANTRO_DEC_EXT_LAST_REG; i++)
+			dev->dec_regs[i] = ioread32(dev->hwregs + i*4);
+#endif
+	} else {
+		/* read all registers from hardware */
+		for (i = 0; i <= HANTRO_G2_DEC_LAST_REG; i++)
+			dev->dec_regs[i] = ioread32(dev->hwregs + i*4);
+	}
+	return 0;
+}
+
+
 static int CheckDecIrq(hantrodec_t *dev)
 {
 	unsigned long flags;
@@ -1085,9 +1144,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 	 * "write" is reversed
 	 */
 	if (_IOC_DIR(cmd) & _IOC_READ)
-		err = !access_ok(VERIFY_WRITE, (void *) arg, _IOC_SIZE(cmd));
+		err = !access_ok((void *) arg, _IOC_SIZE(cmd));
 	else if (_IOC_DIR(cmd) & _IOC_WRITE)
-		err = !access_ok(VERIFY_READ, (void *) arg, _IOC_SIZE(cmd));
+		err = !access_ok((void *) arg, _IOC_SIZE(cmd));
 
 	if (err)
 		return -EFAULT;
@@ -1155,6 +1214,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= cores)
+			return -EFAULT;
+
 		DecFlushRegs(&hantrodec_data[Core.id], &Core);
 		break;
 	}
@@ -1168,6 +1230,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= cores)
+			return -EFAULT;
+
 		PPFlushRegs(&hantrodec_data[Core.id], &Core);
 		break;
 	}
@@ -1181,6 +1246,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= cores)
+			return -EFAULT;
+
 		return DecRefreshRegs(&hantrodec_data[Core.id], &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCS_PP_PULL_REG): {
@@ -1193,6 +1261,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= cores)
+			return -EFAULT;
+
 		return PPRefreshRegs(&hantrodec_data[Core.id], &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCH_DEC_RESERVE): {
@@ -1240,6 +1311,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= cores)
+			return -EFAULT;
+
 		return WaitDecReadyAndRefreshRegs(&hantrodec_data[Core.id], &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCX_PP_WAIT): {
@@ -1252,6 +1326,9 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			return -EFAULT;
 		}
 
+		if (Core.id >= cores)
+			return -EFAULT;
+
 		return WaitPPReadyAndRefreshRegs(&hantrodec_data[Core.id], &Core);
 	}
 	case _IOC_NR(HANTRODEC_IOCG_CORE_WAIT): {
@@ -1277,14 +1354,15 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 
 		PDEBUG("Get DEC Core_id, format = %li\n", arg);
 		id = GetDecCoreID(arg);
-		if ((ctx->core_id == 3) && (id >= 0)) {
-			if (id == 0) {
-				ctx->core_id = 1; //g1
+		if (id == HANTRO_CORE_ID_INVALID || id >= cores)
+			return -EFAULT;
+		if (ctx->core_id == HANTRO_CORE_ID_INVALID) {
+			ctx->core_id = id;
+			if (id == 0 && cores > 1) {
 				/*power off g2*/
 				pm_runtime_put_sync(hantrodec_data[1].dev);
 				hantro_clk_disable(&hantrodec_data[1].clk);
 			} else if (id == 1) {
-				ctx->core_id = 2; //g2
 				/*power off g1*/
 				pm_runtime_put_sync(hantrodec_data[0].dev);
 				hantro_clk_disable(&hantrodec_data[0].clk);
@@ -1308,6 +1386,7 @@ static long hantrodec_ioctl(struct file *filp, unsigned int cmd, unsigned long a
 			PDEBUG("hantrodec: pp_core[%li]  %s\n",
 					tmp, hantrodec_data[tmp].pp_owner == NULL ? "FREE" : "RESERVED");
 		}
+		break;
 	}
 	default:
 		return -ENOTTY;
@@ -1327,7 +1406,7 @@ static int get_hantro_core_desc32(struct core_desc *kp, struct core_desc_32 __us
 {
 	u32 tmp;
 
-	if (!access_ok(VERIFY_READ, up, sizeof(struct core_desc_32)) ||
+	if (!access_ok(up, sizeof(struct core_desc_32)) ||
 				get_user(kp->id, &up->id) ||
 				get_user(kp->size, &up->size) ||
 				get_user(tmp, &up->regs)) {
@@ -1341,7 +1420,7 @@ static int put_hantro_core_desc32(struct core_desc *kp, struct core_desc_32 __us
 {
 	u32 tmp = (u32)((unsigned long)kp->regs);
 
-	if (!access_ok(VERIFY_WRITE, up, sizeof(struct core_desc_32)) ||
+	if (!access_ok(up, sizeof(struct core_desc_32)) ||
 				put_user(kp->id, &up->id) ||
 				put_user(kp->size, &up->size) ||
 				put_user(tmp, &up->regs)) {
@@ -1352,12 +1431,11 @@ static int put_hantro_core_desc32(struct core_desc *kp, struct core_desc_32 __us
 static long hantrodec_ioctl32(struct file *filp, unsigned int cmd, unsigned long arg)
 {
 #define HANTRO_IOCTL32(err, filp, cmd, arg) { \
-		mm_segment_t old_fs = get_fs(); \
-		set_fs(KERNEL_DS); \
+		mm_segment_t old_fs = force_uaccess_begin(); \
 		err = hantrodec_ioctl(filp, cmd, arg); \
 		if (err) \
 			return err; \
-		set_fs(old_fs); \
+		force_uaccess_end(old_fs); \
 	}
 
 	union {
@@ -1426,12 +1504,12 @@ static int hantrodec_open(struct inode *inode, struct file *filp)
 		return -ENOMEM;
 
 	PDEBUG("dev opened: id: %d\n", idx);
-	hantrodec_ctx[idx].core_id = 3;  //unknow
+	hantrodec_ctx[idx].core_id = HANTRO_CORE_ID_INVALID;
 	hantrodec_ctx[idx].inst_id = idx;
 	filp->private_data = (void *)(&hantrodec_ctx[idx]);
 
 	/*not yet know which core id, so power on both g1 and g2 firstly*/
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < cores; i++) {
 		hantro_clk_enable(&hantrodec_data[i].clk);
 		hantro_power_on_disirq(&hantrodec_data[i]);
 	}
@@ -1451,32 +1529,31 @@ static int hantrodec_release(struct inode *inode, struct file *filp)
 	//hantrodec_t *dev = &hantrodec_data;
 	hantrodec_instance *ctx = (hantrodec_instance *)filp->private_data;
 
-	PDEBUG("closing ...\n");
+	PDEBUG("closing core_id:%d...\n", ctx->core_id);
 	for (n = 0; n < cores; n++) {
 		if (hantrodec_data[n].dec_owner == filp) {
 			PDEBUG("releasing dec Core %i lock\n", n);
 			ReleaseDecoder(&hantrodec_data[n]);
 		}
-	}
 
-	for (n = 0; n < 1; n++) {
 		if (hantrodec_data[n].pp_owner == filp) {
 			PDEBUG("releasing pp Core %i lock\n", n);
 			ReleasePostProcessor(&hantrodec_data[n]);
 		}
 	}
 
-	if (ctx->core_id & 0x1) {
-		pm_runtime_put_sync(hantrodec_data[0].dev);
-		hantro_clk_disable(&hantrodec_data[0].clk);
-	}
-	if (ctx->core_id & 0x2) {
-		pm_runtime_put_sync(hantrodec_data[1].dev);
-		hantro_clk_disable(&hantrodec_data[1].clk);
+	if (ctx->core_id == HANTRO_CORE_ID_INVALID) {
+		for (n = 0; n < cores; n++) {
+			pm_runtime_put_sync(hantrodec_data[n].dev);
+			hantro_clk_disable(&hantrodec_data[n].clk);
+		}
+	} else if (ctx->core_id < HXDEC_MAX_CORES) {
+		pm_runtime_put_sync(hantrodec_data[ctx->core_id].dev);
+		hantro_clk_disable(&hantrodec_data[ctx->core_id].clk);
 	}
+
 	hantro_free_instance(ctx->inst_id);
 
-	PDEBUG("closed: id: %d\n", n);
 	return 0;
 }
 
@@ -1705,7 +1782,7 @@ static int ReserveIO(int i)
 				return -EBUSY;
 			}
 
-			hantrodec_data[i].hwregs = (volatile u8 *) ioremap_nocache(multicorebase[i],
+			hantrodec_data[i].hwregs = (volatile u8 *) ioremap(multicorebase[i],
 			hantrodec_data[i].iosize);
 
 			if (hantrodec_data[i].hwregs == NULL) {
@@ -1775,6 +1852,7 @@ static irqreturn_t hantrodec_isr(int irq, void *dev_id)
 			iowrite32(irq_status_dec, hwregs + HANTRODEC_IRQ_STAT_DEC_OFF);
 
 			PDEBUG("decoder IRQ received! Core %d\n", dev->core_id);
+			dev->hw_active = 0;
 
 			atomic_inc(&hantrodec_data[dev->core_id].irq_rx);
 
@@ -1855,12 +1933,13 @@ static int hantro_dev_probe(struct platform_device *pdev)
 {
 	int err = 0;
 	struct resource *res;
-	unsigned long reg_base;
 	int id;
 
 	id = hantro_device_id(&pdev->dev);
-	if (id < 0)
+	if (id == HANTRO_CORE_ID_INVALID || id >= HXDEC_MAX_CORES) {
+		pr_err("hantro: invalid id: %d\n", id);
 		return -ENODEV;
+	}
 
 	hantrodec_data[id].dev = &pdev->dev;
 	hantrodec_data[id].core_id = id;
@@ -1870,11 +1949,7 @@ static int hantro_dev_probe(struct platform_device *pdev)
 		pr_err("hantro: unable to get vpu base addr\n");
 		return -ENODEV;
 	}
-	reg_base = res->start;
-	if ((ulong)reg_base != multicorebase[id]) {
-		pr_err("hantrodec %d: regbase(0x%lX) not equal to expected value(0x%lX)\n", id, reg_base, multicorebase[id]);
-		return -ENODEV;
-	}
+	multicorebase[id] = res->start;
 
 	hantrodec_data[id].clk.dec = clk_get(&pdev->dev, "clk_hantro");
 	hantrodec_data[id].clk.bus = clk_get(&pdev->dev, "clk_hantro_bus");
@@ -1885,10 +1960,7 @@ static int hantro_dev_probe(struct platform_device *pdev)
 	pr_debug("hantro: dec, bus clock: 0x%lX, 0x%lX\n", clk_get_rate(hantrodec_data[id].clk.dec),
 				clk_get_rate(hantrodec_data[id].clk.bus));
 
-	hantro_clk_enable(&hantrodec_data[id].clk);
 	pm_runtime_enable(&pdev->dev);
-	pm_runtime_get_sync(&pdev->dev);
-	hantro_ctrlblk_reset(&hantrodec_data[id]);
 
 	err = hantrodec_init(pdev, id);
 	if (err != 0) {
@@ -1897,22 +1969,24 @@ static int hantro_dev_probe(struct platform_device *pdev)
 	}
 
 #ifdef CONFIG_DEVICE_THERMAL_HANTRO
-	HANTRO_REG_THERMAL_NOTIFIER(&hantro_thermal_hot_notifier);
-	thermal_event = 0;
+	hantrodec_data[id].cooling = thermal_of_cooling_device_register(pdev->dev.of_node,
+		(char *)dev_name(&pdev->dev), &hantrodec_data[id], &hantro_cooling_ops);
+	if (IS_ERR(hantrodec_data[id].cooling))
+		pr_err("hantro[%d] register cooling device failed\n", id);
+	hantrodec_data[id].thermal_event = 0;
 	hantro_dynamic_clock = 0;
 	hantrodec_data[id].thermal_cur = 0;
 #endif
 	hantrodec_data[id].timeout = 0;
 	mutex_init(&hantrodec_data[id].dev_mutex);
 	instance_mask = 0;
+	cores += 1;
 
 	goto out;
 
 error:
 	pr_err("hantro probe failed\n");
 out:
-	pm_runtime_put_sync(&pdev->dev);
-	hantro_clk_disable(&hantrodec_data[id].clk);
 	return err;
 }
 
@@ -1922,39 +1996,56 @@ static int hantro_dev_remove(struct platform_device *pdev)
 
 	hantro_clk_enable(&dev->clk);
 	pm_runtime_get_sync(&pdev->dev);
-
-	hantrodec_cleanup(dev->core_id);
-#if 1 // FIXME: need to identify core id
-	if (hantrodec_major > 0) {
-		device_destroy(hantro_class, MKDEV(hantrodec_major, 0));
-		class_destroy(hantro_class);
-		unregister_chrdev(hantrodec_major, "hantrodec");
-		hantrodec_major = 0;
-	}
+#ifdef CONFIG_DEVICE_THERMAL_HANTRO
+	thermal_cooling_device_unregister(hantrodec_data[dev->core_id].cooling);
 #endif
+	hantrodec_cleanup(dev->core_id);
 	pm_runtime_put_sync(&pdev->dev);
 	pm_runtime_disable(&pdev->dev);
 	hantro_clk_disable(&dev->clk);
+	if (!IS_ERR(dev->clk.dec))
+		clk_put(dev->clk.dec);
+	if (!IS_ERR(dev->clk.bus))
+		clk_put(dev->clk.bus);
 
-#ifdef CONFIG_DEVICE_THERMAL_HANTRO
-	HANTRO_UNREG_THERMAL_NOTIFIER(&hantro_thermal_hot_notifier);
-#endif
+	if (--cores < 0) {
+		pr_err("hantro: decrease cores count incorrect.\n");
+		cores = 0;
+	}
 
 	return 0;
 }
 
 #ifdef CONFIG_PM
-static int hantro_suspend(struct device *dev)
+static int __maybe_unused hantro_suspend(struct device *dev)
 {
+	hantrodec_t *hantrodev = dev_get_drvdata(dev);
+
+	if (hantrodev->dec_owner) {
+		/* polling until hw is idle */
+		while (hantrodev->hw_active) {
+			pr_info("DEC[%d] is still in active when suspend !\n", hantrodev->core_id);
+			usleep_range(5000, 10000);
+		}
+		/* let's backup all registers from H/W to shadow register to support suspend */
+		DecStoreRegs(hantrodev);
+	}
+
 	pm_runtime_put_sync_suspend(dev);   //power off
 	return 0;
 }
-static int hantro_resume(struct device *dev)
+static int __maybe_unused hantro_resume(struct device *dev)
 {
 	hantrodec_t *hantrodev = dev_get_drvdata(dev);
 
 	hantro_power_on_disirq(hantrodev);
 	hantro_ctrlblk_reset(hantrodev);
+
+	if (hantrodev->dec_owner) {
+		/* let's restore registers from shadow register to H/W to support resume */
+		DecRestoreRegs(hantrodev);
+	}
+
 	return 0;
 }
 static int hantro_runtime_suspend(struct device *dev)
@@ -2006,8 +2097,13 @@ static int __init hantro_init(void)
 
 static void __exit hantro_exit(void)
 {
-	//clk_put(hantro_clk);
 	platform_driver_unregister(&mxchantro_driver);
+	if (hantrodec_major > 0) {
+		device_destroy(hantro_class, MKDEV(hantrodec_major, 0));
+		class_destroy(hantro_class);
+		unregister_chrdev(hantrodec_major, "hantrodec");
+		hantrodec_major = 0;
+	}
 }
 
 module_init(hantro_init);
