@@ -1,7 +1,7 @@
 /*
  * Encoder device driver (kernel module)
  *
- * Copyright (c) 2013-2018, VeriSilicon Inc.
+ * Copyright (c) 2013-2019, VeriSilicon Inc.
  * Copyright (C) 2012 Google Finland Oy.
  *
  * This program is free software; you can redistribute it and/or
@@ -72,6 +72,7 @@ static struct clk *hantro_clk_h1_bus;
 #endif
 
 #include <linux/delay.h>
+#include <linux/compat.h>
 
 /* module description */
 MODULE_LICENSE("GPL");
@@ -138,9 +139,12 @@ typedef struct {
 	int irq;
 	unsigned long iobaseaddr;
 	unsigned int iosize;
+	u32 reg_corrupt;
+	struct semaphore core_suspend_sem;
 
 	volatile u8 *hwregs;
 	struct fasync_struct *async_queue;
+	unsigned int mirror_regs[512];
 	struct device *dev;
 	struct mutex dev_mutex;
 } hx280enc_t;
@@ -189,7 +193,7 @@ static int hantro_h1_ctrlblk_reset(struct device *dev)
 
 	//config H1
 	hantro_h1_clk_enable(dev);
-	iobase = (volatile u8 *)ioremap_nocache(BLK_CTL_BASE, 0x10000);
+	iobase = (volatile u8 *)ioremap(BLK_CTL_BASE, 0x10000);
 
 	val = ioread32(iobase);
 	val &= (~0x4);
@@ -272,13 +276,31 @@ static int CheckEncIrq(hx280enc_t *dev)
 
 unsigned int WaitEncReady(hx280enc_t *dev)
 {
-	PDEBUG("WaitEncReady\n");
+	u32 irq_status, is_write1_clr;
+	int i;
+	long ret;
 
-	if (wait_event_interruptible(enc_wait_queue, CheckEncIrq(dev))) {
-		PDEBUG("ENC wait_event_interruptible interrupted\n");
-		return -ERESTARTSYS;
+	PDEBUG("%s\n", __func__);
+	ret = wait_event_timeout(enc_wait_queue, CheckEncIrq(dev), msecs_to_jiffies(200));
+	if (ret == 0) {
+		u32 reg14 = readl(dev->hwregs + 14*4);
+
+		pr_err("%s: wait_event_timeout() timeout !\n", __func__);
+		writel(reg14 & (~1), dev->hwregs + 14*4);
 	}
 
+	/* read register to mirror */
+	for (i = 0; i < dev->iosize; i += 4)
+		dev->mirror_regs[i/4] = readl(dev->hwregs + i);
+
+	/* clear the status bits */
+	is_write1_clr = (dev->mirror_regs[0x4a0/4] & 0x00800000);
+	irq_status = dev->mirror_regs[1];
+	if (is_write1_clr)
+		writel(irq_status, dev->hwregs + 0x04);
+	else
+		writel(irq_status & (~0xf7d), dev->hwregs + 0x04);
+
 	return 0;
 }
 
@@ -338,9 +360,74 @@ void ReleaseEncoder(hx280enc_t *dev)
 	spin_unlock_irqrestore(&owner_lock, flags);
 
 	wake_up_interruptible_all(&enc_hw_queue);
-
 }
 
+static long EncRefreshRegs(hx280enc_t *dev, unsigned int *regs)
+{
+	long ret;
+
+	ret = copy_to_user(regs, dev->mirror_regs, dev->iosize);
+	if (ret) {
+		PDEBUG("%s: copy_to_user failed, returned %li\n", __func__, ret);
+		return -EFAULT;
+	}
+	return 0;
+}
+
+static int hx280enc_write_regs(unsigned long arg)
+{
+	struct enc_regs_buffer regs;
+	hx280enc_t *dev = &hx280enc_data;
+	u32 *reg_buf;
+	u32 i;
+	int ret;
+
+	ret = copy_from_user(&regs, (void *)arg, sizeof(regs));
+	if (ret)
+		return ret;
+	if ((regs.offset + regs.size) > sizeof(dev->mirror_regs)) {
+		pr_err("%s invalid param, offset:%d, size:%d\n",
+			__func__, regs.offset, regs.size);
+		return -EINVAL;
+	}
+
+	reg_buf = &dev->mirror_regs[regs.offset / 4];
+	ret = copy_from_user((void *)reg_buf, (void *)regs.regs, regs.size);
+	if (ret)
+		return ret;
+
+	for (i = 0; i < regs.size / 4; i++)
+		iowrite32(reg_buf[i], (dev->hwregs + regs.offset) + i * 4);
+
+	return ret;
+}
+
+static int hx280enc_read_regs(unsigned long arg)
+{
+	struct enc_regs_buffer regs;
+	hx280enc_t *dev = &hx280enc_data;
+	u32 *reg_buf;
+	u32 i;
+	int ret;
+
+	ret = copy_from_user(&regs, (void *)arg, sizeof(regs));
+	if (ret)
+		return ret;
+	if ((regs.offset + regs.size) > sizeof(dev->mirror_regs)) {
+		pr_err("%s invalid param, offset:%d, size:%d\n",
+			__func__, regs.offset, regs.size);
+		return -EINVAL;
+	}
+
+	reg_buf = &dev->mirror_regs[regs.offset / 4];
+
+	for (i = 0; i < regs.size / 4; i++)
+		reg_buf[i] = ioread32((dev->hwregs + regs.offset) + i * 4);
+
+	ret = copy_to_user((void *)regs.regs, (void *)reg_buf, regs.size);
+
+	return ret;
+}
 
 static long hx280enc_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 {
@@ -363,9 +450,9 @@ static long hx280enc_ioctl(struct file *filp, unsigned int cmd, unsigned long ar
 	* "write" is reversed
 	*/
 	if (_IOC_DIR(cmd) & _IOC_READ)
-		err = !access_ok(VERIFY_WRITE, (void *) arg, _IOC_SIZE(cmd));
+		err = !access_ok((void *) arg, _IOC_SIZE(cmd));
 	else if (_IOC_DIR(cmd) & _IOC_WRITE)
-		err = !access_ok(VERIFY_READ, (void *) arg, _IOC_SIZE(cmd));
+		err = !access_ok((void *) arg, _IOC_SIZE(cmd));
 	if (err)
 		return -EFAULT;
 
@@ -388,11 +475,53 @@ static long hx280enc_ioctl(struct file *filp, unsigned int cmd, unsigned long ar
 		ReleaseEncoder(&hx280enc_data);
 		break;
 	case _IOC_NR(HX280ENC_IOCG_CORE_WAIT): {
-		int ret;
+		unsigned int *regs = (unsigned int *)arg;
+		unsigned int ret1, ret2;
 
-		ret = WaitEncReady(&hx280enc_data);
-		return ret;
+		ret1 = WaitEncReady(&hx280enc_data);
+		ret2 = EncRefreshRegs(&hx280enc_data, regs);
+		if (ret2)
+			return ret2;
+		if (ret1)
+			return ret1;
+		break;
 	}
+	case _IOC_NR(HX280ENC_IOC_WRITE_REGS): {
+		err = hx280enc_write_regs(arg);
+		if (err)
+			return err;
+		break;
+	}
+	case _IOC_NR(HX280ENC_IOC_READ_REGS): {
+		err = hx280enc_read_regs(arg);
+		if (err)
+			return err;
+		break;
+	}
+	case _IOC_NR(HX280ENC_IOCG_EN_CORE): {
+		u32 reg_value;
+
+		PDEBUG("Enable ENC Core\n");
+
+		if (hx280enc_data.is_reserved == 0)
+			return -EPERM;
+		if (hx280enc_data.reg_corrupt == 1) {
+			hx280enc_data.reg_corrupt = 0;
+			return -EAGAIN;
+		}
+
+		if (down_interruptible(&hx280enc_data.core_suspend_sem))
+			return -ERESTARTSYS;
+
+		reg_value = readl(hx280enc_data.hwregs + 14 * 4);
+		reg_value |= 0x01;
+		writel(reg_value, hx280enc_data.hwregs + 14 * 4);
+		// printk("%s reg_value:%d\n", __func__, reg_value);
+
+		break;
+	}
+	default:
+		break;
 	}
 	return 0;
 }
@@ -441,16 +570,16 @@ static int hx280enc_release(struct inode *inode, struct file *filp)
 	return 0;
 }
 
+#ifdef CONFIG_COMPAT
 static long hx280enc_ioctl32(struct file *filp, unsigned int cmd, unsigned long arg)
 {
     long err = 0;
 #define HX280ENC_IOCTL32(err, filp, cmd, arg) { \
-	mm_segment_t old_fs = get_fs(); \
-	set_fs(KERNEL_DS); \
+	mm_segment_t old_fs = force_uaccess_begin(); \
 	err = hx280enc_ioctl(filp, cmd, arg); \
 	if (err) \
 	return err; \
-	set_fs(old_fs); \
+	force_uaccess_end(old_fs); \
 }
 
 union {
@@ -485,14 +614,24 @@ union {
 	    ReleaseEncoder(&hx280enc_data);
 	    break;
 	}
-    case _IOC_NR(HX280ENC_IOCG_CORE_WAIT): {
-	    int ret;
-	    ret = WaitEncReady(&hx280enc_data);
-	    return ret;
+	case _IOC_NR(HX280ENC_IOCG_CORE_WAIT): {
+		HX280ENC_IOCTL32(err, filp, cmd, (unsigned long)up);
+		break;
 	}
+	case _IOC_NR(HX280ENC_IOC_WRITE_REGS): {
+		HX280ENC_IOCTL32(err, filp, cmd, (unsigned long)up);
+		break;
 	}
-    return 0;
+	case _IOC_NR(HX280ENC_IOC_READ_REGS): {
+		HX280ENC_IOCTL32(err, filp, cmd, (unsigned long)up);
+		break;
+	}
+	default:
+		break;
+	}
+	return 0;
 }
+#endif
 
 /* VFS methods */
 static struct file_operations hx280enc_fops = {
@@ -523,6 +662,7 @@ static int __init hx280enc_init(void)
 	hx280enc_data.irq = irq;
 	hx280enc_data.async_queue = NULL;
 	hx280enc_data.hwregs = NULL;
+	sema_init(&hx280enc_data.core_suspend_sem, 1);
 
 	result = register_chrdev(hx280enc_major, "hx280enc", &hx280enc_fops);
 	if (result < 0) {
@@ -601,7 +741,7 @@ static int ReserveIO(void)
 		PDEBUG(KERN_INFO "hx280enc: failed to reserve HW regs\n");
 		return -EBUSY;
 	}
-	hx280enc_data.hwregs = (volatile u8 *) ioremap_nocache(hx280enc_data.iobaseaddr, hx280enc_data.iosize);
+	hx280enc_data.hwregs = (volatile u8 *) ioremap(hx280enc_data.iobaseaddr, hx280enc_data.iosize);
 	if (hx280enc_data.hwregs == NULL)	{
 		PDEBUG(KERN_INFO "hx280enc: failed to ioremap HW regs\n");
 		ReleaseIO();
@@ -677,7 +817,10 @@ irqreturn_t hx280enc_isr(int irq, void *dev_id)
 		dev->irq_status = irq_status & (~0x01);
 		spin_unlock_irqrestore(&owner_lock, flags);
 
-		wake_up_interruptible_all(&enc_wait_queue);
+		if (irq_status & 0x04)
+			up(&hx280enc_data.core_suspend_sem);
+
+		wake_up_all(&enc_wait_queue);
 
 		PDEBUG("IRQ handled!\n");
 		return IRQ_HANDLED;
@@ -808,21 +951,67 @@ static int hantro_h1_dev_remove(struct platform_device *pdev)
 	pm_runtime_disable(&pdev->dev);
 	hantro_h1_clk_disable(&pdev->dev);
 
+	if (!IS_ERR(hantro_clk_h1))
+		clk_put(hantro_clk_h1);
+	if (!IS_ERR(hantro_clk_h1_bus))
+		clk_put(hantro_clk_h1_bus);
+
 	return 0;
 }
 
-#ifdef CONFIG_PM
-static int hantro_h1_suspend(struct device *dev)
+static int hx280enc_suspend(struct device *dev)
 {
+	int ret = 0;
+	int i;
+
+	PDEBUG("%s start..\n", __func__);
+
+	if (hx280enc_data.is_reserved == 0)
+		return ret;
+
+	ret = down_interruptible(&hx280enc_data.core_suspend_sem);
+	if (ret)
+		return ret;
+	hx280enc_data.reg_corrupt = 1;
+	if (hx280enc_data.irq_status & 0x04) {
+		for (i = 0; i < hx280enc_data.iosize; i += 4)
+			hx280enc_data.mirror_regs[i/4] = readl(hx280enc_data.hwregs + i);
+	}
+	up(&hx280enc_data.core_suspend_sem);
+
+	return ret;
+}
+
+static int hx280enc_resume(struct device *dev)
+{
+	int ret = 0;
+	int i;
+
+	PDEBUG("%s start..\n", __func__);
+
+	if (hx280enc_data.is_reserved == 0 || hx280enc_data.reg_corrupt == 0)
+		return ret;
+
+	for (i = 0; i < hx280enc_data.iosize; i += 4)
+		writel(hx280enc_data.mirror_regs[i/4], hx280enc_data.hwregs + i);
+
+	return ret;
+}
+
+#ifdef CONFIG_PM
+static int __maybe_unused hantro_h1_suspend(struct device *dev)
+{
+	hx280enc_suspend(dev);
 	pm_runtime_put_sync_suspend(dev);   //power off
 	return 0;
 }
-static int hantro_h1_resume(struct device *dev)
+static int __maybe_unused hantro_h1_resume(struct device *dev)
 {
 	hx280enc_t *hx280enc = dev_get_drvdata(dev);
 
 	hantro_h1_power_on_disirq(hx280enc);
 	hantro_h1_ctrlblk_reset(dev);
+	hx280enc_resume(dev);
 	return 0;
 }
 static int hantro_h1_runtime_suspend(struct device *dev)
@@ -872,7 +1061,6 @@ static int __init hantro_h1_init(void)
 
 static void __exit hantro_h1_exit(void)
 {
-	//clk_put(hantro_clk);
 	platform_driver_unregister(&mxchantro_h1_driver);
 }
 
