@@ -1,5 +1,6 @@
 /*
  * Copyright 2016 Mans Rullgard <mans@mansr.com>
+ * Copyright (c) Siemens AG, 2016-2020
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,11 +16,13 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <linux/ivshmem.h>
 #include <linux/kernel.h>
 #include <linux/module.h>
 #include <linux/pci.h>
 #include <linux/io.h>
 #include <linux/bitops.h>
+#include <linux/ethtool.h>
 #include <linux/interrupt.h>
 #include <linux/netdevice.h>
 #include <linux/etherdevice.h>
@@ -28,34 +31,27 @@
 
 #define DRV_NAME "ivshmem-net"
 
-#define JAILHOUSE_CFG_SHMEM_PTR	0x40
-#define JAILHOUSE_CFG_SHMEM_SZ	0x48
+#define IVSHM_NET_STATE_UNKNOWN		(~0)
+#define IVSHM_NET_STATE_RESET		0
+#define IVSHM_NET_STATE_INIT		1
+#define IVSHM_NET_STATE_READY		2
+#define IVSHM_NET_STATE_RUN		3
 
-#define IVSHMEM_INTX_ENABLE	0x1
+#define IVSHM_NET_FLAG_RUN		0
 
-#define IVSHM_NET_STATE_RESET	0
-#define IVSHM_NET_STATE_INIT	1
-#define IVSHM_NET_STATE_READY	2
-#define IVSHM_NET_STATE_RUN	3
-
-#define IVSHM_NET_FLAG_RUN	0
-
-#define IVSHM_NET_MTU_MIN 256
-#define IVSHM_NET_MTU_MAX 65535
-#define IVSHM_NET_MTU_DEF 16384
+#define IVSHM_NET_MTU_DEF		16384
 
 #define IVSHM_NET_FRAME_SIZE(s) ALIGN(18 + (s), SMP_CACHE_BYTES)
 
 #define IVSHM_NET_VQ_ALIGN 64
 
-struct ivshmem_regs {
-	u32 intxctrl;
-	u32 istat;
-	u32 ivpos;
-	u32 doorbell;
-	u32 lstate;
-	u32 rstate;
-};
+#define IVSHM_NET_SECTION_TX		0
+#define IVSHM_NET_SECTION_RX		1
+
+#define IVSHM_NET_MSIX_STATE		0
+#define IVSHM_NET_MSIX_TX_RX		1
+
+#define IVSHM_NET_NUM_VECTORS		2
 
 struct ivshm_net_queue {
 	struct vring vr;
@@ -73,7 +69,7 @@ struct ivshm_net_queue {
 };
 
 struct ivshm_net_stats {
-	u32 interrupts;
+	u32 tx_rx_interrupts;
 	u32 tx_packets;
 	u32 tx_notify;
 	u32 tx_pause;
@@ -92,13 +88,12 @@ struct ivshm_net {
 	u32 qlen;
 	u32 qsize;
 
-	spinlock_t tx_free_lock;
-	spinlock_t tx_clean_lock;
-
 	struct napi_struct napi;
 
-	u32 lstate;
-	u32 rstate;
+	struct mutex state_lock;
+	u32 state;
+	u32 last_peer_state;
+	u32 *state_table;
 
 	unsigned long flags;
 
@@ -107,17 +102,19 @@ struct ivshm_net {
 
 	struct ivshm_net_stats stats;
 
-	struct ivshmem_regs __iomem *ivshm_regs;
-	void *shm;
-	phys_addr_t shmaddr;
+	struct ivshm_regs __iomem *ivshm_regs;
+	void *shm[2];
 	resource_size_t shmlen;
 	u32 peer_id;
 
+	u32 tx_rx_vector;
+
 	struct pci_dev *pdev;
 };
 
 static void *ivshm_net_desc_data(struct ivshm_net *in,
 				 struct ivshm_net_queue *q,
+				 unsigned int region,
 				 struct vring_desc *desc,
 				 u32 *len)
 {
@@ -132,7 +129,7 @@ static void *ivshm_net_desc_data(struct ivshm_net *in,
 	if (offs >= in->shmlen)
 		return NULL;
 
-	data = in->shm + offs;
+	data = in->shm[region] + offs;
 
 	if (data < q->data || data >= q->end)
 		return NULL;
@@ -160,18 +157,17 @@ static void ivshm_net_init_queue(struct ivshm_net *in,
 static void ivshm_net_init_queues(struct net_device *ndev)
 {
 	struct ivshm_net *in = netdev_priv(ndev);
-	int ivpos = readl(&in->ivshm_regs->ivpos);
 	void *tx;
 	void *rx;
 	int i;
 
-	tx = in->shm +  ivpos * in->shmlen / 2;
-	rx = in->shm + !ivpos * in->shmlen / 2;
+	tx = in->shm[IVSHM_NET_SECTION_TX];
+	rx = in->shm[IVSHM_NET_SECTION_RX];
 
-	memset(tx, 0, in->shmlen / 2);
+	memset(tx, 0, in->shmlen);
 
-	ivshm_net_init_queue(in, &in->rx, rx, in->qlen);
 	ivshm_net_init_queue(in, &in->tx, tx, in->qlen);
+	ivshm_net_init_queue(in, &in->rx, rx, in->qlen);
 
 	swap(in->rx.vr.used, in->tx.vr.used);
 
@@ -191,16 +187,16 @@ static int ivshm_net_calc_qsize(struct net_device *ndev)
 	for (qlen = 4096; qlen > 32; qlen >>= 1) {
 		vrsize = vring_size(qlen, IVSHM_NET_VQ_ALIGN);
 		vrsize = ALIGN(vrsize, IVSHM_NET_VQ_ALIGN);
-		if (vrsize < in->shmlen / 16)
+		if (vrsize < in->shmlen / 8)
 			break;
 	}
 
-	if (vrsize > in->shmlen / 2)
+	if (vrsize > in->shmlen)
 		return -EINVAL;
 
-	qsize = in->shmlen / 2 - vrsize;
+	qsize = in->shmlen - vrsize;
 
-	if (qsize < 4 * IVSHM_NET_MTU_MIN)
+	if (qsize < 4 * ETH_MIN_MTU)
 		return -EINVAL;
 
 	in->vrsize = vrsize;
@@ -221,7 +217,8 @@ static void ivshm_net_notify_tx(struct ivshm_net *in, unsigned int num)
 	new = in->tx.last_avail_idx;
 
 	if (vring_need_event(evt, new, old)) {
-		writel(in->peer_id << 16, &in->ivshm_regs->doorbell);
+		writel(in->tx_rx_vector | (in->peer_id << 16),
+		       &in->ivshm_regs->doorbell);
 		in->stats.tx_notify++;
 	}
 }
@@ -238,12 +235,13 @@ static void ivshm_net_notify_rx(struct ivshm_net *in, unsigned int num)
 
 	virt_mb();
 
-	evt = vring_used_event(&in->rx.vr);
+	evt = READ_ONCE(vring_used_event(&in->rx.vr));
 	old = in->rx.last_used_idx - num;
 	new = in->rx.last_used_idx;
 
 	if (vring_need_event(evt, new, old)) {
-		writel(in->peer_id << 16, &in->ivshm_regs->doorbell);
+		writel(in->tx_rx_vector | (in->peer_id << 16),
+		       &in->ivshm_regs->doorbell);
 		in->stats.rx_notify++;
 	}
 }
@@ -275,10 +273,12 @@ static size_t ivshm_net_tx_space(struct ivshm_net *in)
 	return space;
 }
 
-static bool ivshm_net_tx_ok(struct ivshm_net *in, unsigned int mtu)
+static bool ivshm_net_tx_ok(struct net_device *ndev)
 {
+	struct ivshm_net *in = netdev_priv(ndev);
+
 	return in->tx.num_free >= 2 &&
-		ivshm_net_tx_space(in) >= 2 * IVSHM_NET_FRAME_SIZE(mtu);
+		ivshm_net_tx_space(in) >= 2 * IVSHM_NET_FRAME_SIZE(ndev->mtu);
 }
 
 static u32 ivshm_net_tx_advance(struct ivshm_net_queue *q, u32 *pos, u32 len)
@@ -294,49 +294,7 @@ static u32 ivshm_net_tx_advance(struct ivshm_net_queue *q, u32 *pos, u32 len)
 	return p;
 }
 
-static int ivshm_net_tx_frame(struct net_device *ndev, struct sk_buff *skb)
-{
-	struct ivshm_net *in = netdev_priv(ndev);
-	struct ivshm_net_queue *tx = &in->tx;
-	struct vring *vr = &tx->vr;
-	struct vring_desc *desc;
-	unsigned int desc_idx;
-	unsigned int avail;
-	u32 head;
-	void *buf;
-
-	BUG_ON(tx->num_free < 1);
-
-	spin_lock(&in->tx_free_lock);
-	desc_idx = tx->free_head;
-	desc = &vr->desc[desc_idx];
-	tx->free_head = desc->next;
-	tx->num_free--;
-	spin_unlock(&in->tx_free_lock);
-
-	head = ivshm_net_tx_advance(tx, &tx->head, skb->len);
-
-	buf = tx->data + head;
-	skb_copy_and_csum_dev(skb, buf);
-
-	desc->addr = buf - in->shm;
-	desc->len = skb->len;
-	desc->flags = 0;
-
-	avail = tx->last_avail_idx++ & (vr->num - 1);
-	vr->avail->ring[avail] = desc_idx;
-	tx->num_added++;
-
-	if (!skb->xmit_more) {
-		virt_store_release(&vr->avail->idx, tx->last_avail_idx);
-		ivshm_net_notify_tx(in, tx->num_added);
-		tx->num_added = 0;
-	}
-
-	return 0;
-}
-
-static void ivshm_net_tx_clean(struct net_device *ndev)
+static bool ivshm_net_tx_clean(struct net_device *ndev)
 {
 	struct ivshm_net *in = netdev_priv(ndev);
 	struct ivshm_net_queue *tx = &in->tx;
@@ -344,22 +302,16 @@ static void ivshm_net_tx_clean(struct net_device *ndev)
 	struct vring *vr = &tx->vr;
 	struct vring_desc *desc;
 	struct vring_desc *fdesc;
+	u16 last = tx->last_used_idx;
 	unsigned int num;
-	u16 used_idx;
-	u16 last;
+	bool tx_ok;
 	u32 fhead;
 
-	if (!spin_trylock(&in->tx_clean_lock))
-		return;
-
-	used_idx = virt_load_acquire(&vr->used->idx);
-	last = tx->last_used_idx;
-
 	fdesc = NULL;
 	fhead = 0;
 	num = 0;
 
-	while (last != used_idx) {
+	while (last != virt_load_acquire(&vr->used->idx)) {
 		void *data;
 		u32 len;
 		u32 tail;
@@ -373,7 +325,8 @@ static void ivshm_net_tx_clean(struct net_device *ndev)
 
 		desc = &vr->desc[used->id];
 
-		data = ivshm_net_desc_data(in, &in->tx, desc, &len);
+		data = ivshm_net_desc_data(in, &in->tx, IVSHM_NET_SECTION_TX,
+					   desc, &len);
 		if (!data) {
 			netdev_err(ndev, "bad tx descriptor, data == NULL\n");
 			break;
@@ -391,22 +344,38 @@ static void ivshm_net_tx_clean(struct net_device *ndev)
 			desc->next = fhead;
 
 		fhead = used->id;
-		last++;
+
+		tx->last_used_idx = ++last;
 		num++;
+		tx->num_free++;
+		BUG_ON(tx->num_free > vr->num);
+
+		tx_ok = ivshm_net_tx_ok(ndev);
+		if (!tx_ok)
+			ivshm_net_enable_tx_irq(in);
 	}
 
-	tx->last_used_idx = last;
-
-	spin_unlock(&in->tx_clean_lock);
-
 	if (num) {
-		spin_lock(&in->tx_free_lock);
 		fdesc->next = tx->free_head;
 		tx->free_head = fhead;
-		tx->num_free += num;
-		BUG_ON(tx->num_free > vr->num);
-		spin_unlock(&in->tx_free_lock);
+	} else {
+		tx_ok = ivshm_net_tx_ok(ndev);
 	}
+
+	return tx_ok;
+}
+
+static void ivshm_net_tx_poll(struct net_device *ndev)
+{
+	struct netdev_queue *txq = netdev_get_tx_queue(ndev, 0);
+
+	if (!__netif_tx_trylock(txq))
+		return;
+
+	if (ivshm_net_tx_clean(ndev) && netif_queue_stopped(ndev))
+		netif_wake_queue(ndev);
+
+	__netif_tx_unlock(txq);
 }
 
 static struct vring_desc *ivshm_net_rx_desc(struct net_device *ndev)
@@ -453,7 +422,7 @@ static int ivshm_net_poll(struct napi_struct *napi, int budget)
 
 	in->stats.napi_poll++;
 
-	ivshm_net_tx_clean(ndev);
+	ivshm_net_tx_poll(ndev);
 
 	while (received < budget) {
 		struct vring_desc *desc;
@@ -465,7 +434,8 @@ static int ivshm_net_poll(struct napi_struct *napi, int budget)
 		if (!desc)
 			break;
 
-		data = ivshm_net_desc_data(in, &in->rx, desc, &len);
+		data = ivshm_net_desc_data(in, &in->rx, IVSHM_NET_SECTION_RX,
+					   desc, &len);
 		if (!data) {
 			netdev_err(ndev, "bad rx descriptor\n");
 			break;
@@ -500,26 +470,59 @@ static int ivshm_net_poll(struct napi_struct *napi, int budget)
 	in->stats.rx_packets += received;
 	in->stats.napi_poll_n[received ? 1 + min(ilog2(received), 8) : 0]++;
 
-	if (ivshm_net_tx_ok(in, ndev->mtu))
-		netif_wake_queue(ndev);
-
 	return received;
 }
 
 static netdev_tx_t ivshm_net_xmit(struct sk_buff *skb, struct net_device *ndev)
 {
 	struct ivshm_net *in = netdev_priv(ndev);
+	struct ivshm_net_queue *tx = &in->tx;
+	bool xmit_more = netdev_xmit_more();
+	struct vring *vr = &tx->vr;
+	struct vring_desc *desc;
+	unsigned int desc_idx;
+	unsigned int avail;
+	u32 head;
+	void *buf;
 
-	ivshm_net_tx_clean(ndev);
+	if (!ivshm_net_tx_clean(ndev)) {
+		netif_stop_queue(ndev);
 
-	if (!ivshm_net_tx_ok(in, ndev->mtu)) {
+		netdev_err(ndev, "BUG: tx ring full when queue awake!\n");
+		return NETDEV_TX_BUSY;
+	}
+
+	desc_idx = tx->free_head;
+	desc = &vr->desc[desc_idx];
+	tx->free_head = desc->next;
+	tx->num_free--;
+
+	head = ivshm_net_tx_advance(tx, &tx->head, skb->len);
+
+	if (!ivshm_net_tx_ok(ndev)) {
 		ivshm_net_enable_tx_irq(in);
 		netif_stop_queue(ndev);
-		skb->xmit_more = 0;
+		xmit_more = false;
 		in->stats.tx_pause++;
 	}
 
-	ivshm_net_tx_frame(ndev, skb);
+	buf = tx->data + head;
+	skb_copy_and_csum_dev(skb, buf);
+
+	desc->addr = buf - in->shm[IVSHM_NET_SECTION_TX];
+	desc->len = skb->len;
+	desc->flags = 0;
+
+	avail = tx->last_avail_idx++ & (vr->num - 1);
+	vr->avail->ring[avail] = desc_idx;
+	tx->num_added++;
+
+	virt_store_release(&vr->avail->idx, tx->last_avail_idx);
+
+	if (!xmit_more) {
+		ivshm_net_notify_tx(in, tx->num_added);
+		tx->num_added = 0;
+	}
 
 	in->stats.tx_packets++;
 	ndev->stats.tx_packets++;
@@ -533,20 +536,23 @@ static netdev_tx_t ivshm_net_xmit(struct sk_buff *skb, struct net_device *ndev)
 static void ivshm_net_set_state(struct ivshm_net *in, u32 state)
 {
 	virt_wmb();
-	WRITE_ONCE(in->lstate, state);
-	writel(state, &in->ivshm_regs->lstate);
+	WRITE_ONCE(in->state, state);
+	writel(state, &in->ivshm_regs->state);
 }
 
 static void ivshm_net_run(struct net_device *ndev)
 {
 	struct ivshm_net *in = netdev_priv(ndev);
 
-	if (in->lstate < IVSHM_NET_STATE_READY)
+	if (in->state < IVSHM_NET_STATE_READY)
 		return;
 
 	if (!netif_running(ndev))
 		return;
 
+	if (in->last_peer_state == IVSHM_NET_STATE_RUN)
+		netif_carrier_on(ndev);
+
 	if (test_and_set_bit(IVSHM_NET_FLAG_RUN, &in->flags))
 		return;
 
@@ -565,6 +571,7 @@ static void ivshm_net_do_stop(struct net_device *ndev)
 	if (!test_and_clear_bit(IVSHM_NET_FLAG_RUN, &in->flags))
 		return;
 
+	netif_carrier_off(ndev);
 	netif_stop_queue(ndev);
 	napi_disable(&in->napi);
 }
@@ -573,15 +580,24 @@ static void ivshm_net_state_change(struct work_struct *work)
 {
 	struct ivshm_net *in = container_of(work, struct ivshm_net, state_work);
 	struct net_device *ndev = in->napi.dev;
-	u32 rstate = readl(&in->ivshm_regs->rstate);
+	u32 peer_state = READ_ONCE(in->state_table[in->peer_id]);
 
-	switch (in->lstate) {
+	mutex_lock(&in->state_lock);
+
+	if (peer_state == in->last_peer_state) {
+		mutex_unlock(&in->state_lock);
+		return;
+	}
+
+	in->last_peer_state = peer_state;
+
+	switch (in->state) {
 	case IVSHM_NET_STATE_RESET:
 		/*
 		 * Wait for the remote to leave READY/RUN before transitioning
 		 * to INIT.
 		 */
-		if (rstate < IVSHM_NET_STATE_READY)
+		if (peer_state < IVSHM_NET_STATE_READY)
 			ivshm_net_set_state(in, IVSHM_NET_STATE_INIT);
 		break;
 
@@ -590,103 +606,107 @@ static void ivshm_net_state_change(struct work_struct *work)
 		 * Wait for the remote to leave RESET before performing the
 		 * initialization and moving to READY.
 		 */
-		if (rstate > IVSHM_NET_STATE_RESET) {
+		if (peer_state > IVSHM_NET_STATE_RESET) {
 			ivshm_net_init_queues(ndev);
 			ivshm_net_set_state(in, IVSHM_NET_STATE_READY);
 
+			mutex_unlock(&in->state_lock);
+
 			rtnl_lock();
 			call_netdevice_notifiers(NETDEV_CHANGEADDR, ndev);
 			rtnl_unlock();
+
+			return;
 		}
 		break;
 
 	case IVSHM_NET_STATE_READY:
-		/*
-		 * Link is up and we are running once the remote is in READY or
-		 * RUN.
-		 */
-		if (rstate >= IVSHM_NET_STATE_READY) {
-			netif_carrier_on(ndev);
-			ivshm_net_run(ndev);
-			break;
-		}
-		/* fall through */
 	case IVSHM_NET_STATE_RUN:
-		/*
-		 * If the remote goes to RESET, we need to follow immediately.
-		 */
-		if (rstate == IVSHM_NET_STATE_RESET) {
-			netif_carrier_off(ndev);
+		if (peer_state >= IVSHM_NET_STATE_READY) {
+			/*
+			 * Link is up and we are running once the remote is in
+			 * READY or RUN.
+			 */
+			ivshm_net_run(ndev);
+		} else if (peer_state == IVSHM_NET_STATE_RESET) {
+			/*
+			 * If the remote goes to RESET, we need to follow
+			 * immediately.
+			 */
 			ivshm_net_do_stop(ndev);
 		}
 		break;
 	}
 
-	virt_wmb();
-	WRITE_ONCE(in->rstate, rstate);
+	mutex_unlock(&in->state_lock);
 }
 
-static void ivshm_net_check_state(struct net_device *ndev)
+static void ivshm_net_check_state(struct ivshm_net *in)
 {
-	struct ivshm_net *in = netdev_priv(ndev);
-	u32 rstate = readl(&in->ivshm_regs->rstate);
-
-	if (rstate != in->rstate || !test_bit(IVSHM_NET_FLAG_RUN, &in->flags))
-		queue_work(in->state_wq, &in->state_work);
+	queue_work(in->state_wq, &in->state_work);
 }
 
-static irqreturn_t ivshm_net_int(int irq, void *data)
+static irqreturn_t ivshm_net_int_state(int irq, void *data)
 {
-	struct net_device *ndev = data;
-	struct ivshm_net *in = netdev_priv(ndev);
+	struct ivshm_net *in = data;
 
-	in->stats.interrupts++;
+	ivshm_net_check_state(in);
+
+	return IRQ_HANDLED;
+}
+
+static irqreturn_t ivshm_net_int_tx_rx(int irq, void *data)
+{
+	struct ivshm_net *in = data;
+
+	in->stats.tx_rx_interrupts++;
 
-	ivshm_net_check_state(ndev);
 	napi_schedule_irqoff(&in->napi);
 
 	return IRQ_HANDLED;
 }
 
+static irqreturn_t ivshm_net_intx(int irq, void *data)
+{
+	ivshm_net_int_state(irq, data);
+	ivshm_net_int_tx_rx(irq, data);
+
+	return IRQ_HANDLED;
+}
+
 static int ivshm_net_open(struct net_device *ndev)
 {
+	struct ivshm_net *in = netdev_priv(ndev);
+
 	netdev_reset_queue(ndev);
 	ndev->operstate = IF_OPER_UP;
+
+	mutex_lock(&in->state_lock);
 	ivshm_net_run(ndev);
+	mutex_unlock(&in->state_lock);
 
 	return 0;
 }
 
 static int ivshm_net_stop(struct net_device *ndev)
 {
+	struct ivshm_net *in = netdev_priv(ndev);
+
 	ndev->operstate = IF_OPER_DOWN;
+
+	mutex_lock(&in->state_lock);
 	ivshm_net_do_stop(ndev);
+	mutex_unlock(&in->state_lock);
 
 	return 0;
 }
 
 static int ivshm_net_change_mtu(struct net_device *ndev, int mtu)
 {
-	struct ivshm_net *in = netdev_priv(ndev);
-	struct ivshm_net_queue *tx = &in->tx;
-
-	if (mtu < IVSHM_NET_MTU_MIN || mtu > IVSHM_NET_MTU_MAX)
-		return -EINVAL;
-
-	if (in->tx.size / mtu < 4)
-		return -EINVAL;
-
-	if (ivshm_net_tx_space(in) < 2 * IVSHM_NET_FRAME_SIZE(mtu))
+	if (netif_running(ndev)) {
+		netdev_err(ndev, "must be stopped to change its MTU\n");
 		return -EBUSY;
-
-	if (in->tx.size - tx->head < IVSHM_NET_FRAME_SIZE(mtu) &&
-	    tx->head < tx->tail)
-		return -EBUSY;
-
-	netif_tx_lock_bh(ndev);
-	if (in->tx.size - tx->head < IVSHM_NET_FRAME_SIZE(mtu))
-		tx->head = 0;
-	netif_tx_unlock_bh(ndev);
+	}
 
 	ndev->mtu = mtu;
 
@@ -715,7 +735,7 @@ static const struct net_device_ops ivshm_net_ops = {
 };
 
 static const char ivshm_net_stats[][ETH_GSTRING_LEN] = {
-	"interrupts",
+	"tx_rx_interrupts",
 	"tx_packets",
 	"tx_notify",
 	"tx_pause",
@@ -758,7 +778,7 @@ static void ivshm_net_get_ethtool_stats(struct net_device *ndev,
 	unsigned int n = 0;
 	unsigned int i;
 
-	st[n++] = in->stats.interrupts;
+	st[n++] = in->stats.tx_rx_interrupts;
 	st[n++] = in->stats.tx_packets;
 	st[n++] = in->stats.tx_notify;
 	st[n++] = in->stats.tx_pause;
@@ -787,8 +807,8 @@ static void ivshm_net_get_regs(struct net_device *ndev,
 	u32 *reg32 = p;
 	u16 *reg16;
 
-	*reg32++ = in->lstate;
-	*reg32++ = in->rstate;
+	*reg32++ = in->state;
+	*reg32++ = in->last_peer_state;
 	*reg32++ = in->qlen;
 
 	reg16 = (u16 *)reg32;
@@ -810,17 +830,28 @@ static const struct ethtool_ops ivshm_net_ethtool_ops = {
 	.get_regs		= ivshm_net_get_regs,
 };
 
-static int ivshm_net_probe(struct pci_dev *pdev,
-			   const struct pci_device_id *id)
+static u64 get_config_qword(struct pci_dev *pdev, unsigned int pos)
 {
+	u32 lo, hi;
+
+	pci_read_config_dword(pdev, pos, &lo);
+	pci_read_config_dword(pdev, pos + 4, &hi);
+	return lo | ((u64)hi << 32);
+}
+
+static int ivshm_net_probe(struct pci_dev *pdev,
+			   const struct pci_device_id *pci_id)
+{
+	phys_addr_t output_sections_addr, section_addr;
+	resource_size_t section_sz, output_section_sz;
+	void *state_table, *output_sections;
+	struct ivshm_regs __iomem *regs;
 	struct net_device *ndev;
 	struct ivshm_net *in;
-	struct ivshmem_regs __iomem *regs;
-	resource_size_t shmaddr;
-	resource_size_t shmlen;
+	unsigned int cap_pos;
 	char *device_name;
-	void *shm;
-	u32 ivpos;
+	int vendor_cap;
+	u32 id, dword;
 	int ret;
 
 	ret = pcim_enable_device(pdev);
@@ -837,39 +868,74 @@ static int ivshm_net_probe(struct pci_dev *pdev,
 
 	regs = pcim_iomap_table(pdev)[0];
 
-	shmlen = pci_resource_len(pdev, 2);
-
-	if (shmlen) {
-		shmaddr = pci_resource_start(pdev, 2);
-	} else {
-		union { u64 v; u32 hl[2]; } val;
-
-		pci_read_config_dword(pdev, JAILHOUSE_CFG_SHMEM_PTR,
-				      &val.hl[0]);
-		pci_read_config_dword(pdev, JAILHOUSE_CFG_SHMEM_PTR + 4,
-				      &val.hl[1]);
-		shmaddr = val.v;
-
-		pci_read_config_dword(pdev, JAILHOUSE_CFG_SHMEM_SZ,
-				      &val.hl[0]);
-		pci_read_config_dword(pdev, JAILHOUSE_CFG_SHMEM_SZ + 4,
-				      &val.hl[1]);
-		shmlen = val.v;
-	}
-
-
-	if (!devm_request_mem_region(&pdev->dev, shmaddr, shmlen, DRV_NAME))
-		return -EBUSY;
-
-	shm = devm_memremap(&pdev->dev, shmaddr, shmlen, MEMREMAP_WB);
-	if (!shm)
-		return -ENOMEM;
-
-	ivpos = readl(&regs->ivpos);
-	if (ivpos > 1) {
-		dev_err(&pdev->dev, "invalid IVPosition %d\n", ivpos);
+	id = readl(&regs->id);
+	if (id > 1) {
+		dev_err(&pdev->dev, "invalid ID %d\n", id);
 		return -EINVAL;
 	}
+	if (readl(&regs->max_peers) > 2) {
+		dev_err(&pdev->dev, "only 2 peers supported\n");
+		return -EINVAL;
+	}
+
+	vendor_cap = pci_find_capability(pdev, PCI_CAP_ID_VNDR);
+	if (vendor_cap < 0) {
+		dev_err(&pdev->dev, "missing vendor capability\n");
+		return -EINVAL;
+	}
+
+	if (pci_resource_len(pdev, 2) > 0) {
+		section_addr = pci_resource_start(pdev, 2);
+	} else {
+		cap_pos = vendor_cap + IVSHM_CFG_ADDRESS;
+		section_addr = get_config_qword(pdev, cap_pos);
+	}
+
+	cap_pos = vendor_cap + IVSHM_CFG_STATE_TAB_SZ;
+	pci_read_config_dword(pdev, cap_pos, &dword);
+	section_sz = dword;
+
+	if (!devm_request_mem_region(&pdev->dev, section_addr, section_sz,
+				     DRV_NAME))
+		return -EBUSY;
+
+	state_table = devm_memremap(&pdev->dev, section_addr, section_sz,
+				    MEMREMAP_WB);
+	if (!state_table)
+		return -ENOMEM;
+
+	output_sections_addr = section_addr + section_sz;
+
+	cap_pos = vendor_cap + IVSHM_CFG_RW_SECTION_SZ;
+	section_sz = get_config_qword(pdev, cap_pos);
+	if (section_sz > 0) {
+		dev_info(&pdev->dev, "R/W section detected - "
+			 "unused by this driver version\n");
+		output_sections_addr += section_sz;
+	}
+
+	cap_pos = vendor_cap + IVSHM_CFG_OUTPUT_SECTION_SZ;
+	output_section_sz = get_config_qword(pdev, cap_pos);
+	if (output_section_sz == 0) {
+		dev_err(&pdev->dev, "Missing input/output sections\n");
+		return -EINVAL;
+	}
+
+	if (!devm_request_mem_region(&pdev->dev, output_sections_addr,
+				     output_section_sz * 2, DRV_NAME))
+		return -EBUSY;
+
+	output_sections = devm_memremap(&pdev->dev, output_sections_addr,
+					output_section_sz * 2, MEMREMAP_WB);
+	if (!output_sections)
+		return -ENOMEM;
+
+	section_addr = output_sections_addr + output_section_sz * id;
+	dev_info(&pdev->dev, "TX memory at %pa, size %pa\n",
+		 &section_addr, &output_section_sz);
+	section_addr = output_sections_addr + output_section_sz * !id;
+	dev_info(&pdev->dev, "RX memory at %pa, size %pa\n",
+		 &section_addr, &output_section_sz);
 
 	device_name = devm_kasprintf(&pdev->dev, GFP_KERNEL, "%s[%s]", DRV_NAME,
 				     dev_name(&pdev->dev));
@@ -885,13 +951,20 @@ static int ivshm_net_probe(struct pci_dev *pdev,
 
 	in = netdev_priv(ndev);
 	in->ivshm_regs = regs;
-	in->shm = shm;
-	in->shmaddr = shmaddr;
-	in->shmlen = shmlen;
-	in->peer_id = !ivpos;
+	in->state_table = state_table;
+
+	in->shm[IVSHM_NET_SECTION_TX] =
+		output_sections + output_section_sz * id;
+	in->shm[IVSHM_NET_SECTION_RX] =
+		output_sections + output_section_sz * !id;
+
+	in->shmlen = output_section_sz;
+
+	in->peer_id = !id;
 	in->pdev = pdev;
-	spin_lock_init(&in->tx_free_lock);
-	spin_lock_init(&in->tx_clean_lock);
+	in->last_peer_state = IVSHM_NET_STATE_UNKNOWN;
+
+	mutex_init(&in->state_lock);
 
 	ret = ivshm_net_calc_qsize(ndev);
 	if (ret)
@@ -907,6 +980,8 @@ static int ivshm_net_probe(struct pci_dev *pdev,
 	ndev->netdev_ops = &ivshm_net_ops;
 	ndev->ethtool_ops = &ivshm_net_ethtool_ops;
 	ndev->mtu = min_t(u32, IVSHM_NET_MTU_DEF, in->qsize / 16);
+	ndev->min_mtu = ETH_MIN_MTU;
+	ndev->max_mtu = min_t(u32, ETH_MAX_MTU, in->qsize / 4);
 	ndev->hw_features = NETIF_F_HW_CSUM | NETIF_F_SG;
 	ndev->features = ndev->hw_features;
 
@@ -917,24 +992,64 @@ static int ivshm_net_probe(struct pci_dev *pdev,
 	if (ret)
 		goto err_wq;
 
-	ret = pci_alloc_irq_vectors(pdev, 1, 1, PCI_IRQ_LEGACY | PCI_IRQ_MSIX);
+	ret = pci_alloc_irq_vectors(pdev, 1, 2, PCI_IRQ_LEGACY | PCI_IRQ_MSIX);
 	if (ret < 0)
 		goto err_alloc_irq;
 
-	ret = request_irq(pci_irq_vector(pdev, 0), ivshm_net_int, 0,
-			  device_name, ndev);
-	if (ret)
-		goto err_request_irq;
+	if (pdev->msix_enabled) {
+		if (ret != 2) {
+			ret = -EBUSY;
+			goto err_request_irq;
+		}
+
+		device_name = devm_kasprintf(&pdev->dev, GFP_KERNEL,
+					     "%s-state[%s]", DRV_NAME,
+					     dev_name(&pdev->dev));
+		if (!device_name) {
+			ret = -ENOMEM;
+			goto err_request_irq;
+		}
+
+		ret = request_irq(pci_irq_vector(pdev, IVSHM_NET_MSIX_STATE),
+				  ivshm_net_int_state, 0, device_name, in);
+		if (ret)
+			goto err_request_irq;
+
+		device_name = devm_kasprintf(&pdev->dev, GFP_KERNEL,
+					     "%s-tx-rx[%s]", DRV_NAME,
+					     dev_name(&pdev->dev));
+		if (!device_name) {
+			ret = -ENOMEM;
+			goto err_request_irq2;
+		}
+
+		ret = request_irq(pci_irq_vector(pdev, IVSHM_NET_MSIX_TX_RX),
+				  ivshm_net_int_tx_rx, 0, device_name, in);
+		if (ret)
+			goto err_request_irq2;
+
+		in->tx_rx_vector = IVSHM_NET_MSIX_TX_RX;
+	} else {
+		ret = request_irq(pci_irq_vector(pdev, 0), ivshm_net_intx, 0,
+				  device_name, in);
+		if (ret)
+			goto err_request_irq;
+
+		in->tx_rx_vector = 0;
+	}
 
 	pci_set_master(pdev);
-	if (!pdev->msix_enabled)
-		writel(IVSHMEM_INTX_ENABLE, &in->ivshm_regs->intxctrl);
 
-	writel(IVSHM_NET_STATE_RESET, &in->ivshm_regs->lstate);
-	ivshm_net_check_state(ndev);
+	pci_write_config_byte(pdev, vendor_cap + IVSHM_CFG_PRIV_CNTL, 0);
+	writel(IVSHM_INT_ENABLE, &in->ivshm_regs->int_control);
+
+	writel(IVSHM_NET_STATE_RESET, &in->ivshm_regs->state);
+	ivshm_net_check_state(in);
 
 	return 0;
 
+err_request_irq2:
+	free_irq(pci_irq_vector(pdev, IVSHM_NET_MSIX_STATE), in);
 err_request_irq:
 	pci_free_irq_vectors(pdev);
 err_alloc_irq:
@@ -952,11 +1067,15 @@ static void ivshm_net_remove(struct pci_dev *pdev)
 	struct net_device *ndev = pci_get_drvdata(pdev);
 	struct ivshm_net *in = netdev_priv(ndev);
 
-	writel(IVSHM_NET_STATE_RESET, &in->ivshm_regs->lstate);
+	writel(IVSHM_NET_STATE_RESET, &in->ivshm_regs->state);
+	writel(0, &in->ivshm_regs->int_control);
 
-	if (!pdev->msix_enabled)
-		writel(0, &in->ivshm_regs->intxctrl);
-	free_irq(pci_irq_vector(pdev, 0), ndev);
+	if (pdev->msix_enabled) {
+		free_irq(pci_irq_vector(pdev, IVSHM_NET_MSIX_STATE), in);
+		free_irq(pci_irq_vector(pdev, IVSHM_NET_MSIX_TX_RX), in);
+	} else {
+		free_irq(pci_irq_vector(pdev, 0), in);
+	}
 	pci_free_irq_vectors(pdev);
 
 	unregister_netdev(ndev);
@@ -966,8 +1085,8 @@ static void ivshm_net_remove(struct pci_dev *pdev)
 }
 
 static const struct pci_device_id ivshm_net_id_table[] = {
-	{ PCI_DEVICE(PCI_VENDOR_ID_REDHAT_QUMRANET, 0x1110),
-		(PCI_CLASS_OTHERS << 16) | (0x01 << 8), 0xffff00 },
+	{ PCI_DEVICE(PCI_VENDOR_ID_SIEMENS, PCI_DEVICE_ID_IVSHMEM),
+	  (PCI_CLASS_OTHERS << 16) | IVSHM_PROTO_NET, 0xffffff },
 	{ 0 }
 };
 MODULE_DEVICE_TABLE(pci, ivshm_net_id_table);
