@@ -1,7 +1,7 @@
 /*
  * Cirrus Logic CS42448/CS42888 Audio CODEC Digital Audio Interface (DAI) driver
  *
- * Copyright (C) 2014-2016 Freescale Semiconductor, Inc.
+ * Copyright (C) 2014 Freescale Semiconductor, Inc.
  *
  * Author: Nicolin Chen <Guangyu.Chen@freescale.com>
  *
@@ -14,7 +14,7 @@
 #include <linux/delay.h>
 #include <linux/module.h>
 #include <linux/of_device.h>
-#include <linux/of_gpio.h>
+#include <linux/gpio/consumer.h>
 #include <linux/pm_runtime.h>
 #include <linux/regulator/consumer.h>
 #include <sound/pcm_params.h>
@@ -33,7 +33,8 @@ static const char *const cs42xx8_supply_names[CS42XX8_NUM_SUPPLIES] = {
 
 #define CS42XX8_FORMATS	(SNDRV_PCM_FMTBIT_S16_LE | \
 			 SNDRV_PCM_FMTBIT_S20_3LE | \
-			 SNDRV_PCM_FMTBIT_S24_LE)
+			 SNDRV_PCM_FMTBIT_S24_LE | \
+			 SNDRV_PCM_FMTBIT_S32_LE)
 
 /* codec private data */
 struct cs42xx8_priv {
@@ -46,8 +47,8 @@ struct cs42xx8_priv {
 	bool is_tdm;
 	unsigned long sysclk;
 	u32 tx_channels;
-	int rate[2];
-	int reset_gpio;
+	struct gpio_desc *gpiod_reset;
+	u32 rate[2];
 };
 
 /* -127.5dB to 0dB with step of 0.5dB */
@@ -131,6 +132,7 @@ static const struct snd_soc_dapm_widget cs42xx8_dapm_widgets[] = {
 	SND_SOC_DAPM_INPUT("AIN2L"),
 	SND_SOC_DAPM_INPUT("AIN2R"),
 
+	SND_SOC_DAPM_SUPPLY("PWR", CS42XX8_PWRCTL, 0, 1, NULL, 0),
 };
 
 static const struct snd_soc_dapm_widget cs42xx8_adc3_dapm_widgets[] = {
@@ -144,28 +146,35 @@ static const struct snd_soc_dapm_route cs42xx8_dapm_routes[] = {
 	/* Playback */
 	{ "AOUT1L", NULL, "DAC1" },
 	{ "AOUT1R", NULL, "DAC1" },
+	{ "DAC1", NULL, "PWR" },
 
 	{ "AOUT2L", NULL, "DAC2" },
 	{ "AOUT2R", NULL, "DAC2" },
+	{ "DAC2", NULL, "PWR" },
 
 	{ "AOUT3L", NULL, "DAC3" },
 	{ "AOUT3R", NULL, "DAC3" },
+	{ "DAC3", NULL, "PWR" },
 
 	{ "AOUT4L", NULL, "DAC4" },
 	{ "AOUT4R", NULL, "DAC4" },
+	{ "DAC4", NULL, "PWR" },
 
 	/* Capture */
 	{ "ADC1", NULL, "AIN1L" },
 	{ "ADC1", NULL, "AIN1R" },
+	{ "ADC1", NULL, "PWR" },
 
 	{ "ADC2", NULL, "AIN2L" },
 	{ "ADC2", NULL, "AIN2R" },
+	{ "ADC2", NULL, "PWR" },
 };
 
 static const struct snd_soc_dapm_route cs42xx8_adc3_dapm_routes[] = {
 	/* Capture */
 	{ "ADC3", NULL, "AIN3L" },
 	{ "ADC3", NULL, "AIN3R" },
+	{ "ADC3", NULL, "PWR" },
 };
 
 struct cs42xx8_ratios {
@@ -175,6 +184,15 @@ struct cs42xx8_ratios {
 	unsigned int ratio[3];
 };
 
+/*
+ * According to reference mannual, define the cs42xx8_ratio struct
+ * MFreq2 | MFreq1 | MFreq0 |     Description     | SSM | DSM | QSM |
+ * 0      | 0      | 0      |1.029MHz to 12.8MHz  | 256 | 128 |  64 |
+ * 0      | 0      | 1      |1.536MHz to 19.2MHz  | 384 | 192 |  96 |
+ * 0      | 1      | 0      |2.048MHz to 25.6MHz  | 512 | 256 | 128 |
+ * 0      | 1      | 1      |3.072MHz to 38.4MHz  | 768 | 384 | 192 |
+ * 1      | x      | x      |4.096MHz to 51.2MHz  |1024 | 512 | 256 |
+ */
 static const struct cs42xx8_ratios cs42xx8_ratios[] = {
 	{ 0, 1029000, 12800000, {256, 128, 64} },
 	{ 2, 1536000, 19200000, {384, 192, 96} },
@@ -186,8 +204,8 @@ static const struct cs42xx8_ratios cs42xx8_ratios[] = {
 static int cs42xx8_set_dai_sysclk(struct snd_soc_dai *codec_dai,
 				  int clk_id, unsigned int freq, int dir)
 {
-	struct snd_soc_codec *codec = codec_dai->codec;
-	struct cs42xx8_priv *cs42xx8 = snd_soc_codec_get_drvdata(codec);
+	struct snd_soc_component *component = codec_dai->component;
+	struct cs42xx8_priv *cs42xx8 = snd_soc_component_get_drvdata(component);
 
 	cs42xx8->sysclk = freq;
 
@@ -197,8 +215,8 @@ static int cs42xx8_set_dai_sysclk(struct snd_soc_dai *codec_dai,
 static int cs42xx8_set_dai_fmt(struct snd_soc_dai *codec_dai,
 			       unsigned int format)
 {
-	struct snd_soc_codec *codec = codec_dai->codec;
-	struct cs42xx8_priv *cs42xx8 = snd_soc_codec_get_drvdata(codec);
+	struct snd_soc_component *component = codec_dai->component;
+	struct cs42xx8_priv *cs42xx8 = snd_soc_component_get_drvdata(component);
 	u32 val;
 
 	cs42xx8->is_tdm = false;
@@ -219,7 +237,7 @@ static int cs42xx8_set_dai_fmt(struct snd_soc_dai *codec_dai,
 		cs42xx8->is_tdm = true;
 		break;
 	default:
-		dev_err(codec->dev, "unsupported dai format\n");
+		dev_err(component->dev, "unsupported dai format\n");
 		return -EINVAL;
 	}
 
@@ -239,7 +257,7 @@ static int cs42xx8_set_dai_fmt(struct snd_soc_dai *codec_dai,
 			cs42xx8->slave_mode[1] = false;
 			break;
 		default:
-			dev_err(codec->dev, "unsupported master/slave mode\n");
+			dev_err(component->dev, "unsupported master/slave mode\n");
 			return -EINVAL;
 		}
 	}
@@ -251,8 +269,8 @@ static int cs42xx8_hw_params(struct snd_pcm_substream *substream,
 			     struct snd_pcm_hw_params *params,
 			     struct snd_soc_dai *dai)
 {
-	struct snd_soc_codec *codec = dai->codec;
-	struct cs42xx8_priv *cs42xx8 = snd_soc_codec_get_drvdata(codec);
+	struct snd_soc_component *component = dai->component;
+	struct cs42xx8_priv *cs42xx8 = snd_soc_component_get_drvdata(component);
 	bool tx = substream->stream == SNDRV_PCM_STREAM_PLAYBACK;
 	u32 ratio[2];
 	u32 rate[2];
@@ -269,25 +287,27 @@ static int cs42xx8_hw_params(struct snd_pcm_substream *substream,
 	ratio[tx] = rate[tx] > 0 ? cs42xx8->sysclk / rate[tx] : 0;
 	ratio[!tx] = rate[!tx] > 0 ? cs42xx8->sysclk / rate[!tx] : 0;
 
+	/* Get functional mode for tx and rx according to rate */
 	for (i = 0; i < 2; i++) {
 		if (cs42xx8->slave_mode[i]) {
 			fm[i] = CS42XX8_FM_AUTO;
 		} else {
-			if (rate[i] < 50000)
+			if (rate[i] < 50000) {
 				fm[i] = CS42XX8_FM_SINGLE;
-			else if (rate[i] > 50000 && rate[i] < 100000)
+			} else if (rate[i] > 50000 && rate[i] < 100000) {
 				fm[i] = CS42XX8_FM_DOUBLE;
-			else if (rate[i] > 100000 && rate[i] < 200000)
+			} else if (rate[i] > 100000 && rate[i] < 200000) {
 				fm[i] = CS42XX8_FM_QUAD;
-			else {
-				dev_err(codec->dev,
-				"unsupported sample rate or rate combine\n");
+			} else {
+				dev_err(component->dev,
+					"unsupported sample rate\n");
 				return -EINVAL;
 			}
 		}
 	}
 
 	for (i = 0; i < ARRAY_SIZE(cs42xx8_ratios); i++) {
+		/* Is the ratio[tx] valid ? */
 		condition1 = ((fm[tx] == CS42XX8_FM_AUTO) ?
 			(cs42xx8_ratios[i].ratio[0] == ratio[tx] ||
 			cs42xx8_ratios[i].ratio[1] == ratio[tx] ||
@@ -296,36 +316,41 @@ static int cs42xx8_hw_params(struct snd_pcm_substream *substream,
 			cs42xx8->sysclk >= cs42xx8_ratios[i].min_mclk &&
 			cs42xx8->sysclk <= cs42xx8_ratios[i].max_mclk;
 
-		if (ratio[tx] <= 0)
+		if (!ratio[tx])
 			condition1 = true;
 
+		/* Is the ratio[!tx] valid ? */
 		condition2 = ((fm[!tx] == CS42XX8_FM_AUTO) ?
 			(cs42xx8_ratios[i].ratio[0] == ratio[!tx] ||
 			cs42xx8_ratios[i].ratio[1] == ratio[!tx] ||
 			cs42xx8_ratios[i].ratio[2] == ratio[!tx]) :
 			(cs42xx8_ratios[i].ratio[fm[!tx]] == ratio[!tx]));
 
-		if (ratio[!tx] <= 0)
+		if (!ratio[!tx])
 			condition2 = true;
 
+		/*
+		 * Both ratio[tx] and ratio[!tx] is valid, then we get
+		 * a proper MFreq.
+		 */
 		if (condition1 && condition2)
 			break;
 	}
 
 	if (i == ARRAY_SIZE(cs42xx8_ratios)) {
-		dev_err(codec->dev, "unsupported sysclk ratio\n");
+		dev_err(component->dev, "unsupported sysclk ratio\n");
 		return -EINVAL;
 	}
 
 	cs42xx8->rate[tx] = params_rate(params);
 
 	if (cs42xx8->is_tdm && !cs42xx8->slave_mode[tx]) {
-		dev_err(codec->dev, "TDM mode is unsupported in master mode\n");
+		dev_err(component->dev, "TDM mode is unsupported in master mode\n");
 		return -EINVAL;
 	}
 
 	if (cs42xx8->is_tdm && (cs42xx8->sysclk < 256 * cs42xx8->rate[tx])) {
-		dev_err(codec->dev, "unsupported sysclk for TDM mode\n");
+		dev_err(component->dev, "unsupported sysclk for TDM mode\n");
 		return -EINVAL;
 	}
 
@@ -340,13 +365,13 @@ static int cs42xx8_hw_params(struct snd_pcm_substream *substream,
 }
 
 static int cs42xx8_hw_free(struct snd_pcm_substream *substream,
-			     struct snd_soc_dai *dai)
+			   struct snd_soc_dai *dai)
 {
-	struct snd_soc_pcm_runtime *rtd = substream->private_data;
-	struct snd_soc_codec *codec = rtd->codec;
-	struct cs42xx8_priv *cs42xx8 = snd_soc_codec_get_drvdata(codec);
+	struct snd_soc_component *component = dai->component;
+	struct cs42xx8_priv *cs42xx8 = snd_soc_component_get_drvdata(component);
 	bool tx = substream->stream == SNDRV_PCM_STREAM_PLAYBACK;
 
+	/* Clear stored rate */
 	cs42xx8->rate[tx] = 0;
 
 	regmap_update_bits(cs42xx8->regmap, CS42XX8_FUNCMOD,
@@ -355,10 +380,10 @@ static int cs42xx8_hw_free(struct snd_pcm_substream *substream,
 	return 0;
 }
 
-static int cs42xx8_digital_mute(struct snd_soc_dai *dai, int mute)
+static int cs42xx8_mute(struct snd_soc_dai *dai, int mute, int direction)
 {
-	struct snd_soc_codec *codec = dai->codec;
-	struct cs42xx8_priv *cs42xx8 = snd_soc_codec_get_drvdata(codec);
+	struct snd_soc_component *component = dai->component;
+	struct cs42xx8_priv *cs42xx8 = snd_soc_component_get_drvdata(component);
 	u8 dac_unmute = cs42xx8->tx_channels ?
 		        ~((0x1 << cs42xx8->tx_channels) - 1) : 0;
 
@@ -373,7 +398,8 @@ static const struct snd_soc_dai_ops cs42xx8_dai_ops = {
 	.set_sysclk	= cs42xx8_set_dai_sysclk,
 	.hw_params	= cs42xx8_hw_params,
 	.hw_free	= cs42xx8_hw_free,
-	.digital_mute	= cs42xx8_digital_mute,
+	.mute_stream	= cs42xx8_mute,
+	.no_capture_mute = 1,
 };
 
 static struct snd_soc_dai_driver cs42xx8_dai = {
@@ -452,17 +478,19 @@ const struct regmap_config cs42xx8_regmap_config = {
 	.volatile_reg = cs42xx8_volatile_register,
 	.writeable_reg = cs42xx8_writeable_register,
 	.cache_type = REGCACHE_RBTREE,
+	.use_single_read = true,
+	.use_single_write = true,
 };
 EXPORT_SYMBOL_GPL(cs42xx8_regmap_config);
 
-static int cs42xx8_codec_probe(struct snd_soc_codec *codec)
+static int cs42xx8_component_probe(struct snd_soc_component *component)
 {
-	struct cs42xx8_priv *cs42xx8 = snd_soc_codec_get_drvdata(codec);
-	struct snd_soc_dapm_context *dapm = snd_soc_codec_get_dapm(codec);
+	struct cs42xx8_priv *cs42xx8 = snd_soc_component_get_drvdata(component);
+	struct snd_soc_dapm_context *dapm = snd_soc_component_get_dapm(component);
 
 	switch (cs42xx8->drvdata->num_adcs) {
 	case 3:
-		snd_soc_add_codec_controls(codec, cs42xx8_adc3_snd_controls,
+		snd_soc_add_component_controls(component, cs42xx8_adc3_snd_controls,
 					ARRAY_SIZE(cs42xx8_adc3_snd_controls));
 		snd_soc_dapm_new_controls(dapm, cs42xx8_adc3_dapm_widgets,
 					ARRAY_SIZE(cs42xx8_adc3_dapm_widgets));
@@ -475,23 +503,21 @@ static int cs42xx8_codec_probe(struct snd_soc_codec *codec)
 
 	/* Mute all DAC channels */
 	regmap_write(cs42xx8->regmap, CS42XX8_DACMUTE, CS42XX8_DACMUTE_ALL);
-	regmap_update_bits(cs42xx8->regmap, CS42XX8_PWRCTL,
-			CS42XX8_PWRCTL_PDN_MASK, 0);
+
 	return 0;
 }
 
-static const struct snd_soc_codec_driver cs42xx8_driver = {
-	.probe = cs42xx8_codec_probe,
-	.idle_bias_off = true,
-
-	.component_driver = {
-		.controls		= cs42xx8_snd_controls,
-		.num_controls		= ARRAY_SIZE(cs42xx8_snd_controls),
-		.dapm_widgets		= cs42xx8_dapm_widgets,
-		.num_dapm_widgets	= ARRAY_SIZE(cs42xx8_dapm_widgets),
-		.dapm_routes		= cs42xx8_dapm_routes,
-		.num_dapm_routes	= ARRAY_SIZE(cs42xx8_dapm_routes),
-	},
+static const struct snd_soc_component_driver cs42xx8_driver = {
+	.probe			= cs42xx8_component_probe,
+	.controls		= cs42xx8_snd_controls,
+	.num_controls		= ARRAY_SIZE(cs42xx8_snd_controls),
+	.dapm_widgets		= cs42xx8_dapm_widgets,
+	.num_dapm_widgets	= ARRAY_SIZE(cs42xx8_dapm_widgets),
+	.dapm_routes		= cs42xx8_dapm_routes,
+	.num_dapm_routes	= ARRAY_SIZE(cs42xx8_dapm_routes),
+	.use_pmdown_time	= 1,
+	.endianness		= 1,
+	.non_legacy_dai_naming	= 1,
 };
 
 const struct cs42xx8_driver_data cs42448_data = {
@@ -516,8 +542,7 @@ EXPORT_SYMBOL_GPL(cs42xx8_of_match);
 
 int cs42xx8_probe(struct device *dev, struct regmap *regmap)
 {
-	const struct of_device_id *of_id = of_match_device(cs42xx8_of_match, dev);
-	struct device_node *np = dev->of_node;
+	const struct of_device_id *of_id;
 	struct cs42xx8_priv *cs42xx8;
 	int ret, val, i;
 
@@ -543,16 +568,12 @@ int cs42xx8_probe(struct device *dev, struct regmap *regmap)
 		return -EINVAL;
 	}
 
-	cs42xx8->reset_gpio = of_get_named_gpio(np, "reset-gpio", 0);
-	if (gpio_is_valid(cs42xx8->reset_gpio)) {
-		ret = devm_gpio_request_one(dev, cs42xx8->reset_gpio,
-				GPIOF_OUT_INIT_LOW, "cs42xx8 reset");
-		if (ret) {
-			dev_err(dev, "unable to get reset gpio\n");
-			return ret;
-		}
-		gpio_set_value_cansleep(cs42xx8->reset_gpio, 1);
-	}
+	cs42xx8->gpiod_reset = devm_gpiod_get_optional(dev, "reset",
+							GPIOD_OUT_HIGH);
+	if (IS_ERR(cs42xx8->gpiod_reset))
+		return PTR_ERR(cs42xx8->gpiod_reset);
+
+	gpiod_set_value_cansleep(cs42xx8->gpiod_reset, 0);
 
 	cs42xx8->clk = devm_clk_get(dev, "mclk");
 	if (IS_ERR(cs42xx8->clk)) {
@@ -563,13 +584,13 @@ int cs42xx8_probe(struct device *dev, struct regmap *regmap)
 
 	cs42xx8->sysclk = clk_get_rate(cs42xx8->clk);
 
-	if (of_property_read_bool(np, "fsl,txm-rxs")) {
+	if (of_property_read_bool(dev->of_node, "fsl,txm-rxs")) {
 		/* 0 --  rx,  1 -- tx */
 		cs42xx8->slave_mode[0] = true;
 		cs42xx8->slave_mode[1] = false;
 	}
 
-	if (of_property_read_bool(np, "fsl,txs-rxm")) {
+	if (of_property_read_bool(dev->of_node, "fsl,txs-rxm")) {
 		/* 0 --  rx,  1 -- tx */
 		cs42xx8->slave_mode[0] = false;
 		cs42xx8->slave_mode[1] = true;
@@ -618,9 +639,9 @@ int cs42xx8_probe(struct device *dev, struct regmap *regmap)
 	/* Each adc supports stereo input */
 	cs42xx8_dai.capture.channels_max = cs42xx8->drvdata->num_adcs * 2;
 
-	ret = snd_soc_register_codec(dev, &cs42xx8_driver, &cs42xx8_dai, 1);
+	ret = devm_snd_soc_register_component(dev, &cs42xx8_driver, &cs42xx8_dai, 1);
 	if (ret) {
-		dev_err(dev, "failed to register codec:%d\n", ret);
+		dev_err(dev, "failed to register component:%d\n", ret);
 		goto err_enable;
 	}
 
@@ -646,10 +667,7 @@ static int cs42xx8_runtime_resume(struct device *dev)
 		return ret;
 	}
 
-	if (gpio_is_valid(cs42xx8->reset_gpio)) {
-		gpio_set_value_cansleep(cs42xx8->reset_gpio, 0);
-		gpio_set_value_cansleep(cs42xx8->reset_gpio, 1);
-	}
+	gpiod_set_value_cansleep(cs42xx8->gpiod_reset, 0);
 
 	ret = regulator_bulk_enable(ARRAY_SIZE(cs42xx8->supplies),
 				    cs42xx8->supplies);
@@ -658,15 +676,11 @@ static int cs42xx8_runtime_resume(struct device *dev)
 		goto err_clk;
 	}
 
-	regmap_update_bits(cs42xx8->regmap, CS42XX8_PWRCTL,
-				CS42XX8_PWRCTL_PDN_MASK, 1);
 	/* Make sure hardware reset done */
 	msleep(5);
 
-	regmap_update_bits(cs42xx8->regmap, CS42XX8_PWRCTL,
-				CS42XX8_PWRCTL_PDN_MASK, 0);
-
 	regcache_cache_only(cs42xx8->regmap, false);
+	regcache_mark_dirty(cs42xx8->regmap);
 
 	ret = regcache_sync(cs42xx8->regmap);
 	if (ret) {
@@ -694,6 +708,8 @@ static int cs42xx8_runtime_suspend(struct device *dev)
 	regulator_bulk_disable(ARRAY_SIZE(cs42xx8->supplies),
 			       cs42xx8->supplies);
 
+	gpiod_set_value_cansleep(cs42xx8->gpiod_reset, 1);
+
 	clk_disable_unprepare(cs42xx8->clk);
 
 	return 0;
@@ -701,7 +717,8 @@ static int cs42xx8_runtime_suspend(struct device *dev)
 #endif
 
 const struct dev_pm_ops cs42xx8_pm = {
-	SET_SYSTEM_SLEEP_PM_OPS(pm_runtime_force_suspend, pm_runtime_force_resume)
+	SET_SYSTEM_SLEEP_PM_OPS(pm_runtime_force_suspend,
+				pm_runtime_force_resume)
 	SET_RUNTIME_PM_OPS(cs42xx8_runtime_suspend, cs42xx8_runtime_resume, NULL)
 };
 EXPORT_SYMBOL_GPL(cs42xx8_pm);
