@@ -32,10 +32,12 @@ struct fsl_micfil {
 	struct platform_device *pdev;
 	struct regmap *regmap;
 	const struct fsl_micfil_soc_data *soc;
+	struct clk *busclk;
 	struct clk *mclk;
 	struct clk *clk_src[MICFIL_CLK_SRC_NUM];
 	struct snd_dmaengine_dai_dma_data dma_params_rx;
 	struct kobject *hwvad_kobject;
+	struct sdma_audio_config audio_config;
 	unsigned int vad_channel;
 	unsigned int dataline;
 	char name[32];
@@ -61,6 +63,8 @@ struct fsl_micfil {
 	int vad_rate_index;
 	atomic_t recording_state;
 	atomic_t hwvad_state;
+	/* spinlock to control HWVAD enable/disable */
+	spinlock_t hwvad_lock;
 	unsigned int pdm_clk_wait;
 };
 
@@ -69,22 +73,33 @@ struct fsl_micfil_soc_data {
 	unsigned int fifo_depth;
 	unsigned int dataline;
 	bool imx;
+	u64  formats;
 };
 
 static char *envp[] = {
-		"EVENT=PDM_VOICE_DETECT",
-		NULL,
-	};
+	"EVENT=PDM_VOICE_DETECT",
+	NULL,
+};
 
 static struct fsl_micfil_soc_data fsl_micfil_imx8mm = {
 	.imx = true,
 	.fifos = 8,
 	.fifo_depth = 8,
 	.dataline =  0xf,
+	.formats = SNDRV_PCM_FMTBIT_S16_LE,
+};
+
+static struct fsl_micfil_soc_data fsl_micfil_imx8mp = {
+	.imx = true,
+	.fifos = 8,
+	.fifo_depth = 32,
+	.dataline =  0xf,
+	.formats = SNDRV_PCM_FMTBIT_S32_LE,
 };
 
 static const struct of_device_id fsl_micfil_dt_ids[] = {
 	{ .compatible = "fsl,imx8mm-micfil", .data = &fsl_micfil_imx8mm },
+	{ .compatible = "fsl,imx8mp-micfil", .data = &fsl_micfil_imx8mp },
 	{}
 };
 MODULE_DEVICE_TABLE(of, fsl_micfil_dt_ids);
@@ -393,62 +408,6 @@ static int gain_info(struct snd_kcontrol *kcontrol,
 	return 0;
 }
 
-static int put_channel_gain(struct snd_kcontrol *kcontrol,
-			    struct snd_ctl_elem_value *ucontrol)
-{
-	struct snd_soc_component *comp = snd_kcontrol_chip(kcontrol);
-	struct fsl_micfil *micfil = snd_soc_component_get_drvdata(comp);
-	struct soc_mixer_control *mc =
-		(struct soc_mixer_control *)kcontrol->private_value;
-	unsigned int shift = mc->shift;
-	int index = shift / 4;
-	int val = ucontrol->value.integer.value[0];
-	int remapped_value;
-	int ret;
-	u32 reg_val;
-
-	/* a value remapping must be done since the gain field have
-	 * the following meaning:
-	 * * 0 : no gain
-	 * * 1 - 7 : +1 to +7 bits gain
-	 * * 8 - 15 : -8 to -1 bits gain
-	 * After the remapp, the scale should start from -8 to +7
-	 */
-
-	micfil->channel_gain[index] = val;
-
-	remapped_value = (val - 8) & 0xF;
-
-	reg_val = remapped_value << shift;
-
-	ret = snd_soc_component_update_bits(comp,
-					    REG_MICFIL_OUT_CTRL,
-					    0xF << shift,
-					    reg_val);
-
-	if (ret < 0)
-		return ret;
-
-	return 0;
-}
-
-static int get_channel_gain(struct snd_kcontrol *kcontrol,
-			    struct snd_ctl_elem_value *ucontrol)
-{
-	struct snd_soc_component *comp = snd_kcontrol_chip(kcontrol);
-	struct fsl_micfil *micfil = snd_soc_component_get_drvdata(comp);
-	struct soc_mixer_control *mc =
-		(struct soc_mixer_control *)kcontrol->private_value;
-	int index;
-
-	/* gain bitfield is 4 bits wide */
-	index = mc->shift / 4;
-
-	ucontrol->value.enumerated.item[0] = micfil->channel_gain[index];
-
-	return 0;
-}
-
 static int hwvad_put_input_gain(struct snd_kcontrol *kcontrol,
 				struct snd_ctl_elem_value *ucontrol)
 {
@@ -683,31 +642,22 @@ static int hwvad_get_zcd_adj(struct snd_kcontrol *kcontrol,
 static DECLARE_TLV_DB_SCALE(gain_tlv, 0, 100, 0);
 
 static const struct snd_kcontrol_new fsl_micfil_snd_controls[] = {
-	SOC_SINGLE_RANGE_EXT_TLV("CH0 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(0),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-	SOC_SINGLE_RANGE_EXT_TLV("CH1 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(1),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-	SOC_SINGLE_RANGE_EXT_TLV("CH2 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(2),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-	SOC_SINGLE_RANGE_EXT_TLV("CH3 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(3),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-	SOC_SINGLE_RANGE_EXT_TLV("CH4 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(4),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-	SOC_SINGLE_RANGE_EXT_TLV("CH5 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(5),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-	SOC_SINGLE_RANGE_EXT_TLV("CH6 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(6),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-	SOC_SINGLE_RANGE_EXT_TLV("CH7 Gain", -1, MICFIL_OUTGAIN_CHX_SHIFT(7),
-				 0x0, 0xF, 0,
-				 get_channel_gain, put_channel_gain, gain_tlv),
-
+	SOC_SINGLE_SX_TLV("CH0 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(0), 0x8, 0xF, gain_tlv),
+	SOC_SINGLE_SX_TLV("CH1 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(1), 0x8, 0xF, gain_tlv),
+	SOC_SINGLE_SX_TLV("CH2 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(2), 0x8, 0xF, gain_tlv),
+	SOC_SINGLE_SX_TLV("CH3 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(3), 0x8, 0xF, gain_tlv),
+	SOC_SINGLE_SX_TLV("CH4 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(4), 0x8, 0xF, gain_tlv),
+	SOC_SINGLE_SX_TLV("CH5 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(5), 0x8, 0xF, gain_tlv),
+	SOC_SINGLE_SX_TLV("CH6 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(6), 0x8, 0xF, gain_tlv),
+	SOC_SINGLE_SX_TLV("CH7 Volume", REG_MICFIL_OUT_CTRL,
+			  MICFIL_OUTGAIN_CHX_SHIFT(7), 0x8, 0xF, gain_tlv),
 	SOC_ENUM_EXT("MICFIL Quality Select",
 		     fsl_micfil_quality_enum,
 		     snd_soc_get_enum_double, snd_soc_put_enum_double),
@@ -811,27 +761,6 @@ static const struct snd_kcontrol_new fsl_micfil_snd_controls[] = {
 
 static int disable_hwvad(struct device *dev, bool sync);
 
-static inline int get_pdm_clk(struct fsl_micfil *micfil,
-			      unsigned int rate);
-
-static inline int get_clk_div(struct fsl_micfil *micfil,
-			      unsigned int rate)
-{
-	u32 ctrl2_reg;
-	long mclk_rate;
-	int osr;
-	int clk_div;
-
-	regmap_read(micfil->regmap, REG_MICFIL_CTRL2, &ctrl2_reg);
-	osr = 16 - ((ctrl2_reg & MICFIL_CTRL2_CICOSR_MASK)
-		    >> MICFIL_CTRL2_CICOSR_SHIFT);
-
-	mclk_rate = clk_get_rate(micfil->mclk);
-
-	clk_div = mclk_rate / (get_pdm_clk(micfil, rate) * 2);
-
-	return clk_div;
-}
 
 static inline int get_pdm_clk(struct fsl_micfil *micfil,
 			      unsigned int rate)
@@ -872,6 +801,22 @@ static inline int get_pdm_clk(struct fsl_micfil *micfil,
 	return bclk;
 }
 
+static inline int get_clk_div(struct fsl_micfil *micfil,
+			      unsigned int rate)
+{
+	u32 ctrl2_reg;
+	long mclk_rate;
+	int clk_div;
+
+	regmap_read(micfil->regmap, REG_MICFIL_CTRL2, &ctrl2_reg);
+
+	mclk_rate = clk_get_rate(micfil->mclk);
+
+	clk_div = mclk_rate / (get_pdm_clk(micfil, rate) * 2);
+
+	return clk_div;
+}
+
 /* The SRES is a self-negated bit which provides the CPU with the
  * capability to initialize the PDM Interface module through the
  * slave-bus interface. This bit always reads as zero, and this
@@ -900,6 +845,9 @@ static int fsl_micfil_reset(struct device *dev)
 		return ret;
 	}
 
+	/* w1c */
+	regmap_write_bits(micfil->regmap, REG_MICFIL_STAT, 0xFF, 0xFF);
+
 	return 0;
 }
 
@@ -956,7 +904,7 @@ static int init_hwvad_internal_filters(struct device *dev)
 	/* sleep for 100ms - it should be enough for bit to stay
 	 * pulsed for more than 2 cycles
 	 */
-	usleep_range(MICFIL_SLEEP_MIN, MICFIL_SLEEP_MAX);
+	mdelay(MICFIL_SLEEP);
 
 	/* Voice Activity Detector Enabled */
 	ret = regmap_update_bits(micfil->regmap, REG_MICFIL_VAD0_CTRL1,
@@ -1071,7 +1019,7 @@ static int init_hwvad_energy_mode(struct device *dev)
 	/* it would be a good idea to wait some time before VADEN
 	 * is set
 	 */
-	usleep_range(5 * MICFIL_SLEEP_MIN, 5 * MICFIL_SLEEP_MAX);
+	mdelay(5 * MICFIL_SLEEP);
 
 	/* Enable Interrupts */
 	ret = configure_hwvad_interrupts(dev, 1);
@@ -1103,7 +1051,7 @@ static int init_hwvad_energy_mode(struct device *dev)
 		if (flag == 0)
 			break;
 
-		usleep_range(MICFIL_SLEEP_MIN, MICFIL_SLEEP_MAX);
+		mdelay(MICFIL_SLEEP);
 	}
 
 	if (i == MICFIL_MAX_RETRY) {
@@ -1237,7 +1185,7 @@ static int init_hwvad_envelope_mode(struct device *dev)
 	/* it would be a good idea to wait some time before VADEN
 	 * is set
 	 */
-	usleep_range(3 * MICFIL_SLEEP_MIN, 3 * MICFIL_SLEEP_MAX);
+	mdelay(3 * MICFIL_SLEEP);
 
 	/* Wait for INITF to be asserted */
 	for (i = 0; i < MICFIL_MAX_RETRY; i++) {
@@ -1252,7 +1200,7 @@ static int init_hwvad_envelope_mode(struct device *dev)
 		if (flag == 0)
 			break;
 
-		usleep_range(MICFIL_SLEEP_MIN, MICFIL_SLEEP_MAX);
+		mdelay(MICFIL_SLEEP);
 	}
 
 	if (i == MICFIL_MAX_RETRY) {
@@ -1407,6 +1355,9 @@ static inline bool clk_in_list(struct clk *p, struct clk *clk_src[])
 	return false;
 }
 
+#define CLK_8K_FREQ    24576000
+#define CLK_11K_FREQ   22579200
+
 static int fsl_micfil_set_mclk_rate(struct fsl_micfil *micfil, int clk_id,
 				    unsigned int freq)
 {
@@ -1424,15 +1375,6 @@ static int fsl_micfil_set_mclk_rate(struct fsl_micfil *micfil, int clk_id,
 	if (atomic_read(&micfil->hwvad_state) == MICFIL_HWVAD_ON)
 		return 0;
 
-	/* check if all clock sources are valid */
-	for (i = 0; i < MICFIL_CLK_SRC_NUM; i++) {
-		if (micfil->clk_src[i])
-			continue;
-
-		dev_err(dev, "Clock Source %d is not valid.\n", i);
-		return -EINVAL;
-	}
-
 	while (p) {
 		struct clk *pp = clk_get_parent(p);
 
@@ -1456,8 +1398,10 @@ static int fsl_micfil_set_mclk_rate(struct fsl_micfil *micfil, int clk_id,
 			 * to any known frequency ???
 			 */
 			clk_rate = roundup(clk_rate - 4, 10);
-			if (do_div(clk_rate, ratio) == 0)
+			if (do_div(clk_rate, ratio) == 0) {
 				npll = micfil->clk_src[i];
+				break;
+			}
 		}
 	} else {
 		/* clock id is offseted by 1 since ID=0 means
@@ -1472,6 +1416,7 @@ static int fsl_micfil_set_mclk_rate(struct fsl_micfil *micfil, int clk_id,
 		return -EINVAL;
 	}
 
+	clk_disable_unprepare(micfil->mclk);
 	if (!clk_is_match(pll, npll)) {
 		ret = clk_set_parent(p, npll);
 		if (ret < 0)
@@ -1479,11 +1424,10 @@ static int fsl_micfil_set_mclk_rate(struct fsl_micfil *micfil, int clk_id,
 				 "failed to set parrent %d\n", ret);
 	}
 
-	clk_disable_unprepare(micfil->mclk);
-	ret = clk_set_rate(micfil->mclk, freq * 1024);
+	clk_rate = freq % 8000 == 0 ? CLK_8K_FREQ : CLK_11K_FREQ;
+	ret = clk_set_rate(micfil->mclk, clk_rate);
 	if (ret)
-		dev_warn(dev, "failed to set rate (%u): %d\n",
-			 freq * 1024, ret);
+		dev_warn(dev, "failed to set rate (%llu): %d\n", clk_rate, ret);
 	clk_prepare_enable(micfil->mclk);
 
 	return ret;
@@ -1495,8 +1439,7 @@ static int fsl_micfil_startup(struct snd_pcm_substream *substream,
 	struct fsl_micfil *micfil = snd_soc_dai_get_drvdata(dai);
 
 	if (!micfil) {
-		dev_err(dai->dev,
-			"micfil dai priv_data not set\n");
+		dev_err(dai->dev, "micfil dai priv_data not set\n");
 		return -EINVAL;
 	}
 
@@ -1533,11 +1476,11 @@ static int fsl_micfil_trigger(struct snd_pcm_substream *substream, int cmd,
 			dev_err(dev, "failed to update DISEL bits\n");
 			return ret;
 		}
-
 		break;
 	case SNDRV_PCM_TRIGGER_STOP:
 	case SNDRV_PCM_TRIGGER_SUSPEND:
 	case SNDRV_PCM_TRIGGER_PAUSE_PUSH:
+
 		ret = regmap_update_bits(micfil->regmap, REG_MICFIL_CTRL1,
 					 MICFIL_CTRL1_DISEL_MASK,
 					 (0 << MICFIL_CTRL1_DISEL_SHIFT));
@@ -1556,7 +1499,7 @@ static int fsl_set_clock_params(struct device *dev, unsigned int rate)
 {
 	struct fsl_micfil *micfil = dev_get_drvdata(dev);
 	int clk_div;
-	int ret = 0;
+	int ret;
 
 	ret = fsl_micfil_set_mclk_rate(micfil, 0, rate);
 	if (ret < 0)
@@ -1621,19 +1564,18 @@ static int fsl_micfil_hw_params(struct snd_pcm_substream *substream,
 		}
 	}
 
-        if(channels == 2){
-            /* enable channels */
-            ret = regmap_update_bits(micfil->regmap, REG_MICFIL_CTRL1,
-                    0xFF, 0x5);
-
-            micfil->dma_params_rx.fifo_off = 1;
-        }
-        else{
-            /* enable channels */
-            ret = regmap_update_bits(micfil->regmap, REG_MICFIL_CTRL1,
-                    0xFF, ((1 << channels) - 1));
-            micfil->dma_params_rx.fifo_off = 0;
-        }
+    if(channels == 2){
+        /* enable channels */
+        ret = regmap_update_bits(micfil->regmap, REG_MICFIL_CTRL1,
+                0xFF, 0x5);
+		micfil->dma_params_rx.fifo_off = 1;
+    }
+    else{
+        /* enable channels */
+        ret = regmap_update_bits(micfil->regmap, REG_MICFIL_CTRL1,
+                0xFF, ((1 << channels) - 1));
+		micfil->dma_params_rx.fifo_off = 0;
+    }
 
 	if (ret) {
 		dev_err(dev, "failed to enable channels %d, reg 0x%X\n", ret,
@@ -1647,14 +1589,16 @@ static int fsl_micfil_hw_params(struct snd_pcm_substream *substream,
 		return ret;
 	}
 
-	micfil->dma_params_rx.fifo_num = channels;
+	micfil->audio_config.src_fifo_num = channels;
+	micfil->audio_config.sw_done_sel = BIT(31);
+	micfil->dma_params_rx.peripheral_config  = &micfil->audio_config;
+	micfil->dma_params_rx.peripheral_size    = sizeof(micfil->audio_config);
 	micfil->dma_params_rx.maxburst = channels * MICFIL_DMA_MAXBURST_RX;
 
-	/* Enable MICFIL module */
+	/* Enable the module */
 	ret = regmap_update_bits(micfil->regmap, REG_MICFIL_CTRL1,
-		MICFIL_CTRL1_PDMIEN_MASK,
-		MICFIL_CTRL1_PDMIEN);
-
+					MICFIL_CTRL1_PDMIEN_MASK,
+					MICFIL_CTRL1_PDMIEN);
 	if (ret) {
 		dev_err(dev, "failed to enable the module\n");
 		return ret;
@@ -1675,13 +1619,12 @@ static int fsl_micfil_hw_free(struct snd_pcm_substream *substream,
 
 	atomic_set(&micfil->recording_state, MICFIL_RECORDING_OFF);
 
-	/* disable MICFIL module */
+	/* Disable the module */
 	ret = regmap_update_bits(micfil->regmap, REG_MICFIL_CTRL1,
-		MICFIL_CTRL1_PDMIEN_MASK,
-		0);
-
+					MICFIL_CTRL1_PDMIEN_MASK,
+					0);
 	if (ret) {
-		dev_err(dev, "failed to disable the module\n");
+		dev_err(dev, "failed to enable the module\n");
 		return ret;
 	}
 
@@ -1732,7 +1675,7 @@ static int fsl_micfil_set_dai_fmt(struct snd_soc_dai *dai, unsigned int fmt)
 	return 0;
 }
 
-static struct snd_soc_dai_ops fsl_micfil_dai_ops = {
+static const struct snd_soc_dai_ops fsl_micfil_dai_ops = {
 	.startup = fsl_micfil_startup,
 	.trigger = fsl_micfil_trigger,
 	.hw_params = fsl_micfil_hw_params,
@@ -1760,7 +1703,7 @@ static int fsl_micfil_dai_probe(struct snd_soc_dai *cpu_dai)
 
 	/* set default gain to 2 */
 	regmap_write(micfil->regmap, REG_MICFIL_OUT_CTRL, 0x22222222);
-	for (i = 0; i < 8; i++)
+	for (i = 0; i < MICFIL_OUTPUT_CHANNELS; i++)
 		micfil->channel_gain[i] = 0xA;
 
 	/* set DC Remover to cut-off at 21Hz */
@@ -1789,8 +1732,6 @@ static int fsl_micfil_dai_probe(struct snd_soc_dai *cpu_dai)
 		return ret;
 	}
 
-	snd_soc_dai_set_drvdata(cpu_dai, micfil);
-
 	return 0;
 }
 
@@ -1898,7 +1839,6 @@ static bool fsl_micfil_writeable_reg(struct device *dev, unsigned int reg)
 static bool fsl_micfil_volatile_reg(struct device *dev, unsigned int reg)
 {
 	switch (reg) {
-	case REG_MICFIL_CTRL1:
 	case REG_MICFIL_STAT:
 	case REG_MICFIL_DATACH0:
 	case REG_MICFIL_DATACH1:
@@ -1908,12 +1848,8 @@ static bool fsl_micfil_volatile_reg(struct device *dev, unsigned int reg)
 	case REG_MICFIL_DATACH5:
 	case REG_MICFIL_DATACH6:
 	case REG_MICFIL_DATACH7:
-	case REG_MICFIL_VAD0_CTRL1:
-	case REG_MICFIL_VAD0_CTRL2:
 	case REG_MICFIL_VAD0_STAT:
-	case REG_MICFIL_VAD0_SCONFIG:
 	case REG_MICFIL_VAD0_NDATA:
-	case REG_MICFIL_VAD0_ZCD:
 		return true;
 	default:
 		return false;
@@ -1943,9 +1879,12 @@ static irqreturn_t voice_detected_fn(int irq, void *devid)
 	int ret;
 
 	/* disable hwvad */
+	spin_lock(&micfil->hwvad_lock);
 	ret = disable_hwvad(dev, true);
+	spin_unlock(&micfil->hwvad_lock);
+
 	if (ret)
-		dev_err(dev, "Failed to disable HWVAD module\n");
+		dev_err(dev, "Failed to disable HWVAD module: %d\n", ret);
 
 	/* notify userspace that voice was detected */
 	kobject_uevent_env(&dev->kobj, KOBJ_CHANGE, envp);
@@ -2114,17 +2053,21 @@ static int enable_hwvad(struct device *dev, bool sync)
 
 	ret = fsl_set_clock_params(dev, rate);
 	if (ret)
-		return ret;
+		goto enable_error;
 
 	ret = fsl_micfil_reset(dev);
 	if (ret)
-		return ret;
+		goto enable_error;
 
 	/* Initialize Hardware Voice Activity */
 	ret = init_hwvad(dev);
+	if (ret == 0)
+		return 0;
 
-	return ret;
 enable_error:
+	if (state == MICFIL_HWVAD_OFF)
+		atomic_cmpxchg(&micfil->hwvad_state,
+			       MICFIL_HWVAD_ON, MICFIL_HWVAD_OFF);
 	if (sync)
 		pm_runtime_put_sync(dev);
 	return ret;
@@ -2136,14 +2079,6 @@ static int disable_hwvad(struct device *dev, bool sync)
 	int ret = 0;
 	u32 state;
 
-	/* This is required because if an arecord was done,
-	 * suspend function will mark regmap as cache only
-	 * and reads/writes in volatile regs will fail
-	 */
-	regcache_cache_only(micfil->regmap, false);
-	regcache_mark_dirty(micfil->regmap);
-	regcache_sync(micfil->regmap);
-
 	/* disable is called with sync = false only from
 	 * system suspend and in this case, you should not
 	 * change the hwvad_state so we know at system_resume
@@ -2157,6 +2092,14 @@ static int disable_hwvad(struct device *dev, bool sync)
 		state = atomic_read(&micfil->hwvad_state);
 
 	if (state == MICFIL_HWVAD_ON) {
+		/* This is required because if an arecord was done,
+		 * suspend function will mark regmap as cache only
+		 * and reads/writes in volatile regs will fail
+		 */
+		regcache_cache_only(micfil->regmap, false);
+		regcache_mark_dirty(micfil->regmap);
+		regcache_sync(micfil->regmap);
+
 		/* Voice Activity Detector Reset */
 		ret |= regmap_update_bits(micfil->regmap,
 					  REG_MICFIL_VAD0_CTRL1,
@@ -2205,14 +2148,13 @@ static int disable_hwvad(struct device *dev, bool sync)
 						  0);
 		}
 
+		if (sync)
+			pm_runtime_put_sync(dev);
 	} else {
 		ret = -EPERM;
 		dev_err(dev, "HWVAD is not enabled %d\n", ret);
 	}
 
-	if (sync)
-		pm_runtime_put_sync(dev);
-
 	return ret;
 }
 
@@ -2231,17 +2173,21 @@ static ssize_t micfil_hwvad_handler(struct kobject *kobj,
 	if (ret < 0)
 		return -EINVAL;
 
+	spin_lock(&micfil->hwvad_lock);
 	if (vad_channel <= 7) {
 		micfil->vad_channel = vad_channel;
 		ret = enable_hwvad(dev, true);
-		if (ret)
-			dev_err(dev, "Failed to enable hwvad");
 	} else {
 		micfil->vad_channel = -1;
 		ret = disable_hwvad(dev, true);
 	}
-	if (ret)
+	spin_unlock(&micfil->hwvad_lock);
+
+	if (ret) {
+		dev_err(dev, "Failed to %s hwvad: %d\n",
+			vad_channel <= 7 ? "enable" : "disable", ret);
 		return ret;
+	}
 
 	return count;
 }
@@ -2254,7 +2200,6 @@ static struct kobj_attribute hwvad_en_attr = __ATTR(enable,
 static int fsl_micfil_probe(struct platform_device *pdev)
 {
 	struct device_node *np = pdev->dev.of_node;
-	const struct of_device_id *of_id;
 	struct fsl_micfil *micfil;
 	struct resource *res;
 	void __iomem *regs;
@@ -2268,11 +2213,7 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 	micfil->pdev = pdev;
 	strncpy(micfil->name, np->name, sizeof(micfil->name) - 1);
 
-	of_id = of_match_device(fsl_micfil_dt_ids, &pdev->dev);
-	if (!of_id || !of_id->data)
-		return -EINVAL;
-
-	micfil->soc = of_id->data;
+	micfil->soc = of_device_get_match_data(&pdev->dev);
 
 	/* ipg_clk is used to control the registers
 	 * ipg_clk_app is used to operate the filter
@@ -2284,6 +2225,13 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 		return PTR_ERR(micfil->mclk);
 	}
 
+	micfil->busclk = devm_clk_get(&pdev->dev, "ipg_clk");
+	if (IS_ERR(micfil->busclk)) {
+		dev_err(&pdev->dev, "failed to get ipg clock: %ld\n",
+			PTR_ERR(micfil->busclk));
+		return PTR_ERR(micfil->busclk);
+	}
+
 	/* get audio pll1 and pll2 */
 	micfil->clk_src[MICFIL_AUDIO_PLL1] = devm_clk_get(&pdev->dev, "pll8k");
 	if (IS_ERR(micfil->clk_src[MICFIL_AUDIO_PLL1]))
@@ -2298,15 +2246,13 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 		micfil->clk_src[MICFIL_CLK_EXT3] = NULL;
 
 	/* init regmap */
-	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
-	regs = devm_ioremap_resource(&pdev->dev, res);
+	regs = devm_platform_get_and_ioremap_resource(pdev, 0, &res);
 	if (IS_ERR(regs))
 		return PTR_ERR(regs);
 
-	micfil->regmap = devm_regmap_init_mmio_clk(&pdev->dev,
-						   "ipg_clk",
-						   regs,
-						   &fsl_micfil_regmap_config);
+	micfil->regmap = devm_regmap_init_mmio(&pdev->dev,
+					       regs,
+					       &fsl_micfil_regmap_config);
 	if (IS_ERR(micfil->regmap)) {
 		dev_err(&pdev->dev, "failed to init MICFIL regmap: %ld\n",
 			PTR_ERR(micfil->regmap));
@@ -2331,10 +2277,8 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 	for (i = 0; i < MICFIL_IRQ_LINES; i++) {
 		micfil->irq[i] = platform_get_irq(pdev, i);
 		dev_err(&pdev->dev, "GET IRQ: %d\n", micfil->irq[i]);
-		if (micfil->irq[i] < 0) {
-			dev_err(&pdev->dev, "no irq for node %s\n", pdev->name);
+		if (micfil->irq[i] < 0)
 			return micfil->irq[i];
-		}
 	}
 
 	if (of_property_read_bool(np, "fsl,shared-interrupt"))
@@ -2352,7 +2296,7 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 	/* Digital Microphone interface voice activity detector event
 	 * interrupt - IRQ 44
 	 */
-	ret = devm_request_threaded_irq(&pdev->dev, micfil->irq[0],
+	ret = devm_request_threaded_irq(&pdev->dev, micfil->irq[2],
 					hwvad_isr, voice_detected_fn,
 					irqflag, micfil->name, micfil);
 	if (ret) {
@@ -2364,7 +2308,7 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 	/* Digital Microphone interface voice activity detector error
 	 * interrupt - IRQ 45
 	 */
-	ret = devm_request_irq(&pdev->dev, micfil->irq[1],
+	ret = devm_request_irq(&pdev->dev, micfil->irq[3],
 			       hwvad_err_isr, irqflag,
 			       micfil->name, micfil);
 	if (ret) {
@@ -2374,22 +2318,22 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 	}
 
 	/* Digital Microphone interface interrupt - IRQ 109 */
-	ret = devm_request_irq(&pdev->dev, micfil->irq[2],
+	ret = devm_request_irq(&pdev->dev, micfil->irq[0],
 			       micfil_isr, irqflag,
 			       micfil->name, micfil);
 	if (ret) {
 		dev_err(&pdev->dev, "failed to claim mic interface irq %u\n",
-			micfil->irq[2]);
+			micfil->irq[0]);
 		return ret;
 	}
 
-	/* Digital Microphone interface error interrupt - IRQ 110 */
-	ret = devm_request_irq(&pdev->dev, micfil->irq[3],
+	/* Digital Microphone interface error interrupt */
+	ret = devm_request_irq(&pdev->dev, micfil->irq[1],
 			       micfil_err_isr, irqflag,
 			       micfil->name, micfil);
 	if (ret) {
 		dev_err(&pdev->dev, "failed to claim mic interface error irq %u\n",
-			micfil->irq[3]);
+			micfil->irq[1]);
 		return ret;
 	}
 
@@ -2401,12 +2345,26 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 
 	/* set default rate to first value in available vad rates */
 	micfil->vad_rate_index = 0;
+	/* init HWVAD enable/disable spinlock */
+	spin_lock_init(&micfil->hwvad_lock);
 
 	platform_set_drvdata(pdev, micfil);
 
 	pm_runtime_enable(&pdev->dev);
 	regcache_cache_only(micfil->regmap, true);
 
+	/*
+	 * Register platform component before registering cpu dai for there
+	 * is not defer probe for platform component in snd_soc_add_pcm_runtime().
+	 */
+	ret = devm_snd_dmaengine_pcm_register(&pdev->dev, NULL, 0);
+	if (ret) {
+		dev_err(&pdev->dev, "failed to pcm register\n");
+		return ret;
+	}
+
+	fsl_micfil_dai.capture.formats = micfil->soc->formats;
+
 	ret = devm_snd_soc_register_component(&pdev->dev, &fsl_micfil_component,
 					      &fsl_micfil_dai, 1);
 	if (ret) {
@@ -2415,16 +2373,6 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 		return ret;
 	}
 
-	if (micfil->soc->imx)
-		ret = imx_pcm_platform_register(&pdev->dev);
-	else
-		ret = devm_snd_dmaengine_pcm_register(&pdev->dev, NULL, 0);
-
-	if (ret) {
-		dev_err(&pdev->dev, "failed to pcm register\n");
-		return ret;
-	}
-
 	/* create sysfs entry used to enable hwvad from userspace */
 	micfil->hwvad_kobject = kobject_create_and_add("hwvad",
 						       &pdev->dev.kobj);
@@ -2442,7 +2390,6 @@ static int fsl_micfil_probe(struct platform_device *pdev)
 	return 0;
 }
 
-#ifdef CONFIG_PM
 static int __maybe_unused fsl_micfil_runtime_suspend(struct device *dev)
 {
 	struct fsl_micfil *micfil = dev_get_drvdata(dev);
@@ -2454,9 +2401,8 @@ static int __maybe_unused fsl_micfil_runtime_suspend(struct device *dev)
 
 	regcache_cache_only(micfil->regmap, true);
 
-	/* Disable the clock only if the hwvad is not enabled */
-	if (state == MICFIL_HWVAD_OFF)
-		clk_disable_unprepare(micfil->mclk);
+	clk_disable_unprepare(micfil->mclk);
+	clk_disable_unprepare(micfil->busclk);
 
 	return 0;
 }
@@ -2477,19 +2423,23 @@ static int __maybe_unused fsl_micfil_runtime_resume(struct device *dev)
 	if (state == MICFIL_HWVAD_ON)
 		return 0;
 
-	ret = clk_prepare_enable(micfil->mclk);
+	ret = clk_prepare_enable(micfil->busclk);
 	if (ret < 0)
 		return ret;
 
+	ret = clk_prepare_enable(micfil->mclk);
+	if (ret < 0) {
+		clk_disable_unprepare(micfil->busclk);
+		return ret;
+	}
+
 	regcache_cache_only(micfil->regmap, false);
 	regcache_mark_dirty(micfil->regmap);
 	regcache_sync(micfil->regmap);
 
 	return 0;
 }
-#endif /* CONFIG_PM*/
 
-#ifdef CONFIG_PM_SLEEP
 static int __maybe_unused fsl_micfil_suspend(struct device *dev)
 {
 	struct fsl_micfil *micfil = dev_get_drvdata(dev);
@@ -2528,7 +2478,6 @@ static int __maybe_unused fsl_micfil_resume(struct device *dev)
 
 	return 0;
 }
-#endif /* CONFIG_PM_SLEEP */
 
 static const struct dev_pm_ops fsl_micfil_pm_ops = {
 	SET_RUNTIME_PM_OPS(fsl_micfil_runtime_suspend,
