@@ -1,6 +1,6 @@
 /*
  * Copyright 2017-2018 NXP
- *  Copyright 2020,2021 Sony Home Entertainment & Sound Products Inc.
+ * Copyright 2022 Sony Corporation
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,15 +33,16 @@
 #include <linux/slab.h>
 #include <linux/smp.h>
 #include <linux/suspend.h>
-#include <soc/imx/fsl_sip.h>
-#include <soc/imx/revision.h>
-#include <soc/imx8/soc.h>
+#include <linux/sys_soc.h>
+
+#define FSL_SIP_DDR_DVFS                0xc2000004
 
 #define HIGH_FREQ_3200MTS	0x0
 #define AUDIO_FREQ_400MTS	0x1
 #define LOW_BUS_FREQ_100MTS	0x2
 #define LOW_BUS_FREQ_667MTS	0x1
 #define WAIT_BUS_FREQ_DONE	0xf
+#define DLL_ON_DRATE		667
 
 static struct device *busfreq_dev;
 static int low_bus_freq_mode;
@@ -53,8 +54,17 @@ static int high_bus_count, audio_bus_count, low_bus_count;
 static int cur_bus_freq_mode;
 static int busfreq_suspended;
 static bool cancel_reduce_bus_freq;
+static bool ddr_400mts;
+
+static unsigned int fsp_table[4];
+static unsigned long origin_noc_rate;
+static int low_bus_mode_fsp_index;
+static int audio_bus_mode_fsp_index;
+/* no bypass or dll off mode support if lowest fsp > 667mts */
+static bool bypass_support = true;
 
 static struct clk *dram_pll_clk;
+static struct clk *dram_pll;
 static struct clk *sys1_pll_800m;
 static struct clk *sys1_pll_400m;
 static struct clk *sys1_pll_160m;
@@ -108,74 +118,54 @@ static void reduce_bus_freq(void)
 	 */
 	if (audio_bus_count) {
 		if (cur_bus_freq_mode == BUS_FREQ_HIGH) {
-
-			if (of_machine_is_compatible("fsl,imx8mq")) {
-				if (imx8_get_soc_revision() < IMX_CHIP_REVISION_2_1) {
-					update_bus_freq(LOW_BUS_FREQ_667MTS);
-
-					/*
-					 * the dram_apb and dram_core clk rate is changed
-					 * in ATF side, below two lines of code is just used
-					 * to upate the clock tree info in kernel side.
-					 */
-					clk_set_rate(dram_apb_pre_div, 160000000);
-					clk_get_rate(dram_pll_clk);
-				} else {
-					/* prepare the necessary clk before frequency change */
-					clk_prepare_enable(sys1_pll_40m);
-					clk_prepare_enable(dram_alt_root);
-					clk_prepare_enable(sys1_pll_100m);
-
-					update_bus_freq(LOW_BUS_FREQ_100MTS);
-
-					clk_set_parent(dram_alt_src, sys1_pll_100m);
-					clk_set_parent(dram_core_clk, dram_alt_root);
-					clk_set_parent(dram_apb_src, sys1_pll_40m);
-					clk_set_rate(dram_apb_pre_div, 20000000);
-					clk_disable_unprepare(sys1_pll_100m);
-					clk_disable_unprepare(sys1_pll_40m);
-					clk_disable_unprepare(dram_alt_root);
-				}
-				/* reduce the NOC & bus clock */
-				rate = clk_get_rate(noc_div);
-				if (rate == 0) {
-					WARN_ON(1);
-					return;
-				}
-				clk_set_rate(noc_div, rate / 8);
-			} else {
+			if (bypass_support) {
 				/* prepare the necessary clk before frequency change */
-				clk_prepare_enable(sys1_pll_160m);
+				clk_prepare_enable(sys1_pll_40m);
 				clk_prepare_enable(dram_alt_root);
-				clk_prepare_enable(sys1_pll_800m);
-
-				update_bus_freq(AUDIO_FREQ_400MTS);
-
-				/* correct the clock tree info */
-				clk_set_parent(dram_alt_src, sys1_pll_800m);
-				clk_set_parent(dram_core_clk, dram_alt_root);
-				clk_set_parent(dram_apb_src, sys1_pll_160m);
-				clk_set_rate(dram_apb_pre_div, 80000000);
-				clk_set_rate(dram_alt_pre_div, 400000000);
-				clk_disable_unprepare(sys1_pll_800m);
-				clk_disable_unprepare(sys1_pll_160m);
-				clk_disable_unprepare(dram_alt_root);
-
-				/* change the NOC rate */
-				rate = clk_get_rate(noc_div);
-				if (rate == 0) {
-					WARN_ON(1);
-					return;
+				if(ddr_400mts){
+					clk_prepare_enable(sys1_pll_800m);
+					update_bus_freq(audio_bus_mode_fsp_index);
+					clk_set_parent(dram_alt_src, sys1_pll_800m);
+				} else {
+					clk_prepare_enable(sys1_pll_100m);
+					update_bus_freq(low_bus_mode_fsp_index);
+					clk_set_parent(dram_alt_src, sys1_pll_100m);
 				}
-				clk_set_rate(noc_div, rate / 5);
+
+				clk_set_parent(dram_core_clk, dram_alt_root);
+				clk_set_parent(dram_apb_src, sys1_pll_40m);
+				clk_set_rate(dram_apb_pre_div, 20000000);
+				if(ddr_400mts){
+					clk_set_rate(dram_alt_src, 400000000);
+					clk_disable_unprepare(sys1_pll_800m);
+				} else {
+					clk_disable_unprepare(sys1_pll_100m);
+				}
+				clk_disable_unprepare(sys1_pll_40m);
+				clk_disable_unprepare(dram_alt_root);
+			} else {
+				update_bus_freq(low_bus_mode_fsp_index);
+				/*
+				 * the dram_apb and dram_core clk rate is changed
+				 * in ATF side, below two lines of code is just used
+				 * to update the clock tree info in kernel side.
+				 */
+				clk_set_rate(dram_apb_pre_div, 160000000);
+				clk_get_rate(dram_pll);
 			}
+			/* change the NOC rate */
+			if (of_machine_is_compatible("fsl,imx8mq"))
+				clk_set_rate(noc_div, origin_noc_rate / 8);
+			else
+				clk_set_rate(noc_div, origin_noc_rate / 5);
+
 			rate = clk_get_rate(ahb_div);
 			if (rate == 0) {
 				WARN_ON(1);
 				return;
 			}
-			clk_set_rate(ahb_div, rate / 3);
-			clk_set_parent(main_axi_src, sys1_pll_100m);
+			clk_set_rate(ahb_div, rate / 6);
+			clk_set_parent(main_axi_src, osc_25m);
 		}
 
 		low_bus_freq_mode = 0;
@@ -183,67 +173,48 @@ static void reduce_bus_freq(void)
 		cur_bus_freq_mode = BUS_FREQ_AUDIO;
 	} else {
 		if (cur_bus_freq_mode == BUS_FREQ_HIGH) {
-			if (of_machine_is_compatible("fsl,imx8mq")) {
-				if (imx8_get_soc_revision() < IMX_CHIP_REVISION_2_1) {
-					update_bus_freq(LOW_BUS_FREQ_667MTS);
-
-					/*
-					 * the dram_apb and dram_core clk rate is changed
-					 * in ATF side, below two lines of code is just used
-					 * to upate the clock tree info in kernel side.
-					 */
-					clk_set_rate(dram_apb_pre_div, 160000000);
-					clk_get_rate(dram_pll_clk);
-				} else {
-					/* prepare the necessary clk before frequency change */
-					clk_prepare_enable(sys1_pll_40m);
-					clk_prepare_enable(dram_alt_root);
-					clk_prepare_enable(sys1_pll_100m);
-
-					update_bus_freq(LOW_BUS_FREQ_100MTS);
-
-					clk_set_parent(dram_alt_src, sys1_pll_100m);
-					clk_set_parent(dram_core_clk, dram_alt_root);
-					clk_set_parent(dram_apb_src, sys1_pll_40m);
-					clk_set_rate(dram_apb_pre_div, 20000000);
-					clk_disable_unprepare(sys1_pll_100m);
-					clk_disable_unprepare(sys1_pll_40m);
-					clk_disable_unprepare(dram_alt_root);
-				}
-				/* reduce the NOC & bus clock */
-				rate = clk_get_rate(noc_div);
-				if (rate == 0) {
-					WARN_ON(1);
-					return;
-				}
-				clk_set_rate(noc_div, rate / 8);
-			} else {
+			if (bypass_support) {
 				/* prepare the necessary clk before frequency change */
-				clk_prepare_enable(sys1_pll_160m);
+				clk_prepare_enable(sys1_pll_40m);
 				clk_prepare_enable(dram_alt_root);
-				clk_prepare_enable(sys1_pll_800m);
-
-				update_bus_freq(AUDIO_FREQ_400MTS);
-
-				/* correct the clock tree info */
-				clk_set_parent(dram_alt_src, sys1_pll_800m);
-				clk_set_parent(dram_core_clk, dram_alt_root);
-				clk_set_parent(dram_apb_src, sys1_pll_160m);
-				clk_set_rate(dram_apb_pre_div, 80000000);
-				clk_set_rate(dram_alt_pre_div, 400000000);
-				clk_disable_unprepare(sys1_pll_800m);
-				clk_disable_unprepare(sys1_pll_160m);
-				clk_disable_unprepare(dram_alt_root);
-
-				/* change the NOC clock rate  */
-				rate = clk_get_rate(noc_div);
-				if (rate == 0) {
-					WARN_ON(1);
-					return;
+				if(ddr_400mts){
+					clk_prepare_enable(sys1_pll_800m);
+					update_bus_freq(audio_bus_mode_fsp_index);
+					clk_set_parent(dram_alt_src, sys1_pll_800m);
+				} else {
+					clk_prepare_enable(sys1_pll_100m);
+					update_bus_freq(low_bus_mode_fsp_index);
+					clk_set_parent(dram_alt_src, sys1_pll_100m);
 				}
-				clk_set_rate(noc_div, rate / 5);
+
+				clk_set_parent(dram_core_clk, dram_alt_root);
+				clk_set_parent(dram_apb_src, sys1_pll_40m);
+				clk_set_rate(dram_apb_pre_div, 20000000);
+				if(ddr_400mts){
+					clk_set_rate(dram_alt_src, 400000000);
+					clk_disable_unprepare(sys1_pll_800m);
+				} else {
+					clk_disable_unprepare(sys1_pll_100m);
+				}
+				clk_disable_unprepare(sys1_pll_40m);
+				clk_disable_unprepare(dram_alt_root);
+			} else {
+				update_bus_freq(low_bus_mode_fsp_index);
+				/*
+				 * the dram_apb and dram_core clk rate is changed
+				 * in ATF side, below two lines of code is just used
+				 * to update the clock tree info in kernel side.
+				 */
+				clk_set_rate(dram_apb_pre_div, 160000000);
+				clk_get_rate(dram_pll);
 			}
 
+			/* change the NOC rate */
+			if (of_machine_is_compatible("fsl,imx8mq"))
+				clk_set_rate(noc_div, origin_noc_rate / 8);
+			else
+				clk_set_rate(noc_div, origin_noc_rate / 5);
+
 			rate = clk_get_rate(ahb_div);
 			if (rate == 0) {
 				WARN_ON(1);
@@ -318,30 +289,7 @@ static int set_high_bus_freq(int high_bus_freq)
 	if (high_bus_freq_mode)
 		return 0;
 
-	if (of_machine_is_compatible("fsl,imx8mq")) {
-		if (imx8_get_soc_revision() < IMX_CHIP_REVISION_2_1) {
-			/* switch the DDR freqeuncy */
-			update_bus_freq(HIGH_FREQ_3200MTS);
-
-			clk_set_rate(dram_apb_pre_div, 200000000);
-			clk_get_rate(dram_pll_clk);
-		} else {
-			/*  enable the clks needed in frequency */
-			clk_prepare_enable(sys1_pll_800m);
-			clk_prepare_enable(dram_pll_clk);
-
-			/* switch the DDR freqeuncy */
-			update_bus_freq(HIGH_FREQ_3200MTS);
-
-			/* correct the clock tree info */
-			clk_set_parent(dram_apb_src, sys1_pll_800m);
-			clk_set_rate(dram_apb_pre_div, 160000000);
-			clk_set_parent(dram_core_clk, dram_pll_clk);
-			clk_disable_unprepare(sys1_pll_800m);
-			clk_disable_unprepare(dram_pll_clk);
-		}
-		clk_set_rate(noc_div, 800000000);
-	} else {
+	if (bypass_support) {
 		/*  enable the clks needed in frequency */
 		clk_prepare_enable(sys1_pll_800m);
 		clk_prepare_enable(dram_pll_clk);
@@ -355,9 +303,15 @@ static int set_high_bus_freq(int high_bus_freq)
 		clk_set_parent(dram_core_clk, dram_pll_clk);
 		clk_disable_unprepare(sys1_pll_800m);
 		clk_disable_unprepare(dram_pll_clk);
-		clk_set_rate(noc_div, 750000000);
+	} else {
+		/* switch the DDR freqeuncy */
+		update_bus_freq(HIGH_FREQ_3200MTS);
+
+		clk_set_rate(dram_apb_pre_div, 200000000);
+		clk_get_rate(dram_pll);
 	}
 
+	clk_set_rate(noc_div, origin_noc_rate);
 	clk_set_rate(ahb_div, 133333333);
 	clk_set_parent(main_axi_src, sys2_pll_333m);
 
@@ -502,9 +456,7 @@ static ssize_t bus_freq_scaling_enable_store(struct device *dev,
 			dev_warn(dev, "busfreq already enabled!\n");
 			return size;
 		}
-
 		dev_info(dev, "enable busfreq.\n");
-
 		bus_freq_scaling_is_active = 1;
 		set_high_bus_freq(1);
 		/*
@@ -514,7 +466,6 @@ static ssize_t bus_freq_scaling_enable_store(struct device *dev,
 		 */
 		schedule_delayed_work(&bus_freq_daemon,
 			usecs_to_jiffies(5000000));
-
 		/*
 		 * Workaround for DRAM clock transition about BUS_FREQ_HIGH to BUS_FREQ_AUDIO
 		 * - transition from BUS_FREQ_HIGH to BUS_FREQ_AUDIO requires the call release_bus_freq.
@@ -528,11 +479,9 @@ static ssize_t bus_freq_scaling_enable_store(struct device *dev,
 		}
 
 		dev_info(dev, "disable busfreq.\n");
-
 		if (bus_freq_scaling_is_active)
 			set_high_bus_freq(1);
 		bus_freq_scaling_is_active = 0;
-
 		/*
 		 * Workaround for DRAM clock transition about BUS_FREQ_HIGH to BUS_FREQ_AUDIO
 		 * - Just in order to balance the reference counter call request_bus_freq here.
@@ -543,6 +492,36 @@ static ssize_t bus_freq_scaling_enable_store(struct device *dev,
 	return size;
 }
 
+static ssize_t bus_freq_scaling_request_store(struct device *dev,
+				struct device_attribute *attr,
+				const char *buf, size_t size)
+{
+
+	if (strncmp(buf, "high", 4) == 0)
+		request_bus_freq(BUS_FREQ_HIGH);
+	else if (strncmp(buf, "audio", 5) == 0)
+		request_bus_freq(BUS_FREQ_AUDIO);
+	else if (strncmp(buf, "low", 3) == 0)
+		request_bus_freq(BUS_FREQ_LOW);
+
+	return size;
+}
+
+static ssize_t bus_freq_scaling_release_store(struct device *dev,
+				struct device_attribute *attr,
+				const char *buf, size_t size)
+{
+
+	if (strncmp(buf, "high", 4) == 0)
+		release_bus_freq(BUS_FREQ_HIGH);
+	else if (strncmp(buf, "audio", 5) == 0)
+		release_bus_freq(BUS_FREQ_AUDIO);
+	else if (strncmp(buf, "low", 3) == 0)
+		release_bus_freq(BUS_FREQ_LOW);
+
+	return size;
+}
+
 static int bus_freq_pm_notify(struct notifier_block *nb, unsigned long event,
 	void *dummy)
 {
@@ -584,6 +563,23 @@ static struct notifier_block imx_busfreq_reboot_notifier = {
 static DEVICE_ATTR(enable, 0644, bus_freq_scaling_enable_show,
 			bus_freq_scaling_enable_store);
 
+static DEVICE_ATTR(release, 0200, NULL,
+			bus_freq_scaling_release_store);
+
+static DEVICE_ATTR(request, 0200, NULL,
+			bus_freq_scaling_request_store);
+
+static struct attribute *attrs[] = {
+	&dev_attr_enable.attr,
+	&dev_attr_request.attr,
+	&dev_attr_release.attr,
+	NULL,
+};
+
+static struct attribute_group attr_group = {
+	.attrs = attrs,
+};
+
 static int imx8mq_init_busfreq_clk(struct platform_device *pdev)
 {
 	dram_pll_clk = devm_clk_get(&pdev->dev, "dram_pll");
@@ -616,16 +612,17 @@ static int imx8mq_init_busfreq_clk(struct platform_device *pdev)
 
 static int imx8mm_init_busfreq_clk(struct platform_device *pdev)
 {
+	dram_pll = devm_clk_get(&pdev->dev, "dram_pll_div");
 	dram_pll_clk = devm_clk_get(&pdev->dev, "dram_pll");
 	dram_alt_src = devm_clk_get(&pdev->dev, "dram_alt_src");
 	dram_alt_root = devm_clk_get(&pdev->dev, "dram_alt_root");
 	dram_core_clk = devm_clk_get(&pdev->dev, "dram_core");
 	dram_apb_src = devm_clk_get(&pdev->dev, "dram_apb_src");
 	dram_apb_pre_div = devm_clk_get(&pdev->dev, "dram_apb_pre_div");
-        dram_alt_pre_div = devm_clk_get(&pdev->dev, "dram_alt_pre_div");
+	dram_alt_pre_div = devm_clk_get(&pdev->dev, "dram_alt_pre_div");
 	sys1_pll_800m = devm_clk_get(&pdev->dev, "sys_pll1_800m");
-	sys1_pll_100m = devm_clk_get(&pdev->dev, "sys_pll1_100m");
 	sys1_pll_160m = devm_clk_get(&pdev->dev, "sys_pll1_160m");
+	sys1_pll_100m = devm_clk_get(&pdev->dev, "sys_pll1_100m");
 	sys1_pll_40m = devm_clk_get(&pdev->dev, "sys_pll1_40m");
 	noc_div = devm_clk_get(&pdev->dev, "noc_div");
 	ahb_div = devm_clk_get(&pdev->dev, "ahb_div");
@@ -635,7 +632,7 @@ static int imx8mm_init_busfreq_clk(struct platform_device *pdev)
 
 	if (IS_ERR(dram_pll_clk) || IS_ERR(dram_alt_src) || IS_ERR(dram_alt_root) ||
 	    IS_ERR(dram_core_clk) || IS_ERR(dram_apb_src) || IS_ERR(dram_apb_pre_div) ||
-	    IS_ERR(sys1_pll_800m) || IS_ERR(sys1_pll_160m) ||  IS_ERR(sys1_pll_100m) || IS_ERR(sys1_pll_40m) ||
+	    IS_ERR(sys1_pll_800m) || IS_ERR(sys1_pll_160m) || IS_ERR(sys1_pll_100m) || IS_ERR(sys1_pll_40m) ||
 	    IS_ERR(osc_25m) || IS_ERR(noc_div) || IS_ERR(main_axi_src) || IS_ERR(ahb_div) ||
 	    IS_ERR(sys2_pll_333m)) {
 		dev_err(&pdev->dev, "failed to get busfreq clk\n");
@@ -653,9 +650,12 @@ static int imx8mm_init_busfreq_clk(struct platform_device *pdev)
  * @return         The function returns 0 on success
  *
  */
+
 static int busfreq_probe(struct platform_device *pdev)
 {
-	int err;
+	int i, err;
+	struct arm_smccc_res res;
+	const struct device_node *np = pdev->dev.of_node;
 
 	busfreq_dev = &pdev->dev;
 
@@ -670,8 +670,42 @@ static int busfreq_probe(struct platform_device *pdev)
 		return err;
 	}
 
+	origin_noc_rate = clk_get_rate(noc_div);
+	if (origin_noc_rate == 0) {
+		WARN_ON(1);
+		return -EINVAL;
+	}
+
+	/*
+	 * Get the supported frequency, normally the lowest frequency point
+	 * is used for low bus & audio bus mode.
+	 */
+	for (i = 0; i < 4; i++) {
+		arm_smccc_smc(FSL_SIP_DDR_DVFS, 0x11, i, 0, 0, 0, 0, 0, &res);
+		err = res.a0;
+		if (err < 0)
+			return -EINVAL;
+
+		fsp_table[i] = res.a0;
+	}
+
+	/* get the lowest fsp index */
+	for (i = 0; i < 4; i++)
+		if (fsp_table[i] == 0)
+			break;
+
+	low_bus_mode_fsp_index = i - 1;
+	audio_bus_mode_fsp_index = i > 2 ? i - 2:low_bus_mode_fsp_index;
+
+	/*
+	 * if lowest fsp data rate higher than 666mts, then no dll off mode or
+	 * bypass mode support.
+	 */
+	if (fsp_table[low_bus_mode_fsp_index] >= DLL_ON_DRATE)
+		bypass_support = false;
+
 	/* create the sysfs file */
-	err = sysfs_create_file(&busfreq_dev->kobj, &dev_attr_enable.attr);
+	err = sysfs_create_group(&busfreq_dev->kobj, &attr_group);
 	if (err) {
 		dev_err(busfreq_dev,
 			"Unable to register sysdev entry for BUSFREQ");
@@ -694,6 +728,11 @@ static int busfreq_probe(struct platform_device *pdev)
 	/* enter low bus mode if no high speed device enabled */
 	schedule_delayed_work(&bus_freq_daemon, msecs_to_jiffies(10000));
 
+	if (of_property_read_bool(np, "enable-ddr-400mts"))
+		ddr_400mts = true;
+	else
+		ddr_400mts = false;
+
 	return 0;
 }
 
@@ -721,7 +760,7 @@ static int __init busfreq_init(void)
 	if (platform_driver_register(&busfreq_driver) != 0)
 		return -ENODEV;
 
-	printk(KERN_INFO "Bus freq driver module loaded\n");
+	pr_info("Bus freq driver module loaded\n");
 
 	return 0;
 }
