@@ -3,6 +3,7 @@
  *
  * Copyright (c) 2018 MediaTek Inc.
  * Author: Hidalgo Huang <hidalgo.huang@mediatek.com>
+ * Copyright 2022 Sony Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 and
@@ -21,8 +22,8 @@
 #include <linux/atomic.h>
 #include <sound/compress_driver.h>
 #include <sound/soc.h>
-#include "mt8570-adsp-common.h"
-#include "hifi4dsp_load/hifi4dsp_load.h"
+#include <soc/mediatek/mt8570-adsp-common.h>
+#include <soc/mediatek/hifi4dsp_load.h>
 
 
 struct mt8570_adsp_compr_stream {
