@@ -22,18 +22,20 @@
 #include <linux/of_address.h>
 #include <linux/workqueue.h>
 #include <linux/atomic.h>
+#include <linux/semaphore.h>
 #include <linux/timekeeping.h>
 #include <linux/dma-mapping.h>
 #include <linux/io.h>
 #include <sound/soc.h>
 #include <sound/pcm_params.h>
-#include "hifi4dsp_load/hifi4dsp_load.h"
-#include "mt8570-adsp-common.h"
-#include "mt8570-adsp-controls.h"
-#include "mt8570-adsp-debug.h"
+#include <soc/mediatek/hifi4dsp/hifi4dsp_load.h>
+#include <soc/mediatek/mt8570-adsp-common.h>
+#include <soc/mediatek/mt8570-adsp-controls.h>
+#include <soc/mediatek/mt8570-adsp-debug.h>
 #ifdef CONFIG_SND_SOC_COMPRESS
-#include "mt8570-adsp-compress.h"
+#include <soc/mediatek/mt8570-adsp-compress.h>
 #endif
+#include <soc/mediatek/mt8570-adsp-pcm.h>
 #ifdef MULTI_DSP_LOAD
 #include <linux/gpio.h>
 #include <linux/of_gpio.h>
@@ -62,6 +64,7 @@
 static struct mt8570_adsp_pcm_priv *g_priv;
 
 #ifdef MULTI_DSP_LOAD
+DEFINE_SEMAPHORE(sem_cs);
 struct mt8570_adsp_pcm_priv *global_priv;
 struct delayed_work mt8570_work;
 struct workqueue_struct *write_spi;
@@ -81,6 +84,35 @@ static struct mt8570_ul_ts_data g_ts_data[MT8570_UL_TS_CNT] = {
 #endif
 };
 
+int mt8570_dsp_select(int number)
+{
+#ifdef MULTI_DSP_LOAD
+	down(&sem_cs);
+	switch (number) {
+	case 1:
+		gpio_set_value(g_priv->dspcs_gpio, 0);
+		break;
+	case 2:
+		gpio_set_value(g_priv->dspcs_gpio, 1);
+		if (number_of_dsp < 3)
+			break;
+		gpio_set_value(g_priv->dspsw2_gpio, 0);
+		break;
+	case 3:
+		gpio_set_value(g_priv->dspcs_gpio, 1);
+		if (number_of_dsp < 3)
+			break;
+		gpio_set_value(g_priv->dspsw2_gpio, 1);
+		break;
+	default:
+		pr_err("dsp number not exists\n");
+	}
+	up(&sem_cs);
+#endif /* MULTI_DSP_LOAD */
+	return 0;
+}
+EXPORT_SYMBOL_GPL(mt8570_dsp_select);
+
 static struct mt8570_ul_ts_data *mt8570_get_ts_data(int id)
 {
 	int i;
@@ -333,7 +365,7 @@ static int dma_fetch_pending_and_update(struct mt8570_adsp_dai_memory *dai_mem)
 	struct mt8570_host_dma_buffer *host_dmab = &dai_mem->host_dmab;
 	int pend = 0;
 	unsigned long flags;
-	struct timespec ts1;
+	struct timespec64 ts1;
 
 	spin_lock_irqsave(&dai_mem->host_dma_lock, flags);
 
@@ -410,7 +442,7 @@ static enum hrtimer_restart dma_elapse_hrtimer_callback(struct hrtimer *hrt)
 
 	elapse = dma_fetch_pending_and_update(dai_mem);
 	if (elapse <= 0) {
-		struct timespec ts1 = ktime_to_timespec(now);
+		struct timespec64 ts1 = ktime_to_timespec64(now);
 
 		dev_info(dai_mem->dev,
 		"%s [%s] no pend elapse(adsp hw 0x%x appl 0x%x)\n",
@@ -421,8 +453,8 @@ static enum hrtimer_restart dma_elapse_hrtimer_callback(struct hrtimer *hrt)
 		dev_info(dai_mem->dev,
 		"%s [%s] elapse(%llu ns) transfer %llu data 0x%x\n",
 		__func__, mt8570_adsp_dai_name(dai_mem->id),
-		(timespec_to_ns(&ts1) -
-		timespec_to_ns(&dai_mem->trigger_start_ts)),
+		(timespec64_to_ns(&ts1) -
+		timespec64_to_ns(&dai_mem->trigger_start_ts)),
 		dai_mem->total_transfer_bytes,
 		atomic_read(&dai_mem->host_dmab.data_size));
 
@@ -524,7 +556,7 @@ static void mt8570_adsp_pcm_fe_shutdown(struct snd_pcm_substream *substream,
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif
 		
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 
 	dev_dbg(rtd->dev, "%s [%s]\n", __func__, mt8570_adsp_dai_name(id));
@@ -559,7 +591,7 @@ static int mt8570_adsp_pcm_fe_hw_params(struct snd_pcm_substream *substream,
 	struct mt8570_adsp_pcm_priv *priv =
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 
@@ -684,7 +716,7 @@ static int mt8570_adsp_pcm_fe_hw_free(struct snd_pcm_substream *substream,
 	struct mt8570_adsp_pcm_priv *priv =
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 	struct mt8570_adsp_dai_memory *dai_mem = &priv->dai_mem[id];
@@ -727,7 +759,7 @@ static int mt8570_adsp_pcm_fe_prepare(struct snd_pcm_substream *substream,
 	struct mt8570_adsp_pcm_priv *priv =
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 
@@ -768,7 +800,7 @@ static int mt8570_adsp_pcm_fe_trigger(struct snd_pcm_substream *substream,
 	struct mt8570_adsp_pcm_priv *priv =
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	struct mt8570_adsp_dai_memory *dai_mem = &priv->dai_mem[id];
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
@@ -795,7 +827,7 @@ static int mt8570_adsp_pcm_fe_trigger(struct snd_pcm_substream *substream,
 				&dai_mem->trigger_start_ts);
 
 			dai_mem->target_elapse_kt =
-				timespec_to_ktime(dai_mem->trigger_start_ts);
+				timespec64_to_ktime(dai_mem->trigger_start_ts);
 			dai_mem->target_elapse_kt =
 				ktime_add_ns(dai_mem->target_elapse_kt,
 					     dai_mem->period_time_ns);
@@ -844,7 +876,7 @@ static int mt8570_adsp_pcm_be_startup(struct snd_pcm_substream *substream,
 				       struct snd_soc_dai *dai)
 {
 	struct snd_soc_pcm_runtime *rtd = substream->private_data;
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 
@@ -871,7 +903,7 @@ static void mt8570_adsp_pcm_be_shutdown(struct snd_pcm_substream *substream,
 					 struct snd_soc_dai *dai)
 {
 	struct snd_soc_pcm_runtime *rtd = substream->private_data;
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 
 	dev_dbg(rtd->dev, "%s [%s]\n", __func__, mt8570_adsp_dai_name(id));
@@ -903,7 +935,7 @@ static int mt8570_adsp_pcm_be_hw_params(struct snd_pcm_substream *substream,
 	struct mt8570_adsp_pcm_priv *priv =
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 	struct host_ipc_msg_hw_param ipc_hw_param;
@@ -1028,7 +1060,7 @@ static int mt8570_adsp_pcm_be_hw_free(struct snd_pcm_substream *substream,
 				       struct snd_soc_dai *dai)
 {
 	struct snd_soc_pcm_runtime *rtd = substream->private_data;
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 
@@ -1051,7 +1083,7 @@ static int mt8570_adsp_pcm_be_prepare(struct snd_pcm_substream *substream,
 				       struct snd_soc_dai *dai)
 {
 	struct snd_soc_pcm_runtime *rtd = substream->private_data;
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 
@@ -1075,7 +1107,7 @@ static int mt8570_adsp_pcm_be_trigger(struct snd_pcm_substream *substream,
 				       struct snd_soc_dai *dai)
 {
 	struct snd_soc_pcm_runtime *rtd = substream->private_data;
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	int scene = mt8570_adsp_get_scene_by_dai_id(id);
 	int ret = 0;
 
@@ -2080,7 +2112,7 @@ static int mt8570_adsp_pcm_pcm_copy(struct snd_pcm_substream *substream,
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif
 		
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	struct mt8570_adsp_dai_memory *dai_mem = &priv->dai_mem[id];
 	ssize_t copy_bytes = frames_to_bytes(runtime, count);
 	//copy_bytes = 32;
@@ -2191,14 +2223,14 @@ static snd_pcm_uframes_t mt8570_adsp_pcm_pcm_pointer(
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif	
 	
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 	struct mt8570_adsp_dai_memory *dai_mem = &priv->dai_mem[id];
 	struct mt8570_host_dma_buffer *host_dmab = &dai_mem->host_dmab;
 	uint32_t hw_offset = host_dmab->hw_offset;
 
 	if (mt8570_adsp_need_dl_dma_copy(id) && snd_pcm_running(substream)) {
-		struct timespec ts;
-		struct timespec prev_ts;
+		struct timespec64 ts;
+		struct timespec64 prev_ts;
 		s64 diff;
 		unsigned long flags;
 
@@ -2211,7 +2243,7 @@ static snd_pcm_uframes_t mt8570_adsp_pcm_pcm_pointer(
 
 		snd_pcm_gettime(runtime, &ts);
 
-		diff = timespec_to_ns(&ts) - timespec_to_ns(&prev_ts);
+		diff = timespec64_to_ns(&ts) - timespec64_to_ns(&prev_ts);
 		if (diff > 0 && diff < dai_mem->period_time_ns) {
 			u64 tmp = diff * runtime->rate;
 
@@ -2250,7 +2282,7 @@ static int mt8570_adsp_pcm_pcm_ack(struct snd_pcm_substream *substream)
 		snd_soc_platform_get_drvdata(rtd->platform);
 #endif	
 	
-	int id = rtd->cpu_dai->id;
+	int id = asoc_rtd_to_cpu(rtd, 0)->id;
 
 	if (mt8570_adsp_check_dma_copy_by_ack(id)) {
 		struct mt8570_adsp_dai_memory *dai_mem = &priv->dai_mem[id];
@@ -2760,11 +2792,11 @@ static ssize_t sw_version_show(struct device *dev,
 	struct mt8570_adsp_pcm_priv *priv = dev_get_drvdata(dev);
 	ssize_t count = 0;
 
-	count += snprintf(buf + count, PAGE_SIZE - count,
+	count += sprintf(buf + count,
 		"Firmware Version: %08x\n",
 		priv->firmware_version);
 
-	count += snprintf(buf + count, PAGE_SIZE - count,
+	count += sprintf(buf + count,
 		"Driver Version: %s\n",
 		MT8570_HOST_DRV_VER);
 
@@ -2794,14 +2826,7 @@ static void fw_writer(struct work_struct *work)
 #ifdef HIFI4DSP_RESET
 	int ret = 0;
 
-	gpio_request(g_priv->dspcs_gpio, "hifi4dsp-cs");
-	gpio_direction_output(g_priv->dspcs_gpio, 1);
-	gpio_set_value(g_priv->dspcs_gpio, 0);
-	if (number_of_dsp > 2) {
-		gpio_request(g_priv->dspsw2_gpio, "hifi4dsp-sw2");
-		gpio_direction_output(g_priv->dspsw2_gpio, 1);
-		gpio_set_value(g_priv->dspsw2_gpio, 0);
-	}
+	mt8570_dsp_select(1);
 	ret = load_hifi4dsp_bin_and_run();
 	if (ret) {
 		pr_info("%s load_hifi4dsp_bin_and_run fail %d\n",
@@ -2809,7 +2834,7 @@ static void fw_writer(struct work_struct *work)
 		return;
 	}
 	msleep(130);
-	gpio_set_value(g_priv->dspcs_gpio, 1);
+	mt8570_dsp_select(2);
 	ret = load_hifi4dsp_bin_and_run();
 	if (number_of_dsp <= 2)
 		return;
@@ -2819,10 +2844,10 @@ static void fw_writer(struct work_struct *work)
 		return;
 	}
 	msleep(130);
-	gpio_set_value(g_priv->dspsw2_gpio, 1);
+	mt8570_dsp_select(3);
 	load_hifi4dsp_bin_and_run();
 #else
-	gpio_set_value(g_priv->dspcs_gpio, 1);
+	mt8570_dsp_select(2);
 	async_load_hifi4dsp_bin_and_run(NULL, NULL);
 #endif
 }
@@ -2830,7 +2855,7 @@ static void fw_writer(struct work_struct *work)
 static void fw_writer3(struct work_struct *work)
 {
 #ifndef HIFI4DSP_RESET
-        gpio_set_value(g_priv->dspsw2_gpio, 1);
+        mt8570_dsp_select(3);
         async_load_hifi4dsp_bin_and_run(NULL, NULL);
 #endif
 }
@@ -2874,16 +2899,7 @@ static int mt8570_adsp_pcm_pcm_probe(struct snd_soc_platform *platform)
 	}
 
 #if defined(MULTI_DSP_LOAD) && !defined(HIFI4DSP_RESET)
-	gpio_request(g_priv->dspcs_gpio, "hifi4dsp-cs");
-	gpio_direction_output(g_priv->dspcs_gpio, 1);
-	gpio_set_value(g_priv->dspcs_gpio, 0);
-
-	if (number_of_dsp > 2) {
-		gpio_request(g_priv->dspsw2_gpio, "hifi4dsp-sw2");
-		gpio_direction_output(g_priv->dspsw2_gpio, 1);
-		gpio_set_value(g_priv->dspsw2_gpio, 0);
-	}
-
+	mt8570_dsp_select(1);
 	ret = async_load_hifi4dsp_bin_and_run(NULL, NULL);
 #endif /* MULTI_DSP_LOAD */
 
@@ -2903,11 +2919,7 @@ static int mt8570_adsp_pcm_pcm_probe(struct snd_soc_platform *platform)
 	write_spi = create_singlethread_workqueue("MT8570 spi");
 	queue_delayed_work(write_spi,
 			&mt8570_work,
-#ifdef HIFI4DSP_RESET
-			msecs_to_jiffies(boot_delay));
-#else
 			msecs_to_jiffies(20000));
-#endif
 
 	if (number_of_dsp > 2) {
                 dev_info(platform->dev,"Re-Re-Load MT8570 FW.\n");
@@ -2915,11 +2927,9 @@ static int mt8570_adsp_pcm_pcm_probe(struct snd_soc_platform *platform)
                 INIT_DELAYED_WORK(&mt8570_work3, fw_writer3);
 
                 write_spi3 = create_singlethread_workqueue("MT8570 spi 3");
-#ifndef HIFI4DSP_RESET
                 queue_delayed_work(write_spi3,
                                 &mt8570_work3,
                                 msecs_to_jiffies(24000));
-#endif
 	}
 #endif /* MULTI_DSP_LOAD */
 
@@ -3077,12 +3087,14 @@ static ssize_t mt8570_adsp_pcm_selector_show(
 #ifdef MULTI_DSP_LOAD
 	int value = 0;
 
+	down(&sem_cs);
 	if (!gpio_get_value(g_priv->dspcs_gpio))
 		value = 1;
 	else if (number_of_dsp < 3 || !gpio_get_value(g_priv->dspsw2_gpio))
 		value = 2;
 	else
 		value = 3;
+	up(&sem_cs);
 	result += snprintf(buf + result, PAGE_SIZE - result, "%.d\n", value);
 #endif
 	return result;
@@ -3095,23 +3107,7 @@ static ssize_t mt8570_adsp_pcm_selector_store(
 	size_t size)
 {
 #ifdef MULTI_DSP_LOAD
-	switch (*buf) {
-	case '1':
-		gpio_set_value(g_priv->dspcs_gpio, 0);
-		break;
-	case '2':
-		gpio_set_value(g_priv->dspcs_gpio, 1);
-		if (number_of_dsp < 3)
-			break;
-		gpio_set_value(g_priv->dspsw2_gpio, 0);
-		break;
-	case '3':
-		gpio_set_value(g_priv->dspcs_gpio, 1);
-		if (number_of_dsp < 3)
-			break;
-		gpio_set_value(g_priv->dspsw2_gpio, 1);
-		break;
-	}
+	mt8570_dsp_select(*buf - '0');
 #endif
 	return size;
 }
@@ -3132,9 +3128,7 @@ static int mt8570_adsp_pcm_pm_suspend(struct device *dev)
 {
 #if defined(MULTI_DSP_LOAD) && defined(HIFI4DSP_RESET)
 	cancel_delayed_work_sync(&mt8570_work);
-	gpio_set_value(g_priv->dspcs_gpio, 0);
-	if (number_of_dsp > 2)
-		gpio_set_value(g_priv->dspsw2_gpio, 0);
+	mt8570_dsp_select(1);
 #endif
 	return 0;
 }
@@ -3142,7 +3136,7 @@ static int mt8570_adsp_pcm_pm_suspend(struct device *dev)
 static int mt8570_adsp_pcm_pm_resume(struct device *dev)
 {
 #if defined(MULTI_DSP_LOAD) && defined(HIFI4DSP_RESET)
-	queue_delayed_work(write_spi, &mt8570_work, msecs_to_jiffies(50));
+	queue_delayed_work(write_spi, &mt8570_work, msecs_to_jiffies(200));
 #endif
 	return 0;
 }
@@ -3195,8 +3189,7 @@ static int mt8570_adsp_pcm_parse_of(struct mt8570_adsp_pcm_priv *priv,
 		goto error;
 	}
 
-	priv->reserved_memory_vaddr = devm_ioremap_nocache(priv->dev,
-		res.start, resource_size(&res));
+	priv->reserved_memory_vaddr = devm_ioremap(priv->dev, res.start, resource_size(&res));
 	if (!priv->reserved_memory_vaddr) {
 		dev_err(priv->dev,
 			"%s Unable to remap addr(0x%llx) size(0x%llx)\n",
@@ -3275,13 +3268,19 @@ static int mt8570_adsp_pcm_dev_probe(struct platform_device *pdev)
 					"mtk8570-dspcs-gpios", 0);
 	if (!gpio_is_valid(priv->dspcs_gpio)) {
 		dev_err(&pdev->dev, "No DSP CS GPIO Supplied.\n");
+	} else {
+		gpio_request(priv->dspcs_gpio, "hifi4dsp-cs");
+		gpio_direction_output(priv->dspcs_gpio, 1);
 	}
 
 	if (number_of_dsp > 2) {
                 priv->dspsw2_gpio = of_get_named_gpio(pdev->dev.of_node,
                                                 "mtk8570-dspsw2-gpios", 0);
-                if (number_of_dsp > 2 && !gpio_is_valid(priv->dspsw2_gpio)) {
+                if (!gpio_is_valid(priv->dspsw2_gpio)) {
                         dev_err(&pdev->dev, "No DSP SW2 GPIO Supplied.\n");
+                } else {
+                        gpio_request(priv->dspsw2_gpio, "hifi4dsp-sw2");
+                        gpio_direction_output(priv->dspsw2_gpio, 1);
                 }
 	}
 #endif /* MULTI_DSP_LOAD */
@@ -3296,6 +3295,22 @@ static int mt8570_adsp_pcm_dev_probe(struct platform_device *pdev)
 	if (ret)
 		return ret;
 
+#ifdef HIFI4DSP_RESET
+	dev_info(dev,"Re-Load MT8570 FW.\n");
+
+	INIT_DELAYED_WORK(&mt8570_work, fw_writer);
+
+	global_priv =priv;
+	write_spi = create_singlethread_workqueue("MT8570 spi");
+	queue_delayed_work(write_spi,
+			&mt8570_work,
+			msecs_to_jiffies(boot_delay));
+
+	if (number_of_dsp > 2) {
+                INIT_DELAYED_WORK(&mt8570_work3, fw_writer3);
+                write_spi3 = create_singlethread_workqueue("MT8570 spi 3");
+	}
+#else
 #ifndef SARD_4_19_35
 	ret = snd_soc_register_platform(dev, &mt8570_adsp_pcm_pcm_platform);
 
@@ -3313,6 +3328,7 @@ static int mt8570_adsp_pcm_dev_probe(struct platform_device *pdev)
 		dev_err(dev, "Failed to register component\n");
 		goto err_platform;
 	}
+#endif
 
 	g_priv = priv;
 
@@ -3322,6 +3338,7 @@ static int mt8570_adsp_pcm_dev_probe(struct platform_device *pdev)
 	dev_info(dev, "%s initialized.\n", __func__);
 	return 0;
 
+#ifndef HIFI4DSP_RESET
 err_platform:
 #ifdef SARD_4_19_35
 	snd_soc_unregister_component(dev);
@@ -3329,18 +3346,21 @@ err_platform:
 	snd_soc_unregister_platform(dev);
 #endif
 	return ret;
+#endif
 }
 
 static int mt8570_adsp_pcm_dev_remove(struct platform_device *pdev)
 {
 	/* remove sysfs */
 	sysfs_remove_group(&pdev->dev.kobj, &mt8570_adsp_pcm_attr_group);
+#ifndef HIFI4DSP_RESET
 #ifdef SARD_4_19_35
 	snd_soc_unregister_component(&pdev->dev);
 #else	
 	snd_soc_unregister_component(&pdev->dev);
 	snd_soc_unregister_platform(&pdev->dev);
 #endif	
+#endif
 	return 0;
 }
 
