@@ -1,8 +1,8 @@
-/* SPDX-License-Identifier: GPL-2.0 */
+// SPDX-License-Identifier: GPL-2.0
 /*
- * ifcon_drv.c - IFCON Communication Driver
+ * IFCON Communication Driver
  *
- * Copyright 2017, 2019, 2020, 2021, 2022 Sony Corporation
+ * Copyright 2017, 2019, 2020, 2021, 2022, 2023 Sony Corporation
  *
  */
 
@@ -25,6 +25,7 @@
 #include <linux/sched/signal.h>
 #include <linux/pm_runtime.h>
 #include <linux/string.h>
+#include <linux/ctype.h>
 #include <linux/ratelimit.h>
 
 #include "ifcon_drv.h"
@@ -95,6 +96,8 @@ do {									\
 				IFCON_DRV_DEVNAME, ##args)
 #define LOG_ERR(fmt, args...) pr_err("%s: [%s] " fmt,\
 				IFCON_DRV_DEVNAME, __func__, ##args)
+#define LOG_ERR_SHORT(fmt, args...)\
+			pr_err("%s: " fmt, IFCON_DRV_DEVNAME, ##args)
 #define LOG_PR_INFO(fmt, args...) pr_info(fmt, ##args)
 #if DEBUG_LOG
 #define LOG_DBG(fmt, args...) pr_info(fmt, ##args)
@@ -169,6 +172,11 @@ enum ifcon_bd_req_pin_status {
 	IFCON_DRV_BD_REQ_ON,
 };
 
+/* debug send / monitor */
+#define IFCON_DRV_PRARAM_STRING_SIZE_MAX 1024 /* kernel module parameters */
+#define IFCON_DRV_DEBUG_SEND_SIZE_MAX	240	/* IFCON_DRV_PACKAGE_SIZE_MAX */
+#define IFCON_DRV_DEBUG_FILTER_SIZE_MAX	4	/* monitor filter size */
+
 //============================
 #define IFCON_DRV_LOSS_OUT_COUNT	(100)		/* 100 times */
 
@@ -194,6 +202,8 @@ struct ifcon_drv_data_t {
 	struct device		*dev;
 	struct task_struct	*kthread_tsk;
 	struct mutex		state_lock;
+	struct mutex		buf_lock;
+	struct mutex		monitor_lock;
 	wait_queue_head_t	rxwait;
 	unsigned int		users;
 	u8			count_stat;
@@ -263,7 +273,7 @@ static int ifcon_drv_cur_frame_all_size = IFCON_DRV_FRAME_BASE_SIZE
 					+ IFCON_DRV_RW_DATA_SIZE;
 
 static void ifcon_mem_dump(char *title, char *p_mem, int size);
-
+static int tx_ring_write(const char __user *buf, int count);
 
 /*--- Module parameters -----------------------------------------------------*/
 
@@ -413,6 +423,259 @@ int test_mode;
 module_param(test_mode, int, 0644);
 #endif
 
+/* command line - common */
+static void ifcon_drv_parse_cmd_send_param(char *args, int arg_pos);
+static void ifcon_drv_parse_cmd_monitor_param(char *args, int arg_pos);
+static int ifcon_drv_hex_to_byte(const char *src, int *pos)
+{
+	u8 byte;
+	int h = hex_to_bin(*src++);
+	int l = hex_to_bin(*src++);
+
+	if ((h < 0) || (l < 0))
+		return -1;/* 2-digit hex is required */
+	byte = (h << 4) | l;
+	(*pos) += 2;
+	return (int)byte;
+}
+static bool ifcon_drv_param_is_delimiter(unsigned char c)
+{
+	bool ret = false;
+
+	if ((c == 0) || (c == '\n'))
+		ret = true;
+	return ret;
+}
+static int ifcon_drv_param_skip_spaces(char *args, int *pos)
+{
+	int ret = 0;
+	int i;
+	char c;
+
+	for (i = *pos; i < IFCON_DRV_PRARAM_STRING_SIZE_MAX; i++) {
+		c = args[i];
+		if (ifcon_drv_param_is_delimiter(c)) {
+			ret = -1;
+			break;
+		} else if (!isspace(c)) {
+			break;
+		}
+		(*pos)++;
+	}
+	return ret;
+}
+static int ifcon_drv_param_set_cmd(const char *val,
+						const struct kernel_param *kp)
+{
+	char args[IFCON_DRV_PRARAM_STRING_SIZE_MAX + 1];
+	int arg_pos = 0;
+	int ret;
+
+	if (!val)
+		return -EINVAL;
+	memcpy(args, val, IFCON_DRV_PRARAM_STRING_SIZE_MAX);
+	args[strnlen(args, IFCON_DRV_PRARAM_STRING_SIZE_MAX) - 1] = 0;
+	ret = ifcon_drv_param_skip_spaces(args, &arg_pos);
+	if (ret < 0) {
+		return 0;
+	} else if (!strncasecmp(&args[arg_pos], "monitor", 7)) {
+		ifcon_drv_parse_cmd_monitor_param(args, arg_pos + 7);
+	} else if (!strncasecmp(&args[arg_pos], "send", 4)) {
+		ifcon_drv_parse_cmd_send_param(args, arg_pos + 4);
+	} else {
+		LOG_ERR_SHORT("USAGE: monitor\n");
+		LOG_ERR_SHORT("       send\n");
+	}
+	return 0;
+}
+static const struct kernel_param_ops ifcon_drv_param_ops_cmd = {
+	.set = ifcon_drv_param_set_cmd,
+};
+module_param_cb(cmd, &ifcon_drv_param_ops_cmd, NULL, 0200);
+/* command line - send */
+static ssize_t ifcon_drv_debug_send(const char *buf, size_t count)
+{
+	ssize_t status = 0;
+
+	if (ifcon_drv_dev->users == 0) {
+		LOG_ERR_SHORT("Not opened error !!\n");
+		return IFCON_DRV_NG;
+	}
+	if (ifcon_drv_power == IFCON_DRV_POWER_SUSPEND) {
+		LOG_ERR_SHORT("Already suspended error !!\n");
+		return -ENETDOWN;
+	}
+	if (count > ifcon_drv_cur_total_data_size) {
+		LOG_ERR_SHORT("Sending data size over error !!\n");
+		return -EMSGSIZE;
+	}
+
+	if (frame_type != ifcon_drv_cur_rx_frame_type) {
+		LOG_ERR_SHORT("Bad frame type error !! rx type=%d\n",
+						ifcon_drv_cur_rx_frame_type);
+		return -IFCON_DRV_BADTYPE;
+	}
+
+	mutex_lock(&ifcon_drv_dev->buf_lock);
+	status = (ssize_t)tx_ring_write(buf, (int)count);
+	mutex_unlock(&ifcon_drv_dev->buf_lock);
+
+	return status;
+}
+static void ifcon_drv_parse_cmd_send_param(char *args, int arg_pos)
+{
+	u8 send_buffer[IFCON_DRV_DEBUG_SEND_SIZE_MAX];
+	int write_pos = 0;
+	int ret;
+	int category;
+	int data;
+	ssize_t send_count;
+
+	if (!isspace(args[arg_pos])
+		|| (ifcon_drv_param_skip_spaces(args, &arg_pos) < 0)) {
+		LOG_ERR_SHORT(
+			"USAGE: send <cat> [hh] [hh] ... [hh]\n");
+		return;
+	}
+	category = ifcon_drv_hex_to_byte(&args[arg_pos], &arg_pos);
+	if (category < 0) {
+		LOG_ERR_SHORT("invalid parameter !!\n");
+		return;
+	}
+	send_buffer[write_pos++] = 0;
+	send_buffer[write_pos++] = 0;
+	send_buffer[write_pos++] = 0;
+	send_buffer[write_pos++] = (u8)category;
+	ret = ifcon_drv_param_skip_spaces(args, &arg_pos);
+	while (!ifcon_drv_param_is_delimiter(args[arg_pos])) {
+		data = ifcon_drv_hex_to_byte(&args[arg_pos], &arg_pos);
+		if (data >= 0) {
+			if (write_pos >= sizeof(send_buffer)) {
+				LOG_ERR_SHORT("too many parameters !!\n");
+				return;
+			}
+			send_buffer[write_pos] = (u8)data;
+			write_pos++;
+		} else {
+			LOG_ERR_SHORT("invalid parameter !!\n");
+			return;
+		}
+		ret = ifcon_drv_param_skip_spaces(args, &arg_pos);
+	}
+	send_buffer[0] = (u8)write_pos;	/* total length */
+	send_count = ifcon_drv_debug_send(send_buffer, (size_t)write_pos);
+}
+/* command line - monitor */
+static int monitor_category; /* If zero, no monitor */
+static int monitor_filter_size; /* filter valid size */
+static u8 monitor_filter_data[IFCON_DRV_DEBUG_FILTER_SIZE_MAX];
+static void ifcon_drv_parse_cmd_monitor_on(char *args, int arg_pos)
+{
+	int ret;
+	int category;
+	int data;
+	int filter_size = 0;
+	u8 filter_data[IFCON_DRV_DEBUG_FILTER_SIZE_MAX];
+
+	if (!isspace(args[arg_pos])
+		|| (ifcon_drv_param_skip_spaces(args, &arg_pos) < 0)) {
+		LOG_ERR_SHORT("USAGE: monitor on <cat> [hh] [hh] [hh] [hh]\n");
+		return;
+	}
+	category = ifcon_drv_hex_to_byte(&args[arg_pos], &arg_pos);
+	if (category < 0) {
+		LOG_ERR_SHORT("invalid parameter !!\n");
+		return;
+	}
+	ret = ifcon_drv_param_skip_spaces(args, &arg_pos);
+	while (!ifcon_drv_param_is_delimiter(args[arg_pos])) {
+		data = ifcon_drv_hex_to_byte(&args[arg_pos], &arg_pos);
+		if (data >= 0) {
+			if (filter_size >= IFCON_DRV_DEBUG_FILTER_SIZE_MAX) {
+				LOG_ERR_SHORT("too many parameters !!\n");
+				return;
+			}
+			filter_data[filter_size] = (u8)data;
+			filter_size++;
+		} else {
+			LOG_ERR_SHORT("invalid parameter !!\n");
+			return;
+		}
+		ret = ifcon_drv_param_skip_spaces(args, &arg_pos);
+	}
+
+	mutex_lock(&ifcon_drv_dev->monitor_lock);
+	for (monitor_filter_size = 0; monitor_filter_size < filter_size;
+							monitor_filter_size++)
+		monitor_filter_data[monitor_filter_size] =
+					filter_data[monitor_filter_size];
+	monitor_category = category;
+	mutex_unlock(&ifcon_drv_dev->monitor_lock);
+}
+static void ifcon_drv_monitor_dump(char *p_mem, int size)
+{
+	int i, len, xpos;
+	char buf[90];
+
+	len = 0;
+	for (i = 0; i < size; i++) {
+		xpos = i % 16;
+		if (xpos == 0) {
+			len += snprintf(&buf[len], 32, "%s: ",
+							IFCON_DRV_DEVNAME);
+			len += snprintf(&buf[len], 7, "[%04x]", i);
+		}
+		len += snprintf(&buf[len], 4, " %02x", p_mem[i]);
+		if (xpos == 15) {
+			LOG_PR_INFO("%s\n", buf);
+			len = 0;
+		}
+	}
+	if (len)
+		LOG_PR_INFO("%s\n", buf);
+}
+static void ifcon_drv_debug_monitor(char *buf, int count)
+{
+	struct ifcon_drv_frame_def *frame
+				= (struct ifcon_drv_frame_def *)buf;
+	int i;
+
+	if (count < (IFCON_DRV_CTL_SIZE + monitor_filter_size))
+		return;
+	mutex_lock(&ifcon_drv_dev->monitor_lock);
+	if (frame->info.dest != (u8)monitor_category)
+		goto monitor_err_exit;
+	for (i = 0; i < monitor_filter_size; i++) {
+		if (frame->data[i] != monitor_filter_data[i])
+			goto monitor_err_exit;
+	}
+	mutex_unlock(&ifcon_drv_dev->monitor_lock);
+	buf = buf + offsetof(struct ifcon_drv_frame_def, info.dest);
+	count -= offsetof(struct ifcon_drv_frame_def, info.dest);
+	ifcon_drv_monitor_dump(buf, count);
+	return;
+
+monitor_err_exit:
+	mutex_unlock(&ifcon_drv_dev->monitor_lock);
+}
+static void ifcon_drv_parse_cmd_monitor_param(char *args, int arg_pos)
+{
+	if (!isspace(args[arg_pos])
+		|| (ifcon_drv_param_skip_spaces(args, &arg_pos) < 0)) {
+		LOG_ERR_SHORT("USAGE: monitor on\n");
+		LOG_ERR_SHORT("       monitor off\n");
+		return;
+	}
+	monitor_category = 0;
+	monitor_filter_size = 0;
+	if (!strncasecmp(&args[arg_pos], "on", 2))
+		ifcon_drv_parse_cmd_monitor_on(args, arg_pos + 2);
+	else if (!strncasecmp(&args[arg_pos], "off", 3 + 1))
+		LOG_INFO("monitor is off\n");
+	else
+		LOG_ERR_SHORT("invalid parameter !!\n");
+}
+
 /*--- Byte order swap -------------------------------------------------------*/
 static inline void ifcon_drv_cpu_to_be32(void *dst, const void *src,
 	__kernel_size_t len)
@@ -1102,6 +1365,7 @@ static int copy_from_frame_to_rx_ring(char *buf)
 		if (sz <= 0)
 			break;
 		sz_total += sz;
+		ifcon_drv_debug_monitor(buf_next, wr_sz);
 	}
 
 	if (sz < 0) {	/* error case */
@@ -1395,10 +1659,10 @@ sync_write_and_read(struct ifcon_drv_data_t *spidev, int len)
 		msg.is_dma_mapped = 1;
 		spi_message_add_tail(&trn, &msg);
 
-		dma_sync_single_for_device(NULL,
+		dma_sync_single_for_device(&spidev->spi->dev,
 			tx_phyadr, IFCON_SPI_FRAME_SIZE, DMA_BIDIRECTIONAL);
 		status = spi_sync(spidev->spi, &msg);
-		dma_sync_single_for_cpu(NULL,
+		dma_sync_single_for_cpu(&spidev->spi->dev,
 			tmp_phyadr, IFCON_SPI_FRAME_SIZE, DMA_BIDIRECTIONAL);
 
 		/* byte order convert */
@@ -1491,12 +1755,25 @@ static int spi_comm_thread(void *arg)
 	long delta_msec;
 	long sum_msec = 0;
 	long sum_count = 0;
+	long interval;
+	unsigned long expire, start;
 
 	LOG_INFO("%s start\n", __func__);
 	while (!kthread_should_stop()) {
 		start_time = ktime_get();
+		interval = msecs_to_jiffies(cycle_msec_val);
+		start = jiffies;
+		expire = start + interval;
 		set_current_state(TASK_INTERRUPTIBLE);
-		schedule_timeout(msecs_to_jiffies(cycle_msec_val));
+		while (!kthread_should_stop() && interval) {
+			interval = schedule_timeout(interval);
+			if (interval != 0) {
+				start = jiffies;
+				if (time_after_eq(start, expire))
+					break;
+				interval = expire - start;
+			}
+		}
 		if (kthread_should_stop())
 			break;
 
@@ -1584,7 +1861,9 @@ static ssize_t ifcon_drv_read(struct file *filp, char __user *buf,
 	}
 
 	/* Read from ring buffer Rx */
+	mutex_lock(&ifcon_drv_dev->buf_lock);
 	status = (ssize_t)rx_ring_read(buf, (int)count);
+	mutex_unlock(&ifcon_drv_dev->buf_lock);
 
 	if ((dump_mode & DUMP_MODE_R) && (status > 0))
 		ifcon_mem_dump("rd:", buf, (int)status);
@@ -1619,7 +1898,9 @@ static ssize_t ifcon_drv_write(struct file *filp, const char __user *buf,
 		return -EFAULT;
 	}
 
+	mutex_lock(&ifcon_drv_dev->buf_lock);
 	status = (ssize_t)tx_ring_write(tmp, (int)count);
+	mutex_unlock(&ifcon_drv_dev->buf_lock);
 
 	if ((dump_mode & DUMP_MODE_W) && (status > 0))
 		ifcon_mem_dump("wr:", tmp, (int)count);
@@ -1642,11 +1923,9 @@ static long ifcon_drv_ioctl(struct file *filp, unsigned int cmd,
 		return -ENOTTY;
 
 	if (_IOC_DIR(cmd) & _IOC_READ)
-		err = !access_ok(VERIFY_WRITE, (void __user *)arg,
-							_IOC_SIZE(cmd));
+		err = !access_ok((void __user *)arg, _IOC_SIZE(cmd));
 	if (err == 0 && _IOC_DIR(cmd) & _IOC_WRITE)
-		err = !access_ok(VERIFY_READ, (void __user *)arg,
-							_IOC_SIZE(cmd));
+		err = !access_ok((void __user *)arg, _IOC_SIZE(cmd));
 	if (err)
 		return -EFAULT;
 
@@ -1938,7 +2217,13 @@ static int ifcon_drv_probe(struct spi_device *spi)
 	/* get platform data */
 	if (spi->dev.of_node) {
 
-		ret = of_dma_configure(&spi->dev, spi->dev.of_node);
+		spi->dev.dma_mask = &spi->dev.coherent_dma_mask;
+		ret = dma_set_mask(&spi->dev, DMA_BIT_MASK(64));
+		if (ret) {
+			LOG_ERR("dma_set_mask error! %d\n", ret);
+			goto out_err_exit;
+		}
+		ret = of_dma_configure(&spi->dev, spi->dev.of_node, true);
 		if (ret) {
 			LOG_ERR("of_dma_configure error! %d\n", ret);
 			goto out_err_exit;
@@ -1949,7 +2234,7 @@ static int ifcon_drv_probe(struct spi_device *spi)
 		if (!tx_frame) {
 			rx_frame = tx_frame;
 			tmp_frame = rx_frame;
-			LOG_ERR("devm_kzalloc error! %p\n", tx_frame);
+			LOG_ERR("dma_alloc_coherent error! %p\n", tx_frame);
 			status = -ENOMEM;
 			goto out_err_exit;
 		} else {
@@ -1959,9 +2244,9 @@ static int ifcon_drv_probe(struct spi_device *spi)
 			tmp_phyadr = tx_phyadr + IFCON_SPI_FRAME_SIZE;
 			rx_frame = (struct ifcon_drv_frame *)
 				((void *)tx_frame + IFCON_SPI_FRAME_SIZE * 2);
-			LOG_INFO("tx_frame alloc vadr %p padr %p\n",
+			LOG_INFO("tx_frame alloc  vadr %px padr %px\n",
 				tx_frame, (void *)tx_phyadr);
-			LOG_INFO("tmp_frame alloc vadr %p padr %p\n",
+			LOG_INFO("tmp_frame alloc vadr %px padr %px\n",
 				tmp_frame, (void *)tmp_phyadr);
 		}
 
@@ -1989,11 +2274,8 @@ static int ifcon_drv_probe(struct spi_device *spi)
 		LOG_INFO("if_reset-gpios=%d\n", GPIO_IFCON_RESET_PIN);
 		GPIO_WATCHDOG_PIN = of_get_named_gpio(spi->dev.of_node,
 			"watchdog-gpios", 0);
-		if (GPIO_WATCHDOG_PIN < 0) {
-			LOG_ERR("watchdog-gpios not found error!\n");
-			status = -ENOMEM;
-		}
-		LOG_INFO("watchdog-gpios=%d\n", GPIO_WATCHDOG_PIN);
+		if (GPIO_WATCHDOG_PIN >= 0)
+			LOG_INFO("watchdog-gpios=%d\n", GPIO_WATCHDOG_PIN);
 		if (status == -ENOMEM)
 			goto out_err_exit;
 	} else {
@@ -2107,6 +2389,12 @@ static void ifcon_drv_shutdown(struct spi_device *spi)
 }
 
 /*-------------------------------------------------------------------------*/
+static const struct spi_device_id ifcon_dev_spi_ids[] = {
+	{ .name = "ifcon_dev" },
+	{ /* sentinel */ }
+};
+MODULE_DEVICE_TABLE(spi, ifcon_dev_spi_ids);
+
 static const struct of_device_id ifcon_dev_dt_ids[] = {
 	{ .compatible = "sony,ifcon_dev", },
 	{ /* sentinel */ }
@@ -2127,6 +2415,7 @@ static struct spi_driver ifcon_driver_st = {
 		.of_match_table = ifcon_dev_dt_ids,
 		.pm    = &ifcon_dev_pmops,
 	},
+	.id_table = ifcon_dev_spi_ids,
 	.probe  = ifcon_drv_probe,
 	.remove = ifcon_drv_remove,
 	.shutdown = ifcon_drv_shutdown,
@@ -2147,6 +2436,8 @@ static int __init ifcon_drv_init(void)
 	}
 
 	mutex_init(&ifcon_drv_dev->state_lock);
+	mutex_init(&ifcon_drv_dev->buf_lock);
+	mutex_init(&ifcon_drv_dev->monitor_lock);
 	rx_frame  = NULL;
 	tmp_frame  = NULL;
 	tmp_phyadr = 0;
@@ -2225,6 +2516,6 @@ module_exit(ifcon_drv_exit);
 
 MODULE_AUTHOR("Sony Corporation");
 MODULE_DESCRIPTION("IfCon driver");
-MODULE_VERSION("2.1.0");
+MODULE_VERSION("2.3.1");
 MODULE_LICENSE("GPL");
 MODULE_ALIAS("spi:" IFCON_DRV_DEVNAME);
