@@ -1,47 +1,36 @@
-/*
- * Copyright 2013-2016 Freescale Semiconductor, Inc.
- * Copyright 2017 NXP.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation.
- *
- */
-#include <linux/busfreq-imx.h>
+// SPDX-License-Identifier: GPL-2.0
+//
+// Copyright 2013 Freescale Semiconductor, Inc.
+
 #include <linux/clk.h>
 #include <linux/cpufreq.h>
 #include <linux/cpu_cooling.h>
 #include <linux/delay.h>
-#include <linux/device.h>
 #include <linux/device_cooling.h>
-#include <linux/init.h>
 #include <linux/interrupt.h>
 #include <linux/io.h>
-#include <linux/kernel.h>
 #include <linux/mfd/syscon.h>
 #include <linux/module.h>
 #include <linux/of.h>
 #include <linux/of_device.h>
-#include <linux/platform_device.h>
 #include <linux/regmap.h>
-#include <linux/slab.h>
 #include <linux/thermal.h>
-#include <linux/types.h>
+#include <linux/nvmem-consumer.h>
 
 #define REG_SET		0x4
 #define REG_CLR		0x8
 #define REG_TOG		0xc
 
-#define MISC0				0x0150
-#define MISC0_REFTOP_SELBIASOFF		(1 << 3)
-#define MISC1				0x0160
-#define MISC1_IRQ_TEMPHIGH		(1 << 29)
-/* Below LOW and PANIC bits are only for TEMPMON_IMX6SX */
-#define MISC1_IRQ_TEMPLOW		(1 << 28)
-#define MISC1_IRQ_TEMPPANIC		(1 << 27)
-
 /* i.MX6 specific */
-#define IMX6_TEMPSENSE0				0X180
+#define IMX6_MISC0				0x0150
+#define IMX6_MISC0_REFTOP_SELBIASOFF		(1 << 3)
+#define IMX6_MISC1				0x0160
+#define IMX6_MISC1_IRQ_TEMPHIGH			(1 << 29)
+/* Below LOW and PANIC bits are only for TEMPMON_IMX6SX */
+#define IMX6_MISC1_IRQ_TEMPLOW			(1 << 28)
+#define IMX6_MISC1_IRQ_TEMPPANIC		(1 << 27)
+
+#define IMX6_TEMPSENSE0				0x0180
 #define IMX6_TEMPSENSE0_ALARM_VALUE_SHIFT	20
 #define IMX6_TEMPSENSE0_ALARM_VALUE_MASK	(0xfff << 20)
 #define IMX6_TEMPSENSE0_TEMP_CNT_SHIFT		8
@@ -50,23 +39,23 @@
 #define IMX6_TEMPSENSE0_MEASURE_TEMP		(1 << 1)
 #define IMX6_TEMPSENSE0_POWER_DOWN		(1 << 0)
 
-#define IMX6_TEMPSENSE1				0X190
+#define IMX6_TEMPSENSE1				0x0190
 #define IMX6_TEMPSENSE1_MEASURE_FREQ		0xffff
 #define IMX6_TEMPSENSE1_MEASURE_FREQ_SHIFT	0
 
-/* Below TEMPSENSE2 is only for TEMPMON_IMX6SX */
-#define TEMPSENSE2			0x0290
-#define TEMPSENSE2_LOW_VALUE_SHIFT	0
-#define TEMPSENSE2_LOW_VALUE_MASK	0xfff
-#define TEMPSENSE2_PANIC_VALUE_SHIFT	16
-#define TEMPSENSE2_PANIC_VALUE_MASK	0xfff0000
-
 #define OCOTP_MEM0			0x0480
 #define OCOTP_ANA1			0x04e0
 
-/* i.MX7D specific */
+/* Below TEMPSENSE2 is only for TEMPMON_IMX6SX */
+#define IMX6_TEMPSENSE2				0x0290
+#define IMX6_TEMPSENSE2_LOW_VALUE_SHIFT		0
+#define IMX6_TEMPSENSE2_LOW_VALUE_MASK		0xfff
+#define IMX6_TEMPSENSE2_PANIC_VALUE_SHIFT	16
+#define IMX6_TEMPSENSE2_PANIC_VALUE_MASK	0xfff0000
+
+/* i.MX7 specific */
 #define IMX7_ANADIG_DIGPROG			0x800
-#define IMX7_TEMPSENSE0				0X300
+#define IMX7_TEMPSENSE0				0x300
 #define IMX7_TEMPSENSE0_PANIC_ALARM_SHIFT	18
 #define IMX7_TEMPSENSE0_PANIC_ALARM_MASK	(0x1ff << 18)
 #define IMX7_TEMPSENSE0_HIGH_ALARM_SHIFT	9
@@ -74,7 +63,7 @@
 #define IMX7_TEMPSENSE0_LOW_ALARM_SHIFT		0
 #define IMX7_TEMPSENSE0_LOW_ALARM_MASK		0x1ff
 
-#define IMX7_TEMPSENSE1				0X310
+#define IMX7_TEMPSENSE1				0x310
 #define IMX7_TEMPSENSE1_MEASURE_FREQ_SHIFT	16
 #define IMX7_TEMPSENSE1_MEASURE_FREQ_MASK	(0xffff << 16)
 #define IMX7_TEMPSENSE1_FINISHED		(1 << 11)
@@ -83,10 +72,6 @@
 #define IMX7_TEMPSENSE1_TEMP_VALUE_SHIFT	0
 #define IMX7_TEMPSENSE1_TEMP_VALUE_MASK		0x1ff
 
-#define IMX6_OCOTP_ANA1		0x04e0
-#define IMX7_OCOTP_ANA1		0x04f0
-#define IMX7_OCOTP_TESTER3	0x0440
-
 /* The driver supports 1 passive trip point and 1 critical trip point */
 enum imx_thermal_trip {
 	IMX_TRIP_PASSIVE,
@@ -94,27 +79,19 @@ enum imx_thermal_trip {
 	IMX_TRIP_NUM,
 };
 
-#define IMX_TEMP_PASSIVE_COOL_DELTA	10000
-
 #define IMX_POLLING_DELAY		2000 /* millisecond */
 #define IMX_PASSIVE_DELAY		1000
 
-#define FACTOR0				10000000
-#define FACTOR1				15423
-#define FACTOR2				4148468
-#define OFFSET				3580661
-
 #define TEMPMON_IMX6Q			1
 #define TEMPMON_IMX6SX			2
-#define TEMPMON_IMX7			3
+#define TEMPMON_IMX7D			3
 
-/* the register offsets and bitfields may change across
- * i.MX SOCs, use below struct as a description of the
- * register.
- */
+#define IMX_TEMP_PASSIVE_COOL_DELTA	10000
 
 struct thermal_soc_data {
-	u32 sensor_ctrl;	/* tempmon sensor basic control */
+	u32 version;
+
+	u32 sensor_ctrl;
 	u32 power_down_mask;
 	u32 measure_temp_mask;
 
@@ -138,8 +115,6 @@ struct thermal_soc_data {
 	u32 low_alarm_ctrl;
 	u32 low_alarm_mask;
 	u32 low_alarm_shift;
-
-	u32 version;
 };
 
 static struct thermal_soc_data thermal_imx6q_data = {
@@ -183,13 +158,17 @@ static struct thermal_soc_data thermal_imx6sx_data = {
 	.high_alarm_mask = IMX6_TEMPSENSE0_ALARM_VALUE_MASK,
 	.high_alarm_shift = IMX6_TEMPSENSE0_ALARM_VALUE_SHIFT,
 
-	.panic_alarm_ctrl = TEMPSENSE2,
-	.panic_alarm_mask = TEMPSENSE2_PANIC_VALUE_MASK,
-	.panic_alarm_shift = TEMPSENSE2_PANIC_VALUE_SHIFT,
+	.panic_alarm_ctrl = IMX6_TEMPSENSE2,
+	.panic_alarm_mask = IMX6_TEMPSENSE2_PANIC_VALUE_MASK,
+	.panic_alarm_shift = IMX6_TEMPSENSE2_PANIC_VALUE_SHIFT,
+
+	.low_alarm_ctrl = IMX6_TEMPSENSE2,
+	.low_alarm_mask = IMX6_TEMPSENSE2_LOW_VALUE_MASK,
+	.low_alarm_shift = IMX6_TEMPSENSE2_LOW_VALUE_SHIFT,
 };
 
 static struct thermal_soc_data thermal_imx7d_data = {
-	.version = TEMPMON_IMX7,
+	.version = TEMPMON_IMX7D,
 
 	.sensor_ctrl = IMX7_TEMPSENSE1,
 	.power_down_mask = IMX7_TEMPSENSE1_POWER_DOWN,
@@ -215,16 +194,14 @@ static struct thermal_soc_data thermal_imx7d_data = {
 	.low_alarm_ctrl = IMX7_TEMPSENSE0,
 	.low_alarm_mask = IMX7_TEMPSENSE0_LOW_ALARM_MASK,
 	.low_alarm_shift = IMX7_TEMPSENSE0_LOW_ALARM_SHIFT,
-
 };
 
 struct imx_thermal_data {
 	struct cpufreq_policy *policy;
 	struct thermal_zone_device *tz;
 	struct thermal_cooling_device *cdev[2];
-	enum thermal_device_mode mode;
 	struct regmap *tempmon;
-	u32 c1, c2; /* See formula in imx_get_sensor_data() */
+	u32 c1, c2; /* See formula in imx_init_calib() */
 	int temp_passive;
 	int temp_critical;
 	int temp_max;
@@ -233,15 +210,10 @@ struct imx_thermal_data {
 	bool irq_enabled;
 	int irq;
 	struct clk *thermal_clk;
-	struct mutex mutex;
 	const struct thermal_soc_data *socdata;
 	const char *temp_grade;
 };
 
-static struct imx_thermal_data *imx_thermal_data;
-static int skip_finish_check;
-static u32 imx7_lpsr_save[2];
-
 static void imx_set_panic_temp(struct imx_thermal_data *data,
 			       int panic_temp)
 {
@@ -249,10 +221,7 @@ static void imx_set_panic_temp(struct imx_thermal_data *data,
 	struct regmap *map = data->tempmon;
 	int critical_value;
 
-	if (data->socdata->version == TEMPMON_IMX7)
-		critical_value = panic_temp / 1000 + data->c1 - 25;
-	else
-		critical_value = (data->c2 - panic_temp) / data->c1;
+	critical_value = (data->c2 - panic_temp) / data->c1;
 
 	regmap_write(map, soc_data->panic_alarm_ctrl + REG_CLR,
 		     soc_data->panic_alarm_mask);
@@ -263,13 +232,13 @@ static void imx_set_panic_temp(struct imx_thermal_data *data,
 static void imx_set_alarm_temp(struct imx_thermal_data *data,
 			       int alarm_temp)
 {
-	const struct thermal_soc_data *soc_data = data->socdata;
 	struct regmap *map = data->tempmon;
+	const struct thermal_soc_data *soc_data = data->socdata;
 	int alarm_value;
 
 	data->alarm_temp = alarm_temp;
 
-	if (data->socdata->version == TEMPMON_IMX7)
+	if (data->socdata->version == TEMPMON_IMX7D)
 		alarm_value = alarm_temp / 1000 + data->c1 - 25;
 	else
 		alarm_value = (data->c2 - alarm_temp) / data->c1;
@@ -286,11 +255,12 @@ static int imx_get_temp(struct thermal_zone_device *tz, int *temp)
 	const struct thermal_soc_data *soc_data = data->socdata;
 	struct regmap *map = data->tempmon;
 	unsigned int n_meas;
-	bool wait;
+	bool wait, run_measurement;
 	u32 val;
 
-	mutex_lock(&data->mutex);
-	if (data->mode == THERMAL_DEVICE_ENABLED) {
+	regmap_read(map, soc_data->sensor_ctrl, &val);
+	run_measurement = val & soc_data->power_down_mask;
+	if (!run_measurement) {
 		/* Check if a measurement is currently in progress */
 		regmap_read(map, soc_data->temp_data, &val);
 		wait = !(val & soc_data->temp_valid_mask);
@@ -300,7 +270,6 @@ static int imx_get_temp(struct thermal_zone_device *tz, int *temp)
 		 * temperature sensor, enable measurements, take a reading,
 		 * disable measurements, power off the temperature sensor.
 		 */
-		clk_prepare_enable(data->thermal_clk);
 		regmap_write(map, soc_data->sensor_ctrl + REG_CLR,
 			    soc_data->power_down_mask);
 		regmap_write(map, soc_data->sensor_ctrl + REG_SET,
@@ -313,38 +282,28 @@ static int imx_get_temp(struct thermal_zone_device *tz, int *temp)
 	 * According to the temp sensor designers, it may require up to ~17us
 	 * to complete a measurement.
 	 */
-	if (wait) {
-		/*
-		 * On i.MX7 TO1.0, the finish bit can only keep 1us after
-		 * the measured data available. It is hard for software to
-		 * polling this bit. So wait for 20ms to make sure the
-		 * measured data is valid.
-		 */
-		if (data->socdata->version == TEMPMON_IMX7 && skip_finish_check)
-			msleep(20);
-		 else
-			usleep_range(20, 50);
-		regmap_read(map, soc_data->temp_data, &val);
-	}
+	if (wait)
+		usleep_range(20, 50);
 
-	if (data->mode != THERMAL_DEVICE_ENABLED) {
+	regmap_read(map, soc_data->temp_data, &val);
+
+	if (run_measurement) {
 		regmap_write(map, soc_data->sensor_ctrl + REG_CLR,
 			     soc_data->measure_temp_mask);
 		regmap_write(map, soc_data->sensor_ctrl + REG_SET,
 			     soc_data->power_down_mask);
-		clk_disable_unprepare(data->thermal_clk);
 	}
 
-	if (!skip_finish_check && ((val & soc_data->temp_valid_mask) == 0)) {
+	if ((val & soc_data->temp_valid_mask) == 0) {
 		dev_dbg(&tz->device, "temp measurement never finished\n");
-		mutex_unlock(&data->mutex);
 		return -EAGAIN;
 	}
 
-	n_meas = (val & soc_data->temp_value_mask) >> soc_data->temp_value_shift;
-	/* See imx_get_sensor_data() for formula derivation */
-	*temp = data->c2 - n_meas * data->c1;
-	if (data->socdata->version == TEMPMON_IMX7)
+	n_meas = (val & soc_data->temp_value_mask)
+		>> soc_data->temp_value_shift;
+
+	/* See imx_init_calib() for formula derivation */
+	if (data->socdata->version == TEMPMON_IMX7D)
 		*temp = (n_meas - data->c1 + 25) * 1000;
 	else
 		*temp = data->c2 - n_meas * data->c1;
@@ -372,36 +331,23 @@ static int imx_get_temp(struct thermal_zone_device *tz, int *temp)
 		data->irq_enabled = true;
 		enable_irq(data->irq);
 	}
-	mutex_unlock(&data->mutex);
 
 	return 0;
 }
 
-static int imx_get_mode(struct thermal_zone_device *tz,
-			enum thermal_device_mode *mode)
+static int imx_change_mode(struct thermal_zone_device *tz,
+			   enum thermal_device_mode mode)
 {
 	struct imx_thermal_data *data = tz->devdata;
-
-	*mode = data->mode;
-
-	return 0;
-}
-
-static int imx_set_mode(struct thermal_zone_device *tz,
-			enum thermal_device_mode mode)
-{
-	struct imx_thermal_data *data = tz->devdata;
-	const struct thermal_soc_data *soc_data = data->socdata;
 	struct regmap *map = data->tempmon;
+	const struct thermal_soc_data *soc_data = data->socdata;
 
 	if (mode == THERMAL_DEVICE_ENABLED) {
-		tz->polling_delay = IMX_POLLING_DELAY;
-		tz->passive_delay = IMX_PASSIVE_DELAY;
-
 		regmap_write(map, soc_data->sensor_ctrl + REG_CLR,
 			     soc_data->power_down_mask);
 		regmap_write(map, soc_data->sensor_ctrl + REG_SET,
 			     soc_data->measure_temp_mask);
+
 		if (!data->irq_enabled) {
 			data->irq_enabled = true;
 			enable_irq(data->irq);
@@ -412,18 +358,12 @@ static int imx_set_mode(struct thermal_zone_device *tz,
 		regmap_write(map, soc_data->sensor_ctrl + REG_SET,
 			     soc_data->power_down_mask);
 
-		tz->polling_delay = 0;
-		tz->passive_delay = 0;
-
 		if (data->irq_enabled) {
 			disable_irq(data->irq);
 			data->irq_enabled = false;
 		}
 	}
 
-	data->mode = mode;
-	thermal_zone_device_update(tz, THERMAL_EVENT_UNSPECIFIED);
-
 	return 0;
 }
 
@@ -458,6 +398,10 @@ static int imx_set_trip_temp(struct thermal_zone_device *tz, int trip,
 {
 	struct imx_thermal_data *data = tz->devdata;
 
+	/* do not allow passive to be set higher than critical */
+	if (temp < 0 || temp > data->temp_critical)
+		return -EINVAL;
+
 	if (trip == IMX_TRIP_CRITICAL) {
 		data->temp_critical = temp;
 		if (data->socdata->version == TEMPMON_IMX6SX)
@@ -474,6 +418,24 @@ static int imx_set_trip_temp(struct thermal_zone_device *tz, int trip,
 	return 0;
 }
 
+static int imx_get_trend(struct thermal_zone_device *tz,
+			 int trip, enum thermal_trend *trend)
+{
+	int ret;
+	int trip_temp;
+
+	ret = imx_get_trip_temp(tz, trip, &trip_temp);
+	if (ret < 0)
+		return ret;
+
+	if (tz->temperature >= (trip_temp - IMX_TEMP_PASSIVE_COOL_DELTA))
+		*trend = THERMAL_TREND_RAISE_FULL;
+	else
+		*trend = THERMAL_TREND_DROP_FULL;
+
+	return 0;
+}
+
 static int imx_bind(struct thermal_zone_device *tz,
 		    struct thermal_cooling_device *cdev)
 {
@@ -509,29 +471,11 @@ static int imx_unbind(struct thermal_zone_device *tz,
 	return 0;
 }
 
-static int imx_get_trend(struct thermal_zone_device *tz,
-	int trip, enum thermal_trend *trend)
-{
-	int ret;
-	int trip_temp;
-
-	ret = imx_get_trip_temp(tz, trip, &trip_temp);
-	if (ret < 0)
-		return ret;
-
-	if (tz->temperature >= (trip_temp - IMX_TEMP_PASSIVE_COOL_DELTA))
-		*trend = THERMAL_TREND_RAISE_FULL;
-	else
-		*trend = THERMAL_TREND_DROP_FULL;
-
-	return 0;
-}
 static struct thermal_zone_device_ops imx_tz_ops = {
 	.bind = imx_bind,
 	.unbind = imx_unbind,
 	.get_temp = imx_get_temp,
-	.get_mode = imx_get_mode,
-	.set_mode = imx_set_mode,
+	.change_mode = imx_change_mode,
 	.get_trip_type = imx_get_trip_type,
 	.get_trip_temp = imx_get_trip_temp,
 	.get_crit_temp = imx_get_crit_temp,
@@ -539,61 +483,90 @@ static struct thermal_zone_device_ops imx_tz_ops = {
 	.get_trend = imx_get_trend,
 };
 
-static inline void imx6_calibrate_data(struct imx_thermal_data *data, u32 val)
-{
-	int t1, t2, n1, n2;
-	u64 temp64;
-	/*
-	 * Sensor data layout:
-	 *   [31:20] - sensor value @ 25C
-	 *    [19:8] - sensor value of hot
-	 *     [7:0] - hot temperature value
-	 * Use universal formula now and only need sensor value @ 25C
-	 * slope = 0.4297157 - (0.0015976 * 25C fuse)
-	 */
-	n1 = val >> 20;
-	n2 = (val & 0xfff00) >> 8;
-	t2 = val & 0xff;
-	t1 = 25; /* t1 always 25C */
-
-	/*
-	 * Derived from linear interpolation:
-	 * slope = 0.4297157 - (0.0015976 * 25C fuse)
-	 * slope = (FACTOR2 - FACTOR1 * n1) / FACTOR0
-	 * offset = OFFSET / 1000000
-	 * (Nmeas - n1) / (Tmeas - t1) = slope
-	 * We want to reduce this down to the minimum computation necessary
-	 * for each temperature read.  Also, we want Tmeas in millicelsius
-	 * and we don't want to lose precision from integer division. So...
-	 * Tmeas = (Nmeas - n1) / slope + t1 + offset
-	 * milli_Tmeas = 1000 * (Nmeas - n1) / slope + 1000 * t1 + OFFSET / 1000
-	 * milli_Tmeas = -1000 * (n1 - Nmeas) / slope + 1000 * t1 + OFFSET /1000
-	 * Let constant c1 = (-1000 / slope)
-	 * milli_Tmeas = (n1 - Nmeas) * c1 + 1000 * t1 + OFFSET / 1000
-	 * Let constant c2 = n1 *c1 + 1000 * t1 + OFFSET / 1000
-	 * milli_Tmeas = c2 - Nmeas * c1
-	 */
-	temp64 = FACTOR0;
-	temp64 *= 1000;
-	do_div(temp64, FACTOR1 * n1 - FACTOR2);
-	data->c1 = temp64;
-	temp64 = OFFSET;
-	do_div(temp64, 1000);
-	data->c2 = n1 * data->c1 + 1000 * t1 + temp64;
-}
-
-/*
- * On i.MX7, we only use the calibration data at 25C to get the temp,
- * Tmeas = ( Nmeas - n1) + 25; n1 is the fuse value for 25C.
- */
-static inline void imx7_calibrate_data(struct imx_thermal_data *data, u32 val)
-{
-	data->c1 = (val >> 9) & 0x1ff;
-}
-
-static int imx_get_sensor_data(struct platform_device *pdev)
+static int imx_init_calib(struct platform_device *pdev, u32 ocotp_ana1)
 {
 	struct imx_thermal_data *data = platform_get_drvdata(pdev);
+	int n1;
+	u64 temp64;
+
+	if (ocotp_ana1 == 0 || ocotp_ana1 == ~0) {
+		dev_err(&pdev->dev, "invalid sensor calibration data\n");
+		return -EINVAL;
+	}
+
+	/*
+	 * On i.MX7D, we only use the calibration data at 25C to get the temp,
+	 * Tmeas = ( Nmeas - n1) + 25; n1 is the fuse value for 25C.
+	 */
+	if (data->socdata->version == TEMPMON_IMX7D) {
+		data->c1 = (ocotp_ana1 >> 9) & 0x1ff;
+		return 0;
+	}
+
+	/*
+	 * The sensor is calibrated at 25 °C (aka T1) and the value measured
+	 * (aka N1) at this temperature is provided in bits [31:20] in the
+	 * i.MX's OCOTP value ANA1.
+	 * To find the actual temperature T, the following formula has to be used
+	 * when reading value n from the sensor:
+	 *
+	 * T = T1 + (N - N1) / (0.4148468 - 0.0015423 * N1) °C + 3.580661 °C
+	 *   = [T1' - N1 / (0.4148468 - 0.0015423 * N1) °C] + N / (0.4148468 - 0.0015423 * N1) °C
+	 *   = [T1' + N1 / (0.0015423 * N1 - 0.4148468) °C] - N / (0.0015423 * N1 - 0.4148468) °C
+	 *   = c2 - c1 * N
+	 *
+	 * with
+	 *
+	 *  T1' = 28.580661 °C
+	 *   c1 = 1 / (0.0015423 * N1 - 0.4297157) °C
+	 *   c2 = T1' + N1 / (0.0015423 * N1 - 0.4148468) °C
+	 *      = T1' + N1 * c1
+	 */
+	n1 = ocotp_ana1 >> 20;
+
+	temp64 = 10000000; /* use 10^7 as fixed point constant for values in formula */
+	temp64 *= 1000; /* to get result in °mC */
+	do_div(temp64, 15423 * n1 - 4148468);
+	data->c1 = temp64;
+	data->c2 = n1 * data->c1 + 28581;
+
+	return 0;
+}
+
+static void imx_init_temp_grade(struct platform_device *pdev, u32 ocotp_mem0)
+{
+	struct imx_thermal_data *data = platform_get_drvdata(pdev);
+
+	/* The maximum die temp is specified by the Temperature Grade */
+	switch ((ocotp_mem0 >> 6) & 0x3) {
+	case 0: /* Commercial (0 to 95 °C) */
+		data->temp_grade = "Commercial";
+		data->temp_max = 95000;
+		break;
+	case 1: /* Extended Commercial (-20 °C to 105 °C) */
+		data->temp_grade = "Extended Commercial";
+		data->temp_max = 105000;
+		break;
+	case 2: /* Industrial (-40 °C to 105 °C) */
+		data->temp_grade = "Industrial";
+		data->temp_max = 105000;
+		break;
+	case 3: /* Automotive (-40 °C to 125 °C) */
+		data->temp_grade = "Automotive";
+		data->temp_max = 125000;
+		break;
+	}
+
+	/*
+	 * Set the critical trip point at 5 °C under max
+	 * Set the passive trip point at 10 °C under max (changeable via sysfs)
+	 */
+	data->temp_critical = data->temp_max - (1000 * 5);
+	data->temp_passive = data->temp_max - (1000 * 10);
+}
+
+static int imx_init_from_tempmon_data(struct platform_device *pdev)
+{
 	struct regmap *map;
 	int ret;
 	u32 val;
@@ -606,63 +579,42 @@ static int imx_get_sensor_data(struct platform_device *pdev)
 		return ret;
 	}
 
-	if (data->socdata->version == TEMPMON_IMX7)
-		ret = regmap_read(map, IMX7_OCOTP_ANA1, &val);
-	else
-		ret = regmap_read(map, IMX6_OCOTP_ANA1, &val);
-
+	ret = regmap_read(map, OCOTP_ANA1, &val);
 	if (ret) {
 		dev_err(&pdev->dev, "failed to read sensor data: %d\n", ret);
 		return ret;
 	}
+	ret = imx_init_calib(pdev, val);
+	if (ret)
+		return ret;
 
-	if (val == 0 || val == ~0) {
-		dev_err(&pdev->dev, "invalid sensor calibration data\n");
-		return -EINVAL;
-	}
-
-	if (data->socdata->version == TEMPMON_IMX7)
-		imx7_calibrate_data(data, val);
-	else
-		imx6_calibrate_data(data, val);
-
-	/* use OTP for thermal grade */
-	if (data->socdata->version == TEMPMON_IMX7)
-		ret = regmap_read(map, IMX7_OCOTP_TESTER3, &val);
-	else
-		ret = regmap_read(map, OCOTP_MEM0, &val);
-
+	ret = regmap_read(map, OCOTP_MEM0, &val);
 	if (ret) {
-		dev_err(&pdev->dev, "failed to read temp grade: %d\n", ret);
+		dev_err(&pdev->dev, "failed to read sensor data: %d\n", ret);
 		return ret;
 	}
+	imx_init_temp_grade(pdev, val);
 
-	/* The maximum die temp is specified by the Temperature Grade */
-	switch ((val >> 6) & 0x3) {
-	case 0: /* Commercial (0 to 95C) */
-		data->temp_grade = "Commercial";
-		data->temp_max = 95000;
-		break;
-	case 1: /* Extended Commercial (-20 to 105C) */
-		data->temp_grade = "Extended Commercial";
-		data->temp_max = 105000;
-		break;
-	case 2: /* Industrial (-40 to 105C) */
-		data->temp_grade = "Industrial";
-		data->temp_max = 105000;
-		break;
-	case 3: /* Automotive (-40 to 125C) */
-		data->temp_grade = "Automotive";
-		data->temp_max = 125000;
-		break;
-	}
+	return 0;
+}
 
-	/*
-	 * Set the critical trip point at 5C under max
-	 * Set the passive trip point at 10C under max (can change via sysfs)
-	 */
-	data->temp_critical = data->temp_max - (1000 * 5);
-	data->temp_passive = data->temp_max - (1000 * 10);
+static int imx_init_from_nvmem_cells(struct platform_device *pdev)
+{
+	int ret;
+	u32 val;
+
+	ret = nvmem_cell_read_u32(&pdev->dev, "calib", &val);
+	if (ret)
+		return ret;
+
+	ret = imx_init_calib(pdev, val);
+	if (ret)
+		return ret;
+
+	ret = nvmem_cell_read_u32(&pdev->dev, "temp_grade", &val);
+	if (ret)
+		return ret;
+	imx_init_temp_grade(pdev, val);
 
 	return 0;
 }
@@ -697,63 +649,79 @@ static const struct of_device_id of_imx_thermal_match[] = {
 };
 MODULE_DEVICE_TABLE(of, of_imx_thermal_match);
 
-static int thermal_notifier_event(struct notifier_block *this,
-					unsigned long event, void *ptr)
+#ifdef CONFIG_CPU_FREQ
+/*
+ * Create cooling device in case no #cooling-cells property is available in
+ * CPU node
+ */
+static int imx_thermal_register_legacy_cooling(struct imx_thermal_data *data)
 {
-	const struct thermal_soc_data *soc_data = imx_thermal_data->socdata;
-	struct regmap *map = imx_thermal_data->tempmon;
+	struct device_node *np;
+	int ret = 0;
 
-	mutex_lock(&imx_thermal_data->mutex);
-
-	switch (event) {
-	/*
-	 * In low_bus_freq_mode, the thermal sensor auto measurement
-	 * can be disabled to low the power consumption.
-	 */
-	case LOW_BUSFREQ_ENTER:
-		regmap_write(map, soc_data->sensor_ctrl + REG_CLR,
-			     soc_data->measure_temp_mask);
-		regmap_write(map, soc_data->sensor_ctrl + REG_SET,
-			     soc_data->power_down_mask);
-		imx_thermal_data->mode = THERMAL_DEVICE_DISABLED;
-		disable_irq(imx_thermal_data->irq);
-		clk_disable_unprepare(imx_thermal_data->thermal_clk);
-		break;
-
-	/* Enabled thermal auto measurement when exiting low_bus_freq_mode */
-	case LOW_BUSFREQ_EXIT:
-		clk_prepare_enable(imx_thermal_data->thermal_clk);
-		regmap_write(map, soc_data->sensor_ctrl + REG_CLR,
-			     soc_data->power_down_mask);
-		regmap_write(map, soc_data->sensor_ctrl + REG_SET,
-			     soc_data->measure_temp_mask);
-		imx_thermal_data->mode = THERMAL_DEVICE_ENABLED;
-		enable_irq(imx_thermal_data->irq);
-		break;
-
-	default:
-		break;
+	data->policy = cpufreq_cpu_get(0);
+	if (!data->policy) {
+		pr_debug("%s: CPUFreq policy not found\n", __func__);
+		return -EPROBE_DEFER;
 	}
-	mutex_unlock(&imx_thermal_data->mutex);
 
-	return NOTIFY_OK;
+	np = of_get_cpu_node(data->policy->cpu, NULL);
+
+	if (!np || !of_find_property(np, "#cooling-cells", NULL)) {
+		data->cdev[0] = cpufreq_cooling_register(data->policy);
+		if (IS_ERR(data->cdev[0])) {
+			ret = PTR_ERR(data->cdev[0]);
+			cpufreq_cpu_put(data->policy);
+		}
+	}
+
+	of_node_put(np);
+	if (ret)
+		return ret;
+
+	data->cdev[1] = devfreq_cooling_register();
+	if (IS_ERR(data->cdev[1])) {
+		ret = PTR_ERR(data->cdev[1]);
+		if (ret != -EPROBE_DEFER) {
+			pr_err("failed to register cpufreq cooling device: %d\n",
+				ret);
+			cpufreq_cooling_unregister(data->cdev[0]);
+		}
+		return ret;
+	}
+
+	return 0;
 }
 
-static struct notifier_block thermal_notifier = {
-	.notifier_call = thermal_notifier_event,
-};
+static void imx_thermal_unregister_legacy_cooling(struct imx_thermal_data *data)
+{
+	cpufreq_cooling_unregister(data->cdev[0]);
+	cpufreq_cpu_put(data->policy);
+	devfreq_cooling_unregister(data->cdev[1]);
+}
+
+#else
+
+static inline int imx_thermal_register_legacy_cooling(struct imx_thermal_data *data)
+{
+	return 0;
+}
+
+static inline void imx_thermal_unregister_legacy_cooling(struct imx_thermal_data *data)
+{
+}
+#endif
 
 static int imx_thermal_probe(struct platform_device *pdev)
 {
 	struct imx_thermal_data *data;
 	struct regmap *map;
 	int measure_freq;
-	int ret, revision;
+	int ret;
 
 	data = devm_kzalloc(&pdev->dev, sizeof(*data), GFP_KERNEL);
 	if (!data)
 		return -ENOMEM;
-	imx_thermal_data = data;
 
 	map = syscon_regmap_lookup_by_phandle(pdev->dev.of_node, "fsl,tempmon");
 	if (IS_ERR(map)) {
@@ -771,14 +739,15 @@ static int imx_thermal_probe(struct platform_device *pdev)
 
 	/* make sure the IRQ flag is clear before enabling irq on i.MX6SX */
 	if (data->socdata->version == TEMPMON_IMX6SX) {
-		regmap_write(map, MISC1 + REG_CLR, MISC1_IRQ_TEMPHIGH |
-			MISC1_IRQ_TEMPLOW | MISC1_IRQ_TEMPPANIC);
+		regmap_write(map, IMX6_MISC1 + REG_CLR,
+			IMX6_MISC1_IRQ_TEMPHIGH | IMX6_MISC1_IRQ_TEMPLOW
+			| IMX6_MISC1_IRQ_TEMPPANIC);
 		/*
 		 * reset value of LOW ALARM is incorrect, set it to lowest
 		 * value to avoid false trigger of low alarm.
 		 */
-		regmap_write(map, TEMPSENSE2 + REG_SET,
-			TEMPSENSE2_LOW_VALUE_MASK);
+		regmap_write(map, data->socdata->low_alarm_ctrl + REG_SET,
+			     data->socdata->low_alarm_mask);
 	}
 
 	data->irq = platform_get_irq(pdev, 0);
@@ -787,20 +756,19 @@ static int imx_thermal_probe(struct platform_device *pdev)
 
 	platform_set_drvdata(pdev, data);
 
-	ret = imx_get_sensor_data(pdev);
-	if (ret) {
-		dev_err(&pdev->dev, "failed to get sensor data\n");
-		return ret;
+	if (of_find_property(pdev->dev.of_node, "nvmem-cells", NULL)) {
+		ret = imx_init_from_nvmem_cells(pdev);
+		if (ret)
+			return dev_err_probe(&pdev->dev, ret,
+					     "failed to init from nvmem\n");
+	} else {
+		ret = imx_init_from_tempmon_data(pdev);
+		if (ret) {
+			dev_err(&pdev->dev, "failed to init from fsl,tempmon-data\n");
+			return ret;
+		}
 	}
 
-	/*
-	 * for i.MX7D TO1.0, finish bit is not available, check the
-	 * SOC revision to skip checking the finish bit status.
-	 */
-	regmap_read(map, IMX7_ANADIG_DIGPROG, &revision);
-	if ((revision & 0xff) == 0x10)
-		skip_finish_check = 1;
-
 	/* Make sure sensor is in known good state for measurements */
 	regmap_write(map, data->socdata->sensor_ctrl + REG_CLR,
 		     data->socdata->power_down_mask);
@@ -808,38 +776,16 @@ static int imx_thermal_probe(struct platform_device *pdev)
 		     data->socdata->measure_temp_mask);
 	regmap_write(map, data->socdata->measure_freq_ctrl + REG_CLR,
 		     data->socdata->measure_freq_mask);
-	if (data->socdata->version != TEMPMON_IMX7)
-		regmap_write(map, MISC0 + REG_SET, MISC0_REFTOP_SELBIASOFF);
+	if (data->socdata->version != TEMPMON_IMX7D)
+		regmap_write(map, IMX6_MISC0 + REG_SET,
+			IMX6_MISC0_REFTOP_SELBIASOFF);
 	regmap_write(map, data->socdata->sensor_ctrl + REG_SET,
 		     data->socdata->power_down_mask);
 
-	data->policy = cpufreq_cpu_get(0);
-	if (!data->policy) {
-		pr_debug("%s: CPUFreq policy not found\n", __func__);
-		return -EPROBE_DEFER;
-	}
-
-	data->cdev[0] = cpufreq_cooling_register(data->policy);
-	if (IS_ERR(data->cdev[0])) {
-		ret = PTR_ERR(data->cdev[0]);
-		if (ret != -EPROBE_DEFER)
-			dev_err(&pdev->dev,
-				"failed to register cpufreq cooling device: %d\n",
-				ret);
-		return ret;
-	}
-
-	data->cdev[1] = devfreq_cooling_register();
-	if (IS_ERR(data->cdev[1])) {
-		ret = PTR_ERR(data->cdev[1]);
-		if (ret != -EPROBE_DEFER) {
-			dev_err(&pdev->dev,
-				"failed to register cpufreq cooling device: %d\n",
-				ret);
-			cpufreq_cooling_unregister(data->cdev[0]);
-		}
-		return ret;
-	}
+	ret = imx_thermal_register_legacy_cooling(data);
+	if (ret)
+		return dev_err_probe(&pdev->dev, ret,
+				     "failed to register cpufreq cooling device\n");
 
 	data->thermal_clk = devm_clk_get(&pdev->dev, NULL);
 	if (IS_ERR(data->thermal_clk)) {
@@ -847,9 +793,7 @@ static int imx_thermal_probe(struct platform_device *pdev)
 		if (ret != -EPROBE_DEFER)
 			dev_err(&pdev->dev,
 				"failed to get thermal clk: %d\n", ret);
-		cpufreq_cooling_unregister(data->cdev[0]);
-		cpufreq_cpu_put(data->policy);
-		return ret;
+		goto legacy_cleanup;
 	}
 
 	/*
@@ -862,16 +806,13 @@ static int imx_thermal_probe(struct platform_device *pdev)
 	ret = clk_prepare_enable(data->thermal_clk);
 	if (ret) {
 		dev_err(&pdev->dev, "failed to enable thermal clk: %d\n", ret);
-		cpufreq_cooling_unregister(data->cdev[0]);
-		devfreq_cooling_unregister(data->cdev[1]);
-		cpufreq_cpu_put(data->policy);
-		return ret;
+		goto legacy_cleanup;
 	}
 
-	mutex_init(&data->mutex);
 	data->tz = thermal_zone_device_register("imx_thermal_zone",
 						IMX_TRIP_NUM,
-						(1 << IMX_TRIP_NUM) - 1, data,
+						BIT(IMX_TRIP_PASSIVE) | BIT(IMX_TRIP_CRITICAL),
+						data,
 						&imx_tz_ops, NULL,
 						IMX_PASSIVE_DELAY,
 						IMX_POLLING_DELAY);
@@ -879,11 +820,7 @@ static int imx_thermal_probe(struct platform_device *pdev)
 		ret = PTR_ERR(data->tz);
 		dev_err(&pdev->dev,
 			"failed to register thermal zone device %d\n", ret);
-		clk_disable_unprepare(data->thermal_clk);
-		cpufreq_cooling_unregister(data->cdev[0]);
-		devfreq_cooling_unregister(data->cdev[1]);
-		cpufreq_cpu_put(data->policy);
-		return ret;
+		goto clk_disable;
 	}
 
 	dev_info(&pdev->dev, "%s CPU temperature grade - max:%dC"
@@ -908,26 +845,28 @@ static int imx_thermal_probe(struct platform_device *pdev)
 		     data->socdata->measure_temp_mask);
 
 	data->irq_enabled = true;
-	data->mode = THERMAL_DEVICE_ENABLED;
+	ret = thermal_zone_device_enable(data->tz);
+	if (ret)
+		goto thermal_zone_unregister;
 
 	ret = devm_request_threaded_irq(&pdev->dev, data->irq,
 			imx_thermal_alarm_irq, imx_thermal_alarm_irq_thread,
 			0, "imx_thermal", data);
 	if (ret < 0) {
 		dev_err(&pdev->dev, "failed to request alarm irq: %d\n", ret);
-		clk_disable_unprepare(data->thermal_clk);
-		thermal_zone_device_unregister(data->tz);
-		cpufreq_cooling_unregister(data->cdev[0]);
-		devfreq_cooling_unregister(data->cdev[1]);
-		cpufreq_cpu_put(data->policy);
-		return ret;
+		goto thermal_zone_unregister;
 	}
 
-	/* register the busfreq notifier called in low bus freq */
-	if (data->socdata->version != TEMPMON_IMX7)
-		register_busfreq_notifier(&thermal_notifier);
-
 	return 0;
+
+thermal_zone_unregister:
+	thermal_zone_device_unregister(data->tz);
+clk_disable:
+	clk_disable_unprepare(data->thermal_clk);
+legacy_cleanup:
+	imx_thermal_unregister_legacy_cooling(data);
+
+	return ret;
 }
 
 static int imx_thermal_remove(struct platform_device *pdev)
@@ -941,79 +880,47 @@ static int imx_thermal_remove(struct platform_device *pdev)
 	if (!IS_ERR(data->thermal_clk))
 		clk_disable_unprepare(data->thermal_clk);
 
-	/* unregister the busfreq notifier called in low bus freq */
-	if (data->socdata->version != TEMPMON_IMX7)
-		unregister_busfreq_notifier(&thermal_notifier);
-
 	thermal_zone_device_unregister(data->tz);
-	cpufreq_cooling_unregister(data->cdev[0]);
-	devfreq_cooling_unregister(data->cdev[1]);
-	cpufreq_cpu_put(data->policy);
+	imx_thermal_unregister_legacy_cooling(data);
 
 	return 0;
 }
 
-#ifdef CONFIG_PM_SLEEP
-static int imx_thermal_suspend(struct device *dev)
+static int __maybe_unused imx_thermal_suspend(struct device *dev)
 {
 	struct imx_thermal_data *data = dev_get_drvdata(dev);
-	struct regmap *map = data->tempmon;
+	int ret;
 
 	/*
 	 * Need to disable thermal sensor, otherwise, when thermal core
 	 * try to get temperature before thermal sensor resume, a wrong
 	 * temperature will be read as the thermal sensor is powered
-	 * down.
+	 * down. This is done in change_mode() operation called from
+	 * thermal_zone_device_disable()
 	 */
-	regmap_write(map, data->socdata->sensor_ctrl + REG_CLR,
-		     data->socdata->measure_temp_mask);
-	regmap_write(map, data->socdata->sensor_ctrl + REG_SET,
-		     data->socdata->power_down_mask);
-
-	/*
-	 * Save the temp sensor registers of i.MX7D as the tempmon
-	 * will lost power in LPSR mode
-	 */
-	if (data->socdata->version == TEMPMON_IMX7) {
-		regmap_read(map, data->socdata->sensor_ctrl, &imx7_lpsr_save[0]);
-		regmap_read(map, data->socdata->high_alarm_ctrl, &imx7_lpsr_save[1]);
-	}
-
-	data->mode = THERMAL_DEVICE_DISABLED;
+	ret = thermal_zone_device_disable(data->tz);
+	if (ret)
+		return ret;
 	clk_disable_unprepare(data->thermal_clk);
 
 	return 0;
 }
 
-static int imx_thermal_resume(struct device *dev)
+static int __maybe_unused imx_thermal_resume(struct device *dev)
 {
 	struct imx_thermal_data *data = dev_get_drvdata(dev);
-	struct regmap *map = data->tempmon;
 	int ret;
 
 	ret = clk_prepare_enable(data->thermal_clk);
 	if (ret)
 		return ret;
-
-	/*
-	 * restore the temp sensor registers of i.MX7D as the tempmon
-	 * will lost power in LPSR mode
-	 */
-	if (data->socdata->version == TEMPMON_IMX7) {
-		regmap_write(map, data->socdata->sensor_ctrl, imx7_lpsr_save[0]);
-		regmap_write(map, data->socdata->high_alarm_ctrl, imx7_lpsr_save[1]);
-	}
-
 	/* Enabled thermal sensor after resume */
-	regmap_write(map, data->socdata->sensor_ctrl + REG_CLR,
-		     data->socdata->power_down_mask);
-	regmap_write(map, data->socdata->sensor_ctrl + REG_SET,
-		     data->socdata->measure_temp_mask);
-	data->mode = THERMAL_DEVICE_ENABLED;
+	ret = thermal_zone_device_enable(data->tz);
+	if (ret)
+		return ret;
 
 	return 0;
 }
-#endif
 
 static SIMPLE_DEV_PM_OPS(imx_thermal_pm_ops,
 			 imx_thermal_suspend, imx_thermal_resume);
