@@ -1,41 +1,27 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
- * Copyright 2012-2016 Freescale Semiconductor, Inc.
- * Copyright 2017 NXP
+ * Copyright 2012 Freescale Semiconductor, Inc.
  * Copyright (C) 2012 Marek Vasut <marex@denx.de>
+ * Copyright 2022 Sony Corporation
  * on behalf of DENX Software Engineering GmbH
- * Copyright 2019,2020 Sony Home Entertainment & Sound Products Inc.
- *
- * The code contained herein is licensed under the GNU General Public
- * License. You may obtain a copy of the GNU General Public License
- * Version 2 or later at the following locations:
- *
- * http://www.opensource.org/licenses/gpl-license.html
- * http://www.gnu.org/copyleft/gpl.html
  */
 
 #include <linux/module.h>
 #include <linux/of_platform.h>
-#include <linux/of_gpio.h>
 #include <linux/platform_device.h>
 #include <linux/pm_runtime.h>
-#include <linux/dma-mapping.h>
 #include <linux/usb/chipidea.h>
 #include <linux/usb/of.h>
 #include <linux/clk.h>
-#include <linux/of_device.h>
-#include <linux/regmap.h>
-#include <linux/mfd/syscon.h>
-#include <linux/regulator/consumer.h>
-#include <linux/busfreq-imx.h>
+#include <linux/pinctrl/consumer.h>
 #include <linux/pm_qos.h>
-#include <linux/usb/of.h>
+#include <linux/busfreq-imx.h>
 
 #include "ci.h"
 #include "ci_hdrc_imx.h"
 
 struct ci_hdrc_imx_platform_flag {
 	unsigned int flags;
-	bool runtime_pm;
 };
 
 static const struct ci_hdrc_imx_platform_flag imx23_usb_data = {
@@ -44,40 +30,37 @@ static const struct ci_hdrc_imx_platform_flag imx23_usb_data = {
 };
 
 static const struct ci_hdrc_imx_platform_flag imx27_usb_data = {
-		CI_HDRC_DISABLE_STREAMING,
+	.flags = CI_HDRC_DISABLE_STREAMING,
 };
 
 static const struct ci_hdrc_imx_platform_flag imx28_usb_data = {
 	.flags = CI_HDRC_IMX28_WRITE_FIX |
 		CI_HDRC_TURN_VBUS_EARLY_ON |
-		CI_HDRC_DISABLE_STREAMING |
-		CI_HDRC_IMX_EHCI_QUIRK,
+		CI_HDRC_DISABLE_STREAMING,
 };
 
 static const struct ci_hdrc_imx_platform_flag imx6q_usb_data = {
 	.flags = CI_HDRC_SUPPORTS_RUNTIME_PM |
 		CI_HDRC_TURN_VBUS_EARLY_ON |
-		CI_HDRC_DISABLE_STREAMING |
-		CI_HDRC_IMX_EHCI_QUIRK,
+		CI_HDRC_DISABLE_STREAMING,
 };
 
 static const struct ci_hdrc_imx_platform_flag imx6sl_usb_data = {
 	.flags = CI_HDRC_SUPPORTS_RUNTIME_PM |
 		CI_HDRC_TURN_VBUS_EARLY_ON |
-		CI_HDRC_DISABLE_HOST_STREAMING |
-		CI_HDRC_IMX_EHCI_QUIRK,
+		CI_HDRC_DISABLE_HOST_STREAMING,
 };
 
 static const struct ci_hdrc_imx_platform_flag imx6sx_usb_data = {
 	.flags = CI_HDRC_SUPPORTS_RUNTIME_PM |
 		CI_HDRC_TURN_VBUS_EARLY_ON |
-		CI_HDRC_DISABLE_HOST_STREAMING |
-		CI_HDRC_IMX_EHCI_QUIRK,
+		CI_HDRC_DISABLE_HOST_STREAMING,
 };
 
 static const struct ci_hdrc_imx_platform_flag imx6ul_usb_data = {
 	.flags = CI_HDRC_SUPPORTS_RUNTIME_PM |
-		CI_HDRC_TURN_VBUS_EARLY_ON,
+		CI_HDRC_TURN_VBUS_EARLY_ON |
+		CI_HDRC_DISABLE_DEVICE_STREAMING,
 };
 
 static const struct ci_hdrc_imx_platform_flag imx7d_usb_data = {
@@ -86,19 +69,9 @@ static const struct ci_hdrc_imx_platform_flag imx7d_usb_data = {
 
 static const struct ci_hdrc_imx_platform_flag imx7ulp_usb_data = {
 	.flags = CI_HDRC_SUPPORTS_RUNTIME_PM |
-		CI_HDRC_IMX_EHCI_QUIRK |
 		CI_HDRC_PMQOS,
 };
 
-static const struct ci_hdrc_imx_platform_flag imx8qm_usb_data = {
-	.flags = CI_HDRC_SUPPORTS_RUNTIME_PM,
-};
-
-static const struct ci_hdrc_imx_platform_flag imx8mm_usb_data = {
-	.flags = CI_HDRC_SUPPORTS_RUNTIME_PM |
-		CI_HDRC_DISABLE_DEVICE_STREAMING,
-};
-
 static const struct of_device_id ci_hdrc_imx_dt_ids[] = {
 	{ .compatible = "fsl,imx23-usb", .data = &imx23_usb_data},
 	{ .compatible = "fsl,imx28-usb", .data = &imx28_usb_data},
@@ -109,8 +82,6 @@ static const struct of_device_id ci_hdrc_imx_dt_ids[] = {
 	{ .compatible = "fsl,imx6ul-usb", .data = &imx6ul_usb_data},
 	{ .compatible = "fsl,imx7d-usb", .data = &imx7d_usb_data},
 	{ .compatible = "fsl,imx7ulp-usb", .data = &imx7ulp_usb_data},
-	{ .compatible = "fsl,imx8qm-usb", .data = &imx8qm_usb_data},
-	{ .compatible = "fsl,imx8mm-usb", .data = &imx8mm_usb_data},
 	{ /* sentinel */ }
 };
 MODULE_DEVICE_TABLE(of, ci_hdrc_imx_dt_ids);
@@ -121,12 +92,11 @@ struct ci_hdrc_imx_data {
 	struct clk *clk;
 	struct imx_usbmisc_data *usbmisc_data;
 	bool supports_runtime_pm;
+	bool override_phy_control;
 	bool in_lpm;
-	struct regmap *anatop;
 	struct pinctrl *pinctrl;
 	struct pinctrl_state *pinctrl_hsic_active;
 	struct regulator *hsic_pad_regulator;
-	const struct ci_hdrc_imx_platform_flag *data;
 	/* SoC before i.mx6 (except imx23/imx28) needs three clks */
 	bool need_three_clks;
 	struct clk *clk_ipg;
@@ -134,50 +104,9 @@ struct ci_hdrc_imx_data {
 	struct clk *clk_per;
 	/* --------------------------------- */
 	struct pm_qos_request pm_qos_req;
+	const struct ci_hdrc_imx_platform_flag *plat_data;
 };
 
-#define DEBOUNCE_TIME		msecs_to_jiffies(50)
-
-static void do_usb_oc(struct work_struct *work)
-{
-
-	struct imx_usbmisc_data *data =
-		container_of(work, struct imx_usbmisc_data, vbus_work.work);
-	unsigned int oc = gpiod_get_value_cansleep(data->vbus_oc_gpio);
-	struct ci_hdrc *ci = platform_get_drvdata(data->ci_pdev);
-
-	if (data->oc_polarity == oc &&
-	  regulator_is_enabled(ci->platdata->reg_vbus)) {
-		enum ci_role role;
-
-		pr_debug("%s:%d Vbus Over Current!\n", __func__, __LINE__);
-
-		mutex_lock(&ci->mutex);
-		role = ci->role;
-		if ((ci->roles[role]->stop) && (role == CI_ROLE_HOST)) {
-			pm_runtime_get_sync(data->dev);
-			disable_irq(ci->irq);
-			ci->vbus_power = CI_VBUS_AUTO;
-			ci->vbus_event = CI_VBUS_OVER_CURRENT;
-			ci->roles[role]->stop(ci);
-			ci->vbus_power = CI_VBUS_OFF;
-			enable_irq(ci->irq);
-			pm_runtime_put_sync(data->dev);
-		}
-		mutex_unlock(&ci->mutex);
-	}
-}
-
-static irqreturn_t oc_thread_isr(int irq, void *priv)
-{
-	struct imx_usbmisc_data *data = priv;
-
-	/* Let the pin settle before processing it */
-	mod_delayed_work(system_wq, &data->vbus_work, DEBOUNCE_TIME);
-
-	return IRQ_HANDLED;
-}
-
 /* Common functions shared by usbmisc drivers */
 
 static struct imx_usbmisc_data *usbmisc_get_init_data(struct device *dev)
@@ -212,83 +141,42 @@ static struct imx_usbmisc_data *usbmisc_get_init_data(struct device *dev)
 	misc_pdev = of_find_device_by_node(args.np);
 	of_node_put(args.np);
 
-	if (!misc_pdev || !platform_get_drvdata(misc_pdev))
+	if (!misc_pdev)
 		return ERR_PTR(-EPROBE_DEFER);
 
+	if (!platform_get_drvdata(misc_pdev)) {
+		put_device(&misc_pdev->dev);
+		return ERR_PTR(-EPROBE_DEFER);
+	}
 	data->dev = &misc_pdev->dev;
 
-	if (of_find_property(np, "disable-over-current", NULL))
+	/*
+	 * Check the various over current related properties. If over current
+	 * detection is disabled we're not interested in the polarity.
+	 */
+	if (of_find_property(np, "disable-over-current", NULL)) {
 		data->disable_oc = 1;
+	} else if (of_find_property(np, "over-current-active-high", NULL)) {
+		data->oc_pol_active_low = 0;
+		data->oc_pol_configured = 1;
+	} else if (of_find_property(np, "over-current-active-low", NULL)) {
+		data->oc_pol_active_low = 1;
+		data->oc_pol_configured = 1;
+	} else {
+		dev_warn(dev, "No over current polarity defined\n");
+	}
 
-	if (of_find_property(np, "over-current-active-high", NULL))
-		data->oc_polarity = 1;
-
-	if (of_find_property(np, "power-polarity-active-high", NULL))
-		data->pwr_polarity = 1;
-
-	if (of_find_property(np, "external-vbus-divider", NULL))
-		data->evdo = 1;
+	data->pwr_pol = of_property_read_bool(np, "power-active-high");
+	data->evdo = of_property_read_bool(np, "external-vbus-divider");
 
 	if (of_usb_get_phy_mode(np) == USBPHY_INTERFACE_MODE_ULPI)
 		data->ulpi = 1;
 
-	if (of_find_property(np, "osc-clkgate-delay", NULL)) {
-		ret = of_property_read_u32(np, "osc-clkgate-delay",
-			&data->osc_clkgate_delay);
-		if (ret) {
-			dev_err(dev,
-				"failed to get osc-clkgate-delay value\n");
-			return ERR_PTR(ret);
-		}
-		/*
-		 * 0 <= osc_clkgate_delay <=7
-		 * - 0x0 (default) is 0.5ms,
-		 * - 0x1-0x7: 1-7ms
-		 */
-		if (data->osc_clkgate_delay > 7) {
-			dev_err(dev,
-				"value of osc-clkgate-delay is incorrect\n");
-			return ERR_PTR(-EINVAL);
-		}
-	}
-
-	of_property_read_u32(np, "picophy,pre-emp-curr-control",
+	of_property_read_u32(np, "samsung,picophy-pre-emp-curr-control",
 			&data->emp_curr_control);
-	of_property_read_u32(np, "picophy,dc-vol-level-adjust",
+	of_property_read_u32(np, "samsung,picophy-dc-vol-level-adjust",
 			&data->dc_vol_level_adjust);
 
-	if(!data->disable_oc) {
-		INIT_DELAYED_WORK(&data->vbus_work, do_usb_oc);
-
-		data->vbus_oc_gpio = devm_gpiod_get(dev, "oc", GPIOD_IN);
-		if (IS_ERR(data->vbus_oc_gpio)) {
-			pr_debug("USB OC GPIO not exist!\n");
-		} else {
-			if (gpiod_get_direction(data->vbus_oc_gpio) != GPIOF_DIR_IN) {
-				pr_debug("USB OC GPIO not in input mode, fixing\n");
-				gpiod_direction_input(data->vbus_oc_gpio);
-			}
-			data->vbus_oc_irq = gpiod_to_irq(data->vbus_oc_gpio);
-			if (data->vbus_oc_irq < 0) {
-				pr_debug("can't get USB OC IRQ: %d\n", data->vbus_oc_irq);
-			} else {
-				int ret;
-				int flag = data->oc_polarity ?
-					IRQF_TRIGGER_RISING : IRQF_TRIGGER_FALLING;
-
-				ret = devm_request_threaded_irq(dev, data->vbus_oc_irq,
-						NULL, oc_thread_isr,
-						IRQF_SHARED | IRQF_ONESHOT |
-						flag ,
-						dev_name(dev), data);
-
-				if (ret < 0) {
-					pr_err("can't request IRQ for USB OC GPIO: %d\n", ret);
-				}
-			}
-		}
-	}
-
 	return data;
 }
 
@@ -390,7 +278,7 @@ static void imx_disable_unprepare_clks(struct device *dev)
 	}
 }
 
-static int ci_hdrc_imx_notify_event(struct ci_hdrc *ci, unsigned event)
+static int ci_hdrc_imx_notify_event(struct ci_hdrc *ci, unsigned int event)
 {
 	struct device *dev = ci->dev->parent;
 	struct ci_hdrc_imx_data *data = dev_get_drvdata(dev);
@@ -398,46 +286,32 @@ static int ci_hdrc_imx_notify_event(struct ci_hdrc *ci, unsigned event)
 	struct imx_usbmisc_data *mdata = data->usbmisc_data;
 
 	switch (event) {
-	case CI_HDRC_CONTROLLER_VBUS_EVENT:
-		if (ci->vbus_active)
-			ret = imx_usbmisc_charger_detection(mdata, true);
-		else
-			ret = imx_usbmisc_charger_detection(mdata, false);
-		break;
 	case CI_HDRC_IMX_HSIC_ACTIVE_EVENT:
-		if (!IS_ERR(data->pinctrl) &&
-			!IS_ERR(data->pinctrl_hsic_active)) {
+		if (data->pinctrl) {
 			ret = pinctrl_select_state(data->pinctrl,
 					data->pinctrl_hsic_active);
 			if (ret)
 				dev_err(dev,
 					"hsic_active select failed, err=%d\n",
 					ret);
-			return ret;
 		}
 		break;
 	case CI_HDRC_IMX_HSIC_SUSPEND_EVENT:
-		if (data->usbmisc_data) {
-			ret = imx_usbmisc_hsic_set_connect(data->usbmisc_data);
-			if (ret)
-				dev_err(dev,
-					"hsic_set_connect failed, err=%d\n",
-					ret);
-			return ret;
-		}
+		ret = imx_usbmisc_hsic_set_connect(mdata);
+		if (ret)
+			dev_err(dev,
+				"hsic_set_connect failed, err=%d\n", ret);
 		break;
-	case CI_HDRC_IMX_TERM_SELECT_OVERRIDE_FS:
-		if (data->usbmisc_data)
-			return imx_usbmisc_term_select_override(
-					data->usbmisc_data, true, 1);
-		break;
-	case CI_HDRC_IMX_TERM_SELECT_OVERRIDE_OFF:
-		if (data->usbmisc_data)
-			return imx_usbmisc_term_select_override(
-					data->usbmisc_data, false, 0);
+	case CI_HDRC_CONTROLLER_VBUS_EVENT:
+		if (ci->vbus_active)
+			ret = imx_usbmisc_charger_detection(mdata, true);
+		else
+			ret = imx_usbmisc_charger_detection(mdata, false);
+		if (ci->usb_phy)
+			schedule_work(&ci->usb_phy->chg_work);
 		break;
 	default:
-		dev_dbg(dev, "unknown event\n");
+		break;
 	}
 
 	return ret;
@@ -452,87 +326,131 @@ static int ci_hdrc_imx_probe(struct platform_device *pdev)
 		.notify_event	= ci_hdrc_imx_notify_event,
 	};
 	int ret;
-	const struct of_device_id *of_id;
 	const struct ci_hdrc_imx_platform_flag *imx_platform_flag;
+	struct device_node *np = pdev->dev.of_node;
 	struct device *dev = &pdev->dev;
-	struct device_node *np = dev->of_node;
-	struct pinctrl_state *pinctrl_hsic_idle;
 
-	of_id = of_match_device(ci_hdrc_imx_dt_ids, dev);
-	if (!of_id)
-		return -ENODEV;
+	imx_platform_flag = of_device_get_match_data(&pdev->dev);
 
-	imx_platform_flag = of_id->data;
-
-	data = devm_kzalloc(dev, sizeof(*data), GFP_KERNEL);
+	data = devm_kzalloc(&pdev->dev, sizeof(*data), GFP_KERNEL);
 	if (!data)
 		return -ENOMEM;
 
-	platform_set_drvdata(pdev, data);
-
-	data->data = imx_platform_flag;
+	data->plat_data = imx_platform_flag;
 	pdata.flags |= imx_platform_flag->flags;
+	platform_set_drvdata(pdev, data);
 	data->usbmisc_data = usbmisc_get_init_data(dev);
 	if (IS_ERR(data->usbmisc_data))
 		return PTR_ERR(data->usbmisc_data);
 
-	data->pinctrl = devm_pinctrl_get(dev);
-	if (IS_ERR(data->pinctrl)) {
-		dev_dbg(dev, "pinctrl get failed, err=%ld\n",
-						PTR_ERR(data->pinctrl));
-	} else {
-		pinctrl_hsic_idle = pinctrl_lookup_state(data->pinctrl, "idle");
-		if (IS_ERR(pinctrl_hsic_idle)) {
-			dev_dbg(dev,
-				"pinctrl_hsic_idle lookup failed, err=%ld\n",
-						PTR_ERR(pinctrl_hsic_idle));
-		} else {
-			ret = pinctrl_select_state(data->pinctrl,
-						pinctrl_hsic_idle);
+	if ((of_usb_get_phy_mode(dev->of_node) == USBPHY_INTERFACE_MODE_HSIC)
+		&& data->usbmisc_data) {
+		pdata.flags |= CI_HDRC_IMX_IS_HSIC;
+		data->usbmisc_data->hsic = 1;
+		data->pinctrl = devm_pinctrl_get(dev);
+		if (PTR_ERR(data->pinctrl) == -ENODEV)
+			data->pinctrl = NULL;
+		else if (IS_ERR(data->pinctrl)) {
+			if (PTR_ERR(data->pinctrl) != -EPROBE_DEFER)
+				dev_err(dev, "pinctrl get failed, err=%ld\n",
+					PTR_ERR(data->pinctrl));
+			return PTR_ERR(data->pinctrl);
+		}
+
+		data->hsic_pad_regulator =
+				devm_regulator_get_optional(dev, "hsic");
+		if (PTR_ERR(data->hsic_pad_regulator) == -ENODEV) {
+			/* no pad regualator is needed */
+			data->hsic_pad_regulator = NULL;
+		} else if (IS_ERR(data->hsic_pad_regulator)) {
+			if (PTR_ERR(data->hsic_pad_regulator) != -EPROBE_DEFER)
+				dev_err(dev,
+					"Get HSIC pad regulator error: %ld\n",
+					PTR_ERR(data->hsic_pad_regulator));
+			return PTR_ERR(data->hsic_pad_regulator);
+		}
+
+		if (data->hsic_pad_regulator) {
+			ret = regulator_enable(data->hsic_pad_regulator);
 			if (ret) {
 				dev_err(dev,
-					"hsic_idle select failed, err=%d\n",
-									ret);
+					"Failed to enable HSIC pad regulator\n");
 				return ret;
 			}
 		}
+	}
+
+	/* HSIC pinctrl handling */
+	if (data->pinctrl) {
+		struct pinctrl_state *pinctrl_hsic_idle;
+
+		pinctrl_hsic_idle = pinctrl_lookup_state(data->pinctrl, "idle");
+		if (IS_ERR(pinctrl_hsic_idle)) {
+			dev_err(dev,
+				"pinctrl_hsic_idle lookup failed, err=%ld\n",
+					PTR_ERR(pinctrl_hsic_idle));
+			return PTR_ERR(pinctrl_hsic_idle);
+		}
+
+		ret = pinctrl_select_state(data->pinctrl, pinctrl_hsic_idle);
+		if (ret) {
+			dev_err(dev, "hsic_idle select failed, err=%d\n", ret);
+			return ret;
+		}
 
 		data->pinctrl_hsic_active = pinctrl_lookup_state(data->pinctrl,
 								"active");
-		if (IS_ERR(data->pinctrl_hsic_active))
-			dev_dbg(dev,
+		if (IS_ERR(data->pinctrl_hsic_active)) {
+			dev_err(dev,
 				"pinctrl_hsic_active lookup failed, err=%ld\n",
 					PTR_ERR(data->pinctrl_hsic_active));
+			return PTR_ERR(data->pinctrl_hsic_active);
+		}
 	}
 
-	ret = imx_get_clks(dev);
-	if (ret)
-		return ret;
+	if (pdata.flags & CI_HDRC_PMQOS)
+		cpu_latency_qos_add_request(&data->pm_qos_req, 0);
 
 #ifdef CONFIG_REDUCE_DRAM_BUSFREQ
 	request_bus_freq(BUS_FREQ_AUDIO);
 #else
 	request_bus_freq(BUS_FREQ_HIGH);
 #endif
-	if (pdata.flags & CI_HDRC_PMQOS)
-		pm_qos_add_request(&data->pm_qos_req,
-			PM_QOS_CPU_DMA_LATENCY, 0);
+	ret = imx_get_clks(dev);
+	if (ret)
+		goto disable_hsic_regulator;
 
 	ret = imx_prepare_enable_clks(dev);
 	if (ret)
-		goto err_bus_freq;
+		goto disable_hsic_regulator;
 
 	data->phy = devm_usb_get_phy_by_phandle(dev, "fsl,usbphy", 0);
 	if (IS_ERR(data->phy)) {
 		ret = PTR_ERR(data->phy);
-		/* Return -EINVAL if no usbphy is available */
-		if (ret == -ENODEV)
-			ret = -EINVAL;
-		goto err_clk;
+		if (ret != -ENODEV)
+			goto err_clk;
+		data->phy = devm_usb_get_phy_by_phandle(dev, "phys", 0);
+		if (IS_ERR(data->phy)) {
+			ret = PTR_ERR(data->phy);
+			if (ret == -ENODEV)
+				data->phy = NULL;
+			else
+				goto err_clk;
+		}
 	}
 
 	pdata.usb_phy = data->phy;
-	data->usbmisc_data->usb_phy = data->phy;
+	if (data->usbmisc_data)
+		data->usbmisc_data->usb_phy = data->phy;
+
+	if ((of_device_is_compatible(np, "fsl,imx53-usb") ||
+	     of_device_is_compatible(np, "fsl,imx51-usb")) && pdata.usb_phy &&
+	    of_usb_get_phy_mode(np) == USBPHY_INTERFACE_MODE_ULPI) {
+		pdata.flags |= CI_HDRC_OVERRIDE_PHY_CONTROL;
+		data->override_phy_control = true;
+		usb_phy_init(pdata.usb_phy);
+	}
+
 	if (pdata.flags & CI_HDRC_SUPPORTS_RUNTIME_PM)
 		data->supports_runtime_pm = true;
 
@@ -541,48 +459,10 @@ static int ci_hdrc_imx_probe(struct platform_device *pdev)
 		pdata.flags &= ~CI_HDRC_SUPPORTS_RUNTIME_PM;
 	}
 
-	if (of_usb_get_phy_mode(dev->of_node) == USBPHY_INTERFACE_MODE_HSIC) {
-		pdata.flags |= CI_HDRC_IMX_IS_HSIC;
-		data->usbmisc_data->hsic = 1;
-		data->hsic_pad_regulator = devm_regulator_get(dev, "pad");
-		if (PTR_ERR(data->hsic_pad_regulator) == -EPROBE_DEFER) {
-			ret = -EPROBE_DEFER;
-			goto err_clk;
-		} else if (PTR_ERR(data->hsic_pad_regulator) == -ENODEV) {
-			/* no pad regualator is needed */
-			data->hsic_pad_regulator = NULL;
-		} else if (IS_ERR(data->hsic_pad_regulator)) {
-			dev_err(dev, "Get hsic pad regulator error: %ld\n",
-					PTR_ERR(data->hsic_pad_regulator));
-			ret = PTR_ERR(data->hsic_pad_regulator);
-			goto err_clk;
-		}
-
-		if (data->hsic_pad_regulator) {
-			ret = regulator_enable(data->hsic_pad_regulator);
-			if (ret) {
-				dev_err(dev,
-					"Fail to enable hsic pad regulator\n");
-				goto err_clk;
-			}
-		}
-	}
-
-	if (of_find_property(np, "fsl,anatop", NULL) && data->usbmisc_data) {
-		data->anatop = syscon_regmap_lookup_by_phandle(np,
-							"fsl,anatop");
-		if (IS_ERR(data->anatop)) {
-			dev_dbg(dev, "failed to find regmap for anatop\n");
-			ret = PTR_ERR(data->anatop);
-			goto disable_hsic_regulator;
-		}
-		data->usbmisc_data->anatop = data->anatop;
-	}
-
 	ret = imx_usbmisc_init(data->usbmisc_data);
 	if (ret) {
 		dev_err(dev, "usbmisc init failed, ret=%d\n", ret);
-		goto disable_hsic_regulator;
+		goto err_clk;
 	}
 
 	data->ci_pdev = ci_hdrc_add_device(dev,
@@ -591,11 +471,24 @@ static int ci_hdrc_imx_probe(struct platform_device *pdev)
 	if (IS_ERR(data->ci_pdev)) {
 		ret = PTR_ERR(data->ci_pdev);
 		if (ret != -EPROBE_DEFER)
-			dev_err(dev,
-				"ci_hdrc_add_device failed, err=%d\n", ret);
-		goto disable_hsic_regulator;
+			dev_err(dev, "ci_hdrc_add_device failed, err=%d\n",
+					ret);
+		goto err_clk;
+	}
+
+	if (data->usbmisc_data) {
+		if (!IS_ERR(pdata.id_extcon.edev) ||
+		    of_property_read_bool(np, "usb-role-switch"))
+			data->usbmisc_data->ext_id = 1;
+
+		if (!IS_ERR(pdata.vbus_extcon.edev) ||
+		    of_property_read_bool(np, "usb-role-switch"))
+			data->usbmisc_data->ext_vbus = 1;
+
+		/* usbmisc needs to know dr mode to choose wakeup setting */
+		data->usbmisc_data->available_role =
+			ci_hdrc_query_available_role(data->ci_pdev);
 	}
-	data->usbmisc_data->ci_pdev = data->ci_pdev;
 
 	ret = imx_usbmisc_init_post(data->usbmisc_data);
 	if (ret) {
@@ -603,17 +496,6 @@ static int ci_hdrc_imx_probe(struct platform_device *pdev)
 		goto disable_device;
 	}
 
-	ret = imx_usbmisc_set_wakeup(data->usbmisc_data, false);
-	if (ret) {
-		dev_err(dev, "usbmisc set_wakeup failed, ret=%d\n", ret);
-		goto disable_device;
-	}
-
-	/* usbmisc needs to know dr mode to choose wakeup setting */
-	if (data->usbmisc_data)
-		data->usbmisc_data->available_role =
-			ci_hdrc_query_available_role(data->ci_pdev);
-
 	if (data->supports_runtime_pm) {
 		pm_runtime_set_active(dev);
 		pm_runtime_enable(dev);
@@ -625,19 +507,20 @@ static int ci_hdrc_imx_probe(struct platform_device *pdev)
 
 disable_device:
 	ci_hdrc_remove_device(data->ci_pdev);
-disable_hsic_regulator:
-	if (data->hsic_pad_regulator)
-		ret = regulator_disable(data->hsic_pad_regulator);
 err_clk:
-	imx_disable_unprepare_clks(&pdev->dev);
-err_bus_freq:
-	if (pdata.flags & CI_HDRC_PMQOS)
-		pm_qos_remove_request(&data->pm_qos_req);
+	imx_disable_unprepare_clks(dev);
+disable_hsic_regulator:
 #ifdef CONFIG_REDUCE_DRAM_BUSFREQ
 	release_bus_freq(BUS_FREQ_AUDIO);
 #else
 	release_bus_freq(BUS_FREQ_HIGH);
 #endif
+	if (data->hsic_pad_regulator)
+		/* don't overwrite original ret (cf. EPROBE_DEFER) */
+		regulator_disable(data->hsic_pad_regulator);
+	if (pdata.flags & CI_HDRC_PMQOS)
+		cpu_latency_qos_remove_request(&data->pm_qos_req);
+	data->ci_pdev = NULL;
 	return ret;
 }
 
@@ -650,17 +533,22 @@ static int ci_hdrc_imx_remove(struct platform_device *pdev)
 		pm_runtime_disable(&pdev->dev);
 		pm_runtime_put_noidle(&pdev->dev);
 	}
-	ci_hdrc_remove_device(data->ci_pdev);
-	imx_disable_unprepare_clks(&pdev->dev);
-	if (data->data->flags & CI_HDRC_PMQOS)
-		pm_qos_remove_request(&data->pm_qos_req);
+	if (data->ci_pdev)
+		ci_hdrc_remove_device(data->ci_pdev);
+	if (data->override_phy_control)
+		usb_phy_shutdown(data->phy);
+	if (data->ci_pdev) {
+		imx_disable_unprepare_clks(&pdev->dev);
 #ifdef CONFIG_REDUCE_DRAM_BUSFREQ
-	release_bus_freq(BUS_FREQ_AUDIO);
+		release_bus_freq(BUS_FREQ_AUDIO);
 #else
-	release_bus_freq(BUS_FREQ_HIGH);
+		release_bus_freq(BUS_FREQ_HIGH);
 #endif
-	if (data->hsic_pad_regulator)
-		regulator_disable(data->hsic_pad_regulator);
+		if (data->plat_data->flags & CI_HDRC_PMQOS)
+			cpu_latency_qos_remove_request(&data->pm_qos_req);
+		if (data->hsic_pad_regulator)
+			regulator_disable(data->hsic_pad_regulator);
+	}
 
 	return 0;
 }
@@ -670,42 +558,38 @@ static void ci_hdrc_imx_shutdown(struct platform_device *pdev)
 	ci_hdrc_imx_remove(pdev);
 }
 
-#ifdef CONFIG_PM
-static int imx_controller_suspend(struct device *dev)
+static int __maybe_unused imx_controller_suspend(struct device *dev,
+						 pm_message_t msg)
 {
 	struct ci_hdrc_imx_data *data = dev_get_drvdata(dev);
-	int ret;
+	int ret = 0;
 
 	dev_dbg(dev, "at %s\n", __func__);
 
-	if (data->usbmisc_data) {
-		ret = imx_usbmisc_hsic_set_clk(data->usbmisc_data, false);
-		if (ret) {
-			dev_err(dev,
-				"usbmisc hsic_set_clk failed, ret=%d\n", ret);
-			return ret;
-		}
+	ret = imx_usbmisc_suspend(data->usbmisc_data,
+				  PMSG_IS_AUTO(msg) || device_may_wakeup(dev));
+	if (ret) {
+		dev_err(dev,
+			"usbmisc suspend failed, ret=%d\n", ret);
+		return ret;
 	}
-    ret = imx_usbmisc_suspend(data->usbmisc_data);
-    if (ret) {
-        dev_err(dev,"usbmisc suspend failed, ret=%d\n", ret);
-        return ret;
-    }
 
 	imx_disable_unprepare_clks(dev);
-	if (data->data->flags & CI_HDRC_PMQOS)
-		pm_qos_remove_request(&data->pm_qos_req);
 #ifdef CONFIG_REDUCE_DRAM_BUSFREQ
 	release_bus_freq(BUS_FREQ_AUDIO);
 #else
 	release_bus_freq(BUS_FREQ_HIGH);
 #endif
+	if (data->plat_data->flags & CI_HDRC_PMQOS)
+		cpu_latency_qos_remove_request(&data->pm_qos_req);
+
 	data->in_lpm = true;
 
 	return 0;
 }
 
-static int imx_controller_resume(struct device *dev)
+static int __maybe_unused imx_controller_resume(struct device *dev,
+						pm_message_t msg)
 {
 	struct ci_hdrc_imx_data *data = dev_get_drvdata(dev);
 	int ret = 0;
@@ -715,68 +599,35 @@ static int imx_controller_resume(struct device *dev)
 	if (!data->in_lpm)
 		return 0;
 
+	if (data->plat_data->flags & CI_HDRC_PMQOS)
+		cpu_latency_qos_add_request(&data->pm_qos_req, 0);
+
 #ifdef CONFIG_REDUCE_DRAM_BUSFREQ
 	request_bus_freq(BUS_FREQ_AUDIO);
 #else
 	request_bus_freq(BUS_FREQ_HIGH);
 #endif
-	if (data->data->flags & CI_HDRC_PMQOS)
-		pm_qos_add_request(&data->pm_qos_req,
-			PM_QOS_CPU_DMA_LATENCY, 0);
 	ret = imx_prepare_enable_clks(dev);
 	if (ret)
-		goto err_bus_freq;
+		return ret;
 
 	data->in_lpm = false;
 
-	ret = imx_usbmisc_power_lost_check(data->usbmisc_data);
-	/* re-init if resume from power lost */
-	if (ret > 0) {
-		ret = imx_usbmisc_init(data->usbmisc_data);
-		if (ret) {
-			dev_err(dev, "usbmisc init failed, ret=%d\n", ret);
-			goto clk_disable;
-		}
-	}
-
-	ret = imx_usbmisc_set_wakeup(data->usbmisc_data, false);
+	ret = imx_usbmisc_resume(data->usbmisc_data,
+				 PMSG_IS_AUTO(msg) || device_may_wakeup(dev));
 	if (ret) {
-		dev_err(dev, "usbmisc set_wakeup failed, ret=%d\n", ret);
+		dev_err(dev, "usbmisc resume failed, ret=%d\n", ret);
 		goto clk_disable;
 	}
 
-	ret = imx_usbmisc_hsic_set_clk(data->usbmisc_data, true);
-	if (ret) {
-		dev_err(dev, "usbmisc hsic_set_clk failed, ret=%d\n", ret);
-		goto hsic_set_clk_fail;
-	}
-
-    ret = imx_usbmisc_resume(data->usbmisc_data);
-    if (ret) {
-        dev_err(dev,"usbmisc resume failed, ret=%d\n", ret);
-        return ret;
-    }
-
-
 	return 0;
 
-hsic_set_clk_fail:
-	imx_usbmisc_set_wakeup(data->usbmisc_data, true);
 clk_disable:
 	imx_disable_unprepare_clks(dev);
-err_bus_freq:
-	if (data->data->flags & CI_HDRC_PMQOS)
-		pm_qos_remove_request(&data->pm_qos_req);
-#ifdef CONFIG_REDUCE_DRAM_BUSFREQ
-	release_bus_freq(BUS_FREQ_AUDIO);
-#else
-	release_bus_freq(BUS_FREQ_HIGH);
-#endif
 	return ret;
 }
 
-#ifdef CONFIG_PM_SLEEP
-static int ci_hdrc_imx_suspend(struct device *dev)
+static int __maybe_unused ci_hdrc_imx_suspend(struct device *dev)
 {
 	int ret;
 
@@ -786,16 +637,7 @@ static int ci_hdrc_imx_suspend(struct device *dev)
 		/* The core's suspend doesn't run */
 		return 0;
 
-	if (device_may_wakeup(dev)) {
-		ret = imx_usbmisc_set_wakeup(data->usbmisc_data, true);
-		if (ret) {
-			dev_err(dev, "usbmisc set_wakeup failed, ret=%d\n",
-					ret);
-			return ret;
-		}
-	}
-
-	ret = imx_controller_suspend(dev);
+	ret = imx_controller_suspend(dev, PMSG_SUSPEND);
 	if (ret)
 		return ret;
 
@@ -803,13 +645,13 @@ static int ci_hdrc_imx_suspend(struct device *dev)
 	return ret;
 }
 
-static int ci_hdrc_imx_resume(struct device *dev)
+static int __maybe_unused ci_hdrc_imx_resume(struct device *dev)
 {
 	struct ci_hdrc_imx_data *data = dev_get_drvdata(dev);
 	int ret;
 
 	pinctrl_pm_select_default_state(dev);
-	ret = imx_controller_resume(dev);
+	ret = imx_controller_resume(dev, PMSG_RESUME);
 	if (!ret && data->supports_runtime_pm) {
 		pm_runtime_disable(dev);
 		pm_runtime_set_active(dev);
@@ -818,32 +660,22 @@ static int ci_hdrc_imx_resume(struct device *dev)
 
 	return ret;
 }
-#endif /* CONFIG_PM_SLEEP */
 
-static int ci_hdrc_imx_runtime_suspend(struct device *dev)
+static int __maybe_unused ci_hdrc_imx_runtime_suspend(struct device *dev)
 {
 	struct ci_hdrc_imx_data *data = dev_get_drvdata(dev);
-	int ret;
 
 	if (data->in_lpm)
 		return 0;
 
-	ret = imx_usbmisc_set_wakeup(data->usbmisc_data, true);
-	if (ret) {
-		dev_err(dev, "usbmisc set_wakeup failed, ret=%d\n", ret);
-		return ret;
-	}
-
-	return imx_controller_suspend(dev);
+	return imx_controller_suspend(dev, PMSG_AUTO_SUSPEND);
 }
 
-static int ci_hdrc_imx_runtime_resume(struct device *dev)
+static int __maybe_unused ci_hdrc_imx_runtime_resume(struct device *dev)
 {
-	return imx_controller_resume(dev);
+	return imx_controller_resume(dev, PMSG_AUTO_RESUME);
 }
 
-#endif /* CONFIG_PM */
-
 static const struct dev_pm_ops ci_hdrc_imx_pm_ops = {
 	SET_SYSTEM_SLEEP_PM_OPS(ci_hdrc_imx_suspend, ci_hdrc_imx_resume)
 	SET_RUNTIME_PM_OPS(ci_hdrc_imx_runtime_suspend,
@@ -863,7 +695,7 @@ static struct platform_driver ci_hdrc_imx_driver = {
 module_platform_driver(ci_hdrc_imx_driver);
 
 MODULE_ALIAS("platform:imx-usb");
-MODULE_LICENSE("GPL v2");
+MODULE_LICENSE("GPL");
 MODULE_DESCRIPTION("CI HDRC i.MX USB binding");
 MODULE_AUTHOR("Marek Vasut <marex@denx.de>");
 MODULE_AUTHOR("Richard Zhao <richard.zhao@freescale.com>");
