@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2016 Freescale Semiconductor, Inc.
+ * Copyright (C) 2011-2021 Freescale Semiconductor, Inc.
  *
  * The code contained herein is licensed under the GNU General Public
  * License. You may obtain a copy of the GNU General Public License
@@ -27,6 +27,7 @@
 #include <linux/irq.h>
 #include <linux/io.h>
 #include <linux/fb.h>
+#include <linux/fbcon.h>
 #include <linux/init.h>
 #include <linux/list.h>
 #include <linux/delay.h>
@@ -1041,12 +1042,6 @@ static int hdmi_phy_configure(struct mxc_hdmi *hdmi, unsigned char pRep,
 			HDMI_PHY_I2CM_SLAVE_ADDR);
 	hdmi_phy_test_clear(hdmi, 0);
 
-	if (hdmi->hdmi_data.video_mode.mPixelClock < 0) {
-		dev_dbg(&hdmi->pdev->dev, "Pixel clock (%d) must be positive\n",
-			hdmi->hdmi_data.video_mode.mPixelClock);
-		return false;
-	}
-
 	if (hdmi->hdmi_data.video_mode.mPixelClock <= 45250000) {
 		switch (cRes) {
 		case 8:
@@ -1078,6 +1073,7 @@ static int hdmi_phy_configure(struct mxc_hdmi *hdmi, unsigned char pRep,
 		case 12:
 			hdmi_phy_i2c_write(hdmi, 0x4142, 0x06);
 			hdmi_phy_i2c_write(hdmi, 0x0005, 0x15);
+			break;
 		default:
 			return false;
 		}
@@ -1094,6 +1090,7 @@ static int hdmi_phy_configure(struct mxc_hdmi *hdmi, unsigned char pRep,
 		case 12:
 			hdmi_phy_i2c_write(hdmi, 0x40a2, 0x06);
 			hdmi_phy_i2c_write(hdmi, 0x000a, 0x15);
+			break;
 		default:
 			return false;
 		}
@@ -1110,6 +1107,7 @@ static int hdmi_phy_configure(struct mxc_hdmi *hdmi, unsigned char pRep,
 		case 12:
 			hdmi_phy_i2c_write(hdmi, 0x4002, 0x06);
 			hdmi_phy_i2c_write(hdmi, 0x000f, 0x15);
+			break;
 		default:
 			return false;
 		}
@@ -1703,8 +1701,8 @@ static void mxc_hdmi_enable_video_path(struct mxc_hdmi *hdmi)
 	hdmi_writeb(0x16, HDMI_FC_CH1PREAM);
 	hdmi_writeb(0x21, HDMI_FC_CH2PREAM);
 
-	clkdis = hdmi_readb(HDMI_MC_CLKDIS);
 	/* Enable pixel clock and tmds data path */
+	clkdis = 0x7F;
 	clkdis &= ~HDMI_MC_CLKDIS_PIXELCLK_DISABLE;
 	hdmi_writeb(clkdis, HDMI_MC_CLKDIS);
 
@@ -1766,6 +1764,8 @@ static void hdmi_disable_overflow_interrupts(void)
 
 static void mxc_hdmi_notify_fb(struct mxc_hdmi *hdmi)
 {
+	int ret;
+
 	dev_dbg(&hdmi->pdev->dev, "%s\n", __func__);
 
 	/* Don't notify if we aren't registered yet */
@@ -1782,9 +1782,9 @@ static void mxc_hdmi_notify_fb(struct mxc_hdmi *hdmi)
 	 */
 	hdmi->fbi->var.activate |= FB_ACTIVATE_FORCE;
 	console_lock();
-	hdmi->fbi->flags |= FBINFO_MISC_USEREVENT;
-	fb_set_var(hdmi->fbi, &hdmi->fbi->var);
-	hdmi->fbi->flags &= ~FBINFO_MISC_USEREVENT;
+	ret = fb_set_var(hdmi->fbi, &hdmi->fbi->var);
+	if (!ret)
+		fbcon_update_vcs(hdmi->fbi, hdmi->fbi->var.activate & FB_ACTIVATE_ALL);
 	console_unlock();
 
 	dev_dbg(&hdmi->pdev->dev, "%s exit\n", __func__);
@@ -1965,8 +1965,9 @@ static void mxc_hdmi_cable_connected(struct mxc_hdmi *hdmi)
 
 	case HDMI_EDID_FAIL:
 		mxc_hdmi_default_edid_cfg(hdmi);
-		/* No break here  */
+		fallthrough;
 	case HDMI_EDID_NO_MODES:
+		fallthrough;
 	default:
 		mxc_hdmi_default_modelist(hdmi);
 		break;
@@ -1995,13 +1996,10 @@ static void mxc_hdmi_power_off(struct mxc_dispdrv_handle *disp,
 
 static void mxc_hdmi_cable_disconnected(struct mxc_hdmi *hdmi)
 {
-	u8 clkdis;
-
 	dev_dbg(&hdmi->pdev->dev, "%s\n", __func__);
-	/* Disable All HDMI clock and bypass cec */
-	clkdis = hdmi_readb(HDMI_MC_CLKDIS);
-	clkdis |= 0x5f;
-	hdmi_writeb(clkdis, HDMI_MC_CLKDIS);
+
+	/* Disable All HDMI clock */
+	hdmi_writeb(0xff, HDMI_MC_CLKDIS);
 
 	mxc_hdmi_phy_disable(hdmi);
 
@@ -2271,10 +2269,11 @@ static void mxc_hdmi_setup(struct mxc_hdmi *hdmi, unsigned long event)
 	hdmi_video_csc(hdmi);
 	hdmi_video_sample(hdmi);
 
+	/* delay 20ms before tmds start work */
+	msleep(20);
 	mxc_hdmi_clear_overflow(hdmi);
 
 	dev_dbg(&hdmi->pdev->dev, "%s exit\n\n", __func__);
-
 }
 
 /* Wait until we are registered to enable interrupts */
@@ -2321,6 +2320,7 @@ static int mxc_hdmi_fb_event(struct notifier_block *nb,
 		return 0;
 
 	switch (val) {
+
 	case FB_EVENT_FB_REGISTERED:
 		dev_dbg(&hdmi->pdev->dev, "event=FB_EVENT_FB_REGISTERED\n");
 		mxc_hdmi_fb_registered(hdmi);
